package cvj2.app;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.cli.BasicParser;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.lang3.SystemUtils;

import cvj2.app.gui.util.MsgBox;

/**
 * コマンドライン解析
 */
public class ParseArg {
  /** 設定ファイルディレクトリ */
  public static File configDir = new File(SystemUtils.getUserHome(), ".ComicViewerJ2").getAbsoluteFile();
  /** 初期表示ファイル */
  public static List<File> initDirs = new ArrayList<File>();
  /** アプリケーションのインストールされているディレクトリ */
  public static File appDir = SystemUtils.getUserDir().getAbsoluteFile();
  /** 最大メモリ設定ファイル */
  public static File javaMaxMemFile = new File(SystemUtils.getUserDir(), "JAVA_MAX_MEM.TXT").getAbsoluteFile();
  
  private ParseArg() {}
  
  /**
   * コマンドライン解析。<br>
   * (エラーが起きたときは、メソッド内部でメッセージボックスを表示する)
   * @return エラーが起きたら、false
   */
  public static boolean parse(String...args) {
    Options opt = new Options();
    opt.addOption("p", true, "設定ファイルディレクトリ");
    
    CommandLine commandLine;
    try {
      commandLine = new BasicParser().parse(opt, args);
    } 
    catch (ParseException e) {
      MsgBox.error("コマンドライン引数エラー", "コマンドライン解析中にエラーが発生しました\n" + e.getMessage());
      return false;
    }

    if (commandLine.hasOption("p")) {
      String dirname = commandLine.getOptionValue("p");
      configDir = new File(dirname).getAbsoluteFile();
    }
    
    for (String f : commandLine.getArgs()) {
      initDirs.add(new File(f));
    }
    
    return true;
  }
}
