package cvj2.app.command;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.action.Action;

import cvj2.gui.swt.ui.input.InputCombination;
import cvj2.gui.swt.ui.input.KeyCombination;

/**
 * コマンドと JFaceのAction を結びつける。<br>
 * メニューの選択やツールバーボタンのクリックが行われると、
 * 結びつけられた {@link Command}が {@link CommandHandler} で実行される。 
 */
public class CommandMapAction {
  private class Element {
    Action action;
    Command command;
    Element(Action action, Command command) {
      this.action = action;
      this.command = command;
    }
  }
  private List<Element> list = new ArrayList<Element>();

  /**
   * 結びつけ追加
   */
  public void add(Action action, Command command) {
    list.add(new Element(action, command));
  }
  
  /**
   * コマンドから Action を探す
   */
  public Action findAction(Command command) {
    for (Element elm : list) {
      if (elm.command == command) return elm.action;
    }
    return null;
  }
  
  /**
   * Action からコマンドを探す
   */
  public Command findCommand(Action action) {
    for (Element elm : list) {
      if (elm.action == action) return elm.command;
    }
    return null;
  }
  
  /**
   * 全ての Action を再設定する
   * @param cmdMap 再設定するキー入力設定
   */
  public void resetActionStatus(CommandMapInput cmdMap) {
    for (Element elm : list) {
      resetActionStatus(cmdMap, elm.action, elm.command);
    }
  }

  private void resetActionStatus(CommandMapInput cmdMap, Action action, Command command) {
    // キー入力を探す
    Set<InputCombination> inputs = cmdMap.reverseFindInputs(command);
    KeyCombination keyInput = null;
    for (InputCombination input : inputs) {
      if (!(input instanceof KeyCombination)) continue;
      keyInput = (KeyCombination) input;
      break;
    }
    
    // 表示文字列
    String text;
    String tooltip;
    if (!command.toString().startsWith(Command.FILE_LAUNCH.toString())) {
      // 通常コマンドの場合
      text = command.getMenuName();
      tooltip = command.getName();
    }
    else {
      // 外部プログラムの場合
      text = action.getText();
      if (text.indexOf('@') > 0) text = text.substring(0, text.indexOf('@'));
      tooltip = text;
    }
    
    // 再設定
    if (keyInput != null) {
      action.setText(text + "@" + keyInput.toString());
      action.setAccelerator(keyInput.getKeyData());
    }
    else {
      action.setText(text);
      action.setAccelerator(0);
    }
    action.setToolTipText(tooltip);
  }
}
