package cvj2.app.command;

import java.util.Arrays;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

import cvj2.gui.swt.ui.input.InputCombination;

/**
 * コマンドと入力(キー、マウス)の結びつけ。<br>
 * 画面でキー入力などが発生すると、結びつけられた {@link Command}が {@link CommandHandler} で実行される。<br> 
 * このクラスのインスタンスは、@{link {@link DefaultCommandMapInputCreator} で生成される。<br>
 */
public class CommandMapInput extends TreeMap<InputCombination, Command> {
  private static final long serialVersionUID = 1L;

  /**
   * コマンド追加
   */
  public void addCommandInput(InputCombination input, Command command) {
    assert super.get(input) == null;
    super.put(input, command);
  }

  /**
   * コマンド削除
   */
  public void removeCommandInput(InputCombination input, Command command) {
    assert super.get(input) != null;
    super.remove(input);
  }
  
  /**
   * コマンドに対応する入力一覧を取得
   */
  public Set<InputCombination> reverseFindInputs(Command command) {
    Set<InputCombination> ret = new TreeSet<InputCombination>();
    for (InputCombination key : super.keySet()) {
      Command c = super.get(key);
      if (command == c) ret.add(key);
    }
    return ret;
  }
  
  /**
   * コマンド一覧を取得
   */
  public Command[] getAllCommands() {
    Command[] ret = super.values().toArray(new Command[0]);
    Arrays.sort(ret);
    return ret;
  }

  /**
   * 入力に対応するコマンドを取得
   */
  public Command findCommand(InputCombination input) {
    return super.get(input);
  }
}
