package cvj2.app.file;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;

import org.apache.commons.io.IOUtils;

import cvj2.app.ParseArg;
import cvj2.util.Log;

/**
 * 同じ設定ファイルから起動されたアプリケーションインスタンスを管理するためのロックファイル
 */
public class ApplicationLock {
  static private String LOCK_FILENAME = "lock.dat";
  static private ApplicationLock instance = new ApplicationLock();
  
  private RandomAccessFile file;
  
  private ApplicationLock() {}

  /**
   * インスタンス取得
   */
  public static ApplicationLock instance() {
    return instance;
  }
  
  /**
   * 初期化
   * @return 他にアプリが起動していないなら、true
   */
  public boolean initialize() {
    Log.start();
    boolean success = false;
    try {
      // ロックファイル
      File lockFile = new File(ParseArg.configDir, LOCK_FILENAME);
      lockFile.createNewFile();
      if (!lockFile.isFile()) {
        throw new IOException("ロックファイルを作成できません: " + lockFile);
      }
      
      // ロックする
      file = new RandomAccessFile(lockFile, "rws");
      if (null == file.getChannel().tryLock()) return false;

      success = true;
      return true;
    }
    catch (IOException e) {
      Log.d(e);
      return false;
    }
    finally {
      Log.d(success ? "lock successed" : "other application started");
      if (!success && file != null) {
        IOUtils.closeQuietly(file);
        file = null;
      }
    }
  }
  
  /**
   * 終了
   */
  public void close() throws IOException {
    if (file != null) file.close();
  }
  
  /**
   * このアプリが最初のアプリかどうか
   */
  public boolean isFirstApp() {
    return file != null;
  }
}
