package cvj2.app.file.thumbnail.fs;

import java.io.IOException;
import java.util.Date;

import cvj2.app.file.thumbnail.ThumbnailCache.Info;
import cvj2.app.gui.util.GlobalErrorHandler;
import cvj2.util.Log;

/**
 * FileCacheManager の初期化をバックグラウンドスレッドで行う。<br>
 * <pre> 
 * 以下の処理を行う
 * 1) 一覧にないのに存在するファイルを消す
 * 2) 一覧にあるのにファイルが存在しないときは一覧から消す
 * </pre>
 */
public class FileCacheBgInitializer extends Thread {
  private long startTime;
  private Info[] initialInfos;

  /**
   * コンストラクタ
   * @param initialInfos 起動時にファイルから読み込んだ全キャッシュ情報
   */
  public FileCacheBgInitializer(Info[] initialInfos) {
    this.initialInfos = initialInfos;
    this.startTime = new Date().getTime();

    this.setDaemon(true);
    this.setPriority(NORM_PRIORITY - 1);
    this.setName("FCBgInit");
  }

  /**
   * スレッド実行
   */
  @Override
  public void run() {
    Log.start();
    
    try {
      // ゴミファイル削除
      FileCacheLoader.clean(initialInfos, startTime);
      
      // 一覧に存在するファイルが実在するかどうか確認する
      for (Info i : initialInfos) {
        if (FileCacheLoader.isExist(i)) continue;
        Log.d("remove info: " + i);
        FileCacheManager.instance().remove(i);
      }
    }
    catch (IOException e) {
      Log.e(e);
      String msg = "サムネイルキャッシュファイルの初期化中に深刻なエラーが発生しました。\n" + 
                   "うまく動かない場合は、アプリケーションを終了させてから\n" + 
                   FileCacheManager.CACHE_DIR + " ディレクトリを削除してみてください。";
      GlobalErrorHandler.instance().setFatalMessage(msg, e);
    }
    finally {
      Log.d("finished");
      FileCacheManager.instance().bgInitializer = null;
    }
  }
}
