package cvj2.app.gui.data;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import cvj2.app.ParseArg;
import cvj2.util.Util;

/**
 * ブックマークデータ。<br>
 * XML読み書き用なので、getter, setter は自動生成
 */
public class Bookmark {
  private static final String filename = "bookmark.xml";
  
  private int focus = -1;
  private ArrayList<Elm> bookmarks = new ArrayList<Elm>();
  
  /**
   * ファイルロード
   */
  public static Bookmark load() throws IOException {
    File f = getFile();
    if (!f.exists()) return new Bookmark();
    Bookmark bm = (Bookmark) Util.readXml(f);
    
    // 取得できるか試してみる
    if (bm.getBookmarks().size() > 0) {
      Elm elm = bm.getBookmarks().get(0);
      elm.getType();
    }
    
    return bm;
  }

  /**
   * ファイルセーブ
   */
  public static void save(Bookmark bm) throws IOException {
    File f = getFile();
    Util.writeXml(f, bm);
  }
  
  private static File getFile() {
    return new File(ParseArg.configDir, filename);
  }

  /**
   * ブックマークの要素。<br>
   * XML読み書き用なので、getter, setter は自動生成
   */
  static public class Elm {
    private String path;
    private Type type;
    
    /**
     * ブックマーク種別
     */
    public enum Type {
      /** サムネイル表示 */
      THUMBNAIL, 
      /** リスト表示 */
      LIST, 
      /** モード指定なし */
      NONE,
    }

    /**
     * コンストラクタ
     */
    public Elm(String path, Type type) {
      this.path = path;
      this.type = type;
    }
    
    /**
     * @deprecated XML用
     */
    public Elm() {
      this("", Type.NONE);
    }

    public String getPath() { return path; }
    public void setPath(String path) { this.path = path; }
    public Type getType() { return type; }
    public void setType(Type type) { this.type = type; }
  }


  
  // 以下は自動生成
  
  public int getFocus() {
    return focus;
  }

  public void setFocus(int focus) {
    this.focus = focus;
  }

  public ArrayList<Elm> getBookmarks() {
    return bookmarks;
  }

  public void setBookmarks(ArrayList<Elm> bookmarks) {
    this.bookmarks = bookmarks;
  }
}
