package cvj2.app.gui.data;

import cvj2.app.Config;
import cvj2.file.fs.FileInfo;
import cvj2.file.fs.FileInfoComparator;
import cvj2.file.util.FileUtil;

/**
 * ListPanel で表示するファイルの列種別
 */
public enum FileListColumn {
  NAME,
  DATE,
  SIZE,
  EXT;
  
  /**
   * ファイル名から取得できる情報かどうか
   */
  public boolean isFilenameAttribute() {
    return this == NAME || this == EXT;
  }
  
  /**
   * 列情報でファイルを比較する
   */
  public int cmpare(FileInfo f1, FileInfo f2) {
    switch (this) {
    case NAME:   return f1.compareTo(f2);
    case DATE:   return Long.valueOf(f1.lastModified()).compareTo(f2.lastModified());
    case SIZE: {
      long l1 = f1.length();
      long l2 = f2.length();
      if (f1.isDirectory() && !f1.isArchiveRoot()) l1 = -1;
      if (f2.isDirectory() && !f2.isArchiveRoot()) l2 = -1;
      return Long.valueOf(l1).compareTo(l2);
    }
    case EXT: {
      String ext1 = FileUtil.splitBasenameExt(f1.getName())[1];
      String ext2 = FileUtil.splitBasenameExt(f2.getName())[1];
      if (Config.get().isMainWindowSortFileExtIgnoreCase()) {
        ext1 = ext1.toLowerCase();
        ext2 = ext2.toLowerCase();
      }
      return FileInfoComparator.customCmp.compare(ext1, ext2);
    }
    }
    throw new RuntimeException("??? " + this);
  }
  
  /**
   * 列の名前を取得する
   */
  public String getName() {
    switch (this) {
    case NAME: return "ファイル名";
    case DATE: return "更新日時";
    case EXT:  return "拡張子";
    case SIZE: return "サイズ";
    }
    throw new RuntimeException("??? " + this);
  }
  
  /**
   * 列表示幅を取得
   */
  public int getWidth() {
    Config conf = Config.get();
    switch (this) {
    case NAME: return conf.getMainWindowListWidthName();
    case DATE: return conf.getMainWindowListWidthDate();
    case EXT:  return conf.getMainWindowListWidthExt();
    case SIZE: return conf.getMainWindowListWidthSize();
    }
    throw new RuntimeException("??? " + this);
  }
}

