package cvj2.app.gui.data;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.widgets.Display;

import cvj2.app.file.imgload.BackgroundImageLoader;
import cvj2.app.file.imgload.CachedImage;
import cvj2.file.fs.FileInfo;
import cvj2.file.fs.FileInfoList;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel.ModelListener;
import cvj2.util.Log;

/**
 * ListPanel の画面データを管理する
 */
public class FileListModel implements GridPanel.Model {
  // カレントディレクトリ
  private FileInfo currentDir;
  // カレントディレクトリ内のファイル/ディレクトリ
  private FileInfoList fileList;
  // 表示列
  private FileListColumn[] columns;
  // ソートされたデータ
  private FileListSort sort;

  // データ更新リスナー
  private List<ModelListener> listeners = new ArrayList<GridPanel.ModelListener>();
  
  /**
   * コンストラクタ 
   */
  public FileListModel(FileInfo currentDir, FileListColumn[] columns) {
    this.currentDir = currentDir;
    this.fileList = currentDir.getList();
    this.columns = columns;
    this.sort = new FileListSort(fileList, columns);
  }
  
  /**
   * カレントディレクトリを取得する
   */
  public FileInfo getCurrentDir() {
    return currentDir;
  }
  
  /**
   * ソート前のリストを取得する
   */
  public FileInfoList getOriginalList() {
    return fileList;
  }
  
  /**
   * ソート済みのリストを取得する
   */
  public FileListSort getSortedList() {
    return sort;
  }

  /**
   * リスト表示の列の表示要素を取得する。<br>
   * ※) 先頭は、必ず NAME
   */
  public FileListColumn[] getColumns() {
    return columns;
  }
  
  /**
   * サムネイル画像ロード。<br>
   * ※) 読み込み完了の通知は、GridSelection と ListPanel に送られる
   */
  public CachedImage getThumbnail(FileInfo f) {
    return BackgroundImageLoader.instance().getThumbnail(f, new ImageNotify(), false);
  }
  private class ImageNotify implements BackgroundImageLoader.Notify {
    @Override
    public void stateChanged(final FileInfo f) {
      Log.start(f);
      Display.getDefault().asyncExec(new Runnable() {
        @Override
        public void run() {
          Throwable err = null;
          for (ModelListener listener : listeners) {
            try {
              listener.dataChanged(getSortedList().find(f));
            }
            catch (Throwable e) {
              Log.e(e);
              err = e;
            }
          }
          if (err != null) throw new RuntimeException(err);
        }
      });
    }
  }

  @Override
  public void addModelListener(ModelListener listener) {
    this.listeners.add(listener);
  }
  @Override
  public void removeModelListener(ModelListener listener) {    
    this.listeners.remove(listener);
  }

  @Override
  public Object get(int index) {
    return sort.get(index);
  }
  
  @Override
  public int size() {
    return fileList.size();
  }
}
