package cvj2.app.gui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.file.fs.FileInfo;

/**
 * サムネイル一括登録ダイアログ (条件設定)
 */
public class AddThumbnailDialog1 extends Dialog {
  private Text text_dir;
  private Button chkButton_dir;
  private Button chkButton_file;
  private Spinner spinner_dirDepth;
  
  private boolean initialized = false;
  private FileInfo dir;

  /** (登録条件) ディレクトリのサムネイルを登録する */
  public boolean checkDir;
  /** (登録条件) ファイルのサムネイルを登録する */
  public boolean checkFile;
  /** (登録条件) 登録ディレクトリの深さ */
  public int depth;
  
  /**
   * コンストラクタ
   */
  public AddThumbnailDialog1(Shell parentShell, FileInfo dir) {
    super(parentShell);
    setShellStyle(SWT.RESIZE | SWT.TITLE | SWT.PRIMARY_MODAL);
    this.dir = dir;
  }

  /**
   * ダイアログ初期化
   */
  private void initializeDialog(ControlEvent e) {
    if (initialized) return;
    initialized = true;
    
    getShell().setText("サムネイル一括登録");
    
    getText_dir().setText(dir.getAbsolutePath());
    
    getSpinner_dirDepth().setFocus();
  }

  /*
   * OKボタンが押された
   * @see org.eclipse.jface.dialogs.Dialog#okPressed()
   */
  @Override
  protected void okPressed() {
    this.checkDir = getChkButton_dir().getSelection();
    this.checkFile = getChkButton_file().getSelection();
    this.depth = getSpinner_dirDepth().getSelection();
    
    super.okPressed();
  }

  /**
   * Create contents of the dialog.
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initializeDialog(e);
      }
    });
    Composite container = (Composite) super.createDialogArea(parent);
    
    text_dir = new Text(container, SWT.BORDER | SWT.READ_ONLY);
    text_dir.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    text_dir.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel = new Label(container, SWT.NONE);
    lblNewLabel.setText("このディレクトリ配下の");
    
    Composite composite = new Composite(container, SWT.NONE);
    GridLayout gl_composite = new GridLayout(3, false);
    gl_composite.marginHeight = 0;
    composite.setLayout(gl_composite);
    
    Label lblNewLabel_2 = new Label(composite, SWT.NONE);
    lblNewLabel_2.setText("深さ");
    
    spinner_dirDepth = new Spinner(composite, SWT.BORDER);
    spinner_dirDepth.setMinimum(-1);
    spinner_dirDepth.setSelection(1);
    
    Label lblNewLabel_3 = new Label(composite, SWT.NONE);
    lblNewLabel_3.setText("までの");
    
    chkButton_dir = new Button(container, SWT.CHECK);
    chkButton_dir.setSelection(true);
    chkButton_dir.setText("画像ファイルを含むフォルダ (&D)");
    
    chkButton_file = new Button(container, SWT.CHECK);
    chkButton_file.setText("画像ファイル (&F)");
    
    Label lblNewLabel_1 = new Label(container, SWT.NONE);
    lblNewLabel_1.setText("のサムネイルを登録します。");
    
    Label lblNewLabel_4 = new Label(container, SWT.NONE);
    lblNewLabel_4.setText("  (深さ0はこのディレクトリのみ。深さ-1は制限なし)");

    return container;
  }

  /**
   * Create contents of the button bar.
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(466, 324);
  }

  private Text getText_dir() {
    return text_dir;
  }
  private Button getChkButton_dir() {
    return chkButton_dir;
  }
  private Button getChkButton_file() {
    return chkButton_file;
  }
  public Spinner getSpinner_dirDepth() {
    return spinner_dirDepth;
  }
}
