package cvj2.app.gui.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import cvj2.file.fs.FileInfo;

/**
 * サムネイル一括登録ダイアログ (実行)
 */
public class AddThumbnailDialog2 extends CommonDialog {
  private Label label_message;

  private boolean initialized = false;
  private AddThumbnailThread thread;
  private String resultMessage;
  
  /**
   * コンストラクタ
   */
  public AddThumbnailDialog2(Shell parentShell, FileInfo dir, boolean checkDir, boolean checkFile, int depth) {
    super(parentShell);
    setShellStyle(SWT.RESIZE | SWT.TITLE | SWT.PRIMARY_MODAL);
    
    // 処理開始
    this.thread = new AddThumbnailThread(this, dir, checkDir, checkFile, depth);
    thread.start();
  }

  /**
   * 処理開始
   */
  protected void initializeDialog(ControlEvent e) {
    if (initialized) return;
    initialized = true;
    
    getShell().setText("サムネイル一括登録中");
    
    updateMessage();
  }
  
  /**
   * 処理終了 (画面を閉じるか、スレッドが完了すると呼ばれる)
   */
  void finishExec(DisposeEvent e) {
    thread.cancel();
    try {
      thread.join();
    }
    catch (InterruptedException ex) {
      throw new RuntimeException(ex);
    }
    
    updateMessage();
  }

  /**
   * 画面のメッセージを更新する
   */
  public void updateMessage() {
    resultMessage = 
        "ディレクトリ数: " + thread.dirCount + "\n" +
        "画像ファイル数: " + thread.imageFileCount + "\n" +
        "画像以外のファイル数: " + thread.fileCount + "\n" +
        "\n" +
        "既に登録されていたディレクトリサムネイル数: " + thread.oldDirThumbnailCount + "\n" +
        "既に登録されていた画像ファイルサムネイル数: " + thread.oldImageThumbnailFileCount + "\n" +
        "\n" +
        "新たに登録されたディレクトリサムネイル数: " + thread.addDirThumbnailCount + "\n" +
        "新たに登録された画像ファイルサムネイル数: " + thread.addImageThumbnailFileCount + "";

    if (getShell() == null || getShell().isDisposed()) return;
    
    // 状況表示
    String msg = 
        "検索中: " + thread.currentFind + "\n" +
        "登録中: " + thread.currentLoad + "\n" +
        "\n" + resultMessage;
    getLabel_message().setText(msg);
    
    // タイマー実行
    getShell().getDisplay().timerExec(100, new Runnable() {
      @Override
      public void run() {
        updateMessage();
      }
    });
  }
  
  /**
   * 結果メッセージを取得する
   */
  public String getResultMessage() {
    return resultMessage;
  }

  /**
   * Create contents of the dialog.
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initializeDialog(e);
      }
    });
    parent.addDisposeListener(new DisposeListener() {
      public void widgetDisposed(DisposeEvent e) {
        finishExec(e);
      }
    });
    Composite container = (Composite) super.createDialogArea(parent);
    container.setLayout(new GridLayout(1, false));
    
    label_message = new Label(container, SWT.BORDER);
    label_message.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    label_message.setText(" ");

    return container;
  }

  /**
   * Create contents of the button bar.
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(574, 300);
  }
  
  private Label getLabel_message() {
    return label_message;
  }
}
