package cvj2.app.gui.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

import cvj2.app.gui.data.Bookmark;
import cvj2.app.gui.data.Bookmark.Elm;
import cvj2.app.gui.data.Bookmark.Elm.Type;
import cvj2.app.gui.util.MsgBox;
import cvj2.util.Log;

/**
 * ブックマーク設定ダイアログ
 */
public class BookmarkDialog extends CommonDialog {
  private Text text_edit;
  private List list_bookmarks;
  
  private Bookmark bm;
  private Bookmark.Elm bmElm;
  
  private boolean initialized = false;
  private Button radio_thumbnailMode;
  private Button radio_listMode;
  private Button radio_noMode;

  /**
   * コンストラクタ
   */
  public BookmarkDialog(Shell parentShell, Bookmark.Elm bmElm) {
    super(parentShell);
    setShellStyle(SWT.BORDER | SWT.RESIZE | SWT.TITLE | SWT.PRIMARY_MODAL);
    this.bmElm = bmElm;
  }

  /**
   * 画面初期化
   */
  private void initializeDialog(ControlEvent e) {
    if (initialized) return;
    initialized = true;
    Log.start();
    
    getShell().setText("ブックマーク編集");
    
    // ブックマークファイル読み込み
    bm = new Bookmark();
    try {
      bm = Bookmark.load();
    }
    catch (Exception ex) {
      Log.e(ex);
      MsgBox.error("エラー", "ブックマークファイルが読み込めませんでした\n" + ex.getMessage());
    }

    // 初期入力ファイル設定
    getText_edit().setText(bmElm.getPath());
    if (bmElm.getType() == Type.NONE) getRadio_noMode().setSelection(true);
    else if (bmElm.getType() == Type.LIST) getRadio_listMode().setSelection(true);
    else getRadio_thumbnailMode().setSelection(true);
    bmElm = null;
    
    // リスト初期化
    initList();
    
    getList_bookmarks().setFocus();
  }
  
  /*
   * リストの初期化
   */
  private void initList() {
    // リスト初期化
    getList_bookmarks().removeAll();
    
    // 要素挿入
    for (Bookmark.Elm elm : bm.getBookmarks()) {
      String type = "　";
      if (elm.getType() == Type.THUMBNAIL) type = "Ｔ";
      if (elm.getType() == Type.LIST) type = "Ｌ";
      getList_bookmarks().add(type + " " + elm.getPath());
    }
    
    // フォーカス設定
    if (bm.getFocus() >= 0 && bm.getFocus() < getList_bookmarks().getItemCount()) {
      getList_bookmarks().select(bm.getFocus());
    }
  }

  /*
   * 追加要素編集領域 トラバース イベントハンドラー
   */
  private void text_edit_keyTraversed(TraverseEvent e) {
    // Enter が押されたら、Add する
    if (e.stateMask == 0 && e.keyCode == SWT.CR) {
      button_add_widgetSelected(null);
      e.doit = false;
    }
  }
  
  /*
   * 追加ボタン イベントハンドラー
   */
  private void button_add_widgetSelected(SelectionEvent e) {
    Type type = Type.NONE;
    if (getRadio_thumbnailMode().getSelection()) type = Type.THUMBNAIL;
    if (getRadio_listMode().getSelection()) type = Type.LIST;
    Bookmark.Elm newElm = new Elm(getText_edit().getText(), type);
    
    int insert = getList_bookmarks().getSelectionIndex() + 1;
    bm.getBookmarks().add(insert, newElm);
    bm.setFocus(insert);
    
    initList();
    
    getList_bookmarks().setFocus();
  }

  /*
   * 削除ボタン イベントハンドラー
   */
  private void button_delete_widgetSelected(SelectionEvent e) {
    try {
      int focus = getList_bookmarks().getSelectionIndex();
      if (focus < 0) return;
      
      // 削除
      bm.getBookmarks().remove(focus);
      
      // フォーカス設定
      focus = Math.min(focus, bm.getBookmarks().size() - 1);
      if (focus >= 0) {
        bm.setFocus(focus);
      }
      
      initList();
    }
    finally {
      getList_bookmarks().setFocus();
    }
  }

  /*
   * ↑ボタン イベントハンドラー
   */
  private void button_up_widgetSelected(SelectionEvent e) {
    moveItem(-1);
  }

  /*
   * ↓ボタン イベントハンドラー
   */
  private void button_down_widgetSelected(SelectionEvent e) {
    moveItem(1);
  }

  /*
   * リストのキーイベントハンドラー
   */
  private void list_bookmarks_keyPressed(KeyEvent e) {
    if (e.stateMask != SWT.ALT) return;
    
    if (e.keyCode == SWT.ARROW_UP) moveItem(-1);
    if (e.keyCode == SWT.ARROW_DOWN) moveItem(1);
  }
  
  /*
   * 項目移動
   */
  private void moveItem(int direction) {
    try {
      int focus = getList_bookmarks().getSelectionIndex();
      if (focus < 0) return;
      
      int newFocus = focus + direction;
      newFocus = Math.max(0, newFocus);
      newFocus = Math.min(newFocus, bm.getBookmarks().size() - 1);
      if (focus == newFocus) return;
      
      Elm sel = bm.getBookmarks().get(focus);
      bm.getBookmarks().remove(focus);
      bm.getBookmarks().add(newFocus, sel);
      
      bm.setFocus(newFocus);

      initList();
    }
    finally {
      getList_bookmarks().setFocus();
    }
  }
  
  /**
   * OKボタン イベントハンドラ
   */
  @Override
  protected void okPressed() {
    // 選択されたファイルを憶えておく
    int focus = getList_bookmarks().getSelectionIndex();
    if (focus >= 0) {
      bmElm = bm.getBookmarks().get(focus);
      bm.setFocus(focus);
    }    

    // 保存
    try {
      Bookmark.save(bm);
    }
    catch (Exception e) {
      Log.e(e);
      MsgBox.error("エラー", "ブックマークファイルの保存に失敗しました\n" + e.getMessage());
      return;
    }

    super.okPressed();
  }
  
  /**
   * 選択されたブックマークを取得
   */
  public Bookmark.Elm getSelectBookmarkElm() {
    return bmElm;
  }

  /**
   * Create contents of the dialog.
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initializeDialog(e);
      }
    });
    Composite container = (Composite) super.createDialogArea(parent);
    
    Composite composite_1 = new Composite(container, SWT.NONE);
    GridLayout gl_composite_1 = new GridLayout(3, false);
    gl_composite_1.marginHeight = 0;
    composite_1.setLayout(gl_composite_1);
    
    radio_thumbnailMode = new Button(composite_1, SWT.RADIO);
    radio_thumbnailMode.setText("サムネイルモード(&T)");
    
    radio_listMode = new Button(composite_1, SWT.RADIO);
    radio_listMode.setText("リストモード(&L)");
    
    radio_noMode = new Button(composite_1, SWT.RADIO);
    radio_noMode.setText("モード指定なし(&N)");
    
    Composite composite_2 = new Composite(container, SWT.NONE);
    composite_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
    GridLayout gl_composite_2 = new GridLayout(2, false);
    gl_composite_2.marginHeight = 0;
    composite_2.setLayout(gl_composite_2);
    
    text_edit = new Text(composite_2, SWT.BORDER);
    text_edit.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Button button_add = new Button(composite_2, SWT.NONE);
    button_add.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_add_widgetSelected(e);
      }
    });
    button_add.setText("追加(&A)");
    text_edit.addTraverseListener(new TraverseListener() {
      public void keyTraversed(TraverseEvent e) {
        text_edit_keyTraversed(e);
      }
    });
    
    list_bookmarks = new List(container, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
    list_bookmarks.addKeyListener(new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent e) {
        list_bookmarks_keyPressed(e);
      }
    });
    list_bookmarks.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    
    Composite composite = new Composite(container, SWT.NONE);
    composite.setLayout(new GridLayout(5, false));
    
    Button button_delete = new Button(composite, SWT.NONE);
    button_delete.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_delete_widgetSelected(e);
      }
    });
    button_delete.setText("削除(&D)");
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("     ");
    
    Button button_up = new Button(composite, SWT.NONE);
    button_up.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_up_widgetSelected(e);
      }
    });
    button_up.setText("上へ ↑");
    
    Button button_down = new Button(composite, SWT.NONE);
    button_down.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_down_widgetSelected(e);
      }
    });
    button_down.setText("下へ ↓");
    
    Label lblNewLabel_1 = new Label(composite, SWT.NONE);
    lblNewLabel_1.setText("(リストを選択して、Alt + ↑↓ でも移動可能)");

    return container;
  }

  /**
   * Create contents of the button bar.
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(700, 400);
  }
  private Text getText_edit() {
    return text_edit;
  }
  private List getList_bookmarks() {
    return list_bookmarks;
  }
  private Button getRadio_thumbnailMode() {
    return radio_thumbnailMode;
  }
  private Button getRadio_listMode() {
    return radio_listMode;
  }
  private Button getRadio_noMode() {
    return radio_noMode;
  }
}
