package cvj2.app.gui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.widgets.Shell;

/**
 * 表示位置を設定ファイルに憶えておくDialogの抽象クラス
 */
public abstract class CommonDialog extends Dialog {
  private boolean initialized = false;

  /**
   * コンストラクタ
   */
  protected CommonDialog(Shell parentShell) {
    super(parentShell);
  }

  /**
   * ダイアログの初期処理
   */
  private void initializeDialog(ShellEvent e) {
    if (initialized) return;
    initialized = true;

    // 画面復元
    DialogUtil.setDialogBoundsFromConfig(CommonDialog.this);

    // 設定保存をタイマーで繰り返し呼ぶ
    saveConfigTimerExec();
  }

  /**
   * 画面設定保存
   */
  private void saveConfigTimerExec() {
    if (getShell() == null || getShell().isDisposed()) return;

    // 設定保存
    DialogUtil.setConfigFromDialogBounds(CommonDialog.this);
    
    // タイマーで自分を呼ぶ
    getShell().getDisplay().timerExec(250, new Runnable() {
      @Override public void run() {
        saveConfigTimerExec();
      }
    });
  }

  /**
   * ダイアログの初期化時に呼ばれるはず
   */
  @Override
  protected void initializeBounds() {
    super.initializeBounds();

    getShell().addShellListener(new ShellAdapter() {
      @Override
      public void shellActivated(ShellEvent e) {
        initializeDialog(e);
      }
    });
  }
}
