package cvj2.app.gui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Shell;

import cvj2.app.Config;
import cvj2.app.Config.WndState;
import cvj2.img.AWT2SWT;
import cvj2.img.SWT2AWT;

/**
 * ウインドウの状態を設定ファイルに保存、復元する
 */
public class DialogUtil {
  private DialogUtil() {}
  
  /**
   * 状態復元
   */
  public static void setDialogBoundsFromConfig(Dialog dlg) {
    restoreConfig(dlg.getClass(), dlg.getShell());
  }
  
  /**
   * 状態保存
   */
  public static void setConfigFromDialogBounds(Dialog dlg) {
    saveConfig(dlg.getClass(), dlg.getShell());
  }

  /**
   * 状態復元
   */
  public static void setWindowBoundsFromConfig(ApplicationWindow wnd) {
    restoreConfig(wnd.getClass(), wnd.getShell());
  }
  
  /**
   * 状態保存
   */
  public static void setConfigFromWindowBounds(ApplicationWindow wnd) {
    saveConfig(wnd.getClass(), wnd.getShell());
  }
  
  /*
   * 状態復元
   */
  private static void restoreConfig(Class<?> clazz, Shell shell) {
    // 設定を取得する
    String name = clazz.getSimpleName();
    WndState state = Config.get().getWindowStates().get(name);
    if (state == null) return;
    
    boolean isDialog = Dialog.class.isAssignableFrom(clazz);
    Rectangle rect = AWT2SWT.toRectangle(state.getBounds());
    
    // 取得した値を修正する
    if (isDialog) {
      // ダイアログの場合は、初期サイズより小さくはできない
      Rectangle orgBounds = shell.getBounds();
      if (rect.width < orgBounds.width) rect.width = orgBounds.width;
      if (rect.height < orgBounds.height) rect.height = orgBounds.height;
    }
    rect.x = Math.max(0, rect.x);
    rect.y = Math.max(0, rect.y);
    Rectangle display = shell.getDisplay().getBounds();
    display.width = Math.max(0, display.width - 100);
    display.height = Math.max(0, display.height - 100);
    if (rect.x > display.width) rect.x = display.width; 
    if (rect.y > display.height) rect.y = display.height; 
    
    shell.setBounds(rect);
    if (!isDialog) {
      if (state.isMaximized()) shell.setMaximized(true);
    }
  }
  
  /*
   * 設定保存
   */
  private static void saveConfig(Class<?> clazz, Shell shell) {
    String name = clazz.getSimpleName();
    WndState state = new WndState();
    state.setBounds(SWT2AWT.toRectangle(shell.getBounds()));
    state.setMaximized(shell.getMaximized());
    Config.get().getWindowStates().put(name, state);
  }
}
