package cvj2.app.gui.dialogs;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.time.StopWatch;
import org.eclipse.swt.widgets.Display;

import cvj2.app.gui.util.GlobalErrorHandler;
import cvj2.util.Log;

/**
 * ファイルプロパティダイアログの表示情報を取得するスレッド
 */
public class FilePropertyDialogThread extends Thread {
  private FilePropertyDialog dlg;
  private File startDir;
  private long size = 0;
  private int failCount = 0;
  private int fileCount = 0;
  private int dirCount = 0;
  private StopWatch sw = new StopWatch();
  private volatile boolean cancel = false;

  /**
   * コンストラクタ
   */
  public FilePropertyDialogThread(FilePropertyDialog dlg, File dir) {
    this.dlg = dlg;
    this.startDir = dir;
    super.setDaemon(true);
    super.setName("FileProp");
  }

  /**
   * キャンセル
   */
  public void cancel() {
    this.cancel = true;
  }
  
  /** 処理開始 */
  @Override public void run() {
    if (!startDir.exists() || !startDir.isDirectory()) return;
    sw.start();
    boolean success = false;
    try {
      dispSize(false);
      count(startDir);
      success = true;
    }
    catch (IOException e) {
      Log.e(e);
      GlobalErrorHandler.instance().setErrorMessage("ディレクトリのサイズを計測中にエラーが発生しました。", e);
    }
    finally {
      if (!success) size = -1L;
      dispSize(true);
    }
  }
  
  /* ディレクトリサイズ計測 */
  private void count(File dir) throws IOException {
    if (cancel) return;
    if (FileUtils.isSymlink(dir)) return;
    if (sw.getTime() > 100) {
      dispSize(false);
      sw.reset();
      sw.start();
    }
    File[] list = dir.listFiles();
    if (list == null) {
      failCount++;
      return;
    }
    for (File f : list) {
      if (f.isDirectory()) {
        dirCount++;
        count(f);
      }
      else {
        fileCount++;
        size += f.length();
      }
    }
  }
  
  /* 表示更新 */
  private void dispSize(final boolean finish) {
    Display.getDefault().asyncExec(new Runnable() {
      @Override public void run() {
        dlg.updateSize(size, failCount, dirCount, fileCount, finish); 
      }
    });
  }
  
}
