package cvj2.app.gui.dialogs;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;

/**
 * メッセージ付き入力ダイアログ。<br>
 * {@link InputDialog} だと、初期表示時のテキスト選択状態制御ができなかったのでこの画面を作成した。
 */
public class InputAndMessageDialog extends Dialog {
  private Text text_input;
  private Label label_message;

  private boolean initialized = false;
  private String title;
  private String input;
  private int selStart;
  private int selEnd;
  private String message;

  /**
   * コンストラクタ
   */
  public InputAndMessageDialog(Shell parentShell) {
    super(parentShell);
  }

  /**
   * 表示するデータの設定
   * @param input 入力初期値
   * @param selStart input の選択開始位置
   * @param selEnd input の選択終了位置 + 1
   */
  public void setData(String title, String input, int selStart, int selEnd, String message) {
    this.title = title;
    this.input = input;
    this.selStart = selStart;
    this.selEnd = selEnd;
    this.message = message;
  }

  /**
   * ダイアログの初期化
   */
  private void initDialog() {
    if (initialized) return;
    initialized = true;

    // タイトル
    getShell().setText(title);
    
    // 入力領域
    getText_input().setText(input);
    if (selStart >= 0 && selEnd >= 0) {
      getText_input().setSelection(selStart, selEnd);
    }
    
    // エラーメッセージ
    if (message != null) getLabel_message().setText(message);
  }

  /**
   * OKボタンが押された
   */
  @Override
  protected void okPressed() {
    input = getText_input().getText();
    super.okPressed();
    setReturnCode(SWT.OK);
  }
  
  /**
   * 入力されたファイル名を取得する
   */
  public String getInputString() {
    return input;
  }

  /**
   * Create contents of the dialog.
   * 
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initDialog();
      }
    });
    Composite container = (Composite) super.createDialogArea(parent);

    text_input = new Text(container, SWT.BORDER);
    text_input.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    label_message = new Label(container, SWT.NONE);
    label_message.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    label_message.setText(" ");

    return container;
  }

  /**
   * Create contents of the button bar.
   * 
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(492, 178);
  }

  /**
   * ファイル名が入った Text を取得する
   */
  private Text getText_input() {
    return text_input;
  }
  private Label getLabel_message() {
    return label_message;
  }
}
