package cvj2.app.gui.dialogs;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.TraverseEvent;
import org.eclipse.swt.events.TraverseListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.app.Main;
import cvj2.migemo.Migemo;
import cvj2.migemo.Sen;
import cvj2.migemo.SenString;
import cvj2.util.CnvRomaji2Kata;
import cvj2.util.Log;

/**
 * 検索文字列の解析テストを行うダイアログ
 */
public class SMSearchTestDialog extends CommonDialog {
  private Text text_afterAnalyze;
  private Text text_input;
  private Text text_inputKana;
  private Text text_migemoString;
  private Composite container;

  private Sen sen = new Sen(false);
  private Migemo migemo = new Migemo(Config.get().isMigemoIgnoreHiraKata(),  false);
  private StyledText text_beforeAnalyze;
  private boolean dialogInitialized = false;
  
  private static final int FIND_SEN = 0x01;
  private static final int FIND_MIGEMO = 0x02;
  
  /**
   * コンストラクタ
   */
  public SMSearchTestDialog(Shell parentShell) {
    super(parentShell);
    setShellStyle(SWT.CLOSE | SWT.RESIZE | SWT.APPLICATION_MODAL);
  }

  /**
   * ダイアログの初期化
   */
  private void initializeDialog() {
    if (dialogInitialized) return;
    dialogInitialized = true;

    getShell().setText("検索文字列判定テスト");
    
    Object text = Main.getClipboard().getContents(TextTransfer.getInstance());
    if (text != null) {
      text_beforeAnalyze.setText(text.toString());
      text_input.setFocus();
    }
  }

  /**
   * トラバースイベントハンドラー
   */
  private void text_input_keyTraversed(TraverseEvent e) {
    Log.start(e);
    
    // ローマ字入力中にESCが押されたら入力内容をクリアする
    if (e.keyCode == SWT.ESC && text_input.getText().length() > 0) {
      text_input.setText("");
      e.doit = false;
    }
  }

  /**
   * 文字列の更新ハンドラ
   */
  private void text_modifyText(ModifyEvent e) {
    String text = text_beforeAnalyze.getText();
    
    // 長さチェック
    if (text.length() > text_beforeAnalyze.getTextLimit()) {
      text_beforeAnalyze.setText(text.substring(0, text_beforeAnalyze.getTextLimit()));
      return;
    }

    // Sen 解析
    SenString ss = sen.analyze(text);
    text_afterAnalyze.setText(ss.toSimpleString("/"));

    // 入力変換
    String katakana = CnvRomaji2Kata.cnv(text_input.getText());
    text_inputKana.setText(katakana);

    // Sen 検索
    int[] analyzedList = new int[text.length()];
    for (int i = 0; i < ss.getElementSize(); i++) {
      int[] find = ss.find(katakana, i);
      if (find == null) break;
      setBits(analyzedList, FIND_SEN, find[0], find[1]);
    }
    
    // Migemo 検索
    migemo.setInputRomaji(text_input.getText());
    text_migemoString.setText(migemo.getMigemoStr());
    for (int i = 0; i < analyzedList.length; i++) {
      int[] find = migemo.foundIn(text.substring(i));
      if (find == null) break;
      setBits(analyzedList, FIND_MIGEMO, i + find[0], i + find[1]);
      i = i + find[1];
    }
    
    // 色付け
    text_beforeAnalyze.setStyleRange(null);
    for (int i = 0; i < analyzedList.length; i++) {
      int bits = analyzedList[i];
      if (bits == 0) continue;
      StyleRange style = new StyleRange();
      style.start = i;
      style.length = 1;
      if ((bits & FIND_SEN) != 0)    style.background = SWTResourceManager.getColor(SWT.COLOR_YELLOW);
      if ((bits & FIND_MIGEMO) != 0) style.foreground = SWTResourceManager.getColor(SWT.COLOR_RED);
      text_beforeAnalyze.setStyleRange(style);
    }
  }
  /*
   * 解析ビットを設定する
   */
  private void setBits(int[] analyzedList, int mask, int start, int end) {
    for (int i = start; i < end; i++) {
      analyzedList[i] |= mask;
    }
  }

  /**
   * Create contents of the dialog.
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initializeDialog();
      }
    });
    container = (Composite) super.createDialogArea(parent);
    
    Composite composite = new Composite(container, SWT.NONE);
    GridLayout gl_composite = new GridLayout(5, false);
    gl_composite.marginHeight = 0;
    gl_composite.horizontalSpacing = 0;
    composite.setLayout(gl_composite);
    new Label(composite, SWT.NONE);
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("検索対象の文章を入力 or 貼り付けしてください ( ");
    
    Label lblNewLabel_3 = new Label(composite, SWT.NONE);
    lblNewLabel_3.setBackground(SWTResourceManager.getColor(SWT.COLOR_YELLOW));
    lblNewLabel_3.setText(" 形態素解析で一致した文字列 ");
    
    Label lblNewLabel_5 = new Label(composite, SWT.NONE);
    lblNewLabel_5.setForeground(SWTResourceManager.getColor(SWT.COLOR_RED));
    lblNewLabel_5.setText(" Migemoで一致した文字列 ");
    
    Label lblNewLabel_7 = new Label(composite, SWT.NONE);
    lblNewLabel_7.setText(" )");
    
    text_beforeAnalyze = new StyledText(container, SWT.BORDER | SWT.WRAP | SWT.V_SCROLL);
    text_beforeAnalyze.setTextLimit(1024);
    text_beforeAnalyze.addModifyListener(new ModifyListener() {
      public void modifyText(ModifyEvent e) {
        text_modifyText(e);
      }
    });
    GridData gd_text_beforeAnalyze = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
    gd_text_beforeAnalyze.heightHint = 20;
    text_beforeAnalyze.setLayoutData(gd_text_beforeAnalyze);
    
    Label lblNewLabel_1 = new Label(container, SWT.NONE);
    lblNewLabel_1.setText("形態素解析した結果 ('/': 単語区切り、'[]': 複数の読み)");
    
    text_afterAnalyze = new Text(container, SWT.BORDER | SWT.READ_ONLY | SWT.WRAP | SWT.V_SCROLL | SWT.MULTI);
    text_afterAnalyze.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    GridData gd_text_afterAnalyze = new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1);
    gd_text_afterAnalyze.heightHint = 20;
    text_afterAnalyze.setLayoutData(gd_text_afterAnalyze);
    
    Label lblNewLabel_2 = new Label(container, SWT.NONE);
    lblNewLabel_2.setText("検索文字列をキーボード入力してください (ローマ字 あるいは、カナ入力)\n" + 
          "* 形態素解析で検索する場合は大文字小文字の区別はありません\n" + 
          "* Migemoで検索する場合は単語の区切りを大文字にしてください (例: '鳥山明の' -> 'toriyamaAkiraNo')");
    
    text_input = new Text(container, SWT.BORDER);
    text_input.addTraverseListener(new TraverseListener() {
      public void keyTraversed(TraverseEvent e) {
        text_input_keyTraversed(e);
      }
    });
    text_input.addModifyListener(new ModifyListener() {
      public void modifyText(ModifyEvent e) {
        text_modifyText(e);
      }
    });
    text_input.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_4 = new Label(container, SWT.NONE);
    lblNewLabel_4.setText("形態素解析した結果に対して検索する文字列");
    
    text_inputKana = new Text(container, SWT.BORDER | SWT.READ_ONLY);
    text_inputKana.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    text_inputKana.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_6 = new Label(container, SWT.NONE);
    lblNewLabel_6.setText("検索対象の文章に対して検索するMigemo正規表現");
    
    text_migemoString = new Text(container, SWT.BORDER | SWT.READ_ONLY);
    text_migemoString.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    text_migemoString.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    container.setTabList(new Control[]{text_beforeAnalyze, text_input});

    return container;
  }

  /**
   * Create contents of the button bar.
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(737, 565);
  }
}
