package cvj2.app.gui.dialogs;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.ArrayUtils;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import cvj2.app.file.imgload.BackgroundImageLoader;
import cvj2.app.gui.data.SystemMonitor;
import cvj2.app.gui.data.SystemMonitor.Data;
import cvj2.app.gui.util.MsgBox;
import cvj2.file.fs.FileInfoComparator;
import cvj2.migemo.Sen;

/**
 * システムモニター表示画面
 */
public class SystemMonitorDialog extends CommonDialog {
  private Table table;
  private TableItem tableItem_maxMem;    
  private TableItem tableItem_freeMem;    
  private TableItem tableItem_totalMem;    
  private TableItem tableItem_usedMem;    
  private TableItem tableItem_wait;    
  private TableItem tableItem_run;    
  private TableItem tableItem_thumbnailCacheNum;    
  private TableItem tableItem_imgCacheNum;    
  private TableItem tableItem_workMem;    
  private TableItem tableItem_pixelThroughput;    
  private TableItem tableItem_fileThroughput;    
  private TableItem tableItem_fileReadNum;

  private boolean initialized = false;
  private Button btnGc;
  private Composite composite;
  private Button btnCountMem;
  
  /**
   * コンストラクター
   */
  public SystemMonitorDialog(Shell parentShell) {
    super(parentShell);
    setShellStyle(SWT.RESIZE | SWT.TITLE);
  }

  /**
   * ダイアログ初期化
   */
  private void initializeDialog() {
    if (initialized) return;
    initialized = true;
    
    getShell().setText("システムモニター");
    
    updateData();
  }

  /*
   * 表示データの更新
   */
  private void updateData() {
    if (getShell() == null || getShell().isDisposed()) return;
    
    SystemMonitor.Data[] datas = SystemMonitor.instance().getDatas();
    ArrayUtils.reverse(datas);
    SystemMonitor.Data d0 = getDataStat(datas, 0, 1);
    SystemMonitor.Data d10 = getDataStat(datas, 0, 10);
    SystemMonitor.Data d60 = getDataStat(datas, 0, 60);
    SystemMonitor.Data d600 = getDataStat(datas, 0, 600);
    
    setTableData(0, d0);
    setTableData(1, d10);
    setTableData(2, d60);
    setTableData(3, d600);
    
    getShell().getDisplay().timerExec(1000, new Runnable() {
      @Override
      public void run() {
        updateData();
      }
    });
  }

  private void setTableData(int index, Data d) {
    tableItem_maxMem.setText           (index + 1, String.format("%4.2f", d.maxMemoryMB));    
    tableItem_freeMem.setText          (index + 1, String.format("%4.2f", d.freeMemoryMB));    
    tableItem_totalMem.setText         (index + 1, String.format("%4.2f", d.totalMemoryMB));    
    tableItem_usedMem.setText          (index + 1, String.format("%4.2f", d.totalMemoryMB - d.freeMemoryMB));    
    tableItem_wait.setText             (index + 1, String.format("%4.2f", d.bgWaitNum));    
    tableItem_run.setText              (index + 1, String.format("%4.2f", d.bgRunNum));    
    tableItem_thumbnailCacheNum.setText(index + 1, String.format("%4.2f", d.thumbnailCacheNum));    
    tableItem_imgCacheNum.setText      (index + 1, String.format("%4.2f", d.imageCacheNum));    
    tableItem_workMem.setText          (index + 1, String.format("%4.2f", d.bgWorkMemoryMB));    
    tableItem_pixelThroughput.setText  (index + 1, String.format("%4.2f", d.bgPixelThroughputMB));    
    tableItem_fileThroughput.setText   (index + 1, String.format("%4.2f", d.bgFileThroughtputMB));    
    tableItem_fileReadNum.setText      (index + 1, String.format("%4.2f", d.bgFileReadNum));
  }

  private Data getDataStat(Data[] datas, int start, int end) {
    int count = 0;
    Data sum = new Data(); 
    for (int i = start; i < datas.length && i < end; i++) {
      count++;
      Data d = datas[i];
      sum.maxMemoryMB += d.maxMemoryMB;
      sum.freeMemoryMB += d.freeMemoryMB;
      sum.totalMemoryMB += d.totalMemoryMB;
      sum.bgWaitNum += d.bgWaitNum;
      sum.bgRunNum += d.bgRunNum;
      sum.thumbnailCacheNum += d.thumbnailCacheNum;
      sum.imageCacheNum += d.imageCacheNum;
      sum.bgWorkMemoryMB += d.bgWorkMemoryMB;
      sum.bgPixelThroughputMB += d.bgPixelThroughputMB;
      sum.bgFileThroughtputMB += d.bgFileThroughtputMB;
      sum.bgFileReadNum += d.bgFileReadNum;
    }
    sum.maxMemoryMB /= count;
    sum.freeMemoryMB /= count;
    sum.totalMemoryMB /= count;
    sum.bgWaitNum /= count;
    sum.bgRunNum /= count;
    sum.thumbnailCacheNum /= count;
    sum.imageCacheNum /= count;
    sum.bgWorkMemoryMB /= count;
    sum.bgPixelThroughputMB /= count;
    sum.bgFileThroughtputMB /= count;
    sum.bgFileReadNum /= count;
    
    return sum;
  }

  /*
   * メモリー使用量概算
   */
  private void btnCountMem_widgetSelected(SelectionEvent e) {
    long thumb = BackgroundImageLoader.instance().getObjectBytesThumbnail();
    long view = BackgroundImageLoader.instance().getObjectBytesView();
    long sen = Sen.getCacheObjectsBytes();
    long sort = FileInfoComparator.customCmp.getObjectBytes();
    String msg = 
        "アプリケーションで使用しているメモリの概算\n" +
        "(画像ロード作業メモリ、PDF/ZIP解析キャッシュなどは入っていません)\n" +
        "  サムネイルキャッシュ: " + FileUtils.byteCountToDisplaySize(thumb) + "\n" +
        "  画像キャッシュ: " + FileUtils.byteCountToDisplaySize(view) + "\n" +
        "  形態素解析キャッシュ: " + FileUtils.byteCountToDisplaySize(sen) + "\n" +
        "  ファイル名ソートキャッシュ: " + FileUtils.byteCountToDisplaySize(sort) + "";
    MsgBox.info("メモリー使用量概算", msg);
  }

  /**
   * Create contents of the dialog.
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initializeDialog();
      }
    });
    Composite container = (Composite) super.createDialogArea(parent);
    
    table = new Table(container, SWT.BORDER | SWT.FULL_SELECTION);
    table.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    table.setHeaderVisible(true);
    table.setLinesVisible(true);
    
    TableColumn tblclmnNewColumn_4 = new TableColumn(table, SWT.NONE);
    tblclmnNewColumn_4.setWidth(384);
    tblclmnNewColumn_4.setText(" ");
    
    TableColumn tblclmnNewColumn_3 = new TableColumn(table, SWT.RIGHT);
    tblclmnNewColumn_3.setWidth(100);
    tblclmnNewColumn_3.setText("現在");
    
    TableColumn tblclmnNewColumn_2 = new TableColumn(table, SWT.RIGHT);
    tblclmnNewColumn_2.setWidth(100);
    tblclmnNewColumn_2.setText("過去10秒");
    
    TableColumn tblclmnNewColumn_1 = new TableColumn(table, SWT.RIGHT);
    tblclmnNewColumn_1.setWidth(100);
    tblclmnNewColumn_1.setText("過去1分");
    
    TableColumn tblclmnNewColumn = new TableColumn(table, SWT.RIGHT);
    tblclmnNewColumn.setWidth(100);
    tblclmnNewColumn.setText("過去10分");
    
    tableItem_maxMem = new TableItem(table, SWT.NONE);
    tableItem_maxMem.setText("Java最大メモリー(MB)");
    
    tableItem_freeMem = new TableItem(table, SWT.NONE);
    tableItem_freeMem.setText("Java空きメモリー(MB)");
    
    tableItem_totalMem = new TableItem(table, SWT.NONE);
    tableItem_totalMem.setText("Java割り当て済みメモリー(MB)");
    
    tableItem_usedMem = new TableItem(table, SWT.NONE);
    tableItem_usedMem.setText("Java使用中メモリー(MB)");
    
    tableItem_wait = new TableItem(table, SWT.NONE);
    tableItem_wait.setText("バックグラウンド読み込み待ち数");
    
    tableItem_run = new TableItem(table, SWT.NONE);
    tableItem_run.setText("バックグラウンド読み込み中のタスク数");
    
    tableItem_thumbnailCacheNum = new TableItem(table, SWT.NONE);
    tableItem_thumbnailCacheNum.setText("メモリーに保存されているサムネイル数");
    
    tableItem_imgCacheNum = new TableItem(table, SWT.NONE);
    tableItem_imgCacheNum.setText("メモリーに保存されている画像数");
    
    tableItem_workMem = new TableItem(table, SWT.NONE);
    tableItem_workMem.setText("バックグラウンドで画像読み込みに使用している作業メモリー(MB)");
    
    tableItem_pixelThroughput = new TableItem(table, SWT.NONE);
    tableItem_pixelThroughput.setText("バックグラウンドで処理している画像データのバイト数(MB/s)");
    
    tableItem_fileThroughput = new TableItem(table, SWT.NONE);
    tableItem_fileThroughput.setText("バックグラウンドで読み込んでいる画像ファイルのバイト数(MB/s)");
    
    tableItem_fileReadNum = new TableItem(table, SWT.NONE);
    tableItem_fileReadNum.setText("バックグラウンドで読み込んでいるファイル数(/s)");
    
    composite = new Composite(container, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    
    btnGc = new Button(composite, SWT.NONE);
    btnGc.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        System.gc();
      }
    });
    btnGc.setText("メモリー掃除");
    
    btnCountMem = new Button(composite, SWT.NONE);
    btnCountMem.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        btnCountMem_widgetSelected(e);
      }
    });
    btnCountMem.setText("メモリー使用量概算");

    return container;
  }

  /**
   * Create contents of the button bar.
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.OK_ID, IDialogConstants.CLOSE_LABEL, true);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(800, 400);
  }
}
