package cvj2.app.gui.dialogs;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.app.file.thumbnail.ThumbnailCache;
import cvj2.app.file.thumbnail.ThumbnailCache.Info;
import cvj2.app.gui.util.MsgBox;
import cvj2.file.fs.FileInfo;
import cvj2.gui.swt.util.GCX;
import cvj2.gui.swt.widgets.WidgetPaintUtil;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel.ModelListener;

/**
 * サムネイル管理ダイアログ
 */
public class ThumbnailDialog extends CommonDialog {
  private Text text_filter;
  private GridPanel gridPanel;

  private final String[] columnNames = { "更新ID", "サイズ", "パス", };
  private final String[] columnDatas = { " 00000000 ", " 9999 x 9999 ", "",  };
  private int[] columnWidths0 = { 0, 0, 0 };
  private int[] columnWidths = { 0, 0, 0 };
  private int listRowHeight;
  
  private Column column;
  private Data data;
  private CellPainter cellPainter;
  private int sortColumn = 2;
  private boolean sortAscend = true;
  private String filter = null;
  
  private FileInfo curDir;
  private boolean initialized = false;
  private Label label_comment;

  /**
   * コンストラクタ
   */
  public ThumbnailDialog(Shell parentShell, FileInfo curDir) {
    super(parentShell);
    setShellStyle(SWT.RESIZE | SWT.TITLE | SWT.PRIMARY_MODAL);
    this.curDir = curDir;
  }

  /**
   * 画面初期化
   */
  private void initializeDialog(ControlEvent e) {
    if (initialized) return;
    initialized = true;
    
    getShell().setText("サムネイルキャッシュファイル一覧");
    
    if (curDir != null) {
      getText_filter().setText(curDir.getAbsolutePath());
    }
   
    // 列クリック
    getGridPanel().setColumnSelectionListener(new GridPanel.ColumnSelectionListener() {
      @Override
      public void columnSelected(int columnIndex) {
        gridPanel_columnSelected(columnIndex);
      }
    });
    
    // 列幅、行高さの計算
    Config c = Config.get();
    GC gc = new GC(gridPanel);
    FontMetrics fm = gc.getFontMetrics();
    int fontHeight = fm.getHeight();
    for (int i = 0; i < columnNames.length; i++) {
      columnWidths0[i] = 0;
      columnWidths0[i] = Math.max(columnWidths0[i], gc.textExtent(columnNames[i]).x);
      columnWidths0[i] = Math.max(columnWidths0[i], gc.textExtent(columnDatas[i]).x);
      columnWidths0[i] += c.getMainWindowListMarginLeft() * 2;
    }
    gc.dispose();
    listRowHeight = fontHeight + c.getMainWindowListMarginTop() + c.getMainWindowListMarginBottom();
    
    // 初期表示
    button_refresh_widgetSelected(null);
    
    // 初期選択
    String find = getText_filter().getText();
    for (int i = 0; i < data.list.length; i++) {
      if (!data.list[i].info.absolutePath.startsWith(find)) continue;
      getGridPanel().getSelection().setFocusIndex(i);
      break;
    }
  }

  /*
   * 絞り込みボタン イベントハンドラー
   */
  private void button_filter_widgetSelected(SelectionEvent e) {
    filter = getText_filter().getText();
    refreshList();
  }

  /*
   * 更新ボタン イベントハンドラー
   */
  private void button_refresh_widgetSelected(SelectionEvent e) {
    filter = null;
    refreshList();
  }
  
  

  /*
   * 削除ボタン イベントハンドラー
   */
  private void button_delete_widgetSelected(SelectionEvent e) {
    try {
      for (int i : getGridPanel().getSelection().getSelectedIndices()) {
        ThumbnailCache.instance().remove(data.list[i].info);
      }
      refreshList();
    }
    catch (IOException ex) {
      MsgBox.error("エラー", "サムネイル削除中にエラーが発生しました。\n" + ex.getMessage());
    }
  }

  /*
   * 列クリックイベントハンドラー
   */
  private void gridPanel_columnSelected(int columnIndex) {
    if (sortColumn == columnIndex) {
      sortAscend = !sortAscend;
    }
    else {
      sortColumn = columnIndex;
      sortAscend = true;
    }
    getGridPanel().getHeader().redraw();
    refreshList();
  }

  /*
   * リストのキーイベントハンドラー
   */
  private void gridPanel_keyPressed(KeyEvent e) {
    getGridPanel().traverse(SWT.TRAVERSE_NONE, e);
    
    if (e.stateMask == SWT.ALT) {
      if (e.keyCode == '1') gridPanel_columnSelected(0);
      if (e.keyCode == '2') gridPanel_columnSelected(1);
      if (e.keyCode == '3') gridPanel_columnSelected(2);
    }
    
    if (e.stateMask == 0) {
      if (e.keyCode == SWT.DEL) button_delete_widgetSelected(null);
      if (e.keyCode == SWT.F5) button_refresh_widgetSelected(null);
    }
  }
  
  /*
   * 表示更新
   */
  private void refreshList() {
    // データ取得
    Info[] infos = ThumbnailCache.instance().getAll();
    ArrayList<RowData> list = new ArrayList<RowData>();
    for (int i = 0; i < infos.length; i++) {
      RowData d = new RowData();
      d.id = i + 1;
      d.info = infos[i];
      if (filter != null && !d.info.absolutePath.startsWith(filter)) continue;
      list.add(d);
    }
    // ソート
    Collections.sort(list, new Comparator<RowData>() {
      @Override
      public int compare(RowData r1, RowData r2) {
        if (sortColumn == 0) {
          return Integer.valueOf(r1.id).compareTo(r2.id) * (sortAscend ? 1 : -1);
        }
        if (sortColumn == 1) {
          int a1 = r1.info.targetSize.width * r1.info.targetSize.height;
          int a2 = r2.info.targetSize.width * r2.info.targetSize.height;
          int x = Integer.valueOf(a1).compareTo(a2) * (sortAscend ? 1 : -1);
          if (x != 0) return x;
        }
        if (sortColumn == 2) {
          int x = r1.info.absolutePath.compareTo(r2.info.absolutePath) * (sortAscend ? 1 : -1);
          if (x != 0) return x;
        }
        return Integer.valueOf(r1.id).compareTo(r2.id);
      }
    });
    // データをセットする
    this.data = new Data(list.toArray(new RowData[0]));
    this.column = new Column();
    this.cellPainter = new CellPainter();
    getGridPanel().setData(data, cellPainter, new Point(0, 0), column);

    getLabel_comment().setText((filter == null ? " " : filter + ": ") + list.size() + " 件");
    getGridPanel().setFocus();
  }

  // ============================================================================================================================
  
  /*
   * 列ヘッダー描画
   */
  private class Column implements GridPanel.ColumnPainter {
    @Override public int[] getColumnWidths(int headerWidth) {
      columnWidths[0] = columnWidths0[0];
      columnWidths[1] = columnWidths0[1];
      columnWidths[2] = headerWidth - columnWidths[0] - columnWidths[1];
      return columnWidths;
    }
    @Override public int getColumnHeight(Control control) {
      return listRowHeight;
    }
    @Override public Object getColumnData(int column) {
      return columnNames[column];
    }
    @Override public boolean paintControl(GC gc, Rectangle rect, Object data, int column, boolean isSelected) {
      Config c = Config.get();
      GCX gcx = new GCX(gc);
      WidgetPaintUtil.drawButtonBg(gcx, rect.x, rect.y, rect.width, rect.height, isSelected);
      gcx.setForeground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_FOREGROUND));
      gcx.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
      String str = "" + data + " " + (sortColumn != column ? "" : (sortAscend ? "△" : "▽"));
      gcx.drawString(str, rect.x + c.getMainWindowListMarginLeft(), rect.y);
      WidgetPaintUtil.drawButtonFg(gcx, rect.x, rect.y, rect.width, rect.height, isSelected);
      return true;
    }
  }

  private class RowData {
    int id;
    ThumbnailCache.Info info;
  }
  /*
   * データ
   */
  private class Data implements GridPanel.Model {
    RowData[] list;
    public Data(RowData[] list) {
      this.list = list;
    }
    @Override public void addModelListener(ModelListener listener) { }
    @Override public void removeModelListener(ModelListener listener) { }
    @Override public Object get(int index) {
      return list[index];
    }
    @Override public int size() {
      return list.length;
    }
  }
  
  /*
   * セル描画
   */
  private class CellPainter implements GridPanel.CellPainter {
    @Override
    public void paintControl(GC gc, Rectangle rect, Object data, int dataIndex, boolean isSelected, boolean hasFocus) {
      Config c = Config.get();
      GCX gcx = new GCX(gc);
      RowData row = (RowData) data;
      
      gcx.setForeground(SWTResourceManager.getColor(isSelected ? SWT.COLOR_LIST_SELECTION_TEXT : SWT.COLOR_LIST_FOREGROUND));
      gcx.setBackground(SWTResourceManager.getColor(isSelected ? SWT.COLOR_LIST_SELECTION : SWT.COLOR_LIST_BACKGROUND));
      gcx.fillRectangle(rect.x, rect.y, rect.width + 1, rect.height + 1);
      
      String id = "" + row.id + " ";
      String size = row.info.targetSize.width + " x " + row.info.targetSize.height;
      gcx.drawString(id, rect.x + columnWidths[0] - gcx.stringExtent(id).x - c.getMainWindowListMarginLeft(), rect.y);
      gcx.drawString(size, rect.x + columnWidths[0] + c.getMainWindowListMarginLeft(), rect.y);
      gcx.drawString(row.info.absolutePath, rect.x + columnWidths[0] + columnWidths[1] + c.getMainWindowListMarginLeft(), rect.y);
      if (hasFocus) {
        gcx.drawFocus(rect.x, rect.y, rect.width, rect.height);
      }
    }
    @Override
    public Point getCellSize() {
      return new Point(0, listRowHeight);
    }
  }

  // ============================================================================================================================
  
  /**
   * Create contents of the dialog.
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initializeDialog(e);
      }
    });
    Composite container = (Composite) super.createDialogArea(parent);
    
    Composite composite = new Composite(container, SWT.NONE);
    composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
    composite.setLayout(new GridLayout(2, false));
    
    text_filter = new Text(composite, SWT.BORDER);
    text_filter.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Button btnfilter = new Button(composite, SWT.NONE);
    btnfilter.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_filter_widgetSelected(e);
      }
    });
    btnfilter.setText("絞り込み(&F)");
    
    gridPanel = new GridPanel(container, SWT.BORDER);
    gridPanel.getCanvas().addKeyListener(new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent e) {
        gridPanel_keyPressed(e);
      }
    });
    gridPanel.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    new Label(gridPanel, SWT.NONE);
    
    Composite composite_1 = new Composite(container, SWT.NONE);
    composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
    composite_1.setLayout(new GridLayout(3, false));
    
    Button btnRrefresh = new Button(composite_1, SWT.NONE);
    btnRrefresh.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_refresh_widgetSelected(e);
      }
    });
    btnRrefresh.setText("再表示(&R)");
    
    Button btndelete = new Button(composite_1, SWT.NONE);
    btndelete.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_delete_widgetSelected(e);
      }
    });
    btndelete.setText("削除(&D)");
    
    label_comment = new Label(composite_1, SWT.NONE);
    label_comment.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    label_comment.setText(" ");

    return container;
  }

  /**
   * Create contents of the button bar.
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.OK_ID, IDialogConstants.CLOSE_LABEL, true);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(700, 520);
  }
  private Text getText_filter() {
    return text_filter;
  }
  private GridPanel getGridPanel() {
    return gridPanel;
  }
  private Label getLabel_comment() {
    return label_comment;
  }
}
