package cvj2.app.gui.dialogs.preference;

import java.util.Arrays;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.gui.swt.util.FontResourceManager;
import cvj2.img.AWT2SWT;
import cvj2.img.SWT2AWT;
import cvj2.util.Log;

/**
 * 基本設定ページ
 */
public class BasicPreferencePage extends CommonPreferencePage {
  private Text text_homeDir;
  private Label lblNewLabel_1;
  private Label lblNewLabel_2;
  private Button radio_newTabCurDir;
  private Text text_font;
  private Text text_imageLoadBackgroundColor;
  private Spinner spinner_thumbWidth;
  private Spinner spinner_thumbHeight;
  private Spinner spinner_thumbCacheNum;
  private Button radio_newTabHomeDir;
  private Spinner spinner_mainWindowListWidthName;
  private Spinner spinner_mainWindowListWidthExt;
  private Spinner spinner_mainWindowListWidthSize;
  private Spinner spinner_mainWindowListWidthDate;
  private Text text_imageBackgroundColor;
  
  private String[] fontDatas;
  private RGB rgbLd;
  private RGB rgbBg;
  private Button check_isDrawThumbnailBackground;
  private Spinner spinner_mainWindowListMarginTop;
  private Spinner spinner_mainWindowListMarginLeft;
  private Spinner spinner_mainWindowListMarginBottom;

  /**
   * コンストラクタ
   */
  public BasicPreferencePage() {
    setTitle("基本");
  }

  /*
   * フォント ボタンイベントハンドラー
   */
  private void button_font_widgetSelected(SelectionEvent e) {
    FontDialog dlg = new FontDialog(getShell());
    dlg.setFontList(FontResourceManager.getFontDatas(fontDatas));
    dlg.setText("フォント選択");
    if (null == dlg.open()) return;
    fontDatas = FontResourceManager.getFontDataStrings(dlg.getFontList());
    updateFontText();
  }
  private void updateFontText() {
    getText_font().setText(Arrays.deepToString(fontDatas));
  }

  /*
   * 背景色 ボタンイベントハンドラー
   */
  protected void button_imageBackgroundColor_widgetSelected(SelectionEvent e) {
    ColorDialog dlg = new ColorDialog(getShell());
    dlg.setRGB(rgbBg);
    dlg.setText("色選択");
    RGB ret = dlg.open();
    if (ret == null) return;
    rgbBg = ret;
    updateColorBg();
  }
  private void updateColorBg() {
    getText_imageBackgroundColor().setBackground(SWTResourceManager.getColor(rgbBg));
  }

  /*
   * 透過色 ボタンイベントハンドラー
   */
  private void button_imageLoadBackgroundColor_widgetSelected(SelectionEvent e) {
    ColorDialog dlg = new ColorDialog(getShell());
    dlg.setRGB(rgbLd);
    dlg.setText("色選択");
    RGB ret = dlg.open();
    if (ret == null) return;
    rgbLd = ret;
    updateColorLd();
  }
  private void updateColorLd() {
    getText_imageLoadBackgroundColor().setBackground(SWTResourceManager.getColor(rgbLd));
  }

  /*
   * ホームディレクトリ ボタンイベントハンドラー
   */
  private void button_homeDir_widgetSelected(SelectionEvent e) {
    DirectoryDialog dlg = new DirectoryDialog(getShell());
    dlg.setFilterPath(getText_homeDir().getText());
    dlg.setText("ディレクトリ選択");
    String s = dlg.open();
    if (s == null) return;
    getText_homeDir().setText(s);
  }

  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;

    Config c = ConfigDialog.getConfig();
    
    getSpinner_thumbWidth().setSelection(c.getMainWindowThumbnailSizeX());
    getSpinner_thumbHeight().setSelection(c.getMainWindowThumbnailSizeY());
    getSpinner_thumbCacheNum().setSelection(c.getThumnailFileCacheCount() / 1000);
    fontDatas = c.getFontDatas();
    updateFontText();
    getSpinner_mainWindowListMarginTop().setSelection(c.getMainWindowListMarginTop());
    getSpinner_mainWindowListMarginLeft().setSelection(c.getMainWindowListMarginLeft());
    getSpinner_mainWindowListMarginBottom().setSelection(c.getMainWindowListMarginBottom());
    getSpinner_mainWindowListWidthName().setSelection(c.getMainWindowListWidthName());
    getSpinner_mainWindowListWidthExt().setSelection(c.getMainWindowListWidthExt());
    getSpinner_mainWindowListWidthSize().setSelection(c.getMainWindowListWidthSize());
    getSpinner_mainWindowListWidthDate().setSelection(c.getMainWindowListWidthDate());
    rgbBg = AWT2SWT.toRGB(c.getImageBackgroundColor());
    updateColorBg();
    getCheck_isDrawThumbnailBackground().setSelection(c.isDrawThumbnailBackground());
    rgbLd = AWT2SWT.toRGB(c.getImageLoadBackgroundColor());
    updateColorLd();
    getText_homeDir().setText(c.getHomeDir());
    getRadio_newTabHomeDir().setSelection(!c.isMainWindowNewTabCurDirHandOver());
    getRadio_newTabCurDir().setSelection(c.isMainWindowNewTabCurDirHandOver());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
    
    c.setMainWindowThumbnailSizeX(getSpinner_thumbWidth().getSelection());
    c.setMainWindowThumbnailSizeY(getSpinner_thumbHeight().getSelection());
    c.setThumnailFileCacheCount(getSpinner_thumbCacheNum().getSelection() * 1000);
    c.setFontDatas(fontDatas);
    c.setMainWindowListMarginTop(getSpinner_mainWindowListMarginTop().getSelection());
    c.setMainWindowListMarginLeft(getSpinner_mainWindowListMarginLeft().getSelection());
    c.setMainWindowListMarginBottom(getSpinner_mainWindowListMarginBottom().getSelection());
    c.setMainWindowListWidthName(getSpinner_mainWindowListWidthName().getSelection());
    c.setMainWindowListWidthExt(getSpinner_mainWindowListWidthExt().getSelection());
    c.setMainWindowListWidthSize(getSpinner_mainWindowListWidthSize().getSelection());
    c.setMainWindowListWidthDate(getSpinner_mainWindowListWidthDate().getSelection());
    c.setImageBackgroundColor(SWT2AWT.toColor(rgbBg));
    c.setDrawThumbnailBackground(getCheck_isDrawThumbnailBackground().getSelection());
    c.setImageLoadBackgroundColor(SWT2AWT.toColor(rgbLd));
    c.setHomeDir(getText_homeDir().getText());
    c.setMainWindowNewTabCurDirHandOver(getRadio_newTabCurDir().getSelection());
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    GridLayout gl_composite = new GridLayout(2, false);
    gl_composite.verticalSpacing = 0;
    composite.setLayout(gl_composite);
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("サムネイルサイズ");
    
    Composite composite_1 = new Composite(composite, SWT.NONE);
    RowLayout rl_composite_1 = new RowLayout(SWT.HORIZONTAL);
    rl_composite_1.center = true;
    composite_1.setLayout(rl_composite_1);
    
    lblNewLabel_1 = new Label(composite_1, SWT.NONE);
    lblNewLabel_1.setText("幅(&W)");
    
    spinner_thumbWidth = new Spinner(composite_1, SWT.BORDER);
    spinner_thumbWidth.setMaximum(512);
    spinner_thumbWidth.setMinimum(16);
    
    lblNewLabel_2 = new Label(composite_1, SWT.NONE);
    lblNewLabel_2.setText("× 高(&H)");
    
    spinner_thumbHeight = new Spinner(composite_1, SWT.BORDER);
    spinner_thumbHeight.setMaximum(512);
    spinner_thumbHeight.setMinimum(16);
    
    Label lblNewLabel_5 = new Label(composite, SWT.NONE);
    lblNewLabel_5.setText("サムネイルキャッシュファイル数(&T)");
    
    Composite composite_5 = new Composite(composite, SWT.NONE);
    composite_5.setLayout(new GridLayout(2, false));
    
    spinner_thumbCacheNum = new Spinner(composite_5, SWT.BORDER);
    spinner_thumbCacheNum.setMaximum(1000);
    
    Label lblNewLabel_6 = new Label(composite_5, SWT.NONE);
    lblNewLabel_6.setText("×1000ファイル");
    
    Label label = new Label(composite, SWT.NONE);
    label.setText("フォント");
    
    Group composite_3 = new Group(composite, SWT.NONE);
    composite_3.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    GridLayout gl_composite_3 = new GridLayout(2, false);
    gl_composite_3.verticalSpacing = 0;
    composite_3.setLayout(gl_composite_3);
    
    Button button_font = new Button(composite_3, SWT.NONE);
    button_font.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_font_widgetSelected(e);
      }
    });
    button_font.setText("選択(&F)");
    
    text_font = new Text(composite_3, SWT.BORDER | SWT.READ_ONLY);
    text_font.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    text_font.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Composite composite_6 = new Composite(composite_3, SWT.NONE);
    composite_6.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    composite_6.setLayout(new GridLayout(6, false));
    
    Label label_3 = new Label(composite_6, SWT.NONE);
    label_3.setText("余白 上(&U)");
    
    spinner_mainWindowListMarginTop = new Spinner(composite_6, SWT.BORDER);
    
    Label label_4 = new Label(composite_6, SWT.NONE);
    label_4.setText("左(&L)");
    
    spinner_mainWindowListMarginLeft = new Spinner(composite_6, SWT.BORDER);
    
    Label label_5 = new Label(composite_6, SWT.NONE);
    label_5.setText("下(&B)");
    
    spinner_mainWindowListMarginBottom = new Spinner(composite_6, SWT.BORDER);
    
    Label lblNewLabel_9 = new Label(composite, SWT.NONE);
    lblNewLabel_9.setText("リスト表示の列幅");
    
    Group composite_8 = new Group(composite, SWT.NONE);
    composite_8.setLayout(new GridLayout(4, false));
    
    Label lblNewLabel_10 = new Label(composite_8, SWT.NONE);
    lblNewLabel_10.setText("ファイル名");
    
    Label lblNewLabel_11 = new Label(composite_8, SWT.NONE);
    lblNewLabel_11.setText("拡張子");
    
    Label lblNewLabel_12 = new Label(composite_8, SWT.NONE);
    lblNewLabel_12.setText("サイズ");
    
    Label lblNewLabel_13 = new Label(composite_8, SWT.NONE);
    lblNewLabel_13.setText("更新日時");
    
    spinner_mainWindowListWidthName = new Spinner(composite_8, SWT.BORDER);
    spinner_mainWindowListWidthName.setMaximum(10000);
    
    spinner_mainWindowListWidthExt = new Spinner(composite_8, SWT.BORDER);
    spinner_mainWindowListWidthExt.setMaximum(10000);
    
    spinner_mainWindowListWidthSize = new Spinner(composite_8, SWT.BORDER);
    spinner_mainWindowListWidthSize.setMaximum(10000);
    
    spinner_mainWindowListWidthDate = new Spinner(composite_8, SWT.BORDER);
    spinner_mainWindowListWidthDate.setMaximum(10000);
    
    Label label_2 = new Label(composite, SWT.NONE);
    label_2.setText("画像表示背景色");
    
    Group composite_9 = new Group(composite, SWT.NONE);
    composite_9.setLayout(new GridLayout(2, false));
    
    Button button_imageBackgroundColor = new Button(composite_9, SWT.NONE);
    button_imageBackgroundColor.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_imageBackgroundColor_widgetSelected(e);
      }
    });
    button_imageBackgroundColor.setText("選択(&B)");
    
    text_imageBackgroundColor = new Text(composite_9, SWT.BORDER | SWT.READ_ONLY);
    text_imageBackgroundColor.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    text_imageBackgroundColor.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, true, false, 1, 1));
    
    check_isDrawThumbnailBackground = new Button(composite_9, SWT.CHECK);
    check_isDrawThumbnailBackground.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    check_isDrawThumbnailBackground.setText("サムネイルの背景もこの色で描画する");
    
    Label label_1 = new Label(composite, SWT.NONE);
    label_1.setText("透過画像の背景色");
    
    Composite composite_4 = new Composite(composite, SWT.NONE);
    composite_4.setLayout(new GridLayout(2, false));
    
    Button button_imageLoadBackgroundColor = new Button(composite_4, SWT.NONE);
    button_imageLoadBackgroundColor.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_imageLoadBackgroundColor_widgetSelected(e);
      }
    });
    button_imageLoadBackgroundColor.setText("選択(&C)");
    
    text_imageLoadBackgroundColor = new Text(composite_4, SWT.BORDER | SWT.READ_ONLY);
    text_imageLoadBackgroundColor.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    text_imageLoadBackgroundColor.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_4 = new Label(composite, SWT.NONE);
    lblNewLabel_4.setText("ホームディレクトリ");
    
    Composite composite_2 = new Composite(composite, SWT.NONE);
    composite_2.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    GridLayout gl_composite_2 = new GridLayout(2, false);
    composite_2.setLayout(gl_composite_2);
    
    Button button_homeDir = new Button(composite_2, SWT.NONE);
    button_homeDir.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_homeDir_widgetSelected(e);
      }
    });
    button_homeDir.setText("選択(&R)");
    
    text_homeDir = new Text(composite_2, SWT.BORDER);
    text_homeDir.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_3 = new Label(composite, SWT.NONE);
    lblNewLabel_3.setText("新規タブのディレクトリ(&N)");
    
    Group composite_7 = new Group(composite, SWT.NONE);
    composite_7.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
    composite_7.setLayout(new GridLayout(1, false));
    
    radio_newTabHomeDir = new Button(composite_7, SWT.RADIO);
    radio_newTabHomeDir.setText("ホームディレクトリを開く");
    
    radio_newTabCurDir = new Button(composite_7, SWT.RADIO);
    radio_newTabCurDir.setText("元タブのカレントディレクトリを引き継ぐ");
    return composite;
  }

  private Spinner getSpinner_thumbWidth() {
    return spinner_thumbWidth;
  }
  private Spinner getSpinner_thumbHeight() {
    return spinner_thumbHeight;
  }
  private Text getText_font() {
    return text_font;
  }
  private Text getText_imageLoadBackgroundColor() {
    return text_imageLoadBackgroundColor;
  }
  private Text getText_homeDir() {
    return text_homeDir;
  }
  private Spinner getSpinner_thumbCacheNum() {
    return spinner_thumbCacheNum;
  }
  private Button getRadio_newTabHomeDir() {
    return radio_newTabHomeDir;
  }
  private Button getRadio_newTabCurDir() {
    return radio_newTabCurDir;
  }
  private Spinner getSpinner_mainWindowListWidthName() {
    return spinner_mainWindowListWidthName;
  }
  private Spinner getSpinner_mainWindowListWidthExt() {
    return spinner_mainWindowListWidthExt;
  }
  private Spinner getSpinner_mainWindowListWidthSize() {
    return spinner_mainWindowListWidthSize;
  }
  private Spinner getSpinner_mainWindowListWidthDate() {
    return spinner_mainWindowListWidthDate;
  }
  private Text getText_imageBackgroundColor() {
    return text_imageBackgroundColor;
  }
  private Button getCheck_isDrawThumbnailBackground() {
    return check_isDrawThumbnailBackground;
  }
  private Spinner getSpinner_mainWindowListMarginTop() {
    return spinner_mainWindowListMarginTop;
  }
  private Spinner getSpinner_mainWindowListMarginLeft() {
    return spinner_mainWindowListMarginLeft;
  }
  private Spinner getSpinner_mainWindowListMarginBottom() {
    return spinner_mainWindowListMarginBottom;
  }
}
