package cvj2.app.gui.dialogs.preference;

import org.eclipse.jface.preference.PreferenceNode;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.swt.graphics.Point;

/**
 * 設定ページの抽象クラス
 */
public abstract class CommonPreferencePage extends PreferencePage {
  private PreferenceNode node;
  
  /** 派生クラスのページで初期化されるときに true に設定される */
  protected boolean initialized = false;
  
  /**
   * ページに対応するノードを取得する
   */
  public PreferenceNode getNode() {
    return node;
  }

  /**
   * ページに対応するノードをセットする
   */
  public void setNode(PreferenceNode node) {
    this.node = node;
  }

  
  @Override
  protected Point doComputeSize() {
    getApplyButton().setText("適用");
    getDefaultsButton().setText("デフォルト");
    return super.doComputeSize();
  }

  /**
   * 入力データ検証
   */
  public abstract boolean validate();

  /**
   * 設定データで画面を更新する
   */
  public abstract void updateDialogFromData(boolean forceUpdate);

  /**
   * 画面入力で設定データを更新する
   */
  public abstract void updateDataFromDialog();

  /**
   * Apply ボタン が押された
   */
  @Override
  protected void performApply() {
    ConfigDialog.dlg.updateDataFromDialog();
  }
  
  /**
   * Default ボタン が押された
   */
  @Override
  protected void performDefaults() {
    ConfigDialog.dlg.updateDialogFromData(true);
  }
}
