package cvj2.app.gui.dialogs.preference;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.PreferenceDialog;
import org.eclipse.jface.preference.PreferenceManager;
import org.eclipse.jface.preference.PreferenceNode;
import org.eclipse.swt.widgets.Shell;

import cvj2.app.Config;
import cvj2.app.gui.window.main.MainWindowCommandHandler;

/**
 * 設定ダイアログ
 */
public class ConfigDialog extends PreferenceDialog {
  static private Config conf;
  static private List<CommonPreferencePage> pages = new ArrayList<CommonPreferencePage>();
  static private MainWindowCommandHandler cmdHandler;
  static ConfigDialog dlg;

  /**
   * ダイアログを表示する
   */
  static public void show(Shell shell, MainWindowCommandHandler cmdHandler) {
    // データ初期化
    ConfigDialog.cmdHandler = cmdHandler;
    conf = Config.get().clone();
    pages.clear();
    
    // ページ初期化
    PreferenceManager pm = new PreferenceManager();
    add(pm, new BasicPreferencePage());
    add(pm, new SystemPreferencePage());
    add(pm, new MainWindowPreferencePage());
    add(pm, new MainWindow2PreferencePage());
    add(pm, new MainWindow3PreferencePage());
    add(pm, new ViewWindowPreferencePage());
    add(pm, new ImageLoadPreferencePage());
    add(pm, new ImageLoad2PreferencePage());
    add(pm, new KeyMapPreferencePage(true));
    add(pm, new KeyMapPreferencePage(false));
    add(pm, new LaunchProgramsPreferencePage());
    add(pm, new TuningPreferencePage());
    add(pm, new DebugPreferencePage());

    // ダイアログ表示
    dlg = new ConfigDialog(shell, pm);
    dlg.open();
    pages.clear();
    dlg = null;
    return;
  }
  /*
   * ページ追加
   */
  static private void add(PreferenceManager pm, CommonPreferencePage page) {
    String nodeName = "node" + (pages.size() + 1);
    PreferenceNode node = new PreferenceNode(nodeName);
    page.setNode(node);
    node.setPage(page);
    pages.add(page);
    pm.addToRoot(node);
  }
 
  /**
   * 設定取得
   */
  static public Config getConfig() {
    return conf;
  }

  /**
   * ダイアログのデータを設定データに保存する
   */
  public boolean updateDataFromDialog() {
    for (CommonPreferencePage page : pages) {
      if (page.getControl() == null) continue;
      if (!page.validate()) {
        showPage(page.getNode());
        return false;
      }
    }
    for (CommonPreferencePage page : pages) {
      if (page.getControl() == null) continue;
      page.updateDataFromDialog();
    }
    
    // 設定更新
    Config.set(conf);
    // 画面更新
    cmdHandler.configChanged();
    
    return true;
  }
  
  /**
   * 設定データをダイアログ画面に表示する
   * @param initByDefault デフォルトの設定にする
   */
  public void updateDialogFromData(boolean initByDefault) {
    if (initByDefault) conf = Config.createDefault();

    for (CommonPreferencePage page : pages) {
      if (page.getControl() == null) continue;
      page.updateDialogFromData(initByDefault);
    }
  }

  /**
   * コンストラクタ
   */
  private ConfigDialog(Shell parentShell, PreferenceManager manager) {
    super(parentShell, manager);
  }
  
  @Override
  protected void update() {
    super.update();
    getShell().setText("設定");
    updateDialogFromData(false);
  }

  @Override
  protected void okPressed() {
    if (!updateDataFromDialog()) return;
    super.okPressed();
  }
}
