package cvj2.app.gui.dialogs.preference;

import java.io.File;

import org.apache.commons.io.FilenameUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import cvj2.app.Config;
import cvj2.app.ParseArg;
import cvj2.gui.swt.widgets.EnumList;
import cvj2.util.Log;

/**
 * デバッグ設定ページ
 */
public class DebugPreferencePage extends CommonPreferencePage {
  private Button check_debugMode;
  private EnumList<Log.Level> list_logLevel;
  private Text text_logFilename;
  private Button check_logStdOut;
  private Button check_logStdoutputSync;
  private Button check_logErrWarnToStderr;
  private Label lblNewLabel_2;
  
  /**
   * コンストラクタ
   */
  public DebugPreferencePage() {
    setTitle("デバッグ");
  }

  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();

    String s = getText_logFilename().getText();
    if (s.length() != 0) {
      File f = new File(ParseArg.configDir, getText_logFilename().getText());
      File f2 = new File(FilenameUtils.normalize(f.getAbsolutePath()));
      if (!ParseArg.configDir.equals(f2.getParentFile())) {
        setErrorMessage("このログファイル名は使用できません。");
        return false;
      }
    }
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;
    
    Config c = ConfigDialog.getConfig();
    
    getCheck_debugMode().setSelection(c.isDebugMode());
    getList_logLevel().initByEnum(Log.Level.values());
    getList_logLevel().setEnumSelection(c.getLogDebugLevel());
    getCheck_logStdOut().setSelection(c.isLogStdout());
    getCheck_logStdoutputSync().setSelection(c.isLogStdoutputSync());
    getCheck_logErrWarnToStderr().setSelection(c.isLogErrWarnToStderr());
    getText_logFilename().setText(c.getLogFilename());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
    
    c.setDebugMode(getCheck_debugMode().getSelection());
    c.setLogDebugLevel(getList_logLevel().getEnumSelection());
    c.setLogStdout(getCheck_logStdOut().getSelection());
    c.setLogStdoutputSync(getCheck_logStdoutputSync().getSelection());
    c.setLogErrWarnToStderr(getCheck_logErrWarnToStderr().getSelection());
    c.setLogFilename(getText_logFilename().getText());
  }

  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(1, false));
    
    lblNewLabel_2 = new Label(composite, SWT.NONE);
    lblNewLabel_2.setText("※) 普段はこの画面の設定は変更しないでください。");
    
    check_debugMode = new Button(composite, SWT.CHECK);
    check_debugMode.setText("デバッグ用のエラーメッセージボックスを画面に表示する");
    
    Label lblNewLabel_1 = new Label(composite, SWT.NONE);
    lblNewLabel_1.setText("ログ出力レベル");
    
    list_logLevel = new EnumList<Log.Level>(composite, SWT.BORDER);
    GridData gd_list_logLevel = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
    gd_list_logLevel.horizontalIndent = 20;
    list_logLevel.setLayoutData(gd_list_logLevel);
    list_logLevel.setItems(new String[] {"a", "b", "c", "d"});
    
    check_logStdOut = new Button(composite, SWT.CHECK);
    check_logStdOut.setText("ログをコンソールに出力する");
    
    check_logStdoutputSync = new Button(composite, SWT.CHECK);
    check_logStdoutputSync.setText("ログをコンソールに出力する度にフラッシュする");
    
    check_logErrWarnToStderr = new Button(composite, SWT.CHECK);
    check_logErrWarnToStderr.setText("ログをコンソールに出力するとき、WARN と ERR を標準エラー出力に出す");
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("ログファイル名 (設定ディレクトリ内に出力されます)");
    
    text_logFilename = new Text(composite, SWT.BORDER);
    GridData gd_text_logFilename = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
    gd_text_logFilename.horizontalIndent = 20;
    text_logFilename.setLayoutData(gd_text_logFilename);
    return composite;
  }
  
  private Button getCheck_debugMode() {
    return check_debugMode;
  }
  private EnumList<Log.Level> getList_logLevel() {
    return list_logLevel;
  }
  private Button getCheck_logStdOut() {
    return check_logStdOut;
  }
  private Button getCheck_logStdoutputSync() {
    return check_logStdoutputSync;
  }
  private Button getCheck_logErrWarnToStderr() {
    return check_logErrWarnToStderr;
  }
  private Text getText_logFilename() {
    return text_logFilename;
  }
}
