package cvj2.app.gui.dialogs.preference;

import java.awt.Image;
import java.awt.image.AffineTransformOp;
import java.util.ArrayList;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import cvj2.app.Config;
import cvj2.gui.swt.widgets.FloatText;
import cvj2.util.Log;

/**
 * 画質設定ページ
 */
public class ImageLoad2PreferencePage extends CommonPreferencePage {
  private FloatText text_imageSizeConverterChangeMethodZoomRatio;
  private Text text_imageSharpnessConverterParamsByZoomRatios;
  private Text text_imageSharpnessConverterParamsByZoomRatiosThumbnail;
  private Button check_imageSizeConverterScaledHint;
  private Button radio_imageSizeConverterAffineType1;
  private Button radio_imageSizeConverterAffineType2;
  private Button radio_imageSizeConverterAffineType3;

  /**
   * コンストラクタ
   */
  public ImageLoad2PreferencePage() {
    setTitle("画質");
  }

  /*
   * 輪郭強調パラメーター設定
   */
  private void setValues(Text text, double[][] ratios) {
    String s = "";
    for (int i = 0; i < ratios.length; i++) {
      s += "" + ratios[i][0] + "   " + ratios[i][1] + "\n";
    }
    text.setText(s);
  }
  
  /*
   * 輪郭強調パラメーター取得
   */
  private double[][] getValues(Text text, StringBuilder msg, String name) {
    int lineNum = 0;
    ArrayList<double[]> ret = new ArrayList<double[]>();
    for (String line : StringUtils.split(text.getText(), "\n")) {
      lineNum++;
      String[] vals = StringUtils.split(line, " \t");
      if (vals.length == 0) continue;
      if (vals.length != 2) {
        if (msg == null) throw new NumberFormatException();
        msg.append(name + " " + lineNum + "行: 値は2つ入力してください");
        return null;
      }
      double[] dvals = new double[2];
      for (int i = 0; i < vals.length; i++) {
        try {
          dvals[i] = Double.parseDouble(vals[i].trim());
        }
        catch (Exception e) {
          if (msg == null) throw new NumberFormatException();
          msg.append(name + " " + lineNum + "行: 値は数値で入力してください");
          return null;
        }
      }
      ret.add(dvals);
    }
    return ret.toArray(new double[0][]);
  }
  
  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();
    
    String msg = getText_imageSizeConverterChangeMethodZoomRatio().getInputErrorMessage("縮小率");
    if (msg != null) {
      setErrorMessage(msg);
      return false;
    }
    
    Text[] texts = {
        getText_imageSharpnessConverterParamsByZoomRatiosThumbnail(),
        getText_imageSharpnessConverterParamsByZoomRatios(),
    };
    String[] names = {
        "輪郭強調パラメーター (サムネイル)",
        "輪郭強調パラメーター (画像)",
    };
    for (int i = 0; i < texts.length; i++) {
      StringBuilder msgSb = new StringBuilder();
      getValues(texts[i], msgSb, names[i]);
      if (msgSb.length() > 0) {
        setErrorMessage(msgSb.toString());
        return false;
      }
    }
    
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;
    
    Config c = ConfigDialog.getConfig();

    getText_imageSizeConverterChangeMethodZoomRatio().setValue(c.getImageSizeConverterChangeMethodZoomRatio());
    getCheck_imageSizeConverterScaledHint().setSelection(c.getImageSizeConverterScaledHint() == Image.SCALE_AREA_AVERAGING);
    getRadio_imageSizeConverterAffineType1().setSelection(c.getImageSizeConverterAffineType() == AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
    getRadio_imageSizeConverterAffineType2().setSelection(c.getImageSizeConverterAffineType() == AffineTransformOp.TYPE_BILINEAR);
    getRadio_imageSizeConverterAffineType3().setSelection(c.getImageSizeConverterAffineType() == AffineTransformOp.TYPE_BICUBIC);
    setValues(getText_imageSharpnessConverterParamsByZoomRatios(), c.getImageSharpnessConverterParamsByZoomRatios());
    setValues(getText_imageSharpnessConverterParamsByZoomRatiosThumbnail(), c.getImageSharpnessConverterParamsByZoomRatiosThumbnail());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
   
    c.setImageSizeConverterChangeMethodZoomRatio(getText_imageSizeConverterChangeMethodZoomRatio().getValue());
    c.setImageSizeConverterScaledHint(getCheck_imageSizeConverterScaledHint().getSelection() ? Image.SCALE_AREA_AVERAGING : Image.SCALE_FAST);
    c.setImageSizeConverterAffineType(getRadio_imageSizeConverterAffineType1().getSelection() ? AffineTransformOp.TYPE_NEAREST_NEIGHBOR :
                                     (getRadio_imageSizeConverterAffineType2().getSelection() ? AffineTransformOp.TYPE_BILINEAR : AffineTransformOp.TYPE_BICUBIC));
    c.setImageSharpnessConverterParamsByZoomRatios(getValues(getText_imageSharpnessConverterParamsByZoomRatios(), null, null));
    c.setImageSharpnessConverterParamsByZoomRatiosThumbnail(getValues(getText_imageSharpnessConverterParamsByZoomRatiosThumbnail(), null, null));
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("サイズ変換方法(&S)");
    
    Group composite_1 = new Group(composite, SWT.NONE);
    composite_1.setLayout(new GridLayout(1, false));
    
    Composite composite_2 = new Composite(composite_1, SWT.NONE);
    GridLayout gl_composite_2 = new GridLayout(3, false);
    gl_composite_2.marginWidth = 0;
    gl_composite_2.marginHeight = 0;
    composite_2.setLayout(gl_composite_2);
    
    Label lblNewLabel_1 = new Label(composite_2, SWT.NONE);
    lblNewLabel_1.setText("縮小率");
    
    text_imageSizeConverterChangeMethodZoomRatio = new FloatText(composite_2, SWT.BORDER);
    text_imageSizeConverterChangeMethodZoomRatio.setMaximum(10.0);
    
    Label lblNewLabel_2 = new Label(composite_2, SWT.NONE);
    lblNewLabel_2.setText("倍");
    
    Label lblNewLabel_4 = new Label(composite_1, SWT.NONE);
    lblNewLabel_4.setText("これ未満なら、基本サイズ変換 を使用する");
    
    Label lblNewLabel_3 = new Label(composite_1, SWT.NONE);
    lblNewLabel_3.setText("これ以上なら、アフィン変換を使用する");
    
    Label lblNewLabel_5 = new Label(composite, SWT.NONE);
    lblNewLabel_5.setText("基本サイズ変換(&B)");
    
    Composite composite_3 = new Composite(composite, SWT.NONE);
    composite_3.setLayout(new GridLayout(1, false));
    
    check_imageSizeConverterScaledHint = new Button(composite_3, SWT.CHECK);
    check_imageSizeConverterScaledHint.setText("画素平均法を使用する (きれい)");
    
    Label lblNewLabel_6 = new Label(composite, SWT.NONE);
    lblNewLabel_6.setText("アフィン変換(&A)");
    
    Group composite_4 = new Group(composite, SWT.NONE);
    composite_4.setLayout(new GridLayout(1, false));
    
    radio_imageSizeConverterAffineType1 = new Button(composite_4, SWT.RADIO);
    radio_imageSizeConverterAffineType1.setText("最近傍補間 (はやい)");
    
    radio_imageSizeConverterAffineType2 = new Button(composite_4, SWT.RADIO);
    radio_imageSizeConverterAffineType2.setText("双一次(バイリニア)補間");
    
    radio_imageSizeConverterAffineType3 = new Button(composite_4, SWT.RADIO);
    radio_imageSizeConverterAffineType3.setText("バイキュービック補間 (きれい)");
    
    Label lblNewLabel_7 = new Label(composite, SWT.NONE);
    lblNewLabel_7.setText("輪郭強調パラメーター(&P)");
    
    Group composite_5 = new Group(composite, SWT.NONE);
    composite_5.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    composite_5.setLayout(new GridLayout(1, false));
    
    Composite composite_6 = new Composite(composite_5, SWT.NONE);
    composite_6.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    composite_6.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel_11 = new Label(composite_6, SWT.NONE);
    lblNewLabel_11.setText("サムネイル");
    
    Label lblNewLabel_12 = new Label(composite_6, SWT.NONE);
    lblNewLabel_12.setText("画像");
    
    text_imageSharpnessConverterParamsByZoomRatiosThumbnail = new Text(composite_6, SWT.BORDER | SWT.V_SCROLL | SWT.MULTI);
    text_imageSharpnessConverterParamsByZoomRatiosThumbnail.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    
    text_imageSharpnessConverterParamsByZoomRatios = new Text(composite_6, SWT.BORDER | SWT.V_SCROLL | SWT.MULTI);
    text_imageSharpnessConverterParamsByZoomRatios.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    
    Label lblNewLabel_8 = new Label(composite_5, SWT.NONE);
    lblNewLabel_8.setText("縮小率[xx]未満の時に、[xx]の値を使用する\nというフォーマットで1行毎に縮小率の小さい順に入力してください。");
    
    Label lblNewLabel_9 = new Label(composite_5, SWT.NONE);
    lblNewLabel_9.setText("値は、正の値が輪郭強調、負の値がぼかしです。");
    
    Label lblNewLabel_10 = new Label(composite_5, SWT.NONE);
    lblNewLabel_10.setText("おおよそ -1.0 ～ 1.0 の間で入力してください。");
    return composite;
  }
  private FloatText getText_imageSizeConverterChangeMethodZoomRatio() {
    return text_imageSizeConverterChangeMethodZoomRatio;
  }
  private Button getCheck_imageSizeConverterScaledHint() {
    return check_imageSizeConverterScaledHint;
  }
  private Button getRadio_imageSizeConverterAffineType1() {
    return radio_imageSizeConverterAffineType1;
  }
  private Button getRadio_imageSizeConverterAffineType2() {
    return radio_imageSizeConverterAffineType2;
  }
  private Button getRadio_imageSizeConverterAffineType3() {
    return radio_imageSizeConverterAffineType3;
  }
  private Text getText_imageSharpnessConverterParamsByZoomRatios() {
    return text_imageSharpnessConverterParamsByZoomRatios;
  }
  private Text getText_imageSharpnessConverterParamsByZoomRatiosThumbnail() {
    return text_imageSharpnessConverterParamsByZoomRatiosThumbnail;
  }
}
