package cvj2.app.gui.dialogs.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import cvj2.app.Config;
import cvj2.util.Log;
import org.eclipse.swt.layout.GridData;

/**
 * 画像ロード設定ページ
 */
public class ImageLoadPreferencePage extends CommonPreferencePage {
  private Text text_imageLoadPdfFitZoomRatio;
  private Spinner spinner_imageLoadMemCacheBytesThumbnail;
  private Spinner spinner_imageLoadMemCacheBytesView;
  private Spinner spinner_imageLoadBackgroundWorkMemory;
  private Spinner spinner_imageLoadMaxFileSizeBytes;
  private Spinner spinner_imageLoadMaxWidth;
  private Spinner spinner_imageLoadMaxHeight;
  private Spinner spinner_imageLoadPdfDpi;
  private Spinner spinner_viewWindowFitImageMaxLoadMagnification;

  /**
   * コンストラクタ
   */
  public ImageLoadPreferencePage() {
    setTitle("画像ロード");
  }
  
  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();

    try {
      double d = Double.parseDouble(getText_imageLoadPdfFitZoomRatio().getText().trim());
      if (d < 0.1 || 4.0 < d) throw new Exception();
    }
    catch (Exception e) {
      setErrorMessage("PDF読み込み倍率は 0.1～4.0 の間で設定してください");
      return false;
    }
    
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;
    
    Config c = ConfigDialog.getConfig();
    getSpinner_imageLoadMemCacheBytesThumbnail().setSelection((int)(c.getImageLoadMemCacheBytesThumbnail() / 1024 / 1024));
    getSpinner_imageLoadMemCacheBytesView().setSelection((int)(c.getImageLoadMemCacheBytesView() / 1024 / 1024));
    getSpinner_imageLoadBackgroundWorkMemory().setSelection((int)(c.getImageLoadBackgroundWorkMemory() / 1024 / 1024));
    getSpinner_imageLoadMaxFileSizeBytes().setSelection((int)(c.getImageLoadMaxFileSizeBytes() / 1024 / 1024));
    getSpinner_imageLoadMaxWidth().setSelection(c.getImageLoadMaxWidth());
    getSpinner_imageLoadMaxHeight().setSelection(c.getImageLoadMaxHeight());
    getSpinner_imageLoadPdfDpi().setSelection(c.getImageLoadPdfDpi());
    getText_imageLoadPdfFitZoomRatio().setText(String.format("%4.2f", c.getImageLoadPdfFitZoomRatio()));
    getSpinner_viewWindowFitImageMaxLoadMagnification().setSelection(c.getViewWindowFitImageMaxLoadMagnification());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
   
    c.setImageLoadMemCacheBytesThumbnail(getSpinner_imageLoadMemCacheBytesThumbnail().getSelection() * 1024L * 1024L);
    c.setImageLoadMemCacheBytesView(getSpinner_imageLoadMemCacheBytesView().getSelection() * 1024L * 1024L);
    c.setImageLoadBackgroundWorkMemory(getSpinner_imageLoadBackgroundWorkMemory().getSelection() * 1024L * 1024L);
    c.setImageLoadMaxFileSizeBytes(getSpinner_imageLoadMaxFileSizeBytes().getSelection() * 1024L * 1024L);
    c.setImageLoadMaxWidth(getSpinner_imageLoadMaxWidth().getSelection());
    c.setImageLoadMaxHeight(getSpinner_imageLoadMaxHeight().getSelection());
    c.setImageLoadPdfDpi(getSpinner_imageLoadPdfDpi().getSelection());
    c.setImageLoadPdfFitZoomRatio(Double.parseDouble(getText_imageLoadPdfFitZoomRatio().getText().trim()));
    c.setViewWindowFitImageMaxLoadMagnification(getSpinner_viewWindowFitImageMaxLoadMagnification().getSelection());
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("サムネイルキャッシュメモリ(&T)");
    
    Composite composite_1 = new Composite(composite, SWT.NONE);
    composite_1.setLayout(new GridLayout(2, false));
    
    spinner_imageLoadMemCacheBytesThumbnail = new Spinner(composite_1, SWT.BORDER);
    spinner_imageLoadMemCacheBytesThumbnail.setMaximum(2048);
    spinner_imageLoadMemCacheBytesThumbnail.setMinimum(32);
    
    Label lblNewLabel_5 = new Label(composite_1, SWT.NONE);
    lblNewLabel_5.setText("MB");
    
    Label lblNewLabel_1 = new Label(composite, SWT.NONE);
    lblNewLabel_1.setText("画像キャッシュメモリ(&I)");
    
    Composite composite_2 = new Composite(composite, SWT.NONE);
    composite_2.setLayout(new GridLayout(2, false));
    
    spinner_imageLoadMemCacheBytesView = new Spinner(composite_2, SWT.BORDER);
    spinner_imageLoadMemCacheBytesView.setMaximum(2048);
    spinner_imageLoadMemCacheBytesView.setMinimum(64);
    
    Label lblNewLabel_6 = new Label(composite_2, SWT.NONE);
    lblNewLabel_6.setText("MB");
    
    Label lblNewLabel_2 = new Label(composite, SWT.NONE);
    lblNewLabel_2.setText("画像ロード作業メモリの目安(&W)");
    
    Composite composite_3 = new Composite(composite, SWT.NONE);
    composite_3.setLayout(new GridLayout(2, false));
    
    spinner_imageLoadBackgroundWorkMemory = new Spinner(composite_3, SWT.BORDER);
    spinner_imageLoadBackgroundWorkMemory.setMaximum(2048);
    spinner_imageLoadBackgroundWorkMemory.setMinimum(96);
    
    Label lblNewLabel_7 = new Label(composite_3, SWT.NONE);
    lblNewLabel_7.setText("MB");
    
    Label lblNewLabel_3 = new Label(composite, SWT.NONE);
    lblNewLabel_3.setText("最大画像ファイルサイズ(&F)");
    
    Composite composite_4 = new Composite(composite, SWT.NONE);
    composite_4.setLayout(new GridLayout(2, false));
    
    spinner_imageLoadMaxFileSizeBytes = new Spinner(composite_4, SWT.BORDER);
    spinner_imageLoadMaxFileSizeBytes.setMaximum(2048);
    spinner_imageLoadMaxFileSizeBytes.setMinimum(1);
    
    Label lblNewLabel_8 = new Label(composite_4, SWT.NONE);
    lblNewLabel_8.setText("MB");
    
    Label lblNewLabel_4 = new Label(composite, SWT.NONE);
    lblNewLabel_4.setText("最大画像ピクセルサイズ(&S)");
    
    Composite composite_5 = new Composite(composite, SWT.NONE);
    composite_5.setLayout(new GridLayout(4, false));
    
    Label lblNewLabel_9 = new Label(composite_5, SWT.NONE);
    lblNewLabel_9.setText("幅");
    
    spinner_imageLoadMaxWidth = new Spinner(composite_5, SWT.BORDER);
    spinner_imageLoadMaxWidth.setMaximum(100000);
    spinner_imageLoadMaxWidth.setMinimum(1024);
    
    Label lblNewLabel_10 = new Label(composite_5, SWT.NONE);
    lblNewLabel_10.setText("× 高");
    
    spinner_imageLoadMaxHeight = new Spinner(composite_5, SWT.BORDER);
    spinner_imageLoadMaxHeight.setMaximum(100000);
    spinner_imageLoadMaxHeight.setMinimum(1024);
    
    Label lblNewLabel_13 = new Label(composite, SWT.NONE);
    lblNewLabel_13.setText("PDF読み込み解像度(&P)\n(原寸表示時)");
    
    Composite composite_7 = new Composite(composite, SWT.NONE);
    composite_7.setLayout(new GridLayout(2, false));
    
    spinner_imageLoadPdfDpi = new Spinner(composite_7, SWT.BORDER);
    spinner_imageLoadPdfDpi.setMaximum(1000);
    spinner_imageLoadPdfDpi.setMinimum(10);
    
    Label lblNewLabel_14 = new Label(composite_7, SWT.NONE);
    lblNewLabel_14.setText("dpi");
    
    Label lblNewLabel_12 = new Label(composite, SWT.NONE);
    lblNewLabel_12.setText("PDF読み込み倍率(&R)\n(画面フィット時)");
    
    Composite composite_6 = new Composite(composite, SWT.NONE);
    composite_6.setLayout(new GridLayout(2, false));
    
    text_imageLoadPdfFitZoomRatio = new Text(composite_6, SWT.BORDER);
    
    Label lblNewLabel_11 = new Label(composite_6, SWT.NONE);
    lblNewLabel_11.setText("倍");
    
    Label lblNewLabel_15 = new Label(composite, SWT.NONE);
    lblNewLabel_15.setText("画像ロード時の拡大率制限(&M)");
    
    Composite composite_8 = new Composite(composite, SWT.NONE);
    composite_8.setLayout(new GridLayout(2, false));
    
    spinner_viewWindowFitImageMaxLoadMagnification = new Spinner(composite_8, SWT.BORDER);
    spinner_viewWindowFitImageMaxLoadMagnification.setMaximum(4);
    spinner_viewWindowFitImageMaxLoadMagnification.setMinimum(1);
    
    Label lblNewLabel_16 = new Label(composite_8, SWT.NONE);
    lblNewLabel_16.setText("倍まで");
    
    Label lblNewLabel_17 = new Label(composite_8, SWT.NONE);
    lblNewLabel_17.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    lblNewLabel_17.setText("(ロード時に内部で画像を拡大しておく倍率です\n 大きくしすぎるとメモリを大量に消費します)");
    return composite;
  }
  private Spinner getSpinner_imageLoadMemCacheBytesThumbnail() {
    return spinner_imageLoadMemCacheBytesThumbnail;
  }
  private Spinner getSpinner_imageLoadMemCacheBytesView() {
    return spinner_imageLoadMemCacheBytesView;
  }
  private Spinner getSpinner_imageLoadBackgroundWorkMemory() {
    return spinner_imageLoadBackgroundWorkMemory;
  }
  private Spinner getSpinner_imageLoadMaxFileSizeBytes() {
    return spinner_imageLoadMaxFileSizeBytes;
  }
  private Spinner getSpinner_imageLoadMaxWidth() {
    return spinner_imageLoadMaxWidth;
  }
  private Spinner getSpinner_imageLoadMaxHeight() {
    return spinner_imageLoadMaxHeight;
  }
  private Spinner getSpinner_imageLoadPdfDpi() {
    return spinner_imageLoadPdfDpi;
  }
  private Text getText_imageLoadPdfFitZoomRatio() {
    return text_imageLoadPdfFitZoomRatio;
  }
  private Spinner getSpinner_viewWindowFitImageMaxLoadMagnification() {
    return spinner_viewWindowFitImageMaxLoadMagnification;
  }
}
