package cvj2.app.gui.dialogs.preference;

import java.util.ArrayList;
import java.util.Set;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;

import cvj2.app.command.Command;
import cvj2.app.command.CommandMapInput;
import cvj2.app.gui.util.MsgBox;
import cvj2.gui.swt.ui.input.InputCombination;

/**
 * キー入力編集ダイアログ
 */
public class KeyEditDialog extends Dialog {
  private List list;
  private Label label_cmdName;

  private boolean initialized = false;
  private Command cmd;
  private ArrayList<InputCombination> inputs;
  private CommandMapInput cmdMap;
  
  /**
   * コンストラクタ
   */
  public KeyEditDialog(Shell parentShell, Command cmd, Set<InputCombination> inputs, CommandMapInput cmdMap) {
    super(parentShell);
    setShellStyle(SWT.RESIZE | SWT.TITLE | SWT.APPLICATION_MODAL);
    
    this.cmd = cmd;
    this.inputs = new ArrayList<InputCombination>(inputs);
    this.cmdMap = cmdMap;
  }

  /**
   * 入力設定を取得
   */
  public java.util.List<InputCombination> getInputs() {
    return inputs;
  }

  /*
   * ダイアログ初期化
   */
  private void initializeDialog(ControlEvent e) {
    if (initialized) return;
    initialized = true;
    
    getShell().setText(" 入力編集");
    getLabel_cmdName().setText(cmd.getGroup() + " - " + cmd.getName());
    
    updateList();
  }
  
  /*
   * リスト更新
   */
  private void updateList() {
    getList().removeAll();
    for (InputCombination i : inputs) {
      getList().add(i.toString());
    }
  }

  /*
   * 追加ボタン イベントハンドラー
   */
  private void buton_add_widgetSelected(SelectionEvent e) {
    int sel = getList().getSelectionIndex();
    if (sel < 0) return;

    // ダイアログ表示
    KeyInputDialog dlg = new KeyInputDialog(getShell());
    if (Window.OK != dlg.open()) return;
    InputCombination input = dlg.getInput();
    if (input == null) return;
    
    // 重複チェック
    Command conflict = cmdMap.findCommand(input);
    if (conflict != null) {
      MsgBox.info("重複しています", "この入力は既に " + conflict.getGroup() + " - " + conflict.getName() + " で使用されています");
      return;
    }
    
    // 追加
    inputs.add(input);
    updateList();
    sel = Math.min(sel, inputs.size() - 1);
    getList().setSelection(sel);
    getList().setFocus();
  }

  /*
   * 削除ボタン イベントハンドラー
   */
  private void button_delete_widgetSelected(SelectionEvent e) {
    int sel = getList().getSelectionIndex();
    if (sel < 0) return;
    inputs.remove(sel);
    updateList();
    sel = Math.min(sel, inputs.size() - 1);
    getList().setSelection(sel);
    getList().setFocus();
  }
  
  /**
   * Create contents of the dialog.
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initializeDialog(e);
      }
    });
    Composite container = (Composite) super.createDialogArea(parent);
    
    label_cmdName = new Label(container, SWT.NONE);
    label_cmdName.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    label_cmdName.setText("New Label");
    
    list = new List(container, SWT.BORDER | SWT.V_SCROLL);
    list.setItems(new String[] {"a", "b", "c"});
    list.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    
    Composite composite = new Composite(container, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    
    Button buton_add = new Button(composite, SWT.NONE);
    buton_add.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        buton_add_widgetSelected(e);
      }
    });
    buton_add.setText("追加(&A)");
    
    Button button_delete = new Button(composite, SWT.NONE);
    button_delete.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_delete_widgetSelected(e);
      }
    });
    button_delete.setText("削除(&D)");

    return container;
  }

  /**
   * Create contents of the button bar.
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(450, 300);
  }
  private List getList() {
    return list;
  }
  private Label getLabel_cmdName() {
    return label_cmdName;
  }
}
