package cvj2.app.gui.dialogs.preference;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.gui.swt.ui.input.InputCombination;
import cvj2.gui.swt.ui.input.KeyCombination;
import cvj2.gui.swt.ui.input.MouseCombination;
import cvj2.gui.swt.widgets.DoubleBufferedCanvas;
import cvj2.util.Log;

import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.events.PaintEvent;

/**
 * 入力設定の入力を取得するダイアログ
 */
public class KeyInputDialog extends Dialog {
  private DoubleBufferedCanvas canvas;
  private Label label_result;

  private boolean initialized = false;
  private InputCombination input;
  
  /**
   * コンストラクタ
   */
  public KeyInputDialog(Shell parentShell) {
    super(parentShell);
    setShellStyle(SWT.RESIZE | SWT.TITLE | SWT.APPLICATION_MODAL);
  }

  /*
   * 画面初期化
   */
  protected void initializeDialog(ControlEvent e) {
    if (initialized) return;
    initialized = true;
    
    getShell().setText("入力");
  }

  /*
   * 描画
   */
  private void canvas_paintControl(PaintEvent e) {
    Log.start(e);
    String msg = "入力してください";
    Point pt = e.gc.stringExtent(msg);
    Rectangle rect = getCanvas().getClientArea();
    e.gc.drawRectangle(rect.x, rect.y, rect.width - 1, rect.height - 1);
    e.gc.drawString(msg, rect.x + (rect.width - pt.x) / 2, rect.y + (rect.height - pt.y) / 2); 
  }

  /*
   * マウスボタン イベントハンドラ
   */
  protected void canvas_mouseDown(MouseEvent e) {
    input = new MouseCombination(e);
    getLabel_result().setText(input.toString());
  }
  
  /*
   * キー入力イベントハンドラ
   */
  private void canvas_keyPressed(KeyEvent e) {
    input = new KeyCombination(e); 
    getLabel_result().setText(input.toString());
  }

  /*
   * 入力を取得
   */
  public InputCombination getInput() {
    return input;
  }
  
  /**
   * Create contents of the dialog.
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initializeDialog(e);
      }
    });
    Composite container = (Composite) super.createDialogArea(parent);
    
    canvas = new DoubleBufferedCanvas(container, SWT.NONE);
    canvas.addPaintListener(new PaintListener() {
      public void paintControl(PaintEvent e) {
        canvas_paintControl(e);
      }
    });
    canvas.addKeyListener(new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent e) {
        canvas_keyPressed(e);
      }
    });
    canvas.addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e) {
        canvas_mouseDown(e);
      }
    });
    canvas.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    canvas.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    
    label_result = new Label(container, SWT.BORDER);
    label_result.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    label_result.setText(" ");

    return container;
  }

  /**
   * Create contents of the button bar.
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(350, 328);
  }

  private Label getLabel_result() {
    return label_result;
  }
  public DoubleBufferedCanvas getCanvas() {
    return canvas;
  }
}
