package cvj2.app.gui.dialogs.preference;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;

import cvj2.app.Config;
import cvj2.app.gui.util.MsgBox;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;

/**
 * 外部起動プログラム編集ダイアログ
 */
public class LaunchProgramDialog extends Dialog {
  private Text text_name;
  private Text text_cmdarray;
  private Spinner spinner_maxFileArgs;
  private Button check_button;
  private Label label_cmdArray;
  
  private boolean initialized = false;
  private Config.LaunchProgram prog;

  /**
   * コンストラクタ
   */
  public LaunchProgramDialog(Shell parentShell, Config.LaunchProgram prog) {
    super(parentShell);
    setShellStyle(SWT.CLOSE | SWT.RESIZE | SWT.TITLE | SWT.APPLICATION_MODAL);
    
    this.prog = prog;
  }

  /**
   * 編集結果取得
   */
  public Config.LaunchProgram getProg() {
    return prog;
  }

  /*
   * 初期化
   */
  private void initializeDialog(ControlEvent e) {
    if (initialized) return;
    initialized = true;

    getShell().setText("外部起動プログラム設定");
    getText_name().setText(prog.getName());
    getSpinner_maxFileArgs().setSelection(prog.getMaxFileArgs());
    getCheck_button().setSelection(prog.isConfirm());
    getText_cmdarray().setText(prog.getCmdarray());
    getLabel_cmdArray().setText(
        "起動引数を改行で区切って入力してください\n" + 
            Config.LaunchProgram.REPLACE_SELECTED_FILES + "は、画面で選択されたファイル群に置き換えられます\n" +
            Config.LaunchProgram.REPLACE_CURRENT_DIR + "は、カレントディレクトリに置き換えられます"); 
    getShell().layout();
  }

  /**
   * OKボタンが押された
   */
  @Override
  protected void okPressed() {
    if (getText_name().getText().trim().length() == 0) {
      MsgBox.info("確認", "名前を入力してください");
    }
    if (getText_cmdarray().getText().trim().length() == 0) {
      MsgBox.info("確認", "コマンドを入力してください");
    }
    prog.setName(getText_name().getText().trim());
    prog.setMaxFileArgs(getSpinner_maxFileArgs().getSelection());
    prog.setConfirm(getCheck_button().getSelection());
    prog.setCmdarray(getText_cmdarray().getText().trim());
    
    super.okPressed();
  }
  
  /**
   * Create contents of the dialog.
   * @param parent
   */
  @Override
  protected Control createDialogArea(Composite parent) {
    parent.addControlListener(new ControlAdapter() {
      @Override
      public void controlResized(ControlEvent e) {
        initializeDialog(e);
      }
    });
    Composite container = (Composite) super.createDialogArea(parent);
    
    Composite composite = new Composite(container, SWT.NONE);
    composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    composite.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
    lblNewLabel.setText("名前");
    
    text_name = new Text(composite, SWT.BORDER);
    text_name.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Composite composite_1 = new Composite(container, SWT.NONE);
    composite_1.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_1 = new Label(composite_1, SWT.NONE);
    lblNewLabel_1.setText("選択ファイルの最大数");
    
    spinner_maxFileArgs = new Spinner(composite_1, SWT.BORDER);
    spinner_maxFileArgs.setMinimum(-1);
    
    Label lblNewLabel_2 = new Label(composite_1, SWT.NONE);
    lblNewLabel_2.setText("(-1 は制限なし)");
    
    Composite composite_2 = new Composite(container, SWT.NONE);
    composite_2.setLayout(new GridLayout(1, false));
    
    check_button = new Button(composite_2, SWT.CHECK);
    check_button.setText("起動時にメッセージボックスを表示して確認する");
    
    text_cmdarray = new Text(container, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL | SWT.CANCEL | SWT.MULTI);
    text_cmdarray.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    
    label_cmdArray = new Label(container, SWT.NONE);
    label_cmdArray.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    label_cmdArray.setText("New Label");

    return container;
  }

  /**
   * Create contents of the button bar.
   * @param parent
   */
  @Override
  protected void createButtonsForButtonBar(Composite parent) {
    createButton(parent, IDialogConstants.OK_ID, IDialogConstants.OK_LABEL, true);
    createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
  }

  /**
   * Return the initial size of the dialog.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(497, 459);
  }
  
  private Text getText_name() {
    return text_name;
  }
  private Spinner getSpinner_maxFileArgs() {
    return spinner_maxFileArgs;
  }
  private Button getCheck_button() {
    return check_button;
  }
  private Text getText_cmdarray() {
    return text_cmdarray;
  }
  private Label getLabel_cmdArray() {
    return label_cmdArray;
  }
}
