package cvj2.app.gui.dialogs.preference;

import java.util.ArrayList;
import java.util.Arrays;

import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;

import cvj2.app.Config;
import cvj2.util.Log;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;

/**
 * 外部起動プログラム設定ページ
 */
public class LaunchProgramsPreferencePage extends CommonPreferencePage {
  private ArrayList<Config.LaunchProgram> progs = new ArrayList<Config.LaunchProgram>();;
  private List list;
  
  /**
   * コンストラクタ
   */
  public LaunchProgramsPreferencePage() {
    setTitle("外部起動プログラム");
  }

  /*
   * 編集ボタン イベントハンドラ
   */
  private void button_edit_widgetSelected(SelectionEvent e) {
    try {
      int sel = getList().getSelectionIndex();
      if (sel < 0) return;
      
      LaunchProgramDialog dlg = new LaunchProgramDialog(getShell(), progs.get(sel));
      if (Window.OK != dlg.open()) return;
      progs.set(sel, dlg.getProg());
      updateList();
    }
    finally {
      getList().setFocus();
    }
  }

  /*
   * 追加ボタン イベントハンドラ
   */
  private void button_add_widgetSelected(SelectionEvent e) {
    try {
      Config.LaunchProgram prog = new Config.LaunchProgram();
      prog.setName("新規外部起動プログラム");
      prog.setCmdarray("program.exe\n" + "-param\n" + Config.LaunchProgram.REPLACE_SELECTED_FILES + "\n" + Config.LaunchProgram.REPLACE_CURRENT_DIR);
      prog.setMaxFileArgs(1);
      prog.setConfirm(true);
      
      LaunchProgramDialog dlg = new LaunchProgramDialog(getShell(), prog);
      if (Window.OK != dlg.open()) return;
      progs.add(dlg.getProg());
      updateList();
      getList().setSelection(progs.size() - 1);
    }
    finally {
      getList().setFocus();
    }
  }

  /*
   * 削除ボタン イベントハンドラ
   */
  private void button_delete_widgetSelected(SelectionEvent e) {
    try {
      int sel = getList().getSelectionIndex();
      if (sel < 0) return;
      
      progs.remove(sel);
      updateList();
    }
    finally {
      getList().setFocus();
    }
  }

  /*
   * 上下ボタン イベントハンドラ
   */
  private void list_move(int moveDirection) {
    try {
      int sel = getList().getSelectionIndex();
      if (sel < 0) return;

      Config.LaunchProgram p = progs.remove(sel);
      int insert = sel + moveDirection;
      insert = Math.min(insert, progs.size());
      insert = Math.max(0, insert);
      progs.add(insert, p);
      updateList();
      getList().setSelection(insert);
    }
    finally {
      getList().setFocus();
    }
  }
  /*
   * リストのキー イベントハンドラ
   */
  protected void list_keyPressed(KeyEvent e) {
    if (e.stateMask != SWT.ALT) return;
    if (e.keyCode == SWT.ARROW_UP) list_move(-1);
    if (e.keyCode == SWT.ARROW_DOWN) list_move(1);
  }

  /*
   * リストの更新
   */
  private void updateList() {
    int sel = getList().getSelectionIndex();
    getList().removeAll();
    for (int i = 0; i < progs.size(); i++) {
      getList().add("(" + (i + 1) + ") " + progs.get(i).getName());
    }
    if (sel > 0) getList().setSelection(sel);
  }

  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;

    Config c = ConfigDialog.getConfig();
    
    progs.clear();
    progs.addAll(Arrays.asList(c.getLaunchPrograms()));
    updateList();
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
    
    c.setLaunchPrograms(progs.toArray(new Config.LaunchProgram[0]));
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(1, false));
    
    Group group_1 = new Group(composite, SWT.NONE);
    group_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    group_1.setLayout(new GridLayout(1, false));
    
    Label lblNewLabel_1 = new Label(group_1, SWT.NONE);
    lblNewLabel_1.setText("外部起動プログラムの登録(&P)");
    
    list = new List(group_1, SWT.BORDER | SWT.V_SCROLL);
    list.addKeyListener(new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent e) {
        list_keyPressed(e);
      }
    });
    list.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    
    Composite composite_1 = new Composite(group_1, SWT.NONE);
    composite_1.setLayout(new GridLayout(6, false));
    
    Button button_edit = new Button(composite_1, SWT.NONE);
    button_edit.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_edit_widgetSelected(e);
      }
    });
    button_edit.setText("編集(&E)");
    
    Button button_add = new Button(composite_1, SWT.NONE);
    button_add.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_add_widgetSelected(e);
      }
    });
    button_add.setText("追加(&A)");
    
    Button button_delete = new Button(composite_1, SWT.NONE);
    button_delete.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_delete_widgetSelected(e);
      }
    });
    button_delete.setText("削除(&D)");
    
    Button button_up = new Button(composite_1, SWT.NONE);
    button_up.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        list_move(-1);
      }
    });
    button_up.setText("上へ↑");
    
    Button button_down = new Button(composite_1, SWT.NONE);
    button_down.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        list_move(1);
      }
    });
    button_down.setText("下へ↓");
    
    Label lblNewLabel_2 = new Label(composite_1, SWT.NONE);
    lblNewLabel_2.setText("(Alt + ↑↓ でも移動可能)");
    
    Label lblNewLabel = new Label(group_1, SWT.NONE);
    lblNewLabel.setText("ここで登録したプログラムは、ショートカットキーから起動できます。\n※) 変更結果をメニューに反映させるには、再起動してください。");
    return composite;
  }

  private List getList() {
    return list;
  }
}
