package cvj2.app.gui.dialogs.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import cvj2.app.Config;
import cvj2.file.fs.FileInfoComparator;
import cvj2.file.fs.FileInfoComparator.Type;
import cvj2.gui.swt.widgets.EnumList;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel.MouseScrollType;
import cvj2.util.Log;
import cvj2.migemo.Sen.MigemoDictUseType;

/**
 * メイン画面2設定ページ
 */
public class MainWindow2PreferencePage extends CommonPreferencePage {
  private EnumList<FileInfoComparator.Type> list_sort;
  private Button check_sortCustomIgnoreCase;
  private Button check_sortCustomIgnoreHiraKata;
  private Button check_sortCustomIgnoreZenHan;
  private Button check_sortCustomSymbolFirst;
  private Button check_sortCustomcnvSjis;
  private Label lblNewLabel;
  private Text text_mainWindowThumbnailSizes;
  private Label lblNewLabel_1;
  private Group group_1;
  private Label lblNewLabel_2;
  private EnumList<MouseScrollType> list_mainWindowMouseScrollType;
  private Label label_2;
  private EnumList<MigemoDictUseType> list_migemoDictType;
  private Label lblNewLabel_3;
  private Button check_migemoIgnoreHiraKata;

  /**
   * コンストラクタ
   */
  public MainWindow2PreferencePage() {
    setTitle("メイン画面2");
  }

  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;

    Config c = ConfigDialog.getConfig();

    getList_sort().initByEnum(FileInfoComparator.Type.values());
    getList_sort().setEnumSelection(c.getMainWindowSortFileEncode());
    getCheck_sortCustomIgnoreCase().setSelection(c.isMainWindowSortCustomIgnoreCase());
    getCheck_sortCustomIgnoreHiraKata().setSelection(c.isMainWindowSortCustomIgnoreHiraKata());
    getCheck_sortCustomIgnoreZenHan().setSelection(c.isMainWindowSortCustomIgnoreZenHan());
    getCheck_sortCustomSymbolFirst().setSelection(c.isMainWindowSortCustomSymbolFirst());
    getCheck_sortCustomcnvSjis().setSelection(c.isMainWindowSortCustomcnvSjis());
    getList_migemoDictType().initByEnum(MigemoDictUseType.values());
    getList_migemoDictType().setEnumSelection(c.getSenUseMigemoDict());
    getCheck_migemoIgnoreHiraKata().setSelection(c.isMigemoIgnoreHiraKata());
    getList_mainWindowMouseScrollType().initByEnum(MouseScrollType.values());
    getList_mainWindowMouseScrollType().setEnumSelection(c.getMainWindowMouseScrollType());
    getText_mainWindowThumbnailSizes().setText(c.getMainWindowThumbnailSizes());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
    
    c.setMainWindowSortFileEncode(getList_sort().getEnumSelection());
    c.setMainWindowSortCustomIgnoreCase(getCheck_sortCustomIgnoreCase().getSelection());
    c.setMainWindowSortCustomIgnoreHiraKata(getCheck_sortCustomIgnoreHiraKata().getSelection());
    c.setMainWindowSortCustomIgnoreZenHan(getCheck_sortCustomIgnoreZenHan().getSelection());
    c.setMainWindowSortCustomSymbolFirst(getCheck_sortCustomSymbolFirst().getSelection());
    c.setMainWindowSortCustomcnvSjis(getCheck_sortCustomcnvSjis().getSelection());
    c.setSenUseMigemoDict(getList_migemoDictType().getEnumSelection());
    c.setMigemoIgnoreHiraKata(getCheck_migemoIgnoreHiraKata().getSelection());
    c.setMainWindowMouseScrollType(getList_mainWindowMouseScrollType().getEnumSelection());
    c.setMainWindowThumbnailSizes(getText_mainWindowThumbnailSizes().getText());
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    
    Label label = new Label(composite, SWT.NONE);
    label.setText("ファイル名ソート順(&S)");
    
    Group group = new Group(composite, SWT.NONE);
    GridLayout gl_group = new GridLayout(1, false);
    gl_group.verticalSpacing = 0;
    group.setLayout(gl_group);
    
    list_sort = new EnumList<Type>(group, SWT.BORDER);
    list_sort.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    list_sort.setToolTipText("Unicodeの文字コード順: \n  ○ 高速\n  × 漢字の並びが読みと無関係になる\nJavaの自然言語比較順: \n  ○ Javaのおすすめ？\n  × Unicodeの4倍遅い\nカスタム比較: \n  ○ 細かい設定ができる\n  × Unicodeの2倍遅い\n");
    list_sort.setItems(new String[] {"a", "b", "c"});
    
    Label label_1 = new Label(group, SWT.NONE);
    label_1.setText("カスタムの場合の設定");
    
    check_sortCustomIgnoreCase = new Button(group, SWT.CHECK);
    check_sortCustomIgnoreCase.setText("大文字小文字の区別をしない");
    
    check_sortCustomIgnoreHiraKata = new Button(group, SWT.CHECK);
    check_sortCustomIgnoreHiraKata.setText("全角ひらがなと全角カタカナの区別をしない");
    
    check_sortCustomIgnoreZenHan = new Button(group, SWT.CHECK);
    check_sortCustomIgnoreZenHan.setText("アルファベットと数字の全角半角の区別をしない");
    
    check_sortCustomSymbolFirst = new Button(group, SWT.CHECK);
    check_sortCustomSymbolFirst.setText("記号で始まる名前を先頭に");
    
    check_sortCustomcnvSjis = new Button(group, SWT.CHECK);
    check_sortCustomcnvSjis.setText("シフトJIS文字コード順(漢字が音読み順になる)");
    
    label_2 = new Label(composite, SWT.NONE);
    label_2.setText("形態素解析時の\nMigemo辞書使用(&M)");
    
    list_migemoDictType = new EnumList<MigemoDictUseType>(composite, SWT.BORDER);
    list_migemoDictType.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    list_migemoDictType.setItems(new String[] {"a", "b", "c"});
    
    lblNewLabel_3 = new Label(composite, SWT.NONE);
    lblNewLabel_3.setText("Migemo検索時に");
    
    check_migemoIgnoreHiraKata = new Button(composite, SWT.CHECK);
    check_migemoIgnoreHiraKata.setText("ひらがなカタカナの区別をしない (新しく開いたタブから有効になります)");
    
    lblNewLabel_2 = new Label(composite, SWT.NONE);
    lblNewLabel_2.setText("マウスホイール\nスクロール(&M)");
    
    list_mainWindowMouseScrollType = new EnumList<MouseScrollType>(composite, SWT.BORDER);
    list_mainWindowMouseScrollType.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    list_mainWindowMouseScrollType.setItems(new String[] {"a", "b", "c"});
    
    lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("サムネイルサイズ\n切り替え候補(&T)");
    
    group_1 = new Group(composite, SWT.NONE);
    group_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    group_1.setLayout(new GridLayout(1, false));
    
    text_mainWindowThumbnailSizes = new Text(group_1, SWT.BORDER | SWT.V_SCROLL | SWT.MULTI);
    text_mainWindowThumbnailSizes.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
    
    lblNewLabel_1 = new Label(group_1, SWT.NONE);
    lblNewLabel_1.setText("幅と高さを小さい順に並べてください");
    return composite;
  }
  private EnumList<FileInfoComparator.Type> getList_sort() {
    return list_sort;
  }
  private Button getCheck_sortCustomIgnoreCase() {
    return check_sortCustomIgnoreCase;
  }
  private Button getCheck_sortCustomIgnoreHiraKata() {
    return check_sortCustomIgnoreHiraKata;
  }
  private Button getCheck_sortCustomIgnoreZenHan() {
    return check_sortCustomIgnoreZenHan;
  }
  private Button getCheck_sortCustomSymbolFirst() {
    return check_sortCustomSymbolFirst;
  }
  private Button getCheck_sortCustomcnvSjis() {
    return check_sortCustomcnvSjis;
  }
  private Text getText_mainWindowThumbnailSizes() {
    return text_mainWindowThumbnailSizes;
  }
  private EnumList<MouseScrollType> getList_mainWindowMouseScrollType() {
    return list_mainWindowMouseScrollType;
  }
  private EnumList<MigemoDictUseType> getList_migemoDictType() {
    return list_migemoDictType;
  }
  private Button getCheck_migemoIgnoreHiraKata() {
    return check_migemoIgnoreHiraKata;
  }
}
