package cvj2.app.gui.dialogs.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

import cvj2.app.Config;
import cvj2.util.Log;

/**
 * メイン画面3設定ページ
 */
public class MainWindow3PreferencePage extends CommonPreferencePage {
  private Spinner spinner_mainWindowThumbnailPrefetchPage;
  private Spinner spinner_mainWindowThumbnailSearchDirDepth;
  private Spinner spinner_windowStatusClearTimeMillisec;
  private Spinner spinner_mainWindowMigemoTooltipHideDelay;
  private Button radio_mainWindowMigemoTooltipUseSWT;
  private Button radio_mainWindowMigemoTooltipUseJface;
  private Button check_mainWindowListShowIcon;
  private Button check_mainWindowThumbnailShowIcon;
  private Spinner spinner_mainWindowMigemoKeyTimeout;
  private Button radio_mainWindowRenameSelectNewNameTrue;
  private Button radio_mainWindowRenameSelectNewNameFalse;
  private Button check_mainWindowUseOsIcon;

  /**
   * コンストラクタ
   */
  public MainWindow3PreferencePage() {
    setTitle("メイン画面3");
  }

  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;

    Config c = ConfigDialog.getConfig();
    
    getSpinner_mainWindowThumbnailPrefetchPage().setSelection(c.getMainWindowThumbnailPrefetchPage());
    getSpinner_mainWindowThumbnailSearchDirDepth().setSelection(c.getMainWindowThumbnailSearchDirDepth());
    getSpinner_windowStatusClearTimeMillisec().setSelection(c.getWindowStatusClearTimeMillisec());
    getSpinner_mainWindowMigemoKeyTimeout().setSelection(c.getMainWindowMigemoKeyTimeout());
    getSpinner_mainWindowMigemoTooltipHideDelay().setSelection(c.getMainWindowMigemoTooltipHideDelay());
    getRadio_mainWindowMigemoTooltipUseSWT().setSelection(c.isMainWindowMigemoTooltipUseSWT());
    getRadio_mainWindowMigemoTooltipUseJface().setSelection(!c.isMainWindowMigemoTooltipUseSWT());
    getCheck_mainWindowListShowIcon().setSelection(c.isMainWindowListShowIcon());
    getCheck_mainWindowThumbnailShowIcon().setSelection(c.isMainWindowThumbnailShowIcon());
    getCheck_mainWindowUseOsIcon().setSelection(c.isMainWindowUseOsIcon());
    getRadio_mainWindowRenameSelectNewNameTrue().setSelection(c.isMainWindowRenameSelectNewName());
    getRadio_mainWindowRenameSelectNewNameFalse().setSelection(!c.isMainWindowRenameSelectNewName());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
    
    c.setMainWindowThumbnailPrefetchPage(getSpinner_mainWindowThumbnailPrefetchPage().getSelection());
    c.setMainWindowThumbnailSearchDirDepth(getSpinner_mainWindowThumbnailSearchDirDepth().getSelection());
    c.setWindowStatusClearTimeMillisec(getSpinner_windowStatusClearTimeMillisec().getSelection());
    c.setMainWindowMigemoKeyTimeout(getSpinner_mainWindowMigemoKeyTimeout().getSelection());
    c.setMainWindowMigemoTooltipHideDelay(getSpinner_mainWindowMigemoTooltipHideDelay().getSelection());
    c.setMainWindowMigemoTooltipUseSWT(getRadio_mainWindowMigemoTooltipUseSWT().getSelection());
    c.setMainWindowListShowIcon(getCheck_mainWindowListShowIcon().getSelection());
    c.setMainWindowThumbnailShowIcon(getCheck_mainWindowThumbnailShowIcon().getSelection());
    c.setMainWindowUseOsIcon(getCheck_mainWindowUseOsIcon().getSelection());
    c.setMainWindowRenameSelectNewName(getRadio_mainWindowRenameSelectNewNameTrue().getSelection());
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel_4 = new Label(composite, SWT.NONE);
    lblNewLabel_4.setText("サムネイル読み込み");
    
    Group composite_3 = new Group(composite, SWT.NONE);
    composite_3.setLayout(new GridLayout(1, false));
    
    Composite composite_4 = new Composite(composite_3, SWT.NONE);
    composite_4.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_5 = new Label(composite_4, SWT.NONE);
    lblNewLabel_5.setText("画面に表示されているサムネイル数の(&F)");
    
    spinner_mainWindowThumbnailPrefetchPage = new Spinner(composite_4, SWT.BORDER);
    spinner_mainWindowThumbnailPrefetchPage.setMaximum(10);
    
    Label lblNewLabel_6 = new Label(composite_4, SWT.NONE);
    lblNewLabel_6.setText("倍まで先読み");
    
    Label lblNewLabel_1 = new Label(composite_3, SWT.NONE);
    lblNewLabel_1.setText("(先読みはサムネイルのメモリキャッシュに収まる範囲にしてください)");
    
    Composite composite_5 = new Composite(composite_3, SWT.NONE);
    composite_5.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_7 = new Label(composite_5, SWT.NONE);
    lblNewLabel_7.setText("深さ(&D)");
    
    spinner_mainWindowThumbnailSearchDirDepth = new Spinner(composite_5, SWT.BORDER);
    spinner_mainWindowThumbnailSearchDirDepth.setMaximum(10);
    
    Label lblNewLabel_10 = new Label(composite_5, SWT.NONE);
    lblNewLabel_10.setText("のサブディレクトリまで画像を探す");
    
    Label lblNewLabel_2 = new Label(composite, SWT.NONE);
    lblNewLabel_2.setText("ステータスメッセージ表示時間(&S)");
    
    Composite composite_2 = new Composite(composite, SWT.NONE);
    GridLayout gl_composite_2 = new GridLayout(2, false);
    composite_2.setLayout(gl_composite_2);
    
    spinner_windowStatusClearTimeMillisec = new Spinner(composite_2, SWT.BORDER);
    spinner_windowStatusClearTimeMillisec.setMaximum(10000);
    
    Label lblNewLabel_3 = new Label(composite_2, SWT.NONE);
    lblNewLabel_3.setText("ミリ秒");
    
    Label label = new Label(composite, SWT.NONE);
    label.setText("検索キー入力タイムアウト(&S)");
    
    Composite composite_6 = new Composite(composite, SWT.NONE);
    composite_6.setLayout(new GridLayout(2, false));
    
    spinner_mainWindowMigemoKeyTimeout = new Spinner(composite_6, SWT.BORDER);
    spinner_mainWindowMigemoKeyTimeout.setMaximum(10000);
    spinner_mainWindowMigemoKeyTimeout.setMinimum(100);
    
    Label label_1 = new Label(composite_6, SWT.NONE);
    label_1.setText("ミリ秒");
    
    Label lblNewLabel_9 = new Label(composite, SWT.NONE);
    lblNewLabel_9.setText("ファイル名検索ツールチップの表示時間(&T)");
    
    Composite composite_7 = new Composite(composite, SWT.NONE);
    composite_7.setLayout(new GridLayout(2, false));
    
    spinner_mainWindowMigemoTooltipHideDelay = new Spinner(composite_7, SWT.BORDER);
    spinner_mainWindowMigemoTooltipHideDelay.setMaximum(10000);
    
    Label label_2 = new Label(composite_7, SWT.NONE);
    label_2.setText("ミリ秒");
    
    Label lblNewLabel_8 = new Label(composite, SWT.NONE);
    lblNewLabel_8.setText("ファイル名検索ツールチップの表示方法");
    
    Group group = new Group(composite, SWT.NONE);
    group.setLayout(new GridLayout(1, false));
    
    radio_mainWindowMigemoTooltipUseSWT = new Button(group, SWT.RADIO);
    radio_mainWindowMigemoTooltipUseSWT.setText("SWTの部品で表示する");
    
    radio_mainWindowMigemoTooltipUseJface = new Button(group, SWT.RADIO);
    radio_mainWindowMigemoTooltipUseJface.setText("JFaceの部品で表示する");
    
    Label lblNewLabel_11 = new Label(group, SWT.NONE);
    lblNewLabel_11.setText("(Jfaceの部品でIMEをONにした状態で検索すると\n Linuxの場合にアプリケーションが固まるようです)");
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("ファイル名にアイコンをつける(&I)");
    
    Group composite_1 = new Group(composite, SWT.NONE);
    composite_1.setLayout(new GridLayout(1, false));
    
    check_mainWindowListShowIcon = new Button(composite_1, SWT.CHECK);
    check_mainWindowListShowIcon.setText("リストモード");
    
    check_mainWindowThumbnailShowIcon = new Button(composite_1, SWT.CHECK);
    check_mainWindowThumbnailShowIcon.setText("サムネイルモード");
    
    check_mainWindowUseOsIcon = new Button(composite_1, SWT.CHECK);
    check_mainWindowUseOsIcon.setText("アプリケーション組み込みのアイコンでなく、OSのアイコンを使用する");
    
    Label lblNewLabel_12 = new Label(composite, SWT.NONE);
    lblNewLabel_12.setText("ファイルリネーム後(&R)");
    
    Group group_1 = new Group(composite, SWT.NONE);
    group_1.setLayout(new GridLayout(1, false));
    
    radio_mainWindowRenameSelectNewNameTrue = new Button(group_1, SWT.RADIO);
    radio_mainWindowRenameSelectNewNameTrue.setText("新しいファイルを選択する");
    
    radio_mainWindowRenameSelectNewNameFalse = new Button(group_1, SWT.RADIO);
    radio_mainWindowRenameSelectNewNameFalse.setText("選択行を移動しない");
    return composite;
  }
  private Spinner getSpinner_mainWindowThumbnailPrefetchPage() {
    return spinner_mainWindowThumbnailPrefetchPage;
  }
  private Spinner getSpinner_mainWindowThumbnailSearchDirDepth() {
    return spinner_mainWindowThumbnailSearchDirDepth;
  }
  private Spinner getSpinner_windowStatusClearTimeMillisec() {
    return spinner_windowStatusClearTimeMillisec;
  }
  private Spinner getSpinner_mainWindowMigemoTooltipHideDelay() {
    return spinner_mainWindowMigemoTooltipHideDelay;
  }
  private Button getRadio_mainWindowMigemoTooltipUseSWT() {
    return radio_mainWindowMigemoTooltipUseSWT;
  }
  private Button getRadio_mainWindowMigemoTooltipUseJface() {
    return radio_mainWindowMigemoTooltipUseJface;
  }
  private Button getCheck_mainWindowListShowIcon() {
    return check_mainWindowListShowIcon;
  }
  private Button getCheck_mainWindowThumbnailShowIcon() {
    return check_mainWindowThumbnailShowIcon;
  }
  private Spinner getSpinner_mainWindowMigemoKeyTimeout() {
    return spinner_mainWindowMigemoKeyTimeout;
  }
  private Button getRadio_mainWindowRenameSelectNewNameTrue() {
    return radio_mainWindowRenameSelectNewNameTrue;
  }
  private Button getRadio_mainWindowRenameSelectNewNameFalse() {
    return radio_mainWindowRenameSelectNewNameFalse;
  }
  public Button getCheck_mainWindowUseOsIcon() {
    return check_mainWindowUseOsIcon;
  }
}
