package cvj2.app.gui.dialogs.preference;

import java.util.Arrays;
import java.util.regex.Pattern;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.gui.swt.util.FontResourceManager;
import cvj2.util.Log;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;

/**
 * メイン画面設定ページ
 */
public class MainWindowPreferencePage extends CommonPreferencePage {
  private Text text_fontTab;
  private Spinner spinner_mainWindowTabMax;
  private Spinner spinner_windowTabHeightPlus;
  private Button check_mainWindowNewTabModeHandOver;
  private Button radio_mainWindowNewTabListMode;
  private Button radio_mainWindowNewTabThumbnailMode;
  private Button check_mainWindowSortFileExtIgnoreCase;
  private Spinner spinner_mainWindowFocusPositionHistoryNum;
  private Button check_mainWindowEnterChildAndShowViewWindowWhenListMode;
  private Button check_mainWindowEnterChildAndShowViewWindowWhenThumbnailMode;
  private Button check_mainWindowEnterChildAndShowViewWindowReferLastFocus;

  private String[] fontDatas;
  private Text text_mainWindowHideRegExp;
  
  /**
   * コンストラクタ
   */
  public MainWindowPreferencePage() {
    setTitle("メイン画面");
  }

  /*
   * フォント ボタンイベントハンドラー
   */
  protected void button_fontTabDatas_widgetSelected(SelectionEvent e) {
    FontDialog dlg = new FontDialog(getShell());
    dlg.setFontList(FontResourceManager.getFontDatas(fontDatas));
    dlg.setText("フォント選択");
    if (null == dlg.open()) return;
    fontDatas = FontResourceManager.getFontDataStrings(dlg.getFontList());
    updateFontText();
  }
  private void updateFontText() {
    getText_fontTab().setText(Arrays.deepToString(fontDatas));
  }

  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();

    try {
      Pattern.compile(getText_mainWindowHideRegExp().getText());
    }
    catch (Exception e) {
      setErrorMessage("非表示設定の正規表現が正しくありません");
      return false;
    }

    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;

    Config c = ConfigDialog.getConfig();
  
    getSpinner_mainWindowTabMax().setSelection(c.getMainWindowTabMax());
    fontDatas = c.getFontTabDatas();
    updateFontText();
    getSpinner_windowTabHeightPlus().setSelection(c.getWindowTabHeightPlus());
    getCheck_mainWindowNewTabModeHandOver().setSelection(c.isMainWindowNewTabModeHandOver());
    getRadio_mainWindowNewTabListMode().setSelection(c.isMainWindowNewTabListMode());
    getRadio_mainWindowNewTabThumbnailMode().setSelection(!c.isMainWindowNewTabListMode());
    getCheck_mainWindowSortFileExtIgnoreCase().setSelection(c.isMainWindowSortFileExtIgnoreCase());
    getSpinner_mainWindowFocusPositionHistoryNum().setSelection(c.getMainWindowFocusPositionHistoryNum());
    getCheck_mainWindowEnterChildAndShowViewWindowWhenListMode().setSelection(c.isMainWindowEnterChildAndShowViewWindowWhenListMode());
    getCheck_mainWindowEnterChildAndShowViewWindowWhenThumbnailMode().setSelection(c.isMainWindowEnterChildAndShowViewWindowWhenThumbnailMode());
    getCheck_mainWindowEnterChildAndShowViewWindowReferLastFocus().setSelection(c.isMainWindowEnterChildAndShowViewWindowReferLastFocus());
    getText_mainWindowHideRegExp().setText(c.getMainWindowHideRegExp());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
    
    c.setMainWindowTabMax(getSpinner_mainWindowTabMax().getSelection());
    c.setFontTabDatas(fontDatas);
    c.setWindowTabHeightPlus(getSpinner_windowTabHeightPlus().getSelection());
    c.setMainWindowNewTabModeHandOver(getCheck_mainWindowNewTabModeHandOver().getSelection());
    c.setMainWindowNewTabListMode(getRadio_mainWindowNewTabListMode().getSelection());
    c.setMainWindowSortFileExtIgnoreCase(getCheck_mainWindowSortFileExtIgnoreCase().getSelection());
    c.setMainWindowFocusPositionHistoryNum(getSpinner_mainWindowFocusPositionHistoryNum().getSelection());
    c.setMainWindowEnterChildAndShowViewWindowWhenListMode(getCheck_mainWindowEnterChildAndShowViewWindowWhenListMode().getSelection());
    c.setMainWindowEnterChildAndShowViewWindowWhenThumbnailMode(getCheck_mainWindowEnterChildAndShowViewWindowWhenThumbnailMode().getSelection());
    c.setMainWindowEnterChildAndShowViewWindowReferLastFocus(getCheck_mainWindowEnterChildAndShowViewWindowReferLastFocus().getSelection());
    c.setMainWindowHideRegExp(getText_mainWindowHideRegExp().getText());
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel_10 = new Label(composite, SWT.NONE);
    lblNewLabel_10.setText("最大タブ数(&T)");
    
    spinner_mainWindowTabMax = new Spinner(composite, SWT.BORDER);
    spinner_mainWindowTabMax.setMaximum(32);
    spinner_mainWindowTabMax.setMinimum(1);
    GridData gd_spinner_mainWindowTabMax = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
    gd_spinner_mainWindowTabMax.horizontalIndent = 5;
    spinner_mainWindowTabMax.setLayoutData(gd_spinner_mainWindowTabMax);
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("タブのフォント");
    
    Composite composite_1 = new Composite(composite, SWT.NONE);
    composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    composite_1.setLayout(new GridLayout(2, false));
    
    Button button_fontTabDatas = new Button(composite_1, SWT.NONE);
    button_fontTabDatas.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_fontTabDatas_widgetSelected(e);
      }
    });
    button_fontTabDatas.setText("選択(&F)");
    
    text_fontTab = new Text(composite_1, SWT.BORDER | SWT.READ_ONLY);
    text_fontTab.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    text_fontTab.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_4 = new Label(composite, SWT.NONE);
    lblNewLabel_4.setText("タブの高さを広げる(&H)");
    
    Composite composite_3 = new Composite(composite, SWT.NONE);
    composite_3.setLayout(new GridLayout(2, false));
    
    spinner_windowTabHeightPlus = new Spinner(composite_3, SWT.BORDER);
    
    Label lblNewLabel_5 = new Label(composite_3, SWT.NONE);
    lblNewLabel_5.setText("ピクセル (要再起動)");
    
    Group composite_6 = new Group(composite, SWT.NONE);
    composite_6.setLayout(new GridLayout(1, false));
    composite_6.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    
    Label lblNewLabel_1 = new Label(composite_6, SWT.NONE);
    lblNewLabel_1.setText("新しいタブを開いたとき、");
    
    check_mainWindowNewTabModeHandOver = new Button(composite_6, SWT.CHECK);
    check_mainWindowNewTabModeHandOver.setText("現在のタブのモード(リスト/サムネイル)を引き継ぐ");
    
    radio_mainWindowNewTabListMode = new Button(composite_6, SWT.RADIO);
    radio_mainWindowNewTabListMode.setText("リストモードで開く");
    
    radio_mainWindowNewTabThumbnailMode = new Button(composite_6, SWT.RADIO);
    radio_mainWindowNewTabThumbnailMode.setText("サムネイルモードで開く");
    
    Composite composite_2 = new Composite(composite, SWT.NONE);
    composite_2.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    composite_2.setLayout(new GridLayout(1, false));
    
    check_mainWindowSortFileExtIgnoreCase = new Button(composite_2, SWT.CHECK);
    check_mainWindowSortFileExtIgnoreCase.setText("拡張子のソートで大文字小文字を区別しない");
    
    Composite composite_4 = new Composite(composite, SWT.NONE);
    composite_4.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    composite_4.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_6 = new Label(composite_4, SWT.NONE);
    lblNewLabel_6.setText("訪れたディレクトリのフォーカス位置を記憶しておく数");
    
    spinner_mainWindowFocusPositionHistoryNum = new Spinner(composite_4, SWT.BORDER);
    spinner_mainWindowFocusPositionHistoryNum.setMaximum(256);
    spinner_mainWindowFocusPositionHistoryNum.setMinimum(1);
    
    Label lblNewLabel_11 = new Label(composite_4, SWT.NONE);
    lblNewLabel_11.setText("(要再起動)");
    
    Group composite_5 = new Group(composite, SWT.NONE);
    composite_5.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    composite_5.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel_2 = new Label(composite_5, SWT.NONE);
    lblNewLabel_2.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    lblNewLabel_2.setText("サブディレクトリに入ったとき先頭のファイルが画像ファイルならそれを表示する");
    
    check_mainWindowEnterChildAndShowViewWindowWhenListMode = new Button(composite_5, SWT.CHECK);
    check_mainWindowEnterChildAndShowViewWindowWhenListMode.setText("リストモードの場合");
    
    check_mainWindowEnterChildAndShowViewWindowWhenThumbnailMode = new Button(composite_5, SWT.CHECK);
    check_mainWindowEnterChildAndShowViewWindowWhenThumbnailMode.setText("サムネイルモードの場合");
    
    check_mainWindowEnterChildAndShowViewWindowReferLastFocus = new Button(composite_5, SWT.CHECK);
    check_mainWindowEnterChildAndShowViewWindowReferLastFocus.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    check_mainWindowEnterChildAndShowViewWindowReferLastFocus.setText("その場合、記憶しておいたフォーカス位置を使用する");
    
    Group group = new Group(composite, SWT.NONE);
    group.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
    group.setLayout(new GridLayout(1, false));
    
    Label label = new Label(group, SWT.NONE);
    label.setText("非表示にするファイル名、ディレクトリ名を正規表現で入力してください(&H)");
    
    text_mainWindowHideRegExp = new Text(group, SWT.BORDER);
    text_mainWindowHideRegExp.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    return composite;
  }

  private Spinner getSpinner_mainWindowTabMax() {
    return spinner_mainWindowTabMax;
  }
  private Text getText_fontTab() {
    return text_fontTab;
  }
  private Spinner getSpinner_windowTabHeightPlus() {
    return spinner_windowTabHeightPlus;
  }
  private Button getCheck_mainWindowNewTabModeHandOver() {
    return check_mainWindowNewTabModeHandOver;
  }
  private Button getRadio_mainWindowNewTabListMode() {
    return radio_mainWindowNewTabListMode;
  }
  private Button getRadio_mainWindowNewTabThumbnailMode() {
    return radio_mainWindowNewTabThumbnailMode;
  }
  private Button getCheck_mainWindowSortFileExtIgnoreCase() {
    return check_mainWindowSortFileExtIgnoreCase;
  }
  private Spinner getSpinner_mainWindowFocusPositionHistoryNum() {
    return spinner_mainWindowFocusPositionHistoryNum;
  }
  private Button getCheck_mainWindowEnterChildAndShowViewWindowWhenListMode() {
    return check_mainWindowEnterChildAndShowViewWindowWhenListMode;
  }
  private Button getCheck_mainWindowEnterChildAndShowViewWindowWhenThumbnailMode() {
    return check_mainWindowEnterChildAndShowViewWindowWhenThumbnailMode;
  }
  private Button getCheck_mainWindowEnterChildAndShowViewWindowReferLastFocus() {
    return check_mainWindowEnterChildAndShowViewWindowReferLastFocus;
  }
  private Text getText_mainWindowHideRegExp() {
    return text_mainWindowHideRegExp;
  }
}
