package cvj2.app.gui.dialogs.preference;

import java.io.IOException;
import java.nio.charset.Charset;

import org.apache.commons.io.FileUtils;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;

import cvj2.app.Config;
import cvj2.app.ParseArg;
import cvj2.app.gui.util.MsgBox;
import cvj2.util.Log;

/**
 * システム設定ページ
 */
public class SystemPreferencePage extends CommonPreferencePage {
  private Text text_zipFileEncoding;
  private Text text_execProgramNoConfirmExts;
  private Spinner spinner_imageLoadBackgroundThreadNum;
  private Spinner spinner_zipFileCacheNum;
  private Spinner spinner_execProgramMaxNum;
  private Spinner spinner_imageLoadPdfCacheSize;
  private Button check_showSplash;
  private Button check_enableZipFolder;
  private Button check_enablePdfFolder;
  private Button check_viewWindowEnable;
  private Label lblNewLabel_8;

  /**
   * コンストラクタ
   */
  public SystemPreferencePage() {
    setTitle("システム");
  }
  
  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();

    try {
      Charset.forName(getText_zipFileEncoding().getText());
    }
    catch (Exception e) {
      setErrorMessage("この文字コードは使用できません: " + getText_zipFileEncoding().getText());
      return false;
    }
    
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;
    
    Config c = ConfigDialog.getConfig();
    
    getCheck_showSplash().setSelection(c.isShowSplash());
    getSpinner_imageLoadBackgroundThreadNum().setSelection(c.getImageLoadBackgroundThreadNum());
    getCheck_viewWindowEnable().setSelection(c.isViewWindowEnable());
    getCheck_enableZipFolder().setSelection(c.isEnableZipFolder());
    getCheck_enablePdfFolder().setSelection(c.isEnablePdfFolder());
    getSpinner_zipFileCacheNum().setSelection(c.getZipFileCacheNum());
    getText_zipFileEncoding().setText(c.getZipFileEncoding());
    getSpinner_imageLoadPdfCacheSize().setSelection(c.getImageLoadPdfCacheSize());
    getText_execProgramNoConfirmExts().setText(c.getExecProgramNoConfirmExts());
    getSpinner_execProgramMaxNum().setSelection(c.getExecProgramMaxNum());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
   
    c.setShowSplash(getCheck_showSplash().getSelection());    
    c.setImageLoadBackgroundThreadNum(getSpinner_imageLoadBackgroundThreadNum().getSelection());
    c.setViewWindowEnable(getCheck_viewWindowEnable().getSelection());
    c.setEnableZipFolder(getCheck_enableZipFolder().getSelection());
    c.setEnablePdfFolder(getCheck_enablePdfFolder().getSelection());
    c.setZipFileCacheNum(getSpinner_zipFileCacheNum().getSelection());
    c.setZipFileEncoding(getText_zipFileEncoding().getText());
    c.setImageLoadPdfCacheSize(getSpinner_imageLoadPdfCacheSize().getSelection());
    c.setExecProgramNoConfirmExts(getText_execProgramNoConfirmExts().getText());
    c.setExecProgramMaxNum(getSpinner_execProgramMaxNum().getSelection());
  }
  
  /*
   * メモリ設定ボタン イベントハンドラ
   */
  private void button_maxMem_widgetSelected(SelectionEvent e) {
    if (!MsgBox.okCancel("確認", "この設定は、" + ParseArg.javaMaxMemFile.getParent() + " に保存され、\n" +
            "同じアプリケーションを使用している他のユーザーやプロファイルと設定を共有します。\n" +
            "続けてもよろしいですか？")) 
    {
      return;
    }
    
    int maxJava = (int)(Runtime.getRuntime().maxMemory() / 1024 / 1024);
    int maxConf = -1;
    try {
      maxConf = Integer.parseInt(FileUtils.readFileToString(ParseArg.javaMaxMemFile).trim());
    }
    catch (Exception ex) { /* */ }
    String msg = "現在の設定は、" + maxConf + "(MB)になっています。\n";
    if (maxConf < 0) msg = "現在の最大メモリー設定を取得で来ませんでした。\n";
    msg += "実際にアプリケーションが使用できるメモリーは " + maxJava + "(MB)です。\n" +
           "(設定した値よりも少し少なくなります)\n" +
           "(大きすぎる設定は起動時にエラーになるかもしれません)\n\n" +
           "設定するメモリーサイズを (MB) で指定してください";
    
    while (true) {
      InputDialog dlg = new InputDialog(getShell(), "最大メモリー設定", msg, "" + Math.max(512, maxConf), null);
      if (Window.OK != dlg.open()) return;
      try {
        maxConf = Integer.parseInt(dlg.getValue());
        if (maxConf < 256 || 4096 < maxConf) throw new Exception();
        break;
      }
      catch (Exception ex) { 
        MsgBox.error("入力値エラー", "最大メモリー設定は、256 ～ 4096 MBで指定してください");
      }
    }
    
    msg = "最大メモリー設定 (" + maxConf + " MB) を\n" +
          "メモリー設定ファイル (" + ParseArg.javaMaxMemFile + ") に書き込みます。\n" +
          "(設定変更により起動できなくなったら、メモリー設定ファイルを削除するか、\n" +
          " アプリケーションをアンインストールして再インストールしてみてください)";
    if (!MsgBox.okCancel("設定を書き込みます", msg)) return;
    
    try {
      FileUtils.writeStringToFile(ParseArg.javaMaxMemFile, "" + maxConf);
      MsgBox.info("成功", "メモリー設定ファイルを更新しました\n新しい設定は再起動後に有効になります");
    }
    catch (IOException ex) {
      msg = "メモリー設定ファイル更新に失敗しました\n" +
            "\t" + ex.getMessage() + "\n\n" +
            "設定ファイル [" + ParseArg.javaMaxMemFile + "] に対して\n" +
            "書き込み権限があるかどうか確認してください。\n" +
            "(この設定はエディターを使って手動で書き換えてもかまいません)";
      MsgBox.error("失敗", msg);
    }
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel_7 = new Label(composite, SWT.NONE);
    lblNewLabel_7.setText("アプリケーション\n最大使用メモリ設定");
    
    Button button_maxMem = new Button(composite, SWT.NONE);
    button_maxMem.addSelectionListener(new SelectionAdapter() {
      @Override
      public void widgetSelected(SelectionEvent e) {
        button_maxMem_widgetSelected(e);
      }
    });
    button_maxMem.setText("設定する");
    
    check_showSplash = new Button(composite, SWT.CHECK);
    check_showSplash.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    check_showSplash.setText("起動時にスプラッシュ画面を表示する");

    Label lblNewLabel_1 = new Label(composite, SWT.NONE);
    lblNewLabel_1.setText("バックグラウンドスレッド数(&T)");
    
    spinner_imageLoadBackgroundThreadNum = new Spinner(composite, SWT.BORDER);
    spinner_imageLoadBackgroundThreadNum.setMinimum(1);
    
    check_viewWindowEnable = new Button(composite, SWT.CHECK);
    check_viewWindowEnable.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    check_viewWindowEnable.setText("内蔵の画像ビュアーを使用する");
    
    check_enableZipFolder = new Button(composite, SWT.CHECK);
    check_enableZipFolder.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    check_enableZipFolder.setText("Zipファイルをフォルダーとして扱う");
    
    check_enablePdfFolder = new Button(composite, SWT.CHECK);
    check_enablePdfFolder.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    check_enablePdfFolder.setText("Pdfファイルをフォルダーとして扱う");
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("Zipファイル解析キャッシュ数(&Z)");
    
    spinner_zipFileCacheNum = new Spinner(composite, SWT.BORDER);
    spinner_zipFileCacheNum.setMaximum(128);
    spinner_zipFileCacheNum.setMinimum(1);
    
    Label lblNewLabel_2 = new Label(composite, SWT.NONE);
    lblNewLabel_2.setText("Zipファイル内部文字コード(&C)");
    
    text_zipFileEncoding = new Text(composite, SWT.BORDER);
    text_zipFileEncoding.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_6 = new Label(composite, SWT.NONE);
    lblNewLabel_6.setText("PDFファイル解析キャッシュ数(&P)");
    
    spinner_imageLoadPdfCacheSize = new Spinner(composite, SWT.BORDER);
    spinner_imageLoadPdfCacheSize.setMaximum(32);
    spinner_imageLoadPdfCacheSize.setMinimum(1);
    
    lblNewLabel_8 = new Label(composite, SWT.NONE);
    lblNewLabel_8.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    lblNewLabel_8.setText("※) Zipファイル解析キャッシュ数とPDFファイル解析キャッシュ数は\n" +
                          "　　バックグラウンドスレッド数以上にするといいかもしれません\n" +
                          "※) PDFファイルキャッシュは大量のメモリを消費します");
    
    Label lblNewLabel_4 = new Label(composite, SWT.NONE);
    lblNewLabel_4.setText("確認なしで起動する拡張子(&E)");
    
    text_execProgramNoConfirmExts = new Text(composite, SWT.BORDER | SWT.V_SCROLL | SWT.MULTI);
    GridData gd_text_execProgramNoConfirmExts = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
    gd_text_execProgramNoConfirmExts.heightHint = 103;
    text_execProgramNoConfirmExts.setLayoutData(gd_text_execProgramNoConfirmExts);
    
    Label lblNewLabel_5 = new Label(composite, SWT.NONE);
    lblNewLabel_5.setText("複数ファイル選択時に確認なしで\n起動する最大数(&L)\n(0は制限なし)");
    
    spinner_execProgramMaxNum = new Spinner(composite, SWT.BORDER);
    
    return composite;
  }

  private Spinner getSpinner_imageLoadBackgroundThreadNum() {
    return spinner_imageLoadBackgroundThreadNum;
  }
  private Spinner getSpinner_zipFileCacheNum() {
    return spinner_zipFileCacheNum;
  }
  private Text getText_zipFileEncoding() {
    return text_zipFileEncoding;
  }
  private Text getText_execProgramNoConfirmExts() {
    return text_execProgramNoConfirmExts;
  }
  private Spinner getSpinner_execProgramMaxNum() {
    return spinner_execProgramMaxNum;
  }
  private Spinner getSpinner_imageLoadPdfCacheSize() {
    return spinner_imageLoadPdfCacheSize;
  }
  private Button getCheck_showSplash() {
    return check_showSplash;
  }
  private Button getCheck_enableZipFolder() {
    return check_enableZipFolder;
  }
  private Button getCheck_enablePdfFolder() {
    return check_enablePdfFolder;
  }
  private Button getCheck_viewWindowEnable() {
    return check_viewWindowEnable;
  }
}
