package cvj2.app.gui.dialogs.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

import cvj2.app.Config;
import cvj2.util.Log;

/**
 * チューニング設定ページ
 */
public class TuningPreferencePage extends CommonPreferencePage {
  private Spinner spinner_fileInfoReadBackgroundWorkTime;
  private Spinner spinner_fileInfoReadBackgroundSleepTime;
  private Spinner spinner_fileInfoReadInitializeTryTime;
  private Spinner spinner_fileInfoReadInitializeTryItemRatio;
  private Spinner spinner_mainWindowFileWatchInterval;
  private Spinner spinner_mainWindowFileWatch1SecDelayFileCount;
  private Button check_propertyDialogHideReadableCheckBox;
  private Button check_propertyDialogHideExecuteCheckBox;
  private Button check_msgboxHeadTailCR;
  private Spinner spinner_progressMonitorDialogPlusWidth;

  /**
   * コンストラクタ
   */
  public TuningPreferencePage() {
    setTitle("細かい設定");
  }


  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;

    Config c = ConfigDialog.getConfig();

    getSpinner_fileInfoReadBackgroundWorkTime().setSelection(c.getFileInfoReadBackgroundWorkTime());
    getSpinner_fileInfoReadBackgroundSleepTime().setSelection(c.getFileInfoReadBackgroundSleepTime());
    getSpinner_fileInfoReadInitializeTryTime().setSelection(c.getFileInfoReadInitializeTryTime());
    getSpinner_fileInfoReadInitializeTryItemRatio().setSelection((int)(c.getFileInfoReadInitializeTryItemRatio() * 100));
    getSpinner_mainWindowFileWatchInterval().setSelection(c.getMainWindowFileWatchInterval());
    getSpinner_mainWindowFileWatch1SecDelayFileCount().setSelection(c.getMainWindowFileWatch1SecDelayFileCount() / 1000);
    getCheck_propertyDialogHideReadableCheckBox().setSelection(c.isPropertyDialogHideReadableCheckBox());
    getCheck_propertyDialogHideExecuteCheckBox().setSelection(c.isPropertyDialogHideExecuteCheckBox());
    getCheck_msgboxHeadTailCR().setSelection(c.isMsgboxHeadTailCR());
    getSpinner_progressMonitorDialogPlusWidth().setSelection(c.getProgressMonitorDialogPlusWidth());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
    
    c.setFileInfoReadBackgroundWorkTime(getSpinner_fileInfoReadBackgroundWorkTime().getSelection());
    c.setFileInfoReadBackgroundSleepTime(getSpinner_fileInfoReadBackgroundSleepTime().getSelection());
    c.setFileInfoReadInitializeTryTime(getSpinner_fileInfoReadInitializeTryTime().getSelection());
    c.setFileInfoReadInitializeTryItemRatio(getSpinner_fileInfoReadInitializeTryItemRatio().getSelection() / 100.0);
    c.setMainWindowFileWatchInterval(getSpinner_mainWindowFileWatchInterval().getSelection());
    c.setMainWindowFileWatch1SecDelayFileCount(getSpinner_mainWindowFileWatch1SecDelayFileCount().getSelection() * 1000);
    c.setPropertyDialogHideReadableCheckBox(getCheck_propertyDialogHideReadableCheckBox().getSelection());
    c.setPropertyDialogHideExecuteCheckBox(getCheck_propertyDialogHideExecuteCheckBox().getSelection());
    c.setMsgboxHeadTailCR(getCheck_msgboxHeadTailCR().getSelection());
    c.setProgressMonitorDialogPlusWidth(getSpinner_progressMonitorDialogPlusWidth().getSelection());
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(1, false));
    
    Group composite_2 = new Group(composite, SWT.NONE);
    composite_2.setLayout(new GridLayout(1, false));
    
    Label lblNewLabel_2 = new Label(composite_2, SWT.NONE);
    lblNewLabel_2.setText("バックグラウンドでファイル情報(日付やサイズ)を読み込むタイミング");
    
    Composite composite_6 = new Composite(composite_2, SWT.NONE);
    composite_6.setLayout(new GridLayout(4, false));
    
    spinner_fileInfoReadBackgroundWorkTime = new Spinner(composite_6, SWT.BORDER);
    
    Label lblNewLabel_3 = new Label(composite_6, SWT.NONE);
    lblNewLabel_3.setText("ミリ秒読んで、");
    
    spinner_fileInfoReadBackgroundSleepTime = new Spinner(composite_6, SWT.BORDER);
    
    Label lblNewLabel_4 = new Label(composite_6, SWT.NONE);
    lblNewLabel_4.setText("ミリ秒スリープする、を繰り返す");
    
    Label lblNewLabel_5 = new Label(composite_2, SWT.NONE);
    lblNewLabel_5.setText("(連続して読みすぎると、画面の動作がにぶくなります)");
    
    Group composite_7 = new Group(composite, SWT.NONE);
    composite_7.setLayout(new GridLayout(1, false));
    
    Label lblNewLabel_8 = new Label(composite_7, SWT.NONE);
    lblNewLabel_8.setText("ファイルサイズや更新日時でソートしようとしたときに、\nファイル情報を取得する制限時間");
    
    Composite composite_8 = new Composite(composite_7, SWT.NONE);
    composite_8.setLayout(new GridLayout(4, false));
    
    spinner_fileInfoReadInitializeTryTime = new Spinner(composite_8, SWT.BORDER);
    
    Label lblNewLabel_9 = new Label(composite_8, SWT.NONE);
    lblNewLabel_9.setText("ミリ秒の間読み込んでみて");
    
    spinner_fileInfoReadInitializeTryItemRatio = new Spinner(composite_8, SWT.BORDER);
    
    Label lblNewLabel_10 = new Label(composite_8, SWT.NONE);
    lblNewLabel_10.setText("％しか読めなかったら、");
    
    Label lblNewLabel_11 = new Label(composite_7, SWT.NONE);
    lblNewLabel_11.setText("そのまま続けてもいいか確認画面を出す");
    
    Group composite_3 = new Group(composite, SWT.NONE);
    composite_3.setLayout(new GridLayout(1, false));
    
    Composite composite_4 = new Composite(composite_3, SWT.NONE);
    composite_4.setLayout(new GridLayout(2, false));
    
    spinner_mainWindowFileWatchInterval = new Spinner(composite_4, SWT.BORDER);
    spinner_mainWindowFileWatchInterval.setMaximum(10000);
    
    Label label = new Label(composite_4, SWT.NONE);
    label.setText("ミリ秒毎に、ファイルの追加や削除があったかどうかを監視する");
    
    Composite composite_5 = new Composite(composite_3, SWT.NONE);
    composite_5.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_6 = new Label(composite_5, SWT.NONE);
    lblNewLabel_6.setText("ただし、ディレクトリ内に");
    
    spinner_mainWindowFileWatch1SecDelayFileCount = new Spinner(composite_5, SWT.BORDER);
    spinner_mainWindowFileWatch1SecDelayFileCount.setMaximum(10000);
    
    Label label_1 = new Label(composite_5, SWT.NONE);
    label_1.setText("×1000ファイルある毎に、監視時間を1秒伸ばす");
    
    Label lblNewLabel_7 = new Label(composite_3, SWT.NONE);
    GridData gd_lblNewLabel_7 = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
    gd_lblNewLabel_7.horizontalIndent = 5;
    lblNewLabel_7.setLayoutData(gd_lblNewLabel_7);
    lblNewLabel_7.setText("(たくさんのファイルがあるとCPUに負荷がかかります)");
    
    Group composite_9 = new Group(composite, SWT.NONE);
    composite_9.setLayout(new GridLayout(1, false));
    
    Label lblNewLabel_12 = new Label(composite_9, SWT.NONE);
    lblNewLabel_12.setText("ファイルプロパティダイアログにおいて、");
    
    Composite composite_10 = new Composite(composite_9, SWT.NONE);
    composite_10.setLayout(new RowLayout(SWT.HORIZONTAL));
    
    check_propertyDialogHideReadableCheckBox = new Button(composite_10, SWT.CHECK);
    check_propertyDialogHideReadableCheckBox.setText("読み取り可");
    
    check_propertyDialogHideExecuteCheckBox = new Button(composite_10, SWT.CHECK);
    check_propertyDialogHideExecuteCheckBox.setText("実行可");
    
    Label lblNewLabel_13 = new Label(composite_9, SWT.NONE);
    lblNewLabel_13.setText("の属性を隠す(Windowsでは表示しても無意味かもしれません)");
    
    check_msgboxHeadTailCR = new Button(composite, SWT.CHECK);
    check_msgboxHeadTailCR.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    check_msgboxHeadTailCR.setText("メッセージボックスのメッセージの先頭と末尾に余白を入れる\n(上下の文字が表示されない場合があるので、その時にONにする)");
    
    Group composite_1 = new Group(composite, SWT.NONE);
    composite_1.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 2, 1));
    composite_1.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel = new Label(composite_1, SWT.NONE);
    lblNewLabel.setText("プログレス(進捗表示)ダイアログの画面幅を少し広げる");
    
    spinner_progressMonitorDialogPlusWidth = new Spinner(composite_1, SWT.BORDER);
    
    Label lblNewLabel_1 = new Label(composite_1, SWT.NONE);
    lblNewLabel_1.setText("ピクセル");
    
    Label lblNewLabel_14 = new Label(composite_1, SWT.NONE);
    lblNewLabel_14.setLayoutData(new GridData(SWT.LEFT, SWT.CENTER, false, false, 3, 1));
    lblNewLabel_14.setText("プログレスダイアログの画面サイズは変更できないので、\nファイル名が表示できない時にこの値を調整してください");
    return composite;
  }
  private Spinner getSpinner_fileInfoReadBackgroundWorkTime() {
    return spinner_fileInfoReadBackgroundWorkTime;
  }
  private Spinner getSpinner_fileInfoReadBackgroundSleepTime() {
    return spinner_fileInfoReadBackgroundSleepTime;
  }
  private Spinner getSpinner_fileInfoReadInitializeTryTime() {
    return spinner_fileInfoReadInitializeTryTime;
  }
  private Spinner getSpinner_fileInfoReadInitializeTryItemRatio() {
    return spinner_fileInfoReadInitializeTryItemRatio;
  }
  private Spinner getSpinner_mainWindowFileWatchInterval() {
    return spinner_mainWindowFileWatchInterval;
  }
  private Spinner getSpinner_mainWindowFileWatch1SecDelayFileCount() {
    return spinner_mainWindowFileWatch1SecDelayFileCount;
  }
  private Button getCheck_propertyDialogHideReadableCheckBox() {
    return check_propertyDialogHideReadableCheckBox;
  }
  private Button getCheck_propertyDialogHideExecuteCheckBox() {
    return check_propertyDialogHideExecuteCheckBox;
  }
  private Button getCheck_msgboxHeadTailCR() {
    return check_msgboxHeadTailCR;
  }
  private Spinner getSpinner_progressMonitorDialogPlusWidth() {
    return spinner_progressMonitorDialogPlusWidth;
  }
}
