package cvj2.app.gui.dialogs.preference;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;

import cvj2.app.Config;
import cvj2.gui.swt.widgets.FloatText;
import cvj2.util.Log;

/**
 * 画像表示画面設定ページ
 */
public class ViewWindowPreferencePage extends CommonPreferencePage {
  private FloatText text_viewWindowZoomInRatio;
  private FloatText text_viewWindowZoomOutRatio;
  private FloatText text_viewWindowZoomInRatioMax;
  private FloatText text_viewWindowZoomOutRatioMin;
  private Spinner spinner_viewWindowForwardLoadNum;
  private Spinner spinner_viewWindowForwardLoadInCacheRatio;
  private Spinner spinner_viewWindowMoveCursorColumnDelay;
  private Spinner spinner_viewWindowMoveCursorRowDelay;
  private Spinner spinner_viewWindowScrollVisibleTime;
  private Spinner spinner_viewWindowFullscreenHideCursorTime;
  private Button check_viewWindowShowImageInfoDebugMessage;
  private Spinner spinner_viewWindowMoveRatio;

  /**
   * コンストラクタ
   */
  public ViewWindowPreferencePage() {
    setTitle("画像表示画面");
  }


  /**
   * 入力データ検証
   */
  @Override
  public boolean validate() {
    Log.start();
    
    FloatText[] texts = {
        getText_viewWindowZoomInRatio(),
        getText_viewWindowZoomOutRatio(),
        getText_viewWindowZoomInRatioMax(),
        getText_viewWindowZoomOutRatioMin(),
    };
    String[] names = {
        "ズーム倍率 (拡大)",
        "ズーム倍率 (縮小)",
        "ズーム倍率制限 (拡大)",
        "ズーム倍率制限 (縮小)",
    };
    for (int i = 0; i < texts.length; i++) {
      String msg = texts[i].getInputErrorMessage(names[i]);
      if (msg == null) continue;
      setErrorMessage(msg);
      return false;
    }
    
    setErrorMessage(null);
    return true;
  }

  /**
   * 設定データで画面を更新する
   */
  @Override
  public void updateDialogFromData(boolean forceUpdate) {
    Log.start(forceUpdate);
    if (!forceUpdate && super.initialized) return;
    super.initialized = true;

    Config c = ConfigDialog.getConfig();
    
    getSpinner_viewWindowForwardLoadNum().setSelection(c.getViewWindowForwardLoadNum());
    getSpinner_viewWindowForwardLoadInCacheRatio().setSelection((int)(c.getViewWindowForwardLoadInCacheRatio() * 100));
    getSpinner_viewWindowMoveCursorColumnDelay().setSelection(c.getViewWindowMoveCursorColumnDelay());
    getSpinner_viewWindowMoveCursorRowDelay().setSelection(c.getViewWindowMoveCursorRowDelay());
    getText_viewWindowZoomInRatio().setValue(c.getViewWindowZoomInRatio());
    getText_viewWindowZoomOutRatio().setValue(c.getViewWindowZoomOutRatio());
    getText_viewWindowZoomInRatioMax().setValue(c.getViewWindowZoomInRatioMax());
    getText_viewWindowZoomOutRatioMin().setValue(c.getViewWindowZoomOutRatioMin());
    getSpinner_viewWindowMoveRatio().setSelection((int)(c.getViewWindowMoveRatio() * 100));
    getSpinner_viewWindowScrollVisibleTime().setSelection(c.getViewWindowScrollVisibleTime());
    getSpinner_viewWindowFullscreenHideCursorTime().setSelection(c.getViewWindowFullscreenHideCursorTime());
    getCheck_viewWindowShowImageInfoDebugMessage().setSelection(c.isViewWindowShowImageInfoDebugMessage());
  }

  /**
   * 画面入力で設定データを更新する
   */
  @Override
  public void updateDataFromDialog() {
    Log.start();
    Config c = ConfigDialog.getConfig();
    
    c.setViewWindowForwardLoadNum(getSpinner_viewWindowForwardLoadNum().getSelection());
    c.setViewWindowForwardLoadInCacheRatio(getSpinner_viewWindowForwardLoadInCacheRatio().getSelection() / 100.0);
    c.setViewWindowMoveCursorColumnDelay(getSpinner_viewWindowMoveCursorColumnDelay().getSelection());
    c.setViewWindowMoveCursorRowDelay(getSpinner_viewWindowMoveCursorRowDelay().getSelection());
    c.setViewWindowZoomInRatio(getText_viewWindowZoomInRatio().getValue());
    c.setViewWindowZoomOutRatio(getText_viewWindowZoomOutRatio().getValue());
    c.setViewWindowZoomInRatioMax(getText_viewWindowZoomInRatioMax().getValue());
    c.setViewWindowZoomOutRatioMin(getText_viewWindowZoomOutRatioMin().getValue());
    c.setViewWindowMoveRatio(getSpinner_viewWindowMoveRatio().getSelection() / 100.0);
    c.setViewWindowScrollVisibleTime(getSpinner_viewWindowScrollVisibleTime().getSelection());
    c.setViewWindowFullscreenHideCursorTime(getSpinner_viewWindowFullscreenHideCursorTime().getSelection());
    c.setViewWindowShowImageInfoDebugMessage(getCheck_viewWindowShowImageInfoDebugMessage().getSelection());
  }
  
  @Override
  protected Control createContents(Composite parent) {
    Composite composite = new Composite(parent, SWT.NONE);
    composite.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel = new Label(composite, SWT.NONE);
    lblNewLabel.setText("先読み画像(&L)");
    
    Group composite_1 = new Group(composite, SWT.NONE);
    GridLayout gl_composite_1 = new GridLayout(1, false);
    gl_composite_1.verticalSpacing = 0;
    composite_1.setLayout(gl_composite_1);
    
    Composite composite_2 = new Composite(composite_1, SWT.NONE);
    composite_2.setLayout(new GridLayout(2, false));
    
    Label lblNewLabel_1 = new Label(composite_2, SWT.NONE);
    lblNewLabel_1.setText("最大先読み数");
    
    spinner_viewWindowForwardLoadNum = new Spinner(composite_2, SWT.BORDER);
    spinner_viewWindowForwardLoadNum.setMaximum(16);
    
    Composite composite_3 = new Composite(composite_1, SWT.NONE);
    composite_3.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_2 = new Label(composite_3, SWT.NONE);
    lblNewLabel_2.setText("画像キャッシュメモリの最大");
    
    spinner_viewWindowForwardLoadInCacheRatio = new Spinner(composite_3, SWT.BORDER);
    spinner_viewWindowForwardLoadInCacheRatio.setMaximum(75);
    
    Label lblNewLabel_3 = new Label(composite_3, SWT.NONE);
    lblNewLabel_3.setText("％まで先読みする");
    
    Label lblNewLabel_4 = new Label(composite, SWT.NONE);
    lblNewLabel_4.setText("画像表示タイミング制限(&T)\n(キーを押しつづけた場合の動作)");
    
    Group composite_4 = new Group(composite, SWT.NONE);
    GridLayout gl_composite_4 = new GridLayout(1, false);
    gl_composite_4.verticalSpacing = 0;
    composite_4.setLayout(gl_composite_4);
    
    Composite composite_5 = new Composite(composite_4, SWT.NONE);
    composite_5.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_5 = new Label(composite_5, SWT.NONE);
    lblNewLabel_5.setText("次/前の画像を表示するのに少なくとも");
    
    spinner_viewWindowMoveCursorColumnDelay = new Spinner(composite_5, SWT.BORDER);
    spinner_viewWindowMoveCursorColumnDelay.setMaximum(10000);
    
    Label lblNewLabel_6 = new Label(composite_5, SWT.NONE);
    lblNewLabel_6.setText("ミリ秒待つ");
    
    Composite composite_6 = new Composite(composite_4, SWT.NONE);
    composite_6.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_7 = new Label(composite_6, SWT.NONE);
    lblNewLabel_7.setText("次/前の行の画像を表示するのに少なくとも");
    
    spinner_viewWindowMoveCursorRowDelay = new Spinner(composite_6, SWT.BORDER);
    spinner_viewWindowMoveCursorRowDelay.setMaximum(10000);
    
    Label lblNewLabel_8 = new Label(composite_6, SWT.NONE);
    lblNewLabel_8.setText("ミリ秒待つ");
    
    Label lblNewLabel_9 = new Label(composite, SWT.NONE);
    lblNewLabel_9.setText("ズーム倍率(&Z)");
    
    Group composite_7 = new Group(composite, SWT.NONE);
    GridLayout gl_composite_7 = new GridLayout(1, false);
    gl_composite_7.verticalSpacing = 0;
    composite_7.setLayout(gl_composite_7);
    
    Composite composite_10 = new Composite(composite_7, SWT.NONE);
    composite_10.setLayout(new GridLayout(2, false));
    
    text_viewWindowZoomInRatio = new FloatText(composite_10, SWT.BORDER);
    text_viewWindowZoomInRatio.setMaximum(10.0);
    text_viewWindowZoomInRatio.setMinimum(1.0);
    text_viewWindowZoomInRatio.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_13 = new Label(composite_10, SWT.NONE);
    lblNewLabel_13.setText("倍づつ拡大する");
    
    Composite composite_11 = new Composite(composite_7, SWT.NONE);
    composite_11.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_14 = new Label(composite_11, SWT.NONE);
    lblNewLabel_14.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
    lblNewLabel_14.setText("1/ ");
    
    text_viewWindowZoomOutRatio = new FloatText(composite_11, SWT.BORDER);
    text_viewWindowZoomOutRatio.setMinimum(1.0);
    text_viewWindowZoomOutRatio.setMaximum(10.0);
    text_viewWindowZoomOutRatio.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_15 = new Label(composite_11, SWT.NONE);
    lblNewLabel_15.setText("倍づつ縮小する");
    
    Label lblNewLabel_11 = new Label(composite, SWT.NONE);
    lblNewLabel_11.setText("ズーム倍率制限(&R)");
    
    Group composite_9 = new Group(composite, SWT.NONE);
    GridLayout gl_composite_9 = new GridLayout(1, false);
    gl_composite_9.verticalSpacing = 0;
    composite_9.setLayout(gl_composite_9);
    
    Composite composite_12 = new Composite(composite_9, SWT.NONE);
    composite_12.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_12 = new Label(composite_12, SWT.NONE);
    lblNewLabel_12.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
    lblNewLabel_12.setText("拡大は");
    
    text_viewWindowZoomInRatioMax = new FloatText(composite_12, SWT.BORDER);
    text_viewWindowZoomInRatioMax.setMinimum(1.0);
    text_viewWindowZoomInRatioMax.setMaximum(100.0);
    text_viewWindowZoomInRatioMax.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_16 = new Label(composite_12, SWT.NONE);
    lblNewLabel_16.setText("倍まで");
    
    Composite composite_13 = new Composite(composite_9, SWT.NONE);
    composite_13.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_17 = new Label(composite_13, SWT.NONE);
    lblNewLabel_17.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
    lblNewLabel_17.setText("縮小は");
    
    text_viewWindowZoomOutRatioMin = new FloatText(composite_13, SWT.BORDER);
    text_viewWindowZoomOutRatioMin.setMinimum(1.0E-4);
    text_viewWindowZoomOutRatioMin.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
    
    Label lblNewLabel_18 = new Label(composite_13, SWT.NONE);
    lblNewLabel_18.setText("倍まで");
    
    Label lblNewLabel_10 = new Label(composite, SWT.NONE);
    lblNewLabel_10.setText("ズーム時のスクロール移動(&S)");
    
    Composite composite_8 = new Composite(composite, SWT.NONE);
    composite_8.setLayout(new GridLayout(3, false));
    
    Label lblNewLabel_19 = new Label(composite_8, SWT.NONE);
    lblNewLabel_19.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
    lblNewLabel_19.setText("画面に表示されている大きさの");
    
    spinner_viewWindowMoveRatio = new Spinner(composite_8, SWT.BORDER);
    spinner_viewWindowMoveRatio.setMinimum(1);
    
    Label lblNewLabel_20 = new Label(composite_8, SWT.NONE);
    lblNewLabel_20.setText("％づつ");
    
    Label lblNewLabel_21 = new Label(composite, SWT.NONE);
    lblNewLabel_21.setText("ズーム時のスクロールバー(&B)");
    
    Composite composite_14 = new Composite(composite, SWT.NONE);
    composite_14.setLayout(new GridLayout(2, false));
    
    spinner_viewWindowScrollVisibleTime = new Spinner(composite_14, SWT.BORDER);
    spinner_viewWindowScrollVisibleTime.setMaximum(10000);
    
    Label lblNewLabel_22 = new Label(composite_14, SWT.NONE);
    lblNewLabel_22.setText("ミリ秒経ったら消す");
    
    Label lblNewLabel_23 = new Label(composite, SWT.NONE);
    lblNewLabel_23.setText("フルスクリーン時のマウスカーソル(&M)");
    
    Composite composite_15 = new Composite(composite, SWT.NONE);
    composite_15.setLayout(new GridLayout(2, false));
    
    spinner_viewWindowFullscreenHideCursorTime = new Spinner(composite_15, SWT.BORDER);
    spinner_viewWindowFullscreenHideCursorTime.setMaximum(10000);
    
    Label label = new Label(composite_15, SWT.NONE);
    label.setText("ミリ秒経ったら消す");
    
    Label lblNewLabel_24 = new Label(composite, SWT.NONE);
    lblNewLabel_24.setText("画像情報表示(&I)");
    
    check_viewWindowShowImageInfoDebugMessage = new Button(composite, SWT.CHECK);
    check_viewWindowShowImageInfoDebugMessage.setText("画像ロード時のデバッグメッセージも表示する");
    return composite;
  }
  private Spinner getSpinner_viewWindowForwardLoadNum() {
    return spinner_viewWindowForwardLoadNum;
  }
  private Spinner getSpinner_viewWindowForwardLoadInCacheRatio() {
    return spinner_viewWindowForwardLoadInCacheRatio;
  }
  private Spinner getSpinner_viewWindowMoveCursorColumnDelay() {
    return spinner_viewWindowMoveCursorColumnDelay;
  }
  private Spinner getSpinner_viewWindowMoveCursorRowDelay() {
    return spinner_viewWindowMoveCursorRowDelay;
  }
  private FloatText getText_viewWindowZoomInRatio() {
    return text_viewWindowZoomInRatio;
  }
  private FloatText getText_viewWindowZoomOutRatio() {
    return text_viewWindowZoomOutRatio;
  }
  private FloatText getText_viewWindowZoomInRatioMax() {
    return text_viewWindowZoomInRatioMax;
  }
  private FloatText getText_viewWindowZoomOutRatioMin() {
    return text_viewWindowZoomOutRatioMin;
  }
  private Spinner getSpinner_viewWindowScrollVisibleTime() {
    return spinner_viewWindowScrollVisibleTime;
  }
  private Spinner getSpinner_viewWindowFullscreenHideCursorTime() {
    return spinner_viewWindowFullscreenHideCursorTime;
  }
  private Button getCheck_viewWindowShowImageInfoDebugMessage() {
    return check_viewWindowShowImageInfoDebugMessage;
  }
  private Spinner getSpinner_viewWindowMoveRatio() {
    return spinner_viewWindowMoveRatio;
  }
}
