package cvj2.app.gui.util;

import java.io.File;
import java.util.HashMap;
import java.util.HashSet;

import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Display;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.app.ParseArg;
import cvj2.file.util.FileUtil;
import cvj2.util.Log;

/**
 * 拡張子別のアイコン画像
 */
public class ExtIcons {
  private static ExtIcons instance = new ExtIcons();
  private boolean disposed = false;
  private HashMap<String, Image> map = new HashMap<String, Image>();
  private HashSet<String> failExts = new HashSet<String>();
  private HashMap<String, Image> progMap = new HashMap<String, Image>();
  
  /**
   * インスタンス取得
   */
  public static ExtIcons instance() {
    return instance;
  }
  
  /*
   * コンストラクタ
   */
  private ExtIcons() {
    File iconDir = new File(ParseArg.appDir, "icons");
    if (!iconDir.isDirectory()) return;
    for (File f : iconDir.listFiles()) {
      try {
        if (!f.isFile()) continue;
        String base = FileUtil.splitBasenameExt(f)[0].toLowerCase();
        Log.ut("ext = ", base, ", icon file = ", f);
        Image img = SWTResourceManager.getImage(f.getAbsolutePath());
        map.put(base, img);
      }
      catch (Exception e) {
        Log.w(e);
      }
    }
  }

  /**
   * ファイルの拡張子に対応するアイコンを取得
   * @return 見つからない時は、null
   */
  public Image find(String name) {
    if (disposed) return null;
    
    String ext = FileUtil.splitBasenameExt(name)[1];
    if (ext.length() == 0) return null;

    // アプリに組み込みのアイコンを使用する
    if (!Config.get().isMainWindowUseOsIcon()) {
      return map.get(ext.substring(1).toLowerCase());
    }
    
    Image img = progMap.get(ext);
    if (img != null) return img;
    
    if (failExts.contains(ext)) return null;
    
    Program p = Program.findProgram(ext);
    if (p != null) {
      ImageData id = p.getImageData();
      if (id != null) {
        img = new Image(Display.getDefault(), id);
        progMap.put(ext, img);
        return img;
      }
    }
    failExts.add(ext);
    return null;
  }

  /**
   * 終了処理
   */
  public void dispose() {
    disposed = true;
    for (Image img : progMap.values()) {
      img.dispose();
    }
    progMap.clear();
    map.clear();
  }
}
