package cvj2.app.gui.util;

import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import cvj2.app.Config;
import cvj2.app.gui.dialogs.MsgBoxDialog;
import cvj2.app.gui.window.WindowSelector;

/**
 * メッセージボックス表示ユーティリティー。<br>
 * (このクラスのメソッドは、バックグラウンドスレッドからも呼べる)
 */
public class MsgBox {
  private MsgBox() {}
  
  /** メッセージボックスの戻り値 */
  public enum Ret {
    OK,
    CANCEL,
    YES,
    NO,
    ALL,
  }
  
  /** OK Cancel */
  static public boolean okCancel(String title, String msg) {
    int ret = showMsgBoxDialog(title, SWT.ICON_QUESTION, new String[] { "&OK", "キャンセル(&C)" }, new char[] { 'o', 'c' }, 0, msg, null);
    return ret == 0;
  }

  /** OK All Cancel */
  static public Ret okAllCancel(String title, String msg) {
    int ret = showMsgBoxDialog(title, SWT.ICON_QUESTION, new String[] { "&OK", "全て(&A)", "キャンセル(&C)" }, new char[] { 'o', 'a', 'c' }, 0, msg, null);
    if (ret == 0) return Ret.OK;
    if (ret == 1) return Ret.ALL;
    return Ret.CANCEL;
  }

  /** Yes All No Cancel */
  static public Ret yesAllNoCancel(String title, String msg) {
    int ret = showMsgBoxDialog(title, SWT.ICON_QUESTION, new String[] { "はい(&Y)", "全て(&A)", "いいえ(&N)", "キャンセル(&C)" }, new char[] { 'y', 'a', 'n', 'c' }, 0, msg, null);
    if (ret == 0) return Ret.YES;
    if (ret == 1) return Ret.ALL;
    if (ret == 2) return Ret.NO;
    return Ret.CANCEL;
  }
  
  /** Yes No Cancel */
  static public Ret yesNoCancel(String title, String msg) {
    int ret = showMsgBoxDialog(title, SWT.ICON_QUESTION, new String[] { "はい(&Y)", "いいえ(&N)", "キャンセル(&C)" }, new char[] { 'y', 'n', 'c' }, 0, msg, null);
    if (ret == 0) return Ret.YES;
    if (ret == 1) return Ret.NO;
    return Ret.CANCEL;
  }
  
  /** Error */
  static public void error(String title, String msg) {
    showMsgBoxDialog(title, SWT.ICON_ERROR, new String[] { "閉じる(&C)" }, new char[] { 'c' }, 0, msg, null);
  }

  /** Error */
  static public void error(String title, String dispMsg, String innerMsg) {
    showMsgBoxDialog(title, SWT.ICON_ERROR, new String[] { "閉じる(&C)" }, new char[] { 'c' }, 0, dispMsg, innerMsg);
  }
  
  /** Info */
  static public void info(String title, String msg) {
    showMsgBoxDialog(title, SWT.ICON_INFORMATION, new String[] { "閉じる(&C)" }, new char[] { 'c' }, 0, msg, null);
  }

  /**
   * {@link MsgBoxDialog} を使用してメッセージボックスを表示する
   */
  static public int showMsgBoxDialog(
      final String title, final int icon, final String[] buttonNames, final char[] shotcutChars, final int defaultButtonIndex, 
      final String dispMsg) 
  {
    return showMsgBoxDialog(title, icon, buttonNames, shotcutChars, defaultButtonIndex, dispMsg, null);
  }
  
  /**
   * {@link MsgBoxDialog} を使用してメッセージボックスを表示する
   */
  static public int showMsgBoxDialog(
      final String title, final int icon, final String[] buttonNames, final char[] shotcutChars, final int defaultButtonIndex, 
      final String dispMsg, final String innerMsg) 
  {
    if (Display.getCurrent() != null) {
      // UIスレッドから呼ばれたら、そのまま処理する
      return showMsgBoxDialog_inUiThread(title, icon, buttonNames, shotcutChars, defaultButtonIndex, dispMsg, innerMsg);
    }
    else {
      // バックグラウンドスレッドから呼ばれたら、syncExec で処理する
      final int[] ret = { 0 };
      Display.getDefault().syncExec(new Runnable() {
        @Override
        public void run() {
          ret[0] = showMsgBoxDialog_inUiThread(title, icon, buttonNames, shotcutChars, defaultButtonIndex, dispMsg, innerMsg);
        }
      });
      return ret[0];
    }
  }
  // このメソッドは、UIスレッド中からのみ呼ばれる
  static private int showMsgBoxDialog_inUiThread(String title, int icon, String[] buttonNames, char[] shotcutChars, int defaultButtonIndex, String dispMsg, String innerMsg) {
    Shell shell = Display.getCurrent().getActiveShell();
    if (shell == null) {
      shell = WindowSelector.instance().getCurrentShell();
    }
    boolean createShell = false;
    if (shell == null) {
      shell = new Shell();
      createShell = true;
    }
    try {
      MsgBoxDialog dlg = new MsgBoxDialog(shell, title, icon, buttonNames, shotcutChars, defaultButtonIndex);
      if (Config.get().isMsgboxHeadTailCR()) {
        dispMsg = "\n" + dispMsg + "\n";
      }
      return dlg.showMessage(dispMsg, innerMsg);
    }
    finally {
      if (createShell) shell.dispose();
    }
  }
}
