package cvj2.app.gui.widgets;

import java.text.DecimalFormat;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;

import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DragSource;
import org.eclipse.swt.dnd.DragSourceAdapter;
import org.eclipse.swt.dnd.DragSourceEvent;
import org.eclipse.swt.dnd.DropTarget;
import org.eclipse.swt.dnd.DropTargetAdapter;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;

import cvj2.app.Config;
import cvj2.app.Main;
import cvj2.app.file.FileInfoReader;
import cvj2.app.file.imgload.BackgroundImageLoader;
import cvj2.app.gui.data.FileListColumn;
import cvj2.app.gui.data.FileListModel;
import cvj2.app.gui.data.FileListSortElement;
import cvj2.app.gui.util.MsgBox;
import cvj2.app.gui.window.WindowController;
import cvj2.app.gui.window.main.MainWindowCommandHandler;
import cvj2.file.fs.FileInfo;
import cvj2.gui.swt.migemo.SWTMigemoKeyHandler;
import cvj2.gui.swt.ui.input.KeyCombination;
import cvj2.gui.swt.ui.input.MouseCombination;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel;
import cvj2.gui.swt.widgets.gdidpanel.GridSelection;
import cvj2.migemo.ListSearch;
import cvj2.migemo.Migemo;
import cvj2.migemo.Sen;
import cvj2.util.CnvRomaji2Kata;
import cvj2.util.Log;

/**
 * メイン画面のリスト、サムネイル表示パネル
 */
public class ListPanel extends GridPanel {
  // データ
  private FileListModel model;
  // 表示モード
  private boolean isThumbnailMode = false;
  // キー、マウスイベントのハンドラー
  private MainWindowCommandHandler cmdHandler;
  // ウインドウを操作するためのクラス
  private WindowController wndController;
  // フォーカス履歴
  private ListPanelFocusHistory focusHistory = new ListPanelFocusHistory(); 

  // ファイル名検索
  private ListSearch<FileInfo> listSearch;
  private class ListSearchCnv implements ListSearch.StringConverter<FileInfo> {
    @Override public String cnv(FileInfo element) {
      return element.getName();
    }
  }
  // ファイル名検索結果を表示するためのツールチップ
  private ListPanelToolTip tooltip;
  
  // ViewPanel で画像表示するための状態
  private ViewPanelState viewPanelState = new ViewPanelState();
  /** ViewPanelState の取得。 
   * (この情報の生存期間が ListPanel と同じなので、あえてここで管理する) */
  public ViewPanelState getViewPanelState() { return viewPanelState; };
  /** ViewPanelState の設定 */
  public void setViewPanelState(ViewPanelState state) { this.viewPanelState = state; };
  
  /**
   * コンストラクタ
   */
  public ListPanel(Composite parent, MainWindowCommandHandler cmdHandler, WindowController wndController) {
    super(parent, SWT.BORDER, DND.DROP_COPY | DND.DROP_MOVE);
    this.cmdHandler = cmdHandler;
    this.wndController = wndController;

    Config conf = Config.get();
    
    getHeader().setFont(conf.getFont());
    getCanvas().setFont(conf.getFont());
    getCanvas().setMenu(new ListPanelPopupMenu(getCanvas(), cmdHandler));
    
    Migemo migemo = new Migemo(conf.isMigemoIgnoreHiraKata(), true);
    Sen sen = new Sen(true);
    listSearch = new ListSearch<FileInfo>(migemo, sen, new ListSearchCnv());
    listSearch.setEnableLastSearchCache(true);
    tooltip = new ListPanelToolTip(this);
    
    DragSource dragSource = super.getDragSource();
    dragSource.setTransfer(new Transfer[] { FileTransfer.getInstance() });
    dragSource.addDragListener(new ListDragSource());
    
    DropTarget dropTarget = super.getDropTarget();
    dropTarget.setTransfer(new Transfer[] { FileTransfer.getInstance() });
    dropTarget.addDropListener(new ListDropTarget());
    
    super.getCanvas().addKeyListener(new KeyAdapter() {
      @Override
      public void keyPressed(KeyEvent e) {
        control_keyPressed(e);
      }
    });
    super.getCanvas().addMouseListener(new MouseAdapter() {
      @Override
      public void mouseDown(MouseEvent e) {
        control_mouseDown(e);
      }
    });
    getCanvas().addPaintListener(new PaintListener() {
      @Override
      public void paintControl(PaintEvent e) {
        startThumbnailBackgroundLoad(false);
      }
    });
    
    super.setSelectionListener(new GridSelectionListener() {
      @Override
      public void selectionUpdated() {
        updateApplicationWindowState();
        startThumbnailBackgroundLoad(true);
      }
    });
    
    super.setColumnSelectionListener(new ColumnSelectionListener() {
      @Override
      public void columnSelected(int columnIndex) {
        columnClicked(columnIndex);
      }
    });
    
    getCanvas().addFocusListener(new FocusAdapter() {
      @Override
      public void focusGained(FocusEvent e) {
        startThumbnailBackgroundLoad(true);
      }
    });
  }

  /**
   * ドラッグ
   */
  private class ListDragSource extends DragSourceAdapter {
    /** 開始 */
    @Override
    public void dragStart(DragSourceEvent event) {
      Log.start(event);
      if (getSelection().getSelectedIndices().size() == 0) {
        event.doit = false;
      }
    }
    /** データセット */
    @Override
    public void dragSetData(DragSourceEvent event) {
      Log.start(event);
      if (!FileTransfer.getInstance().isSupportedType(event.dataType)) return;
      event.data = ListPanelDndUtil.getAbsoluteFilenameArray(model.getSortedList(), getSelection().getSelectedIndices());
    }
    /** 完了 */
    @Override
    public void dragFinished(DragSourceEvent event) {
      Log.start(event);
    }
  }
  
  /**
   * ドロップ
   */
  private class ListDropTarget extends DropTargetAdapter {
    @Override
    public void drop(DropTargetEvent event) {
      Log.start(event);
      ListPanelDndUtil.handleDrop(ListPanel.this, event);
    }
  }

  private void updateApplicationWindowState() {
    updateApplicationWindowState(wndController);
  }
  
  /**
   * アプリケーションのウインドウ状態を更新する
   */
  public void updateApplicationWindowState(WindowController wndController) {
    if (model == null) return;

    // タブ名を更新
    wndController.updateTabNames();
    
    // ウインドウタイトル更新
    wndController.setTitle("" + model.getCurrentDir().getAbsolutePath());
    
    // ステータスバー更新
    FileInfo focusFile = getFocusItem();
    if (focusFile != null) {
      wndController.setStatusMessage(focusFile.getName());
    }
    else {
      wndController.setStatusMessage("");
    }

    // ステータスバー(ファイル領域)更新
    int focusIndex = getSelection().getFocusIndex();
    Set<Integer> sel = getSelection().getSelectedIndices();
    long totalBytes = 0;
    int unknownCount = 0;
    for (int index : sel) {
      FileInfo f = model.getSortedList().get(index);
      if (index == focusIndex) f.length();
      if (!f.isAttributeInitialized()) {
        unknownCount++;
      }
      else if (!f.isDirectory() || f.isArchiveRoot()) {
        totalBytes += f.length();
      }
    }
    String s = "";
    if (sel.size() > 0) {
      s = DecimalFormat.getNumberInstance().format(totalBytes) + " バイト";
      if (unknownCount > 0) s += " 以上？";
    }
    s += " (選択 " + sel.size() + " / 全 " + model.getSortedList().size();
    if (model.getOriginalList().getHideCount() > 0) {
      s += " 非表示 " + model.getOriginalList().getHideCount();
    }
    s += ")";
    wndController.setStatusFileMessage(s);
    

    // メニュー状態更新
    wndController.updateMenuStatus();
  }

  /*
   * パネルのキーイベント処理
   */
  private void control_keyPressed(KeyEvent e) {
    // Migemo のインクリメンタル検索
    if (Main.getMigemoKey().handleKeyEvent(e)) {
      searchList(1, false);
      return;
    }

    // その他コマンド
    cmdHandler.handle(new KeyCombination(e));
  }
  
  /*
   * パネルのマウスイベント処理
   */
  private void control_mouseDown(MouseEvent e) {
    Log.start(new MouseCombination(e));
    cmdHandler.handle(new MouseCombination(e));
  }
  
  /*
   * 列ヘッダークリック処理
   */
  private void columnClicked(int columnIndex) {
    columnClicked(model.getColumns()[columnIndex]);
  }
  /**
   * 列ヘッダーがクリックされた
   */
  public void columnClicked(FileListColumn column) {
    Log.start(column);

    // クリックされたのがファイル名以外なら、ファイル情報の読み込みを試す
    if (!column.isFilenameAttribute()) {
      if (!FileInfoReader.instance().tryReadFg(getShell(), model.getOriginalList())) return;
    }
    
    // ソートする
    model.getSortedList().clickColumn(column);
    // 全て再描画
    getSelection().dataUpdated(true);
    getHeader().redraw();
    // ソートされた順序でファイル情報のバックグラウンド読み込みを再開する
    startBgFileInfoRead();
  }
  
  /*
   * バックグラウンドでファイル情報の読み込みを開始する (リスト表示モードの時)
   */
  private void startBgFileInfoRead() {
    // 読み込み開始
    FileInfoReader.instance().readBg(model.getSortedList(), new Observer() {
      @Override
      public void update(Observable o, Object arg) {
        if (isDisposed()) return;
        getDisplay().asyncExec(new Runnable() {
          @Override
          public void run() {
            if (getScroll().isDisposed()) return;
            getScroll().redraw();
          }
        });
      }
    });
  }
  
  /*
   * バックグラウンドでサムネイルの読み込みを行う
   */
  private void startThumbnailBackgroundLoad(final boolean clearOldTask) {
    // あまり頻繁に呼ばないようにする
    // バックグラウンドスレッドでエラーが頻繁に起きると、エラーメッセージボックスの表示ができなくなるため
    if (thumbnailBackgroundLoading) return;
    thumbnailBackgroundLoading = true;
    getDisplay().timerExec(Config.get().getMainWindowThumbnailBackgroundLoadSleep(), new Runnable() {
      @Override public void run() {
        startThumbnailBackgroundLoad_internal(clearOldTask);
        thumbnailBackgroundLoading = false;
      }
    });
  }
  private boolean thumbnailBackgroundLoading = false;
  private void startThumbnailBackgroundLoad_internal(boolean clearOldTask) {
    Log.start(clearOldTask);
    if (!isThumbnailMode) return;
    if (isDisposed()) return;
    if (!isVisible()) return;
    Config conf = Config.get();
    GridSelection sel = getSelection();

    // 古いタスクをキャンセルする
    if (clearOldTask) {
      BackgroundImageLoader.instance().cancelWaitTasks();
    }
    
    int bgLoadMax = conf.getImageLoadBackgroundThreadNum() - BackgroundImageLoader.instance().getWaitNum();
    int directioion = sel.getLastMoveDirection();
    if (directioion == 0) directioion = 1;

    // 画面内を読む
    bgLoadMax = startThumbnailBackgroundLoad_internal(bgLoadMax, 0);
    // 画面外を読む
    bgLoadMax = startThumbnailBackgroundLoad_internal(bgLoadMax, directioion);
  }
  private int startThumbnailBackgroundLoad_internal(int bgLoadMax, int direction) {
    Config conf = Config.get();
    GridSelection sel = getSelection();
    
    // 画面に表示されているものを読む
    int start = sel.getScrollIndex() * sel.getColumn();
    int end = start + (sel.getVisibleRow() + 1) * sel.getColumn();
    
    if (direction > 0) {
      // 下のページを読む
      start = (sel.getScrollIndex() + sel.getVisibleRow()) * sel.getColumn();
      end = start + (sel.getColumn() * sel.getVisibleRow() * conf.getMainWindowThumbnailPrefetchPage());
    }
    else if (direction < 0) {
      // 上のページを読む
      start = sel.getScrollIndex() * sel.getColumn();
      end = start - (sel.getColumn() * sel.getVisibleRow() * conf.getMainWindowThumbnailPrefetchPage());
    }
    
    // ロード依頼した画像数
    int bgLoadNum = 0;

    if (direction == 0) direction = 1;
    for (int i = start; i >= 0 && i < sel.getDataSize() && i != end; i += direction) {
      // 読み込み数チェック
      if (bgLoadNum >= bgLoadMax) return 0;
      
      // 読み込み対象チェック
      FileInfo f = model.getSortedList().get(i);
      if (!f.mayGetThumbnail()) continue;
      if (BackgroundImageLoader.instance().existsInThumbCache(f)) continue;
      if (BackgroundImageLoader.instance().existsThumbnailTask(f)) continue;

      // 読み込む
      if (!BackgroundImageLoader.instance().isShutdown()) {
        model.getThumbnail(f);
        sel.redrawCell(i);
      }
      bgLoadNum++;
    }
    return bgLoadMax - bgLoadNum;
  }
  /*
   * バックグラウンドでサムネイルの読み込みをキャンセル
   */
  private void stopThumbnailBackgroundLoad() {
    BackgroundImageLoader.instance().cancelWaitTasks();
  }

  /**
   * リスト内検索 
   */
  public void searchList(int direction, boolean startNext) {
    SWTMigemoKeyHandler migemoKey = Main.getMigemoKey();
    GridSelection sel = getSelection();
    int[] find = listSearch.find(migemoKey.getInputStr(), model.getSortedList(), sel.getFocusIndex(), startNext, direction);
    String input = migemoKey.getInputStr() + " (" + CnvRomaji2Kata.cnv(migemoKey.getInputStr()) + ")";
    if (find == null) {
      String msg = input + "\n" +
          "\tローマ字アルファベットで入力してください\n" + 
          "\t/ で始めると、ファイル名の先頭に一致します\n" +
          "\tctrl+n: 次検索、ctrl+p: 前検索\n" +
          "\t(カタカナか漢字でも入力できます)";
      tooltip.setTextAndShow(msg);
      return;
    }
    
    sel.setFocusIndex(find[0]);
    FileInfo f = model.getSortedList().get(find[0]);

    tooltip.setTextAndShow(input + " | " + f.getName().substring(find[1], find[2]));
  }
  
  /**
   * サムネイルモードかどうか
   */
  public boolean isThumbnailMode() {
    return isThumbnailMode;
  }
  
  /**
   * 画面表示モードの切り替え
   */
  public void setThumbnailMode(boolean b) {
    // フォーカス位置を記憶
    focusHistory.save(this, model);

    // モード切り替え
    setGridPanelData(b);

    // フォーカス位置を復元
    focusHistory.restore(this, true, true);
  }
  
  /*
   * 現在の FileListModel データで、画面表示モードを切り替える
   */
  private void setGridPanelData(boolean thumbMode) {
    Log.start(thumbMode);
    Config conf = Config.get();
    this.isThumbnailMode = thumbMode;
    if (model == null) return;

    // モード設定に従って、GridPanel に描画情報を再設定する
    if (isThumbnailMode) {
      setData(
          model,
          new ListPanelThumbnailPainter(this, model), 
          new Point(conf.getMainWindowThumbnailMarginX(), conf.getMainWindowThumbnailMarginY()), 
          null);
      model.addModelListener(new ModelListener() {
        @Override public void dataChanged(int updateIndex) {
          if (isDisposed()) return;
          startThumbnailBackgroundLoad(false);
          getScroll().redraw();
        }
      });
      super.setScrollBgPaintListener(new ListPanelScrollBgPainter(model, true));

      // サムネイルのバックグラウンド読み込み
      startThumbnailBackgroundLoad(true);
    }
    else {
      ListPanelColumnPainter columnPainter = new ListPanelColumnPainter(model);
      setData( 
          model, 
          new ListPanelRowPainter(this, model, columnPainter), 
          new Point(0, 0), 
          columnPainter);
      super.setScrollBgPaintListener(new ListPanelScrollBgPainter(model, false));

      // リストモードになったら、サムネイルの読み込みをやめる
      stopThumbnailBackgroundLoad();
    }
  }
  
  /**
   * ディレクトリ移動
   * @param dir 移動先
   * @param selFile 移動後に選択するファイル
   */
  public void setCurrentDir(FileInfo dir, FileInfo selFile) {
    Log.start(dir, selFile);

    // 移動前のフォーカス位置を記憶
    focusHistory.save(this, model);

    FileListModel oldModel = model;
    
    // 渡された引数の確認
    FileInfo newDir = dir;
    while (!newDir.exists() || !newDir.isDirectory()) {
      newDir = newDir.getParent();
    }
    if (!newDir.equals(dir)) {
      MsgBox.info("存在しないディレクトリ", 
          "  " + dir + "\n" +
          "このディレクトリは存在しません\n" + 
          "  " + newDir + "\n" + 
          "に移動します。");
      dir = newDir;
      selFile = null;
    }
    
    // 新しいモデル生成
    model = new FileListModel(dir, new FileListColumn[] { FileListColumn.NAME, FileListColumn.EXT, FileListColumn.SIZE, FileListColumn.DATE });
    // 描画情報設定
    setGridPanelData(isThumbnailMode);
    
    // 現在と同じカレントディレクトリなら(refreshされたとき)、ソート条件も復元する
    boolean sameDir = (oldModel != null && model.getCurrentDir().equals(oldModel.getCurrentDir()));
    if (sameDir) {
      boolean doSort = true;
      // ソートの復元にファイル情報が必要なら、読み込む
      if (oldModel.getSortedList().isNeedFileAttribute()) {
        if (!FileInfoReader.instance().tryReadFg(getShell(), model.getOriginalList())) doSort = false;
      }
      if (doSort) model.getSortedList().resetSortColumns(oldModel.getSortedList());
    }

    if (selFile != null) {
      // 選択ファイルが指定されていたら、それをセットする
      int i = model.getSortedList().find(selFile);
      if (i >= 0) getSelection().setFocusIndex(i);
      else getSelection().setFocusIndex(-1 * (i + 1));
      
      // 以前のスクロール位置を復元
      focusHistory.restore(this, false, false);
    }
    else {
      // 以前のフォーカスとスクロール位置を復元
      focusHistory.restore(this, true, sameDir);
    }

    // ソート条件が変わったかもしれないので、ヘッダーを再描画
    getHeader().redraw();
    // バックグラウンドでファイル名解析 (解析中にソートされると困るので、オリジナルを渡す)
    listSearch.analyzeBackground(model.getOriginalList());
    // ファイル情報のバックグラウンド読み込み
    startBgFileInfoRead();
  }

  /**
   * フォーカスのあるファイル取得
   */
  public FileInfo getFocusItem() {
    int sel = getSelection().getFocusIndex();
    if (sel >= 0) return model.getSortedList().get(sel);
    return null;
  }

  /**
   * データモデル取得
   */
  public FileListModel getModel() {
    return model;
  }
  
  /**
   * カレントディレクトリを取得
   */
  public FileInfo getCurrentDir() {
    return model.getCurrentDir();
  }
  
  /**
   * ソート状態取得
   */
  public FileListSortElement getSort() {
    return model.getSortedList().getFirst();
  }
}
