package cvj2.app.gui.widgets;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Control;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.app.gui.data.FileListModel;
import cvj2.app.gui.data.FileListColumn;
import cvj2.app.gui.data.FileListSortElement;
import cvj2.gui.swt.util.GCX;
import cvj2.gui.swt.util.SWTUtil;
import cvj2.gui.swt.widgets.WidgetPaintUtil;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel;
import cvj2.util.Log;

/**
 * 列ヘッダー描画
 */
public class ListPanelColumnPainter implements GridPanel.ColumnPainter {
  /** 昇順アイコン */
  protected Image iconAsc  = SWTResourceManager.getImage(this.getClass(), "sort_asc.png");
  /** 降順アイコン */
  protected Image iconDesc = SWTResourceManager.getImage(this.getClass(), "sort_desc.png");
  
  private int fontHeight;
  private FileListModel model;

  public ListPanelColumnPainter(FileListModel model) {
    this.model = model;
  }

  @Override
  public Object getColumnData(int column) {
    return model.getColumns()[column].getName();
  }

  @Override
  public int getColumnHeight(Control control) {
    GC gc = new GC(control);
    FontMetrics fm = gc.getFontMetrics();
    Log.d(gc.getFont().getFontData()[0], ", ", SWTUtil.toString(fm));
    fontHeight = fm.getHeight();
    gc.dispose();
    Config c = Config.get();
    return fontHeight + c.getMainWindowListMarginTop() + c.getMainWindowListMarginBottom();
  }

  @Override
  public int[] getColumnWidths(int headerWidth) {
    FileListColumn[] columns = model.getColumns();
    int[] ret = new int[columns.length];
    for (int i = 1; i < ret.length; i++) {
      ret[i] = columns[i].getWidth();
      headerWidth -= ret[i];
    }
    ret[0] = Math.max(FileListColumn.NAME.getWidth(), headerWidth);
    return ret;
  }

  @Override
  public boolean paintControl(GC gc, Rectangle rect, Object data, int column, boolean isSelected) {
    GCX gcx = new GCX(gc);
    Config conf = Config.get();
    
    // ボタン描画
    WidgetPaintUtil.drawButtonBg(gcx, rect.x, rect.y, rect.width, rect.height, isSelected);
    
    gcx.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
    gcx.setForeground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_FOREGROUND));

    int x = rect.x + conf.getMainWindowListMarginLeft();
    // ソート描画
    FileListSortElement sort = model.getSortedList().getFirst();
    if (sort != null && model.getSortedList().getColumnIndex(sort.getColumn()) == column) {
      Image ico = sort.isAscent() ? iconAsc : iconDesc;
      ImageData id = ico.getImageData();
      gcx.drawImage(ico, 0, 0, id.width, id.height, x, 0, rect.height, rect.height);
      x += rect.height + conf.getMainWindowListMarginLeft();
    }
    
    // タイトル描画
    gcx.drawString("" + data, x, rect.y + conf.getMainWindowListMarginTop());

    // ボタン描画
    WidgetPaintUtil.drawButtonFg(gcx, rect.x, rect.y, rect.width, rect.height, isSelected);
    
    return true;
  }
}
