package cvj2.app.gui.widgets;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;

import cvj2.app.Main;
import cvj2.app.gui.util.GlobalErrorHandler;
import cvj2.app.gui.window.main.MainWindowFileOperation;
import cvj2.file.fs.FileInfo;
import cvj2.file.fs.FileInfoFactory;
import cvj2.file.fs.FileInfoList;
import cvj2.gui.swt.dnd.MyFileTransfer;
import cvj2.gui.swt.dnd.MyFileTransferData;
import cvj2.util.Log;

/**
 * ListPanel で使用するユーティリティー
 */
public class ListPanelDndUtil {
  private ListPanelDndUtil() {}
  
  /**
   * 絶対パス名の配列を取得
   */
  static public String[] getAbsoluteFilenameArray(FileInfoList fileList, Set<Integer> sel) {
    List<String> files = new ArrayList<String>();
    for (int i : sel) {
      files.add(fileList.get(i).getAbsolutePath());
    }
    return files.toArray(new String[0]);
  }
  
  /**
   * ファイル名の配列を取得
   */
  static public String getFilenames(FileInfoList fileList, Set<Integer> sel) {
    StringBuilder sb = new StringBuilder();
    for (int i : sel) {
      if (sb.length() != 0) sb.append("\r\n");
      sb.append(fileList.get(i).getName());
    }
    return sb.toString();
  }
  
  /**
   * ファイル名のリストをメッセージボックス用に変換する
   */
  static public String toMsgBoxString(String[] files, String prefix, int max) {
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < files.length ;i++) {
      sb.append(prefix);
      if (i >= max) {
        sb.append("(以下略) 全 " + files.length + "\n");
        break;
      }
      sb.append(files[i]).append('\n');
    }
    return sb.toString();
  }

  /**
   * D&D の drop イベントを処理する
   * @param lp Drop されたListPanel 
   */
  static public void handleDrop(ListPanel lp, DropTargetEvent event) {
    try {
      // FileTransfer しか受け付けなくしている
      String[] fileNames = (String[]) event.data;
      
      // ドラッグ元からカレントディレクトリのファイルが渡されたかどうか確認する
      for (String s : fileNames) {
        FileInfo d = FileInfoFactory.create(new File(s).getParentFile());
        if (lp.getCurrentDir().equals(d)) return;
      }
      
      // 操作確認
      boolean isCopy;
      switch (event.detail) { 
      case DND.DROP_COPY: 
        isCopy = true; 
        break;
      case DND.DROP_MOVE: 
        isCopy = false; 
        break;
      default:
        Log.e("invalid event.detail = ", event.detail);
        GlobalErrorHandler.instance().setDebugMessage("D&D error", null);
        return;
      }
      
      // コピー、移動
      MainWindowFileOperation.copyOrMove(lp, fileNames, isCopy, false);
    }
    finally {
      event.feedback = DND.FEEDBACK_NONE;
      event.operations = DND.DROP_NONE;
      event.detail = DND.DROP_NONE;
    }
  }
  
  /**
   * カット、コピー機能のハンドラ
   * @param list ファイルリスト
   * @param indices ファイルリストの選択ファイルのインデックス
   * @return 処理するファイルが存在しない時は false
   */
  static public boolean handleCutCopy(FileInfoList list, Set<Integer> indices, boolean isCopy) {
    if (indices.size() == 0) return false;
    
    Main.getClipboard().setContents(
        new Object[] { 
            getAbsoluteFilenameArray(list, indices),
            new MyFileTransferData(isCopy, getAbsoluteFilenameArray(list, indices)),
            getFilenames(list, indices),
        },
        new Transfer[] {
            FileTransfer.getInstance(),
            MyFileTransfer.getInstance(),
            TextTransfer.getInstance(),
        });
    return true;
  }
}
