package cvj2.app.gui.widgets;

import java.util.Set;

import org.apache.commons.lang3.ObjectUtils;

import cvj2.app.Config;
import cvj2.app.gui.data.FileListModel;
import cvj2.app.gui.data.FileListSortElement;
import cvj2.file.fs.FileInfo;
import cvj2.file.fs.FileInfoList;
import cvj2.util.CacheMap;
import cvj2.util.DelimString;
import cvj2.util.Log;

/**
 * ディレクトリを移動しても、リスト内のフォーカス位置とスクロール位置を覚えておく
 */
public class ListPanelFocusHistory {
  private CacheMap<FileInfo, State> cache = new CacheMap<FileInfo, State>(Config.get().getMainWindowFocusPositionHistoryNum());

  private class State {
    FileInfo focusFile;
    boolean isThumbnailMode;
    int focusIndex;
    int scrollRow;
    int listSize;
    Set<Integer> sel;
    FileListSortElement sort;
    int fileListHash;

    @Override
    public String toString() {
      DelimString s = new DelimString(", ", "=");
      s.append("focusFile", focusFile);
      s.append("isThumbnailMode", isThumbnailMode);
      s.append("focusIndex", focusIndex);
      s.append("scrollRow", scrollRow);
      s.append("listSize", listSize);
      s.append("sel.size", sel.size());
      return s.toString();
    }
  }
  
  private int hash(FileInfoList fil) {
    return ObjectUtils.hashCodeMulti(fil.toArray());
  }

  /**
   * 状態保存
   */
  public void save(ListPanel lp, FileListModel model) {
    if (model == null) return;
    State state = new State();
    state.focusFile = lp.getFocusItem();
    state.isThumbnailMode = lp.isThumbnailMode();
    state.focusIndex = lp.getSelection().getFocusIndex();
    state.scrollRow = lp.getSelection().getScrollIndex();
    state.listSize = model.getOriginalList().size();
    state.sel = lp.getSelection().getSelectedIndices();
    state.sort = model.getSortedList().getFirst();
    state.fileListHash = hash(model.getOriginalList());
    if (state.focusFile == null) return;
    cache.put(lp.getCurrentDir(), state);
    Log.ut(lp.getCurrentDir(), ", state dump: ", state);
  }

  /**
   * 状態復元
   */
  public void restore(ListPanel lp, boolean restoreFocus, boolean restoreSelection) {
    FileListModel model = lp.getModel();
    if (model == null) return;
    State state = cache.get(lp.getCurrentDir());
    if (state == null) return;
    Log.ut(lp.getCurrentDir(), ", state dump: ", state);
    
    int focusIndex = model.getSortedList().find(state.focusFile);

    if (restoreFocus) {
      // フォーカス位置復元
      if (focusIndex < 0) {
        lp.getSelection().setFocusIndex(state.focusIndex);
      }
      else {
        lp.getSelection().setFocusIndex(focusIndex);
      }
    }
    
    // 全ての条件が一致したら、スクロール位置と選択状態も復元する
    if (state.isThumbnailMode == lp.isThumbnailMode() && 
        state.focusIndex == focusIndex && 
        state.listSize == model.getOriginalList().size() &&
        model.getSortedList().getFirst() == state.sort && 
        hash(model.getOriginalList()) == state.fileListHash)
    {
      lp.getSelection().setScrollIndex(state.scrollRow);
      lp.getSelection().scrollToFocus();
      
      if (restoreSelection) {
        lp.getSelection().setSelectedIndices(state.sel);
      }
    }
  }
}
