package cvj2.app.gui.widgets;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.app.gui.util.ExtIcons;
import cvj2.file.fs.FileInfo;
import cvj2.gui.swt.util.GCX;
import cvj2.gui.swt.util.SWTUtil;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel;
import cvj2.util.Log;

/**
 * ListPanel のリスト表示、サムネイル表示の描画を行う抽象クラス
 */
abstract public class ListPanelPainter implements GridPanel.CellPainter {
  protected ListPanel listPanel;
  
  // リスト、サムネイル 両方で使用するリソース
  protected Image iconArchive = SWTResourceManager.getImage(this.getClass(), "archive.png");
  protected Image iconFolder = SWTResourceManager.getImage(this.getClass(), "folder.png");
  protected Image iconFile = SWTResourceManager.getImage(this.getClass(), "file.png");
  protected Image iconPict = SWTResourceManager.getImage(this.getClass(), "pict.png");
  
  protected Color selBg = SWTResourceManager.getColor(SWT.COLOR_LIST_SELECTION); 
  protected Color selFg = SWTResourceManager.getColor(SWT.COLOR_LIST_SELECTION_TEXT); 
  protected Color border = SWTResourceManager.getColor(SWT.COLOR_WIDGET_BORDER);
  protected Color fg = SWTResourceManager.getColor(SWT.COLOR_LIST_FOREGROUND);
  protected Color bg = SWTResourceManager.getColor(SWT.COLOR_LIST_BACKGROUND);

  protected int fontHeight;

  /**
   * コンストラクタ
   */
  public ListPanelPainter(ListPanel lp) {
    this.listPanel = lp;
    GC gc = new GC(lp);
    FontMetrics fm = gc.getFontMetrics();
    Log.d(gc.getFont().getFontData()[0], ", ", SWTUtil.toString(fm));
    fontHeight = fm.getHeight();
    gc.dispose();
  }

  public void drawIconAndFilename(GCX gcx, FileInfo f, Rectangle rect, boolean showIcon, boolean isSelected) {
    Config conf = Config.get();
    gcx.setClipping(rect);

    // アイコン
    if (showIcon) {
      drawIcon(gcx, f, rect.x, rect.y, rect.height);
    }
    // ファイル名
    gcx.setForeground(isSelected ? selFg : fg);
    gcx.drawString(f.getName(), 
        rect.x + conf.getMainWindowListMarginLeft() + (showIcon ? rect.height : 0), 
        rect.y + conf.getMainWindowListMarginTop());
  }
  
  /**
   * ファイルに対応したアイコンを描画
   */
  public void drawIcon(GCX gcx, FileInfo f, int x, int y, int size) {
    Image image;
    if (f.isArchiveRoot()) {
      image = iconArchive;
    }
    else if (f.isDirectory()) {
      image = iconFolder;
    }
    else if (f.isImageFile()) {
      image = iconPict;
    }
    else {
      image = ExtIcons.instance().find(f.getName());
      if (image == null) image = iconFile;
    }
    ImageData id = image.getImageData();
    gcx.drawImage(image, 0, 0, id.width, id.height, x, y, size, size);
  }
}
