package cvj2.app.gui.widgets;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import cvj2.app.Config;
import cvj2.app.gui.data.FileListModel;
import cvj2.app.gui.data.FileListColumn;
import cvj2.file.fs.FileInfo;
import cvj2.file.util.FileUtil;
import cvj2.gui.swt.util.GCX;

/**
 * ListPanel の行描画
 */
public class ListPanelRowPainter extends ListPanelPainter {
  private FileListModel model;
  private ListPanelColumnPainter columnPainter;
  
  // 列幅のキャッシュ
  private int lastWidth = -1;
  private int[] lastColumnWidths;

  /**
   * コンストラクタ
   */
  public ListPanelRowPainter(ListPanel gridPanel, FileListModel model, ListPanelColumnPainter columnPainter) {
    super(gridPanel);
    this.model = model;
    this.columnPainter = columnPainter;
  }
  
  @Override
  public void paintControl(GC gc, Rectangle rect, Object data, int dataIndex, boolean isSelected, boolean hasFocus) {
    GCX gcx = new GCX(gc);
    FileInfo f = (FileInfo) data;
    Config conf = Config.get();
    
    // 列幅取得
    int[] columnWidths = null;
    if (rect.width == lastWidth) columnWidths = lastColumnWidths;
    if (columnWidths == null) {
      columnWidths = columnPainter.getColumnWidths(rect.width);
      lastWidth = rect.width;
      lastColumnWidths = columnWidths;
    }
    
    // 背景
    gcx.setBackground(isSelected ? selBg : bg);
    gcx.fillRectangle(rect);
    
    gcx.setForeground(isSelected ? selFg : fg);
    int x = rect.x;
    int y = rect.y;

    // ファイル名
    super.drawIconAndFilename(gcx, f, 
        new Rectangle(x, y, columnWidths[0], rect.height), 
        conf.isMainWindowListShowIcon(),
        isSelected);
    x += columnWidths[0];

    // 日付など
    for (int i = 1; i < columnWidths.length; i++) {
      Rectangle r = new Rectangle(
          x + conf.getMainWindowListMarginLeft(), 
          y + conf.getMainWindowListMarginTop(), 
          columnWidths[i] - conf.getMainWindowListMarginLeft() * 2, 
          rect.height - conf.getMainWindowListMarginTop() - conf.getMainWindowListMarginBottom());
      gcx.setClipping(r);
      paint(gcx, f, model.getColumns()[i], r);
      x += columnWidths[i];
    }
    gcx.setClipping(rect);
    
    // 枠 (右と下)
    gcx.setForeground(border);
    gcx.drawRectangle(rect.x, rect.y - 1, rect.width, rect.height + 1);

    // フォーカス
    if (hasFocus) {
      gcx.drawFocus(rect.x + 1, rect.y, rect.width - 2, rect.height - 1);
    }
  }
  
  private SimpleDateFormat datef = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
  private DecimalFormat sizef = new DecimalFormat(",###");
  private void paint(GCX gcx, FileInfo f, FileListColumn column, Rectangle r) {
    if (r.width == 0) return;
    boolean left = true;
    String s = "";
    switch (column) {
    case EXT: 
      s = FileUtil.splitBasenameExt(f.getName())[1];
      if (s.length() > 1) s = s.substring(1);
      break;
    case DATE:
      s = datef.format(new Date(f.lastModified()));
      left = false;
      break;
    case SIZE:
      if (!f.isDirectory() || f.isArchiveRoot()) {
        s = sizef.format(f.length());
        left = false;
      }
      break;
    }
    if (left) {
      gcx.drawString(s, r.x, r.y);
    }
    else {
      int x = r.x + r.width - gcx.stringExtent(s).x;
      gcx.drawString(s, x, r.y);
    }
  }
  
  @Override
  public Point getCellSize() {
    Config c = Config.get();
    return new Point(0, fontHeight + c.getMainWindowListMarginTop() + c.getMainWindowListMarginBottom());
  }
}
