package cvj2.app.gui.widgets;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.file.imgload.BackgroundImageLoader;
import cvj2.app.gui.data.FileListModel;
import cvj2.file.fs.FileInfo;
import cvj2.file.fs.FileInfoList;
import cvj2.gui.swt.util.GCX;

/**
 * スクロールバーの背景描画
 */
public class ListPanelScrollBgPainter implements PaintListener {
  private FileListModel model;
  private boolean isThumbnailMode;

  private Color readBg = SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND);
  private Color unreadBg = SWTResourceManager.getColor(SWT.COLOR_WIDGET_NORMAL_SHADOW);

  /**
   * コンストラクタ
   */
  public ListPanelScrollBgPainter(FileListModel model, boolean isThumbnailMode) {
    this.model = model;
    this.isThumbnailMode = isThumbnailMode;
  }

  /**
   * 描画
   */
  @Override
  public void paintControl(PaintEvent e) {
    GCX gcx = new GCX(e.gc);
    Rectangle rect = (Rectangle) e.data;
    
    FileInfoList list = model.getSortedList();
    int oldY = rect.y;
    int read = 0;
    int unread = 0;
    
    for (int i = 0; i < list.size(); i++) {
      int newY = (int)(rect.y + (double)rect.height * (i + 1) / list.size());
      boolean dataOk = false;
      
      FileInfo f = list.get(i);
      if (isThumbnailMode) {
        // サムネイルモードの時
        if (f.isAttributeInitialized()) {
          dataOk = !f.mayGetThumbnail() || BackgroundImageLoader.instance().existsInThumbCache(f);
        }
      }
      else {
        // リストモードの時
        dataOk = f.isAttributeInitialized();
      }

      if (dataOk) {
        read++;
      }
      else {
        unread++;
      }
      
      // Y座標が前回と同じならループする
      if (newY == oldY) continue;
      
      // 描画
      paint(gcx, rect, oldY, newY, read >= unread);
      read = 0;
      unread = 0;
      oldY = newY;
    }
    paint(gcx, rect, oldY, rect.y + rect.height, read >= unread);
  }
  
  private void paint(GCX gcx, Rectangle rect, int oldY, int newY, boolean read) {
    gcx.setBackground(read ? readBg : unreadBg);
    gcx.fillRectangle(rect.x, oldY, rect.width, newY);
  }
}
