package cvj2.app.gui.widgets;

import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.app.file.imgload.BackgroundImageLoader;
import cvj2.app.file.imgload.CachedImage;
import cvj2.app.gui.data.FileListModel;
import cvj2.file.fs.FileInfo;
import cvj2.gui.swt.util.GCX;
import cvj2.img.AWT2SWT;
import cvj2.img.ImageUtil;
import cvj2.img.SWT2AWT;

/**
 * ListPanel のサムネイル描画
 */
public class ListPanelThumbnailPainter extends ListPanelPainter {
  private Image iconNormalFile = SWTResourceManager.getImage(this.getClass(), "file.png");
  private Image iconNowLoading = SWTResourceManager.getImage(this.getClass(), "now_loading.png");
  private Image iconPictUnknown = SWTResourceManager.getImage(this.getClass(), "pict_unknown.png");
  private Image iconFolderUnknown = SWTResourceManager.getImage(this.getClass(), "folder_unknown.png");

  private FileListModel model;
  
  /**
   * コンストラクタ
   * @param model 
   */
  public ListPanelThumbnailPainter(ListPanel gridPanel, FileListModel model) {
    super(gridPanel);
    this.model = model;
  }

  @Override
  public void paintControl(GC gc, Rectangle rect, Object data, int dataIndex, boolean isSelected, boolean hasFocus) {
    GCX gcx = new GCX(gc);
    Config conf = Config.get();
    FileInfo f = (FileInfo) data;
  
    // 背景
    gcx.setBackground(isSelected ? selBg : bg);
    gcx.fillRectangle(rect);
    if (!isSelected && conf.isDrawThumbnailBackground()) {
      gcx.setBackground(SWTResourceManager.getColor(AWT2SWT.toRGB(conf.getImageBackgroundColor())));
      gcx.fillRectangle(rect.x, rect.y, rect.width, conf.getMainWindowThumbnailSizeY() + 1);
      gcx.setBackground(isSelected ? selBg : bg);
    }
    
    // ファイル名
    super.drawIconAndFilename(gcx, f, 
        new Rectangle(rect.x, rect.y + conf.getMainWindowThumbnailSizeY(), rect.width, rect.height - conf.getMainWindowThumbnailSizeY()), 
        conf.isMainWindowThumbnailShowIcon(),
        isSelected);
    gcx.setClipping(rect);
    
    if (f.mayGetThumbnail()) {
      // サムネイル画像があるかもしれない (まだロードしてないかもしれないけど)
      CachedImage img = null;
      if (BackgroundImageLoader.instance().existsInThumbCache(f)) {
        img = model.getThumbnail(f);
        if (img != null && img.ref()) {
          // サムネイルがロードされていた
          try {
            Rectangle imgDrawArea = new Rectangle(rect.x, rect.y, conf.getMainWindowThumbnailSizeX(), conf.getMainWindowThumbnailSizeY());
            Rectangle imgDrawRect = AWT2SWT.toRectangle(ImageUtil.calcImageFitRect(img.getSize(), SWT2AWT.toRectangle(imgDrawArea)));
            gcx.drawImage(img.getImage(), imgDrawRect.x, imgDrawRect.y);
          }
          finally {
            img.unref();
          }
        }
        else {
          // サムネイルは消えていたので、ロード中画像を出しておく
          gcx.drawImage(iconNowLoading, rect.x, rect.y);
        }
      }
      else if (BackgroundImageLoader.instance().existsThumbnailRunTask(f)) {
        // サムネイルロード中
        gcx.drawImage(iconNowLoading, rect.x, rect.y);
      }
      else if (f.isDirectory()) {
        // サムネイルロード待ち (フォルダー)
        gcx.drawImage(iconFolderUnknown, rect.x, rect.y);
      }
      else {
        // サムネイルロード待ち (画像)
        gcx.drawImage(iconPictUnknown, rect.x, rect.y);
      }
    }
    else {
      // サムネイル画像はありえない
      gcx.drawImage(iconNormalFile, rect.x, rect.y);
    }
    
    // 枠
    gcx.setClipping(rect);
    gcx.drawRectangle(rect);
    if (hasFocus) {
      gcx.drawFocus(rect.x + 1, rect.y + 1, rect.width - 2, rect.height - 2);
    }
  }

  @Override
  public Point getCellSize() {
    Config c = Config.get();
    Point p = new Point(c.getMainWindowThumbnailSizeX(), c.getMainWindowThumbnailSizeY());
    p.y += fontHeight + c.getMainWindowListMarginTop() + c.getMainWindowListMarginBottom();
    return p;
  }
}
