package cvj2.app.gui.widgets;

import org.eclipse.jface.window.DefaultToolTip;
import org.eclipse.jface.window.ToolTip;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import cvj2.app.Config;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel;
import cvj2.gui.swt.widgets.gdidpanel.GridSelection;

/**
 * ListPanel で Migemo 検索結果を表示するための Tooltip。<br>
 * 
 * ※) Linux で JFace の DefaultTooltip と IME の共存ができなくてフリーズしたので、
 *     SWT と JFace を切り替えられるようにする。
 */
public class ListPanelToolTip {
  /** 表示時間 */
  public static int autoHideDelay = 1000;

  private GridPanel gridPanel;

  private DefaultToolTip jfaceTooltip;
  private org.eclipse.swt.widgets.ToolTip swtTooltip;

  /**
   * コンストラクタ
   */
  public ListPanelToolTip(GridPanel gridPanel) {
    jfaceTooltip = new DefaultToolTip(gridPanel.getCanvas(), ToolTip.RECREATE, true);
    swtTooltip = new org.eclipse.swt.widgets.ToolTip(gridPanel.getShell(), SWT.BALLOON);
    this.gridPanel = gridPanel;
    jfaceTooltip.setHideDelay(autoHideDelay);
  }

  /**
   * ツールチップを表示する
   */
  public void setTextAndShow(String s) {
    if (Config.get().isMainWindowMigemoTooltipUseSWT()) {
      setTextAndShow_swt(s);
    }
    else {
      setTextAndShow_jface(s);
    }
  }

  private String lastMessage;
  private void setTextAndShow_swt(String s) {
    final String msg = new String(s);
    lastMessage = msg;
    
    Rectangle rect = new Rectangle(0, 0, 0, 0);
    GridSelection sel = gridPanel.getSelection();
    int focus = sel.getFocusIndex();
    if (focus >= 0) rect = sel.getCellRect(focus); 
    
    swtTooltip.setVisible(false);
    swtTooltip.setLocation(gridPanel.getCanvas().toDisplay(rect.x, rect.y + rect.height));
    swtTooltip.setText(msg);
    swtTooltip.setVisible(true);
    
    gridPanel.getShell().getDisplay().timerExec(autoHideDelay, new Runnable() {
      @Override
      public void run() {
        if (msg == lastMessage) swtTooltip.setVisible(false);
      }
    });
  }

  private void setTextAndShow_jface(String s) {
    Rectangle rect = new Rectangle(0, 0, 0, 0);
    GridSelection sel = gridPanel.getSelection();
    int focus = sel.getFocusIndex();
    if (focus >= 0) rect = sel.getCellRect(focus); 
    jfaceTooltip.setText(s);
    jfaceTooltip.show(new Point(rect.x, rect.y + rect.height));
  }
}
