package cvj2.app.gui.widgets;

import java.awt.Dimension;

import org.eclipse.swt.graphics.Rectangle;

import cvj2.app.Config;
import cvj2.app.file.imgload.BackgroundImageLoader;
import cvj2.app.file.imgload.CachedImage;
import cvj2.app.file.imgload.ImageLoadProperty;
import cvj2.file.fs.FileInfo;
import cvj2.img.ImageInfo;
import cvj2.img.ImageLocation;
import cvj2.img.ImageRotate;
import cvj2.img.ImageUtil;
import cvj2.img.SWT2AWT;
import cvj2.img.ZoomLevel;

/**
 * ViewPanel に表示する画像情報の管理。及びその情報を元に画像をロードする
 */
public class ViewPanelState implements Cloneable {
  /** 画像 */
  public FileInfo file;
  /** 2ページ表示の場合の画像 */
  public FileInfo file2;
  /** ズーム倍率 */
  public ZoomLevel zoom = new ZoomLevel();
  /** ズーム時の表示位置 */
  public ImageLocation location = new ImageLocation();
  /** 回転 */
  public ImageRotate rotate = new ImageRotate();
  /** 原寸表示 */
  public boolean originalSize;
  /** 2ページ表示 */
  public boolean page2;
  /** 2ページ表示の左右入れ替え */
  public boolean page2Reverse;

  /**
   * 画面表示用の画像を取得する
   * @param vp 画像を読み込んだときに再描画するパネル
   */
  public CachedImage getImage(final ViewPanel vp) {
    if (file == null) return null;
    
    // 画像ロード通知
    BackgroundImageLoader.Notify notify = new BackgroundImageLoader.Notify() {
      @Override
      public void stateChanged(FileInfo f) {
        if (vp.getCanvas().isDisposed()) return;
        vp.getCanvas().getDisplay().asyncExec(new Runnable() {
          @Override
          public void run() {
            if (vp.getCanvas().isDisposed()) return;
            vp.getCanvas().redraw();
          }
        });
      }
    };
    
    // 画像ロード
    return BackgroundImageLoader.instance().getViewImage(getImageLoadProperty(vp), notify);
  }
  
  /**
   * 画像ロード情報を取得
   */
  public ImageLoadProperty getImageLoadProperty(final ViewPanel vp) {
    // ロードする画像サイズ、ズーム の情報を決定する
    Dimension size = null;
    Double zoomRatio = null;
    if (originalSize) {
      // 原寸モード (描画時に拡大縮小する)
      zoomRatio = 1.0;
    }
    else {
      // 画面フィットモード
      // (回転してからサイズ変換するので、縦横は画面のままでいい)
      Rectangle canvasRect = vp.getCanvas().getClientArea();
      double z = Math.min(zoom.getRatio(), Config.get().getViewWindowFitImageMaxLoadMagnification()); 
      size = new Dimension((int)(canvasRect.width * z), (int)(canvasRect.height * z));
    }

    return new ImageLoadProperty(file, file2, rotate, size, zoomRatio);
  }
  
  /**
   * 現在の画像情報で、画面に表示する画像の大きさを取得 (拡大縮小回転した大きさ)
   */
  public Dimension getDisplayImageSize(CachedImage img, Rectangle canvasRect) {
    ImageInfo orgInfo = img.getLoadResult().originalImageInfo;
    double z = zoom.getRatio();

    if (originalSize) {
      // 原寸モード (描画時に拡大縮小する)
      if (rotate.isHorizontal()) {
        return new Dimension((int)(orgInfo.width * z), (int)(orgInfo.height * z));
      }
      else {
        return new Dimension((int)(orgInfo.height * z), (int)(orgInfo.width * z));
      }
    }
    else {
      // 画面フィットモード
      Dimension fitSize;
      if (rotate.isHorizontal()) {
        fitSize = ImageUtil.calcImageFitSize(new Dimension(orgInfo.width, orgInfo.height), SWT2AWT.toDimension(canvasRect));
      }
      else {
        fitSize = ImageUtil.calcImageFitSize(new Dimension(orgInfo.height, orgInfo.width), SWT2AWT.toDimension(canvasRect));
      }
      return new Dimension((int)(fitSize.width * z), (int)(fitSize.height * z));
    }
  }

  @Override
  public ViewPanelState clone() {
    try {
      ViewPanelState other = (ViewPanelState) super.clone();
      other.zoom = this.zoom.clone();
      other.location = this.location.clone();
      other.rotate = this.rotate.clone();
      return other;
    }
    catch (CloneNotSupportedException e) {
      throw new RuntimeException(e);
    }
  }
}
