package cvj2.app.gui.window.main;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuCreator;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.StatusLineContributionItem;
import org.eclipse.jface.action.StatusLineManager;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.wb.swt.ResourceManager;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.app.Main;
import cvj2.app.command.Command;
import cvj2.app.command.CommandMapAction;
import cvj2.app.gui.data.FileListColumn;
import cvj2.app.gui.data.FileListSortElement;
import cvj2.app.gui.dialogs.DialogUtil;
import cvj2.app.gui.util.GlobalErrorHandler;
import cvj2.app.gui.widgets.ListPanel;
import cvj2.app.gui.window.CommonApplicationWindow;
import cvj2.util.Log;
import org.eclipse.swt.custom.CTabFolder2Adapter;
import org.eclipse.swt.custom.CTabFolderEvent;
import org.eclipse.swt.custom.CTabItem;

/**
 * アプリケーションのメイン画面
 */
public class MainWindow extends ApplicationWindow implements CommonApplicationWindow {
  private MyAction action_file_exit;
  private MyAction action_file_newFile;
  private MyAction action_file_openDir;
  private MyAction action_file_exec;
  private MyAction action_file_newDir;
  private MyAction action_file_property;
  private MyAction action_edit_cut;
  private MyAction action_edit_copy;
  private MyAction action_edit_delete;
  private MyAction action_edit_paste;
  private MyAction action_edit_rename;
  private MyAction action_edit_config;
  private MyAction action_edit_thumbnail;
  private MyAction action_edit_addThumbnail;
  private MyAction action_edit_bookmark;
  private MyAction action_view_homeDir;
  private MyAction action_view_refresh;
  private MyAction action_view_hideFiles;
  private MyAction action_view_clearCache;
  private MyAction action_view_prevDir;
  private MyAction action_view_nextDir;
  private MyAction action_view_parentDir;
  private MyAction action_view_enterChild;
  private MyAction action_view_listMode;
  private MyAction action_view_thumbnailMode;
  private MyAction action_view_thumbnailShrink;
  private MyAction action_view_thumbnailExpand;
  private MyAction action_tab_open;
  private MyAction action_tab_next;
  private MyAction action_tab_prev;
  private MyAction action_tab_close;
  private MyAction action_view_sort;
  private MyAction action_help_smSearch;
  private MyAction action_help_about;
  private MyAction action_help_sysmon;
  private MyAction action_view_sort_name;
  private MyAction action_view_sort_ext;
  private MyAction action_view_sort_size;
  private MyAction action_view_sort_date;
  private CTabFolder tabFolder;
  private StatusLineContributionItem statusFileItem;

  private boolean appInitialized = false;
  private CommandMapAction actionCommandMap = new CommandMapAction();
  private MainWindowCommandHandler cmd;
  private MainWindowFileWatchThread fileWatcher;
  
  /**
   * コンストラクタ
   */
  public MainWindow() {
    super(null);
    createActions();
    addToolBar(SWT.FLAT | SWT.WRAP);
    addMenuBar();
    addStatusLine();
    setExceptionHandler(new SWTExceptionHandler());
  }

  /**
   * エラーハンドラー
   */
  class SWTExceptionHandler implements Window.IExceptionHandler {
    @Override
    public void handleException(Throwable e) {
      Log.e(e);
      GlobalErrorHandler.instance().setFatalMessage("メインウインドウで、プログラム内部エラーが発生しました", e);
    }
  }
  
  /**
   * アプリ起動直後の初期化
   */
  private void initializeApp() {
    if (appInitialized) return;
    appInitialized = true;

    Log.start();
    Config conf = Config.get();
    getShell().setImage(Main.getAppIcon());
    
    // コマンドマップの初期化
    cmd = new MainWindowCommandHandler(this);
    actionCommandMap.add(action_file_exit, Command.FILE_EXIT);
    actionCommandMap.add(action_file_exec, Command.FILE_EXEC);
    actionCommandMap.add(action_file_openDir, Command.FILE_OPEN_DIR);
    actionCommandMap.add(action_file_newFile, Command.FILE_NEW_FILE);
    actionCommandMap.add(action_file_newDir, Command.FILE_NEW_DIR);
    actionCommandMap.add(action_file_property, Command.FILE_PROPERTY);
    actionCommandMap.add(action_edit_cut, Command.EDIT_CUT);
    actionCommandMap.add(action_edit_copy, Command.EDIT_COPY);
    actionCommandMap.add(action_edit_delete, Command.EDIT_DELETE);
    actionCommandMap.add(action_edit_paste, Command.EDIT_PASTE);
    actionCommandMap.add(action_edit_rename, Command.EDIT_RENAME);
    actionCommandMap.add(action_edit_config, Command.EDIT_CONFIG);
    actionCommandMap.add(action_edit_thumbnail, Command.EDIT_THUMBNAIL);
    actionCommandMap.add(action_edit_addThumbnail, Command.EDIT_ADD_THUMBNAIL);
    actionCommandMap.add(action_edit_bookmark, Command.EDIT_BOOKMARK);
    actionCommandMap.add(action_view_homeDir, Command.VIEW_HOME_DIR);
    actionCommandMap.add(action_view_refresh, Command.VIEW_REFRESH);
    actionCommandMap.add(action_view_hideFiles, Command.VIEW_HIDE_FILES);
    actionCommandMap.add(action_view_clearCache, Command.VIEW_CLEAR_CACHE);
    actionCommandMap.add(action_view_sort_name, Command.VIEW_SORT_NAME);
    actionCommandMap.add(action_view_sort_ext, Command.VIEW_SORT_EXT);
    actionCommandMap.add(action_view_sort_size, Command.VIEW_SORT_SIZE);
    actionCommandMap.add(action_view_sort_date, Command.VIEW_SORT_DATE);
    actionCommandMap.add(action_view_prevDir, Command.VIEW_PREV_DIR);
    actionCommandMap.add(action_view_nextDir, Command.VIEW_NEXT_DIR);
    actionCommandMap.add(action_view_parentDir, Command.VIEW_PARENT_DIR);
    actionCommandMap.add(action_view_enterChild, Command.VIEW_ENTER_CHILD);
    actionCommandMap.add(action_view_listMode, Command.VIEW_LIST_MODE);
    actionCommandMap.add(action_view_thumbnailMode, Command.VIEW_THUMBNAIL_MODE);
    actionCommandMap.add(action_view_thumbnailShrink, Command.VIEW_THUMBNAIL_SIZE_SHRINK);
    actionCommandMap.add(action_view_thumbnailExpand, Command.VIEW_THUMBNAIL_SIZE_EXPAND);
    actionCommandMap.add(action_tab_open, Command.TAB_OPEN);
    actionCommandMap.add(action_tab_next, Command.TAB_NEXT);
    actionCommandMap.add(action_tab_prev, Command.TAB_PREV);
    actionCommandMap.add(action_tab_close, Command.TAB_CLOSE);
    actionCommandMap.add(action_help_about, Command.HELP_ABOUT);
    actionCommandMap.add(action_help_smSearch, Command.HELP_SMSEARCH_TEST);
    actionCommandMap.add(action_help_sysmon, Command.HELP_SYSMON);
    actionCommandMap.add(action_view_sort, Command.VIEW_SORT);
    
    MenuManager mmFile = (MenuManager) getMenuBarManager().getItems()[0];
    MenuManager mmFileLaunch = new MenuManager("プログラム起動(&L)");
    mmFile.insertAfter("ID_EXEC", mmFileLaunch);
    Config.LaunchProgram[] launchProgs = Config.get().getLaunchPrograms();
    for (int i = 0; i < launchProgs.length; i++) {
      boolean hasId = i < Command.MAX_LAUNCH;
      Config.LaunchProgram p = launchProgs[i];
      MyAction a = new MyAction(p.getName() + (hasId ? "(&" + (i+1)%10 + ")" : ""));
      a.setId("" + i);
      mmFileLaunch.add(a);
      if (hasId) {
        actionCommandMap.add(a, Command.values()[Command.FILE_LAUNCH_0.ordinal() + i]);
      }
      else {
        actionCommandMap.add(a, Command.FILE_LAUNCH);
      }
    }
    
    MyAction.cmd = cmd;
    MyAction.actionCommandMap = actionCommandMap;
    actionCommandMap.resetActionStatus(Config.get().getCmdMapMain());
    
    // アプリ起動直後にしか行えない Main の初期化を行う
    Main.initializeByMainWindow();

    // サイズ復元
    DialogUtil.setWindowBoundsFromConfig(this);
    
    // タブを少し高くする
    getTabFolder().setTabHeight((int)(getTabFolder().getTabHeight() + conf.getWindowTabHeightPlus()));
    // タブのフォント
    getTabFolder().setFont(conf.getFontTab());
    
    // これをしないと、Linux の Java 1.7 でウインドウサイズを手動でリサイズしないとステータスバーが表示されなかった
    getShell().layout();
    
    // 初期タブオープン
    // (Windows で実行したとき、遅延させないとキー入力ができなかった)
    Main.setSplashMessage("タブを開いています");
    getShell().getDisplay().timerExec(10, new Runnable() {
      @Override
      public void run() {
        cmd.restoreTabState();
        Main.closeSplash();
      }
    });
    
    // ファイル監視スレッド開始
    fileWatcher = new MainWindowFileWatchThread(this, cmd);
    fileWatcher.start();
  }

  /**
   * メイン画面を閉じる前の準備
   */
  public void shutdownMainWindow() {
    cmd.saveTabState();
    DialogUtil.setConfigFromWindowBounds(this);
    Main.shutdownByMainWindow(getShell());
  }
  
  /**
   * メニューの再設定
   */
  public void resetMenu() {
    actionCommandMap.resetActionStatus(Config.get().getCmdMapMain());
  }
  
  /**
   * フォントセット
   */
  public void setFont(Font font) {
    getShell().setFont(font);
    getTabFolder().setFont(font);
    for (Control c : getTabFolder().getChildren()) {
      c.setFont(font);
    }
  }
  
  /**
   * アプリ終了前処理
   */
  @Override
  protected boolean canHandleShellCloseEvent() {
    Log.start();
    shutdownMainWindow();
    return true;
  }
  
  /**
   * ステータスバーのメッセージ
   */
  @Override
  public void setStatus(String message, boolean isTemporary) {
    super.setStatus(message);
  }
  
  /**
   * ファイルステータスバーのメッセージ
   */
  public void setStatusFileMessage(String s) {
    statusFileItem.setText(s);
  }

  /**
   * タブ取得
   */
  @Override
  public CTabFolder getTabFolder() {
    return tabFolder;
  }
  
  /**
   * メニュー状態を更新する
   * @param thumbnailMode 現在の画面がサムネイルモードか
   * @param sort 画面のソート状態 
   */
  public void updateMenuState(boolean thumbnailMode, FileListSortElement sort) {
    int tabCount = getTabFolder().getChildren().length;
    
    action_file_newDir.setEnabled(tabCount > 0);
    action_file_newFile.setEnabled(tabCount > 0);
    action_file_exec.setEnabled(tabCount > 0);
    action_file_property.setEnabled(tabCount > 0);
    
    action_edit_cut.setEnabled(tabCount > 0);
    action_edit_copy.setEnabled(tabCount > 0);
    action_edit_delete.setEnabled(tabCount > 0);
    action_edit_paste.setEnabled(tabCount > 0);
    action_edit_rename.setEnabled(tabCount > 0);
    action_edit_bookmark.setEnabled(tabCount > 0);
    action_edit_addThumbnail.setEnabled(tabCount > 0);

    action_view_homeDir.setEnabled(tabCount > 0);
    action_view_refresh.setEnabled(tabCount > 0);
    action_view_hideFiles.setChecked(Config.get().isMainWindowHideFiles());
    action_view_prevDir.setEnabled(tabCount > 0);
    action_view_nextDir.setEnabled(tabCount > 0);
    action_view_parentDir.setEnabled(tabCount > 0);
    action_view_enterChild.setEnabled(tabCount > 0);

    action_view_sort.setEnabled(tabCount > 0);
    action_view_sort_name.setEnabled(tabCount > 0);
    action_view_sort_ext.setEnabled(tabCount > 0);
    action_view_sort_size.setEnabled(tabCount > 0);
    action_view_sort_date.setEnabled(tabCount > 0);
    
    action_view_listMode.setChecked(!thumbnailMode);
    action_view_listMode.setEnabled(tabCount > 0);
    
    action_view_thumbnailMode.setChecked(thumbnailMode);
    action_view_thumbnailMode.setEnabled(tabCount > 0);
    
    action_view_thumbnailShrink.setEnabled(tabCount > 0 && thumbnailMode);
    action_view_thumbnailExpand.setEnabled(tabCount > 0 && thumbnailMode);
    
    action_tab_prev.setEnabled(tabCount > 1);
    action_tab_next.setEnabled(tabCount > 1);
    action_tab_close.setEnabled(tabCount > 0);

    String sort_icon;
    if (sort == null) sort_icon = "name_0";
    else if (sort.getColumn() == FileListColumn.NAME &&  sort.isAscent()) sort_icon = "name_0";
    else if (sort.getColumn() == FileListColumn.NAME && !sort.isAscent()) sort_icon = "name_1";
    else if (sort.getColumn() == FileListColumn.EXT  &&  sort.isAscent()) sort_icon = "ext_0";
    else if (sort.getColumn() == FileListColumn.EXT  && !sort.isAscent()) sort_icon = "ext_1";
    else if (sort.getColumn() == FileListColumn.SIZE &&  sort.isAscent()) sort_icon = "size_0";
    else if (sort.getColumn() == FileListColumn.SIZE && !sort.isAscent()) sort_icon = "size_1";
    else if (sort.getColumn() == FileListColumn.DATE &&  sort.isAscent()) sort_icon = "date_0";
    else if (sort.getColumn() == FileListColumn.DATE && !sort.isAscent()) sort_icon = "date_1";
    else sort_icon = "name_0";
    action_view_sort.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_sort_" + sort_icon + ".png"));
  }
  
  /**
   * Create contents of the application window.
   * @param parent
   */
  @Override
  protected Control createContents(Composite parent) {
    Composite container = new Composite(parent, SWT.NONE);
    container.setLayout(new FillLayout(SWT.HORIZONTAL));
    {
      tabFolder = new CTabFolder(container, SWT.BORDER | SWT.CLOSE | SWT.FLAT);
      tabFolder.addCTabFolder2Listener(new CTabFolder2Adapter() {
        @Override
        public void close(CTabFolderEvent event) {
          cmd.closeTab((CTabItem) event.item);
        }
      });
      tabFolder.setMinimumCharacters(8);
      tabFolder.setSelectionForeground(SWTResourceManager.getColor(SWT.COLOR_LIST_SELECTION_TEXT));
      tabFolder.setSelectionBackground(SWTResourceManager.getColor(SWT.COLOR_LIST_SELECTION));
    }

    return container;
  }

  /**
   * Command に結びついた Action のサブクラス
   */
  static public class MyAction extends Action {
    static MainWindowCommandHandler cmd;
    static CommandMapAction actionCommandMap;
    
    public MyAction(String text) {
      super(text);
    }
    public MyAction(String text, int style) {
      super(text, style);
    }
    @Override
    public void run() {
      // IDを通知する
      cmd.setLastInvokeActionId(getId());
      // アクセラレーターキーが押されたら、Migemo の入力を中断する
      Main.getMigemoKey().cancelKeySequence();
      // コマンド実行
      cmd.execCmd(actionCommandMap.findCommand(this));
    }
  }
  /** ツールバーのソートメニュー */
  public static class SortAction extends MyAction {
    public SortAction(String text) {
      super("", Action.AS_DROP_DOWN_MENU);
      setMenuCreator(new MenuCreator());
    }

    public class MenuCreator implements IMenuCreator {
      private Menu menu;
      public Menu getMenu(Control parent) {
        menu = new Menu(parent);
        new SortMenuItem(menu, FileListColumn.NAME.getName(), Command.VIEW_SORT_NAME);
        new SortMenuItem(menu, FileListColumn.EXT.getName(), Command.VIEW_SORT_EXT);
        new SortMenuItem(menu, FileListColumn.SIZE.getName(), Command.VIEW_SORT_SIZE);
        new SortMenuItem(menu, FileListColumn.DATE.getName(), Command.VIEW_SORT_DATE);
        return menu;
      }
      public Menu getMenu(Menu parent) { return null; }
      public void dispose() { menu.dispose(); }
      public class SortMenuItem extends MenuItem {
        public SortMenuItem(Menu parent, String text, final Command command) {
          super(parent, SWT.NONE);
          setText(text);
          addSelectionListener(new SelectionAdapter() {
            @Override public void widgetSelected(SelectionEvent e) { cmd.execCmd(command); }
          });
        }
        @Override protected void checkSubclass() {}
      }
    }
  }  
  
  /**
   * Create the actions.
   */
  private void createActions() {
    {
      action_file_exit = new MyAction("&Exit") {
      };
      action_file_exit.setText("E&xit");
      action_file_exit.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "file_exit.png"));
    }
    {
      action_file_openDir = new MyAction("&Open Dir") {
      };
      action_file_openDir.setText("&Open Dir ...");
      action_file_openDir.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "file_openDir.png"));
    }
    {
      action_edit_cut = new MyAction("Cu&t") {
      };
      action_edit_cut.setText("Cut");
      action_edit_cut.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_cut.png"));
    }
    {
      action_edit_copy = new MyAction("&Copy") {
      };
      action_edit_copy.setText("Copy");
      action_edit_copy.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_copy.png"));
    }
    {
      action_edit_delete = new MyAction("&Delete") {
      };
      action_edit_delete.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_delete.png"));
    }
    {
      action_edit_paste = new MyAction("&Paste") {
      };
      action_edit_paste.setText("Paste");
      action_edit_paste.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_paste.png"));
    }
    {
      action_edit_rename = new MyAction("&Rename") {
      };
      action_edit_rename.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_rename.png"));
    }
    {
      action_view_refresh = new MyAction("&Refresh") {
      };
      action_view_refresh.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_refresh.png"));
    }
    {
      action_view_enterChild = new MyAction("&Enter Child Dir") {
      };
      action_view_enterChild.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_enterChildDir.png"));
    }
    {
      action_view_prevDir = new MyAction("&Prev Dir") {
      };
      action_view_prevDir.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_prevDir.png"));
    }
    {
      action_view_parentDir = new MyAction("P&arent Dir") {
      };
      action_view_parentDir.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_parentDir.png"));
    }
    {
      action_view_nextDir = new MyAction("&Next Dir") {
      };
      action_view_nextDir.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_nextDir.png"));
    }
    {
      action_tab_open = new MyAction("&Open") {
      };
      action_tab_open.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "tab_open.png"));
    }
    {
      action_tab_next = new MyAction("&Next Tab") {
      };
      action_tab_next.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "tab_next.png"));
    }
    {
      action_tab_prev = new MyAction("&Prev Tab") {
      };
      action_tab_prev.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "tab_prev.png"));
    }
    {
      action_tab_close = new MyAction("&Close") {
      };
      action_tab_close.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "tab_close.png"));
    }
    {
      action_help_about = new MyAction("&About") {
      };
      action_help_about.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "help_about.png"));
    }
    {
      action_view_listMode = new MyAction("&List mode") {
      };
      action_view_listMode.setChecked(true);
      action_view_listMode.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_listMode.png"));
    }
    {
      action_view_thumbnailMode = new MyAction("&Thumbnail mode") {
      };
      action_view_thumbnailMode.setChecked(true);
      action_view_thumbnailMode.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_thumbnailMode.png"));
    }
    {
      action_edit_config = new MyAction("&Config") {
      };
      action_edit_config.setText("&Config ...");
      action_edit_config.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_config.png"));
    }
    {
      action_view_sort = new SortAction("Sort") {
      };
    }
    {
      action_help_smSearch = new MyAction("Text Search Test") {
      };
      action_help_smSearch.setText("Text Search Test ...");
      action_help_smSearch.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "help_smsearch.png"));
    }
    {
      action_edit_bookmark = new MyAction("&Bookmark") {
      };
      action_edit_bookmark.setText("&Bookmark ...");
      action_edit_bookmark.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_bookmark.png"));
    }
    {
      action_edit_thumbnail = new MyAction("&Thumbnail") {
      };
      action_edit_thumbnail.setText("&Thumbnail ...");
      action_edit_thumbnail.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_thumbnail.png"));
    }
    {
      action_view_sort_name = new MyAction("Name") {
      };
      action_view_sort_name.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_sort_name_0.png"));
    }
    {
      action_view_sort_ext = new MyAction("Ext") {
      };
      action_view_sort_ext.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_sort_ext_0.png"));
    }
    {
      action_view_sort_size = new MyAction("Size") {
      };
      action_view_sort_size.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_sort_size_0.png"));
    }
    {
      action_view_sort_date = new MyAction("Date") {
      };
      action_view_sort_date.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_sort_date_0.png"));
    }
    {
      action_file_newDir = new MyAction("&New Dir ...") {
      };
      action_file_newDir.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "file_newDir.png"));
    }
    {
      action_file_exec = new MyAction("Exec") {
      };
      action_file_exec.setId("ID_EXEC");
      action_file_exec.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "file_exec.png"));
    }
    {
      action_file_property = new MyAction("&Property ...") {
      };
      action_file_property.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "file_property.png"));
    }
    {
      action_view_clearCache = new MyAction("Clear Cache") {
      };
      action_view_clearCache.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_clearCache.png"));
    }
    {
      action_edit_addThumbnail = new MyAction("Add Thumbnail ...") {
      };
      action_edit_addThumbnail.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_addThumbnail.png"));
    }
    {
      action_help_sysmon = new MyAction("System Monitor ...") {
      };
      action_help_sysmon.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "help_sysmon.png"));
    }
    {
      action_view_homeDir = new MyAction("Go Home") {
      };
      action_view_homeDir.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_homeDir.png"));
    }
    {
      action_view_hideFiles = new MyAction("Hide Files") {
      };
      action_view_hideFiles.setChecked(true);
      action_view_hideFiles.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_hideFiles.png"));
    }
    {
      action_file_newFile = new MyAction("New File ...") {
      };
      action_file_newFile.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "file_newFile.png"));
    }
    {
      action_view_thumbnailShrink = new MyAction("Thumbnail Shrink") {
      };
      action_view_thumbnailShrink.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_thumbnailShrink.png"));
    }
    {
      action_view_thumbnailExpand = new MyAction("Thumbnail Expand") {
      };
      action_view_thumbnailExpand.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_thumbnailExpand.png"));
    }
  }
  
  /**
   * Create the menu manager.
   * @return the menu manager
   */
  @Override
  protected MenuManager createMenuManager() {
    MenuManager menuManager = new MenuManager("menu");
    {
      MenuManager menuManager_file = new MenuManager("ファイル(&F)");
      menuManager.add(menuManager_file);
      menuManager_file.add(action_file_openDir);
      menuManager_file.add(action_file_newFile);
      menuManager_file.add(action_file_newDir);
      menuManager_file.add(action_file_exec);
      menuManager_file.add(action_file_property);
      menuManager_file.add(new Separator());
      menuManager_file.add(action_file_exit);
    }
    {
      MenuManager menuManager_edit = new MenuManager("編集(&E)");
      menuManager.add(menuManager_edit);
      menuManager_edit.add(action_edit_cut);
      menuManager_edit.add(action_edit_copy);
      menuManager_edit.add(action_edit_paste);
      menuManager_edit.add(new Separator());
      menuManager_edit.add(action_edit_delete);
      menuManager_edit.add(action_edit_rename);
      menuManager_edit.add(new Separator());
      menuManager_edit.add(action_edit_bookmark);
      menuManager_edit.add(action_edit_thumbnail);
      menuManager_edit.add(action_edit_addThumbnail);
      menuManager_edit.add(action_edit_config);
    }
    {
      MenuManager menuManager_view = new MenuManager("表示(&V)");
      menuManager.add(menuManager_view);
      menuManager_view.add(action_view_refresh);
      menuManager_view.add(action_view_clearCache);
      menuManager_view.add(action_view_hideFiles);
      {
        MenuManager menuManager_view_sort = new MenuManager("ソート(&S)");
        menuManager_view.add(menuManager_view_sort);
        menuManager_view_sort.add(action_view_sort_name);
        menuManager_view_sort.add(action_view_sort_ext);
        menuManager_view_sort.add(action_view_sort_size);
        menuManager_view_sort.add(action_view_sort_date);
      }
      menuManager_view.add(action_view_homeDir);
      menuManager_view.add(new Separator());
      menuManager_view.add(action_view_listMode);
      menuManager_view.add(action_view_thumbnailMode);
      menuManager_view.add(new Separator());
      menuManager_view.add(action_view_thumbnailShrink);
      menuManager_view.add(action_view_thumbnailExpand);
      menuManager_view.add(new Separator());
      menuManager_view.add(action_view_prevDir);
      menuManager_view.add(action_view_nextDir);
      menuManager_view.add(new Separator());
      menuManager_view.add(action_view_parentDir);
      menuManager_view.add(action_view_enterChild);
    }
    {
      MenuManager menuManager_tab = new MenuManager("タブ(&T)");
      menuManager.add(menuManager_tab);
      menuManager_tab.add(action_tab_open);
      menuManager_tab.add(action_tab_close);
      menuManager_tab.add(new Separator());
      menuManager_tab.add(action_tab_prev);
      menuManager_tab.add(action_tab_next);
    }
    {
      MenuManager menuManager_help = new MenuManager("ヘルプ(&H)");
      menuManager.add(menuManager_help);
      menuManager_help.add(action_help_smSearch);
      menuManager_help.add(action_help_sysmon);
      menuManager_help.add(action_help_about);
    }
    return menuManager;
  }

  /**
   * Create the toolbar manager.
   * @return the toolbar manager
   */
  @Override
  protected ToolBarManager createToolBarManager(int style) {
    ToolBarManager toolBarManager = new ToolBarManager(style);
    toolBarManager.add(action_file_openDir);
    toolBarManager.add(action_file_exit);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_edit_cut);
    toolBarManager.add(action_edit_copy);
    toolBarManager.add(action_edit_paste);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_edit_delete);
    toolBarManager.add(action_edit_rename);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_edit_bookmark);
    toolBarManager.add(action_edit_config);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_view_hideFiles);
    toolBarManager.add(action_view_refresh);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_view_listMode);
    toolBarManager.add(action_view_thumbnailMode);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_view_prevDir);
    toolBarManager.add(action_view_nextDir);
    toolBarManager.add(action_view_parentDir);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_tab_open);
    toolBarManager.add(action_tab_close);
    toolBarManager.add(action_view_sort);
    return toolBarManager;
  }

  /**
   * Create the status line manager.
   * @return the status line manager
   */
  @Override
  protected StatusLineManager createStatusLineManager() {
    StatusLineManager statusLineManager = new StatusLineManager();
    statusFileItem = new StatusLineContributionItem("FILE_STATUS", StatusLineContributionItem.CALC_TRUE_WIDTH);
    statusLineManager.add(statusFileItem);
    return statusLineManager;
  }

  /**
   * Configure the shell.
   * @param newShell
   */
  @Override
  protected void configureShell(Shell newShell) {
    super.configureShell(newShell);
    newShell.setText(Config.APP_NAME);
    newShell.addShellListener(new ShellAdapter() {
      @Override
      public void shellActivated(ShellEvent e) {
        initializeApp();
        setupFocus();
      }
    });
  }

  private void setupFocus() {
    ListPanel lp = cmd.getCurrentListPanel();
    if (lp != null) lp.setFocus();
  }
  
  /**
   * Return the initial size of the window.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(450, 300);
  }
}
