 package cvj2.app.gui.window.main;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.lang3.StringUtils;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.custom.CTabItem;
import org.eclipse.swt.widgets.Control;

import cvj2.app.Config;
import cvj2.app.Main;
import cvj2.app.ParseArg;
import cvj2.app.command.Command;
import cvj2.app.command.CommandHandler;
import cvj2.app.file.ApplicationLock;
import cvj2.app.file.imgload.BackgroundImageLoader;
import cvj2.app.gui.data.Bookmark.Elm;
import cvj2.app.gui.data.FileListColumn;
import cvj2.app.gui.dialogs.AboutDialog;
import cvj2.app.gui.dialogs.AddThumbnailDialog1;
import cvj2.app.gui.dialogs.AddThumbnailDialog2;
import cvj2.app.gui.dialogs.BookmarkDialog;
import cvj2.app.gui.dialogs.FilePropertyDialog;
import cvj2.app.gui.dialogs.SMSearchTestDialog;
import cvj2.app.gui.dialogs.SystemMonitorDialog;
import cvj2.app.gui.dialogs.ThumbnailDialog;
import cvj2.app.gui.dialogs.preference.ConfigDialog;
import cvj2.app.gui.util.MsgBox;
import cvj2.app.gui.widgets.ListPanel;
import cvj2.app.gui.widgets.ListPanelDndUtil;
import cvj2.app.gui.window.WindowController;
import cvj2.app.gui.window.WindowSelector;
import cvj2.app.gui.window.WindowSelector.Type;
import cvj2.file.fs.FileInfo;
import cvj2.file.fs.FileInfoComparator;
import cvj2.file.fs.FileInfoFactory;
import cvj2.file.fs.FileInfoList;
import cvj2.file.fs.FileInfoPDFManager;
import cvj2.file.fs.FileInfoZipManager;
import cvj2.gui.swt.ui.input.InputCombination;
import cvj2.gui.swt.util.WaitCursor;
import cvj2.migemo.Sen;
import cvj2.util.Log;

/**
 * MainWindow で発生したコマンドを処理するクラス。<br>
 * 
 * MainWindow と ListPanel の間で処理のやりとりを行う
 */
public class MainWindowCommandHandler extends CommandHandler {
  private MainWindow wnd;
  private WindowController wndController;
  
  private String lastInvokeActionId = "";
  
  /**
   * コンストラクタ
   */
  public MainWindowCommandHandler(MainWindow mainWindow) {
    this.wnd = mainWindow;
    this.wndController = new WindowController(mainWindow, mainWindow);
  }

  @Override
  public void execCmd(Command command) {
    // Windowsでは画面が非表示になっていてもキー入力が行われる場合があるのでチェックする
    if (wnd.getShell() == null || wnd.getShell().isDisposed() || !wnd.getShell().isVisible()) return;
    super.execCmd(command);
  }

  /**
   * ListPanel の入力ハンドラー
   */
  public void handle(InputCombination input) {
    // Windowsでは画面が非表示になっていてもキー入力が行われる場合があるのでチェックする
    if (wnd.getShell() == null || wnd.getShell().isDisposed() || !wnd.getShell().isVisible()) return;
    
    ListPanel lp = getCurrentListPanel();
    if (lp != null) lp.setFocus();

    Command cmd = Config.get().getCmdMapMain().findCommand(input);
    if (cmd == null) return;
    execCmd(cmd);
  }
  
  /**
   * Commandが同じメニューが起動されたとき、MainWindow から呼ばれる
   */
  public void setLastInvokeActionId(String actionId) {
    lastInvokeActionId = actionId;
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_FILE_OPEN_DIR() {
    if (getCurrentListPanel() == null) handle_TAB_OPEN();
    MainWindowFileOperation.openDir(getCurrentListPanel());
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_FILE_NEW_DIR() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    MainWindowFileOperation.newDir(lp);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_FILE_NEW_FILE() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    MainWindowFileOperation.newFile(lp);
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_FILE_EXEC() {
    MainWindowProgramExecutor.exec(getCurrentListPanel(), wndController);
  }

  /** 
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_FILE_LAUNCH() {
    try {
      int id = Integer.parseInt(lastInvokeActionId);
      MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), id, wndController);
    }
    catch (Exception e) { /* */ }
  }
  
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_0() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 0, wndController); }
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_1() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 1, wndController); }
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_2() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 2, wndController); }
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_3() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 3, wndController); }
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_4() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 4, wndController); }
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_5() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 5, wndController); }
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_6() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 6, wndController); }
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_7() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 7, wndController); }
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_8() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 8, wndController); }
  /** ※) このメソッドは、CommandHandler からリフレクションで呼ばれる*/
  public void handle_FILE_LAUNCH_9() { MainWindowProgramExecutor.launchProgram(getCurrentListPanel(), 9, wndController); }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_FILE_PROPERTY() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    if (lp.getFocusItem() == null) return;

    FilePropertyDialog dlg = new FilePropertyDialog(wnd.getShell(), lp.getFocusItem());
    dlg.open();
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_FILE_EXIT() {
    wnd.shutdownMainWindow();
    wnd.close();
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_CUT() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    if (lp.getCurrentDir() == FileInfo.ROOT) return;
    
    if (!ListPanelDndUtil.handleCutCopy(lp.getModel().getSortedList(), lp.getSelection().getSelectedIndices(), false)) return;
    wndController.setStatusMessageTemporary("ファイルをクリップボードに送りました");
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_COPY() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    if (lp.getCurrentDir() == FileInfo.ROOT) return;
    
    if (!ListPanelDndUtil.handleCutCopy(lp.getModel().getSortedList(), lp.getSelection().getSelectedIndices(), true)) return;
    wndController.setStatusMessageTemporary("ファイルをクリップボードに送りました");
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_PASTE() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    
    MainWindowFileOperation.paste(lp);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_DELETE() {
    MainWindowFileOperation.delete(getCurrentListPanel());
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_RENAME() {
    MainWindowFileOperation.rename(getCurrentListPanel());
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_CONFIG() {
    if (!checkFirstApp()) return;

    ConfigDialog.show(wnd.getShell(), this);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_BOOKMARK() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    
    FileInfo selFile = lp.getFocusItem();
    if (selFile == null) selFile = lp.getCurrentDir();
    Elm bmElm = new Elm(selFile.getAbsolutePath(), lp.isThumbnailMode() ? Elm.Type.THUMBNAIL : Elm.Type.LIST);
    BookmarkDialog dlg = new BookmarkDialog(wnd.getShell(), bmElm);
    dlg.open();
    bmElm = dlg.getSelectBookmarkElm();
    if (bmElm == null) return;
    
    FileInfo file = FileInfoFactory.create(new File(bmElm.getPath()));
    lp.setCurrentDir(file.getParent(), file);
    switch (bmElm.getType()) {
    case THUMBNAIL: lp.setThumbnailMode(true); break;
    case LIST: lp.setThumbnailMode(false); break;
    }
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_THUMBNAIL() {
    if (!checkFirstApp()) return;
    ListPanel lp = getCurrentListPanel();
    ThumbnailDialog dlg = new ThumbnailDialog(wnd.getShell(), lp == null ? null : lp.getCurrentDir());
    dlg.open();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_ADD_THUMBNAIL() {
    if (!checkFirstApp()) return;
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;

    // 条件設定
    AddThumbnailDialog1 dlg1 = new AddThumbnailDialog1(wnd.getShell(), lp.getCurrentDir());
    int ret = dlg1.open();
    if (ret != Dialog.OK) return;
    
    // 実行
    AddThumbnailDialog2 dlg2 = new AddThumbnailDialog2(wnd.getShell(), lp.getCurrentDir(), dlg1.checkDir, dlg1.checkFile, dlg1.depth);
    dlg2.open();
    MsgBox.info("サムネイル登録結果", dlg2.getResultMessage());
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_SEARCH_PREV() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    lp.searchList(-1, true);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_SEARCH_NEXT() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    lp.searchList(1, true);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_LIST_MODE() {
    switchListThumbnailMode();
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_THUMBNAIL_MODE() {
    switchListThumbnailMode();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_THUMBNAIL_SIZE_SHRINK() {
    thumbnailSizeChange(-1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_THUMBNAIL_SIZE_EXPAND() {
    thumbnailSizeChange(1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_HOME_DIR() {
    wndController.moveToHome();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_REFRESH() {
    wndController.refresh();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_HIDE_FILES() {
    Config conf = Config.get();
    conf.setMainWindowHideFiles(!conf.isMainWindowHideFiles());
    FileInfoList.hideFiles = conf.isMainWindowHideFiles();
    wndController.refresh();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_CLEAR_CACHE() {
    BackgroundImageLoader.instance().clearCache();
    FileInfoZipManager.cache.clear();
    FileInfoPDFManager.infoCache.clear();
    FileInfoPDFManager.pdfCache.clear();
    FileInfoComparator.customCmp.clearCache();
    Sen.clearAnalyzeCache();
    
    wndController.refresh();
    wndController.setStatusMessageTemporary("画像のキャッシュをクリアしました");
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_PREV_DIR() {
    wndController.moveSiblingDir(-1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_NEXT_DIR() {
    wndController.moveSiblingDir(1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_PARENT_DIR() {
    wndController.moveToParent();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ENTER_CHILD() {
    Config conf = Config.get();
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    
    FileInfo sel = lp.getFocusItem();
    if (sel == null) return;
    
    // 複数選択の場合
    if (lp.getSelection().getSelectedIndices().size() > 1) {
      return;
    }
    
    // 画像なら表示する
    if (!sel.isDirectory() && sel.isImageFile() && Config.get().isViewWindowEnable()) {
      openViewWindow();
      return;
    }

    // ディレクトリでないなら実行する
    if (!sel.isDirectory()) {
      MainWindowProgramExecutor.exec(lp, wndController);
      return;
    }
    
    // 子に移動
    wndController.setRealtimeStatusMessageTemporary("子ディレクトリに移動");
    lp.setCurrentDir(sel, null);
    wndController.setStatusMessageTemporary("子ディレクトリに移動しました");

    // 子ディレクトリ中の最初のファイルが画像なら、それを表示する
    if ((lp.isThumbnailMode() && conf.isMainWindowEnterChildAndShowViewWindowWhenThumbnailMode()) ||
        (!lp.isThumbnailMode() && conf.isMainWindowEnterChildAndShowViewWindowWhenListMode())) 
    {
      FileInfoList list = lp.getModel().getSortedList();
      if (list.size() == 0) return;
      int focusIndex = lp.getSelection().getFocusIndex();
      if (!conf.isMainWindowEnterChildAndShowViewWindowReferLastFocus()) focusIndex = 0;
      FileInfo file = list.get(focusIndex);
      if (!file.isDirectory() && file.isImageFile()) {
        lp.getSelection().setFocusIndex(focusIndex);
        if (Config.get().isViewWindowEnable()) openViewWindow();
      }
    }
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_PARENT_DIR_ON_LIST_MODE() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    if (lp.isThumbnailMode()) return;
    handle_VIEW_PARENT_DIR();
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ENTER_CHILD_ON_LIST_MODE() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    if (lp.isThumbnailMode()) return;
    handle_VIEW_ENTER_CHILD();
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_SORT() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    
    if (lp.getSort() == null) {
      lp.columnClicked(FileListColumn.NAME);
    }
    else {
      lp.columnClicked(lp.getSort().getColumn());
    }
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_SORT_NAME() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    
    lp.columnClicked(FileListColumn.NAME);
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_SORT_EXT() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    
    lp.columnClicked(FileListColumn.EXT);
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_SORT_SIZE() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    
    lp.columnClicked(FileListColumn.SIZE);
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_SORT_DATE() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    
    lp.columnClicked(FileListColumn.DATE);
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_TAB_OPEN() {
    Config conf = Config.get();
    
    // 最大数確認
    if (wnd.getTabFolder().getChildren().length >= conf.getMainWindowTabMax()) {
      MsgBox.info("お知らせ", "これ以上タブは開けません。\nもっと多くのタブを開きたいときは、設定を変更してください。");
      return;
    }
    
    ListPanel currentLp = getCurrentListPanel();

    // タブをつくる
    ListPanel newLp = createNewTab();
    
    wndController.setRealtimeStatusMessageTemporary("新しいタブを開きます");
    boolean isListMode = conf.isMainWindowNewTabListMode();
    FileInfo curDir = FileInfoFactory.create(new File(conf.getHomeDir()).getAbsoluteFile());
    if (currentLp != null) {
      // 現在のタブがあれば、引き継ぐ      
      if (conf.isMainWindowNewTabCurDirHandOver()) curDir = currentLp.getCurrentDir();
      if (conf.isMainWindowNewTabModeHandOver()) isListMode = !currentLp.isThumbnailMode();
    }
    newLp.setThumbnailMode(!isListMode);
    newLp.setCurrentDir(curDir, null);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_TAB_CLOSE() {
    // タブを閉じる
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    CTabItem item = wnd.getTabFolder().getSelection();
    if (item == null) return;
    closeTab(item);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_TAB_PREV() {
    // タブ移動
    wndController.moveTabSelect(-1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_TAB_NEXT() {
    // タブ移動
    wndController.moveTabSelect(1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_HELP_SMSEARCH_TEST() {
    SMSearchTestDialog dlg = new SMSearchTestDialog(wnd.getShell());
    dlg.open();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_HELP_SYSMON() {
    SystemMonitorDialog dlg = new SystemMonitorDialog(wnd.getShell());
    dlg.open();
  }
  
  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_HELP_ABOUT() {
    AboutDialog dlg = new AboutDialog(wnd.getShell());
    dlg.open();
  }
  
  
  /**
   * 前回のタブを復元する
   */
  public void restoreTabState() {
    Log.start();
    
    // 時間がかかるかもしれないので、カーソルを砂時計にする
    WaitCursor.start(wnd.getShell(), new WaitCursor.Work<Object>() {
      @Override public Object run() {
        wnd.getShell().getDisplay().syncExec(new Runnable() {
          @Override public void run() {
            // 本体
            if (ParseArg.initDirs.size() > 0) {
              openInitDirs(ParseArg.initDirs);
            }
            else {
              restoreTabState_body();
            }
          }
        });
        return null;
      }
    });
  }
  private void openInitDirs(List<File> initDirs) {
    Log.start();

    // 初期ディレクトリオープン
    for (File f : initDirs) {
      ListPanel lp = createNewTab();
      FileInfo fileInfo = FileInfoFactory.create(f);
      if (fileInfo.isDirectory()) {
        lp.setCurrentDir(fileInfo, null);
      }
      else {
        lp.setCurrentDir(fileInfo.getParent(), fileInfo);
      }
    }
  }
  private void restoreTabState_body() {
    Log.start();
    
    Config.TabState[] tabStates = Config.get().getMainWindowTabStates();
    
    // タブが空だったら、デフォルトのタブを開く
    if (tabStates.length == 0) {
      handle_TAB_OPEN();
      return;
    }
    
    wndController.setRealtimeStatusMessageTemporary("前回のタブを復元しています。少々おまちください。");
    wndController.setStatusFileMessage("前回のタブを復元しています。少々おまちください。");
    
    // 復元
    for (int i = 0; i < tabStates.length; i++) {
      ListPanel lp = createNewTab();
      FileInfo fileInfo = FileInfoFactory.create(new File(tabStates[i].getFile()));
      lp.setThumbnailMode(tabStates[i].isThumbnailMode());
      lp.setCurrentDir(fileInfo.getParent(), fileInfo);
    }
    int focus = Config.get().getMainWindowTabStatesFocus();
    focus = Math.max(0, focus);
    focus = Math.min(focus, wnd.getTabFolder().getChildren().length - 1);
    wnd.getTabFolder().setSelection(focus);
    getCurrentListPanel().setFocus();

    // ステータス表示を更新
    getCurrentListPanel().updateApplicationWindowState(wndController);
  }
  
  /**
   * タブ状態を設定ファイルに保存する
   */
  public void saveTabState() {
    Log.start();
    List<Config.TabState> tabStates = new ArrayList<Config.TabState>();
    for (Control c : wnd.getTabFolder().getChildren()) {
      ListPanel lp = (ListPanel) c;
      Config.TabState s = new Config.TabState();
      if (lp.getFocusItem() != null) {
        s.setFile(lp.getFocusItem().getAbsolutePath());
      }
      else {
        s.setFile(lp.getCurrentDir().getAbsolutePath());
      }
      s.setThumbnailMode(lp.isThumbnailMode());
      tabStates.add(s);
    }
    Config.get().setMainWindowTabStates(tabStates.toArray(new Config.TabState[0]));
    Config.get().setMainWindowTabStatesFocus(wnd.getTabFolder().getSelectionIndex());
  }

  /**
   * 設定が更新されたときに、画面を更新する
   */
  public void configChanged() {
    Main.setupByConfig();
    wnd.resetMenu();
    handle_VIEW_CLEAR_CACHE();
    handle_VIEW_LIST_MODE();
    handle_VIEW_LIST_MODE();
  }

  /**
   * タブを作成する
   */
  private ListPanel createNewTab() {
    CTabFolder tab = wnd.getTabFolder();
    CTabItem item = new CTabItem(tab, SWT.NONE);
    ListPanel newLp = new ListPanel(tab, this, wndController);
    item.setControl(newLp);
    tab.setSelection(item);
    newLp.setMouseScrollType(Config.get().getMainWindowMouseScrollType());
    newLp.setFont(Config.get().getFont());
    newLp.setFocus();
    return newLp;
  }
  
  /**
   * タブを閉じる
   */
  public void closeTab(CTabItem item) {
    item.getControl().dispose();
    item.dispose();
    
    // ステータス表示を更新
    if (getCurrentListPanel() != null) {
      wndController.refresh();
      getCurrentListPanel().updateApplicationWindowState(wndController);
      getCurrentListPanel().setFocus();
    }
    else {
      wndController.updateMenuStatus();
      wndController.setStatusFileMessage("");
      wndController.setTitle("");
    }
  }
  
  /**
   * サムネイルモードとリストモードを切り替える
   */
  private void switchListThumbnailMode() {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;

    lp.setThumbnailMode(!lp.isThumbnailMode());
  }

  /**
   * サムネイルサイズ変更
   */
  private void thumbnailSizeChange(int changeLevel) {
    ListPanel lp = getCurrentListPanel();
    if (lp == null) return;
    if (!lp.isThumbnailMode()) return;
    assert changeLevel == -1 || changeLevel == 1;
    
    Config conf = Config.get();
    
    // 設定読み込み (小さい順に並んでいる)
    ArrayList<int[]> list = new ArrayList<int[]>(); // [面積, width, height]
    for (String s : StringUtils.split(conf.getMainWindowThumbnailSizes(), "\r\n")) {
      String[] ss = StringUtils.split(s.trim(), "　\t ");
      if (ss.length != 2) continue;
      try {
        int[] vals = { 0, Integer.parseInt(ss[0].trim()), Integer.parseInt(ss[1].trim()) };
        vals[0] = vals[1] * vals[2];
        list.add(vals);
      }
      catch (Exception e) { }
    }
    
    int currentArea = conf.getMainWindowThumbnailSizeX() * conf.getMainWindowThumbnailSizeY();
    
    // 候補検索
    int index = -1;
    if (changeLevel > 0) {
      // 拡大
      for (int i = 0; i < list.size(); i++) {
        if (list.get(i)[0] <= currentArea) continue;
        index = i;
        break;
      }
    }
    else {
      // 縮小
      for (int i = list.size() - 1; i >= 0; i--) {
        if (list.get(i)[0] >= currentArea) continue;
        index = i;
        break;
      }
    }
    if (index < 0) return;

    // サイズ変更
    conf.setMainWindowThumbnailSizeX(list.get(index)[1]);
    conf.setMainWindowThumbnailSizeY(list.get(index)[2]);
    handle_VIEW_CLEAR_CACHE();
  }
  
  /**
   * 画像表示画面を開く
   */
  private void openViewWindow() {
    if (Config.get().viewWindowFullscreen) {
      WindowSelector.instance().select(Type.VIEW_FULL);
    }
    else {
      WindowSelector.instance().select(Type.VIEW);
    }
  }

  /*
   * 最初のアプリかどうかのチェック
   */
  private boolean checkFirstApp() {
    if (!ApplicationLock.instance().isFirstApp()) {
      String msg = "このウインドウよりも先に起動された" + Config.APP_NAME + "が存在するのでこの操作は実行できません。\n" +
            "最初に起動されたウインドウ以外には、タイトルに \"(*)\" がついています。\n" +
            "\n" +
            "別の設定を使用して起動された場合はこの制限はありません。\n" +
            "現在の設定ディレクトリは、" + ParseArg.configDir + " です。";
      MsgBox.info("実行できません", msg);
      return false;
    }
    return true;
  }
  
  /**
   * 現在のタブの ListPanel を取得
   */
  public ListPanel getCurrentListPanel() {
    return wndController.getCurrentListPanel();
  }
}
