package cvj2.app.gui.window.main;

import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.time.StopWatch;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.swt.SWT;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Shell;

import cvj2.app.Config;
import cvj2.app.Main;
import cvj2.app.gui.dialogs.FileRenameDialog;
import cvj2.app.gui.dialogs.InputAndMessageDialog;
import cvj2.app.gui.dialogs.MyProgressMonitorDialog;
import cvj2.app.gui.util.MsgBox;
import cvj2.app.gui.util.MsgBox.Ret;
import cvj2.app.gui.widgets.ListPanel;
import cvj2.app.gui.widgets.ListPanelDndUtil;
import cvj2.file.fs.FileInfo;
import cvj2.file.fs.FileInfoFactory;
import cvj2.file.fs.FileInfoList;
import cvj2.file.util.BPSCout;
import cvj2.file.util.FileOperationCancellerJFace;
import cvj2.file.util.FileOperationCopy;
import cvj2.file.util.FileOperationDelete;
import cvj2.file.util.FileOperationException;
import cvj2.file.util.FileOperationMove;
import cvj2.gui.swt.dnd.MyFileTransfer;
import cvj2.gui.swt.dnd.MyFileTransferData;

/**
 * MainWindow で行うファイル操作を処理するヘルパークラス
 */
public class MainWindowFileOperation {
  private MainWindowFileOperation() {}
  
  /**
   * ファイル削除
   */
  static public void delete(ListPanel lp) {
    if (lp == null) return;
    if (lp.getCurrentDir() == FileInfo.ROOT) return;
    
    // フルパス名のリストを作る
    FileInfoList fileList = lp.getModel().getSortedList();
    List<File> list = new ArrayList<File>();
    List<String> names = new ArrayList<String>();
    for (int i : lp.getSelection().getSelectedIndices()) {
      list.add(fileList.get(i).getFile());
      names.add(fileList.get(i).getName());
    }
    if (list.size() == 0) return;
    
    // 確認
    String filenames = ListPanelDndUtil.toMsgBoxString(names.toArray(new String[0]), "  ", 10);
    if (!MsgBox.okCancel("削除", "削除してもよろしいですか\n\n" + filenames)) {
      return;
    }
    
    // プログレスダイアログを表示する
    MyProgressMonitorDialog dlg = new MyProgressMonitorDialog(lp.getShell());
    dlg.open();
    dlg.getShell().setText("削除");
    try {
      dlg.run(true, true, new DeleteTask(dlg.getShell(), list.toArray(new File[0])));
    }
    catch (InvocationTargetException e) {
      if (e.getCause() instanceof FileOperationException) {
        MsgBox.error("エラー", "ファイル削除中にエラーが発生しました。\n\n" + e.getCause().getLocalizedMessage());
      }
      else {
        throw new RuntimeException(e);
      }
    }
    catch (InterruptedException e) {
      throw new RuntimeException(e);
    }
    
    // 再表示
    int focus = lp.getSelection().getFocusIndex();
    FileInfo f = null;
    if (focus >= 0) f = lp.getModel().getSortedList().get(focus);
    lp.setCurrentDir(lp.getModel().getCurrentDir(), f);
  }
  /*
   * 削除タスク
   */
  private static class DeleteTask implements IRunnableWithProgress {
    Shell shell;
    File[] files;
    IProgressMonitor monitor;
    /* コンストラクタ */
    public DeleteTask(Shell shell, File[] files) {
      this.shell = shell;
      this.files = files;
    }
    /* 削除処理 */
    @Override
    public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
      this.monitor = monitor;
      try {
        FileOperationDelete op = new FileOperationDelete(new DeleteTaskNotify(), new FileOperationCancellerJFace(monitor));
        monitor.setTaskName("削除するファイルを確認しています...");
        int count = op.count(files);
        if (monitor.isCanceled()) return;
        monitor.beginTask("削除します", count);
        op.delete(files);
        monitor.done();
      }
      catch (IOException e) {
        throw new InvocationTargetException(e);
      }
    }
    /* 削除処理の通知 */
    private class DeleteTaskNotify implements FileOperationDelete.Notify {
      private boolean allDelete = false;
      @Override public void progressDir(File dir) {
        monitor.setTaskName("" + dir);
        monitor.worked(1);
      }
      @Override public void progressFile(File f) {
        monitor.subTask("" + f.getName());
        monitor.worked(1);
      }
      @Override public boolean askDelete(final File f) {
        if (allDelete) return true;
        final MsgBox.Ret[] ret = { null };
        shell.getDisplay().syncExec(new Runnable() {
          @Override public void run() {
            ret[0] = MsgBox.okAllCancel("削除", f + "\nは読み取り専用です。削除しますか？");
          }
        });
        if (ret[0] == Ret.ALL) allDelete = true;
        return ret[0] != Ret.CANCEL;
      }
    }
  }
  
  /**
   * ファイル名変更
   */
  static public void rename(ListPanel lp) {
    if (lp == null) return;
    if (lp.getCurrentDir() == FileInfo.ROOT) return;
    
    if (lp.getSelection().getSelectedIndices().size() != 1) {
      MsgBox.info("名称変更", "ファイルは1つだけ選択してください");
      return;
    }
    
    FileInfo oldFile = lp.getModel().getSortedList().get(lp.getSelection().getFocusIndex());
    String newName = oldFile.getName();
    String msg = null;
    File newFile = null;
    while (true) {
      // ダイアログ表示
      InputAndMessageDialog dlg = new FileRenameDialog(lp.getShell(), newName, msg);
      if (SWT.OK != dlg.open()) return;
      
      newName = dlg.getInputString();
      newFile = new File(oldFile.getParent().getFile(), newName);

      // チェック
      if (newFile.exists()) {
        msg = newFile.getAbsolutePath() + " は既に存在します";
        continue;
      }
      if (newFile.getParent() == null || !newFile.getParentFile().equals(oldFile.getParent().getFile())) {
        msg = "正しいファイル名ではありません";
        continue;
      }

      // リネーム
      if (oldFile.getFile().renameTo(newFile)) break;
      
      // 失敗
      msg = "名前の変更ができませんでした";
    }
    
    // 再表示
    if (Config.get().isMainWindowRenameSelectNewName()) {
      lp.setCurrentDir(lp.getModel().getCurrentDir(), FileInfoFactory.create(newFile));
    }
    else {
      lp.setCurrentDir(lp.getModel().getCurrentDir(), oldFile);
    }
  }

  /**
   * ディレクトリのオープン
   */
  public static void openDir(ListPanel lp) {
    DirectoryDialog dlg = new DirectoryDialog(lp.getShell());
    dlg.setText("閲覧するディレクトリを選択してください");
    dlg.setFilterPath(lp.getCurrentDir().getAbsolutePath());
    String path = dlg.open();
    if (path == null) return;
    FileInfo dir = FileInfoFactory.create(new File(path));
    if (!dir.isDirectory()) {
      MsgBox.error("お知らせ", "このディレクトリは開けません\n" + path);
      return;
    }
    lp.setCurrentDir(dir, null);
  }

  /**
   * 新規ディレクトリ作成
   */
  public static void newDir(ListPanel lp) {
    if (lp == null) return;
    if (lp.getCurrentDir() == FileInfo.ROOT) return;
    
    String dirName = "";
    String msg = "作成するディレクトリの名前を入力してください";
    File newDir;
    while (true) {
      // ダイアログ表示
      InputAndMessageDialog dlg = new InputAndMessageDialog(lp.getShell());
      dlg.setData("新規ディレクトリ作成", dirName, 0, 0, msg);
      if (SWT.OK != dlg.open()) return;
      
      dirName = dlg.getInputString();
      newDir = new File(lp.getCurrentDir().getFile(), dirName);
      
      // チェック
      if (newDir.exists()) {
        msg = newDir.getAbsolutePath() + " は既に存在しています。";
        continue;
      }
      if (newDir.getParent() == null || !lp.getCurrentDir().getFile().equals(newDir.getParentFile())) {
        msg = "正しいディレクトリ名ではありません";
        continue;
      }
      
      // 作成
      if (newDir.mkdir()) break;
      
      // 失敗
      msg = "このディレクトリ名では作成できません";
    }

    // 再表示
    lp.setCurrentDir(lp.getModel().getCurrentDir(), FileInfoFactory.create(newDir));
  }

  /**
   * 新規ファイル作成
   */
  public static void newFile(ListPanel lp) {
    if (lp == null) return;
    if (lp.getCurrentDir() == FileInfo.ROOT) return;
    
    String filename = "新規ファイル.txt";
    String msg = "作成するファイルの名前を入力してください";
    File newFile;
    while (true) {
      // ダイアログ表示
      InputAndMessageDialog dlg = new InputAndMessageDialog(lp.getShell());
      dlg.setData("新規ファイル作成", filename, 0, msg.length(), msg);
      if (SWT.OK != dlg.open()) return;
      
      filename = dlg.getInputString();
      newFile = new File(lp.getCurrentDir().getFile(), filename);
      
      // チェック
      if (newFile.exists()) {
        msg = newFile.getAbsolutePath() + " は既に存在しています。";
        continue;
      }
      if (newFile.getParent() == null || !lp.getCurrentDir().getFile().equals(newFile.getParentFile())) {
        msg = "正しいファイル名ではありません";
        continue;
      }
      
      // 作成
      try {
        FileUtils.touch(newFile);
        if (newFile.isFile()) break;
        
        // 失敗
        msg = "このファイル名では作成できません";
      }
      catch (IOException e) {
        msg = "ファイルを作成できません";
      }
    }

    // 再表示
    lp.setCurrentDir(lp.getModel().getCurrentDir(), FileInfoFactory.create(newFile));
  }
  
  /**
   * ペースト
   */
  public static void paste(ListPanel lp) {
    if (lp == null) return;
    if (lp.getCurrentDir() == FileInfo.ROOT) return;

    String[] filenames;
    boolean isCopy;
    boolean fromOtherApp;
    
    // クリップボードからファイル名を取得
    MyFileTransferData myData = (MyFileTransferData) Main.getClipboard().getContents(MyFileTransfer.getInstance());
    if (myData != null) {
      fromOtherApp = false;
      isCopy = myData.isCopy;
      filenames = myData.files.toArray(new String[0]);
    }
    else {
      filenames = (String[]) Main.getClipboard().getContents(FileTransfer.getInstance());
      if (filenames == null) return;
      fromOtherApp = true;
      isCopy = true;
    }
    
    // 処理
    copyOrMove(lp, filenames, isCopy, fromOtherApp);
  }
  
  /**
   * ファイルのコピー、移動
   * @param lp コピー先ディレクトリのListPanel
   * @param fromList コピー元
   * @param isCopy コピーするか
   * @param fromOtherApp 外部アプリからで、コピーか移動かわからない場合
   */
  public static void copyOrMove(ListPanel lp, String[] fromList, boolean isCopy, boolean fromOtherApp) {
    if (lp.getCurrentDir() == FileInfo.ROOT) return;

    // 確認メッセージ
    String nameMoveCopy = (isCopy ? "コピー" : "移動");
    String msg;
    if (fromOtherApp) {
      msg = "外部のアプリケーションからクリップボード経由で送られてきた\n" +
            "以下のファイルあるいはディレクトリを、カレントディレクトリにコピーします";
    }
    else if (isCopy) {
      msg = "以下のファイルあるいはディレクトリを、カレントディレクトリにコピーします";
    }
    else {
      msg = "以下のファイルあるいはディレクトリを、カレントディレクトリに移動します";
    }
    if (!MsgBox.okCancel(nameMoveCopy + "の確認", msg + "\n\n" + ListPanelDndUtil.toMsgBoxString(fromList, "\t", 10))) return;

    // 処理
    File[] fromFileList = new File[fromList.length];
    for (int i = 0; i < fromList.length; i++) fromFileList[i] = new File(fromList[i]);
    
    try {
      if (!isCopy) {
        // 移動
        move(lp, fromFileList, lp.getCurrentDir().getFile());
      }
      else {
        // コピー
        copy(lp, fromFileList, lp.getCurrentDir().getFile());
      }
    }
    catch (InvocationTargetException e) {
      if (e.getCause() instanceof FileOperationException || e.getCause() instanceof IOException) {
        String errMsg = "ファイル" + nameMoveCopy + "中にエラーが発生しました。\n\n" + 
            e.getCause().getLocalizedMessage() + "\n\n" +
            "※) このアプリケーションは、複雑な移動やコピーには対応していません。\n" + 
            "そのような処理をしたいときは、\n" + 
            "  (1) CTRL+Enter で転送先のディレクトリを" + Config.OS_NAME + "のウインドウで別に開いて\n" +
            "  (2) ドラッグ・アンド・ドロップでファイルをそのウインドウに渡してください。\n" +
            "そうすれば、" + Config.OS_NAME + "がファイルの移動やコピーをしてくれます。";
        MsgBox.error("お知らせ", errMsg);
      }
      else {
        throw new RuntimeException(e);
      }
    }
    catch (InterruptedException e) {
      throw new RuntimeException(e);
    }

    // 再表示
    lp.setCurrentDir(lp.getModel().getCurrentDir(), lp.getFocusItem());
  }
  
  /**
   * 移動する
   */
  private static void move(ListPanel lp, File[] fromList, File destDir) throws InvocationTargetException, InterruptedException {
    MyProgressMonitorDialog dlg = new MyProgressMonitorDialog(lp.getShell());
    dlg.open();
    dlg.getShell().setText("移動");
    dlg.run(true, true, new MoveTask(fromList, destDir));
  }
  /*
   * 移動タスク
   */
  private static class MoveTask implements IRunnableWithProgress {
    File[] fromList;
    File destDir;
    IProgressMonitor monitor;
    StopWatch sw = new StopWatch();
    public MoveTask(File[] fromList, File destDir) {
      this.fromList = fromList;
      this.destDir = destDir;
    }
    /* 処理の本体 */
    @Override
    public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
      this.monitor = monitor;
      try {
        FileOperationMove op = new FileOperationMove(new MoveTaskNotify(), new FileOperationCancellerJFace(monitor));
        sw.start();
        monitor.setTaskName("移動するファイルを確認しています...");
        op.tryMove(fromList, destDir);
        if (monitor.isCanceled()) return;
        monitor.beginTask("移動します", fromList.length);
        op.move(fromList, destDir);
        monitor.done();
      }
      catch (IOException e) {
        throw new InvocationTargetException(e);
      }
    }
    /* 処理の通知 */
    private class MoveTaskNotify implements FileOperationMove.Notify {
      private boolean answerAll = false;
      @Override public void progress(File f) {
        if (sw.getTime() > 50) {
          monitor.setTaskName("" + f);
          sw.reset();
          sw.start();
        }
        monitor.worked(1);
      }
      @Override public boolean confirmOverwrite(File f) {
        if (answerAll) return true;
        Ret ret = MsgBox.okAllCancel("ファイルの上書き", "このファイルを上書きしてもよろしいですか？\n" + f);
        switch (ret) {
        case OK: return true;
        case CANCEL: return false;
        case ALL: 
          answerAll = true;
          return true;
        default:
          throw new RuntimeException("" + ret);
        }
      }
    }
  }
  
  /**
   * コピーする
   */
  private static void copy(ListPanel lp, File[] fromList, File destDir) throws InvocationTargetException, InterruptedException {
    MyProgressMonitorDialog dlg = new MyProgressMonitorDialog(lp.getShell());
    dlg.open();
    dlg.getShell().setText("コピー");
    dlg.run(true, true, new CopyTask(fromList, destDir));
  }
  /*
   * コピータスク
   */
  private static class CopyTask implements IRunnableWithProgress {
    private static final int PROGRESS_TOTAL = 1000;
    File[] fromList;
    File destDir;
    IProgressMonitor monitor;
    long totalCount;
    long finishedBytes = 0;
    long finishedCount = 0;
    int lastProgress = 0;
    BPSCout bps;
    StopWatch sw;
    public CopyTask(File[] fromList, File destDir) {
      this.fromList = fromList;
      this.destDir = destDir;
    }
    @Override
    public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
      this.monitor = monitor;
      try {
        FileOperationCopy op = new FileOperationCopy(new CopyTaskNotify(), new FileOperationCancellerJFace(monitor));
        monitor.setTaskName("コピーするファイルを確認しています...");
        totalCount = op.tryCopy(fromList, destDir);
        if (monitor.isCanceled()) return;
        monitor.beginTask("コピーします", PROGRESS_TOTAL);
        bps = new BPSCout(500, 6);
        sw = new StopWatch();
        sw.start();
        op.copy(fromList, destDir);
        monitor.done();
      }
      catch (IOException e) {
        throw new InvocationTargetException(e);
      }
    }
    /* 処理の通知 */
    private class CopyTaskNotify implements FileOperationCopy.Notify {
      boolean allFileRename = false;
      boolean allFileIgnore = false;
      boolean allFileOverwrite = false;
      boolean allDirRename = false;
      boolean allDirIgnore = false;
      boolean allDirMerge = false;
      @Override
      public void fileProgress(File f, long copySize) {
        // 転送バイト数更新
        finishedCount += copySize;
        finishedBytes += copySize;
        bps.add(copySize);
        int progress = totalCount == 0 ? 0 : (int)(finishedCount * PROGRESS_TOTAL / totalCount);
        if (progress > lastProgress) {
          monitor.worked(progress - lastProgress);
          lastProgress = progress;
        }
        // 表示更新
        if (sw.getTime() > 50) {
          monitor.setTaskName("" + f);
          monitor.subTask(DecimalFormat.getNumberInstance().format(finishedBytes) + " バイトをコピーしました " +
              "(" + FileUtils.byteCountToDisplaySize(bps.getBPS()) + "/s)");
          sw.reset();
          sw.start();
        }
      }
      @Override
      public void fileFinish(File f, long size) { 
        finishedCount++;
      }
      @Override
      public DirAnswer confirmExistDir(File dir) {
        if (allDirRename) return DirAnswer.Rename;
        if (allDirIgnore) return DirAnswer.Ignore;
        if (allDirMerge) return DirAnswer.Merge;
        int ret = MsgBox.showMsgBoxDialog("コピー先に既にディレクトリが存在します", SWT.ICON_QUESTION, 
            new String[] { "名前変更(&R)", "全て名前変更(&A)", "無視(&I)", "全て無視(&G)", "マージ(&M)", "全てマージ(&E)", "キャンセル(&C)" },
            new char[] { 'r', 'a', 'i', 'g', 'm', 'e', 'c' }, 6, 
            "" + dir);
        if (ret == 0) return DirAnswer.Rename;
        if (ret == 1) { allDirRename = true; return DirAnswer.Rename; }
        if (ret == 2) return DirAnswer.Ignore;
        if (ret == 3) { allDirIgnore = true; return DirAnswer.Ignore; }
        if (ret == 4) return DirAnswer.Merge;
        if (ret == 5) { allDirMerge = true; return DirAnswer.Merge; }
        return DirAnswer.Cancel;
      }
      @Override
      public FileAnswer confirmExistFile(File file) {
        if (allFileRename) return FileAnswer.Rename;
        if (allFileIgnore) return FileAnswer.Ignore;
        if (allFileOverwrite) return FileAnswer.OverWrite;
        int ret = MsgBox.showMsgBoxDialog("コピー先に既にファイルが存在します", SWT.ICON_QUESTION, 
            new String[] { "名前変更(&R)", "全て名前変更(&A)", "無視(&I)", "全て無視(&G)", "上書き(&O)", "全て上書き(&V)", "キャンセル(&C)" },
            new char[] { 'r', 'a', 'i', 'g', 'o', 'v', 'c' }, 6, 
            "" + file);
        if (ret == 0) return FileAnswer.Rename;
        if (ret == 1) { allFileRename = true; return FileAnswer.Rename; }
        if (ret == 2) return FileAnswer.Ignore;
        if (ret == 3) { allFileIgnore = true; return FileAnswer.Ignore; }
        if (ret == 4) return FileAnswer.OverWrite;
        if (ret == 5) { allFileOverwrite = true; return FileAnswer.OverWrite; }
        return FileAnswer.Cancel;
      }
    }
  }
}
