package cvj2.app.gui.window.main;

import java.io.File;
import java.util.Arrays;
import java.util.HashSet;

import org.apache.commons.lang3.time.StopWatch;
import org.eclipse.swt.widgets.Display;

import cvj2.app.Config;
import cvj2.app.file.imgload.BackgroundImageLoader;
import cvj2.app.gui.dialogs.MyProgressMonitorDialog;
import cvj2.app.gui.util.GlobalErrorHandler;
import cvj2.app.gui.widgets.ListPanel;
import cvj2.file.fs.FileInfo;
import cvj2.file.fs.FileInfoList;
import cvj2.util.Log;

/**
 * 別アプリからのファイル更新を監視するスレッド。<br>
 * ファイル一覧を定期的に取得し、現在の一覧と相異があれば画面を再表示する
 */
public class MainWindowFileWatchThread extends Thread {
  private static boolean BENCH_MODE = false;
  
  private MainWindow wnd;
  private MainWindowCommandHandler cmdHandler;

  private FileInfo oldDir;
  private FileInfoList oldList;
  
  /**
   * コンストラクタ
   */
  public MainWindowFileWatchThread(MainWindow wnd, MainWindowCommandHandler cmdHandler) {
    this.wnd = wnd;
    this.cmdHandler = cmdHandler;
    super.setDaemon(true);
    super.setName("FileWatch");
  }

  /**
   * 処理本体
   */
  @Override
  public void run() {
    Log.start();
    try {
      StopWatch BENCH_SW = new StopWatch();
      BENCH_SW.start();
      int BEHCH_COUNT = 0;
      
      while (true) {
        // スリープする
        StopWatch sw = new StopWatch();
        sw.start();
        while (true) {
          if (BENCH_MODE) break;
          if (sw.getTime() >= Config.get().getMainWindowFileWatchInterval()) break;
          Thread.sleep(250);
        }
        // 忙しいときは何もしない
        if (BackgroundImageLoader.instance().getRunNum() > 0 || BackgroundImageLoader.instance().getWaitNum() > 0) continue;
        if (MyProgressMonitorDialog.isRunning()) continue;
        // 更新されたかチェック
        if (fileUpdateCheck()) refresh();

        BEHCH_COUNT++;
        if (BENCH_MODE && BENCH_SW.getTime() > 5000) {
          Log.d("count = ", BEHCH_COUNT, ", time (ms) = ", (double)BENCH_SW.getNanoTime() / BEHCH_COUNT / 1000 / 1000);
          BEHCH_COUNT = 0;
          BENCH_SW.reset();
          BENCH_SW.start();
        }
      }
    }
    catch (InterruptedException e) {
      Log.e(e);
      GlobalErrorHandler.instance().setErrorMessage("ファイル更新監視中にエラーが発生しました", e);
    }
  }

  /**
   * 更新されたかチェック
   * @return 更新されていたら、true
   */
  private boolean fileUpdateCheck() throws InterruptedException {
    // 現在のカレントディレクトリとファイルリストを取得する
    if (!updateOld()) return false;
    
    // ファイルがたくさんある時は、スリープする
    int sleepSec = 0;
    if (Config.get().getMainWindowFileWatch1SecDelayFileCount() > 0) {
      sleepSec = oldList.size() / Config.get().getMainWindowFileWatch1SecDelayFileCount();
    }
    if (sleepSec > 0) {
      if (!BENCH_MODE) Thread.sleep(sleepSec * 1000);
    }

    // 元ディレクトリの存在確認
    if (oldDir.isArchiveRoot()) {
      if (!oldDir.getFile().exists()) return true; 
      if (oldDir.lastModified() != oldDir.getFile().lastModified()) return true;
      return false;
    }
    else if (oldDir.isArchiveChild()) {
      if (!oldDir.getArchiveRoot().getFile().isFile()) return true; 
      return false;
    }

    if (oldDir == FileInfo.ROOT) return false;
    if (!oldDir.getFile().isDirectory()) return true;
    
    // 最新のリストを取得する
    File[] newList = oldDir.getFile().listFiles();
    HashSet<File> newFileList = new HashSet<File>(Arrays.asList(newList == null ? new File[0] : newList));

    // 比較
    HashSet<File> oldFileList = new HashSet<File>();
    for (FileInfo f : oldList.getNoFilterList()) oldFileList.add(f.getFile());
    if (oldFileList.equals(newFileList)) return false;
    
    return true;
  }

  /*
   * 画面の情報を取得する
   */
  private boolean updateOld() {
    if (Display.getDefault().isDisposed()) return false;
    Getter g = new Getter();
    Display.getDefault().syncExec(g);
    return g.ret;  
  }
  /*
   * 画面の情報を取得するクラス
   */
  private class Getter implements Runnable {
    boolean ret = false;
    @Override
    public void run() {
      if (wnd.getShell() == null || wnd.getShell().isDisposed()) return;
      ListPanel lp = cmdHandler.getCurrentListPanel();
      if (lp == null || lp.isDisposed() || lp.getModel() == null) return;
      oldDir = lp.getModel().getCurrentDir();
      oldList = lp.getModel().getOriginalList();
      ret = true;
    }
  }

  /*
   * 画面更新
   */
  private void refresh() {
    Log.start(oldDir);
    
    if (Display.getDefault().isDisposed()) return;
    Refresher r = new Refresher();
    Display.getDefault().syncExec(r);
  }
  /*
   * 画面更新クラス
   */
  private class Refresher implements Runnable {
    @Override
    public void run() {
      if (wnd.getShell() == null || wnd.getShell().isDisposed()) return;
      ListPanel lp = cmdHandler.getCurrentListPanel();
      if (lp == null || lp.isDisposed() || lp.getModel() == null) return;
      if (oldDir != lp.getModel().getCurrentDir()) return;
      if (oldList != lp.getModel().getOriginalList()) return;
      cmdHandler.handle_VIEW_REFRESH();
    }
  }
}
