package cvj2.app.gui.window.splash;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import cvj2.app.Config;
import cvj2.app.Main;

/**
 * スプラッシュウインドウ
 */
public class SplashWindow extends Shell {
  private String msg = "アプリケーション起動中・・・";
  private Image bg;
  
  
  /**
   * コンストラクタ
   */
  public SplashWindow() {
    super(Display.getDefault(), SWT.NO_TRIM | SWT.ON_TOP); // ON_TOP でないと、画像表示が欠ける
    addPaintListener(new PaintListener() {
      public void paintControl(PaintEvent e) {
        canvas_paintControl(e);
      }
    });
    setLayout(new FillLayout(SWT.HORIZONTAL));
    createContents();
  }

  /**
   * ウインドウを表示する
   */
  public void showSplashWindow() {
    setImage(Main.getAppIcon());
    bg = new Image(getDisplay(), this.getClass().getResourceAsStream("splash.png"));
    Rectangle dRect = super.getDisplay().getBounds();
    Rectangle rect = super.getBounds();
    rect.width = bg.getBounds().width;
    rect.height = bg.getBounds().height;
    rect.x = (dRect.width - rect.width) / 2;
    rect.y = (dRect.height - rect.height) / 2;
    super.setBounds(rect);
    super.layout();
    super.open();
  }
  
  /**
   * ウインドウを閉じる
   */
  public void closeSplashWindow() {
    if (!super.isDisposed()) {
      super.close();
      super.dispose();
    }
    if (bg != null) {
      bg.dispose();
      bg = null;
    }
  }
  
  /**
   * メッセージを表示する
   */
  public void setMessage(String msg) {
    this.msg = msg;
    redraw();
  }

  /**
   * 画面描画
   */
  protected void canvas_paintControl(PaintEvent e) {
    e.gc.drawImage(bg, 0, 0);
    int fontHeight = e.gc.getFontMetrics().getHeight();
    e.gc.drawString(msg, 3, bg.getBounds().height - fontHeight - 3, true);
  }

  /**
   * Create contents of the shell.
   */
  protected void createContents() {
    setText(Config.APP_NAME);
    setSize(800, 600);
  }

  @Override
  protected void checkSubclass() { }
}
