package cvj2.app.gui.window.view;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.action.StatusLineContributionItem;
import org.eclipse.jface.action.StatusLineManager;
import org.eclipse.jface.action.ToolBarManager;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.wb.swt.ResourceManager;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.app.Config;
import cvj2.app.Main;
import cvj2.app.command.Command;
import cvj2.app.command.CommandMapAction;
import cvj2.app.gui.data.FileListSortElement;
import cvj2.app.gui.util.GlobalErrorHandler;
import cvj2.app.gui.window.CommonApplicationWindow;
import cvj2.app.gui.window.WindowSelector;
import cvj2.app.gui.window.WindowSelector.Type;
import cvj2.app.gui.window.main.MainWindow;
import cvj2.util.Log;

/**
 * 一枚画像表示用の画面
 */
public class ViewWindow extends ApplicationWindow implements CommonApplicationWindow {
  private Action action_screen_close;
  private Action action_edit_copy;
  private Action action_edit_delete;
  private Action action_edit_rename;
  private Action action_view_fullscreen;
  private Action action_view_showImageInfo;
  private Action action_view_refresh;
  private Action action_view_prevDir;
  private Action action_view_nextDir;
  private Action action_tab_prevTab;
  private Action action_tab_nextTab;
  private Action action_image_rotate;
  private Action action_image_rotateReverse;
  private Action action_zoom_in;
  private Action action_zoom_fitToWindow;
  private Action action_zoom_originalSize;
  private Action action_zoom_out;
  private Action action_image_toggle2PageMode;
  private Action action_image_toggle2PageRL;
  private CTabFolder tabFolder;
  private StatusLineContributionItem statusFileItem;

  private MainWindow parent;
  private boolean appInitialized = false;
  private CommandMapAction actionCommandMap = new CommandMapAction();
  private ViewWindowCommandHandler cmd;
  
  public ViewWindow(MainWindow parent) {
    this();
    this.parent = parent;
  }
  
  /**
   * コンストラクタ
   */
  public ViewWindow() {
    super(null);
    createActions();
    addToolBar(SWT.FLAT | SWT.WRAP);
    addMenuBar();
    addStatusLine();
    setExceptionHandler(new SWTExceptionHandler());
  }

  /**
   * エラーハンドラー
   */
  class SWTExceptionHandler implements Window.IExceptionHandler {
    @Override
    public void handleException(Throwable e) {
      Log.e(e);
      GlobalErrorHandler.instance().setFatalMessage("画像ウインドウで、プログラム内部エラーが発生しました", e);
    }
  }
  
  /**
   * アプリ起動直後の初期化
   */
  private void initializeApp() {
    if (appInitialized) return;
    appInitialized = true;

    Log.start();
    Config conf = Config.get();
    getShell().setImage(Main.getAppIcon());

    // コマンドマップの初期化
    cmd = new ViewWindowCommandHandler(this, parent);
    actionCommandMap.add(action_screen_close, Command.VIEW_CLOSE);
    actionCommandMap.add(action_edit_copy, Command.EDIT_COPY);
    actionCommandMap.add(action_edit_delete, Command.EDIT_DELETE);
    actionCommandMap.add(action_edit_rename, Command.EDIT_RENAME);
    actionCommandMap.add(action_view_fullscreen, Command.VIEW_SWITCH_FULLSCREEN);
    actionCommandMap.add(action_view_showImageInfo, Command.VIEW_SHOW_IMAGE_INFO);
    actionCommandMap.add(action_view_refresh, Command.VIEW_REFRESH);
    actionCommandMap.add(action_view_prevDir, Command.VIEW_PREV_DIR);
    actionCommandMap.add(action_view_nextDir, Command.VIEW_NEXT_DIR);
    actionCommandMap.add(action_tab_prevTab, Command.TAB_PREV);
    actionCommandMap.add(action_tab_nextTab, Command.TAB_NEXT);
    actionCommandMap.add(action_image_rotate, Command.VIEW_ROTATE);
    actionCommandMap.add(action_image_rotateReverse, Command.VIEW_ROTATE_REVERSE);
    actionCommandMap.add(action_zoom_in, Command.VIEW_ZOOM_IN);
    actionCommandMap.add(action_zoom_fitToWindow, Command.VIEW_ZOOM_FIT_TO_WINDOW);
    actionCommandMap.add(action_zoom_originalSize, Command.VIEW_ZOOM_ORIGINAL_SIZE);
    actionCommandMap.add(action_zoom_out, Command.VIEW_ZOOM_OUT);
    actionCommandMap.add(action_image_toggle2PageMode, Command.VIEW_TOGGLE_2PAGE_MODE);
    actionCommandMap.add(action_image_toggle2PageRL, Command.VIEW_TOGGLE_2PAGE_RL);
    MyAction.cmd = cmd;
    MyAction.actionCommandMap = actionCommandMap;
    actionCommandMap.resetActionStatus(Config.get().getCmdMapView());
    
    // 画面の大きさを設定 (Windowsの場合はここでセットしなければならない)
    getShell().setBounds(WindowSelector.instance().getMainWindowBounds());
    getShell().setMaximized(WindowSelector.instance().isMainWindowMaximized());
    
    // タブを少し高くする
    getTabFolder().setTabHeight((int)(getTabFolder().getTabHeight() + conf.getWindowTabHeightPlus()));
    // タブのフォント
    getTabFolder().setFont(conf.getFontTab());

    // タブオープン
    getShell().getDisplay().timerExec(10, new Runnable() {
      @Override
      public void run() {
        if (ViewWindow.this.getShell() == null || ViewWindow.this.getShell().isDisposed()) return;
        cmd.createInitialTab();
        cmd.updateViewPanelFile();
      }
    });
  }

  /**
   * アプリ終了前処理
   */
  @Override
  protected boolean canHandleShellCloseEvent() {
    Log.start();
    
    WindowSelector.instance().select(Type.MAIN);
    
    // select() で閉じているので、false をかえす
    return false; 
  }
  
  /**
   * ステータスバーのメッセージ
   */
  @Override
  public void setStatus(String message, boolean isTemporary) {
    super.setStatus(message);
  }
  
  /**
   * ファイルステータスバーのメッセージ
   */
  @Override
  public void setStatusFileMessage(String s) {
    statusFileItem.setText(s);
  }

  /**
   * メニュー状態を更新する
   */
  @Override
  public void updateMenuState(boolean thumbnailMode, FileListSortElement sort) {
    int tabCount = getTabFolder().getChildren().length;
    
    action_tab_prevTab.setEnabled(tabCount > 1);
    action_tab_nextTab.setEnabled(tabCount > 1);
  }
  
  /**
   * タブ取得
   */
  @Override
  public CTabFolder getTabFolder() {
    return tabFolder;
  }
  
  /**
   * Create contents of the application window.
   * @param parent
   */
  @Override
  protected Control createContents(Composite parent) {
    Composite container = new Composite(parent, SWT.NONE);
    container.setLayout(new FillLayout(SWT.HORIZONTAL));
    {
      tabFolder = new CTabFolder(container, SWT.BORDER | SWT.FLAT);
      tabFolder.setMinimumCharacters(8);
      tabFolder.setSelectionForeground(SWTResourceManager.getColor(SWT.COLOR_LIST_SELECTION_TEXT));
      tabFolder.setSelectionBackground(SWTResourceManager.getColor(SWT.COLOR_LIST_SELECTION));
    }

    return container;
  }

  /**
   * Command に結びついた Action のサブクラス
   */
  static public class MyAction extends Action {
    static ViewWindowCommandHandler cmd;
    static CommandMapAction actionCommandMap;
    
    public MyAction(String text) {
      super(text);
    }
    @Override
    public void run() {
      // コマンド実行
      cmd.execCmd(actionCommandMap.findCommand(this));
    }
  }
  
  /**
   * Create the actions.
   */
  private void createActions() {
    // Create the actions
    {
      action_screen_close = new MyAction("&Close") {
      };
      action_screen_close.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "file_exit.png"));
    }
    {
      action_edit_copy = new MyAction("&Copy") {
      };
      action_edit_copy.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_copy.png"));
    }
    {
      action_edit_delete = new MyAction("&Delete") {
      };
      action_edit_delete.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_delete.png"));
    }
    {
      action_edit_rename = new MyAction("&Rename") {
      };
      action_edit_rename.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "edit_rename.png"));
    }
    {
      action_view_fullscreen = new MyAction("&Fullscreen") {
      };
      action_view_fullscreen.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_fullscreen.png"));
    }
    {
      action_view_showImageInfo = new MyAction("&Show image info") {
      };
      action_view_showImageInfo.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_showImageInfo.png"));
    }
    {
      action_view_refresh = new MyAction("&Refresh") {
      };
      action_view_refresh.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_refresh.png"));
    }
    {
      action_view_prevDir = new MyAction("&Prev Dir") {
      };
      action_view_prevDir.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_prevDir.png"));
    }
    {
      action_view_nextDir = new MyAction("&Next Dir") {
      };
      action_view_nextDir.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_nextDir.png"));
    }
    {
      action_tab_prevTab = new MyAction("&Prev Tab") {
      };
      action_tab_prevTab.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "tab_prev.png"));
    }
    {
      action_tab_nextTab = new MyAction("&Next Tab") {
      };
      action_tab_nextTab.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "tab_next.png"));
    }
    {
      action_image_rotate = new MyAction("Rotate") {
      };
      action_image_rotate.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_rotate_right.png"));
    }
    {
      action_image_rotateReverse = new MyAction("Rotate Reverse") {
      };
      action_image_rotateReverse.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_rotate_left.png"));
    }
    {
      action_zoom_in = new MyAction("Zoom In") {
      };
      action_zoom_in.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_zoomIn.png"));
    }
    {
      action_zoom_fitToWindow = new MyAction("Fit to Window") {
      };
      action_zoom_fitToWindow.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_zoomFitToWindow.png"));
    }
    {
      action_zoom_originalSize = new MyAction("Original Size") {
      };
      action_zoom_originalSize.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_zoomOriginalSize.png"));
    }
    {
      action_zoom_out = new MyAction("Zoom Out") {
      };
      action_zoom_out.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_zoomOut.png"));
    }
    {
      action_image_toggle2PageMode = new MyAction("Toggle 2 page mode") {
      };
      action_image_toggle2PageMode.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_toggle2pageMode.png"));
    }
    {
      action_image_toggle2PageRL = new MyAction("Toggle 2 page Right Left") {
      };
      action_image_toggle2PageRL.setImageDescriptor(ResourceManager.getImageDescriptor(MainWindow.class, "view_toggle2PageRL.png"));
    }
  }

  /**
   * Create the menu manager.
   * @return the menu manager
   */
  @Override
  protected MenuManager createMenuManager() {
    MenuManager menuManager = new MenuManager("menu");
    {
      MenuManager menuManager_screen = new MenuManager("画面(&S)");
      menuManager.add(menuManager_screen);
      menuManager_screen.add(action_screen_close);
    }
    {
      MenuManager menuManager_edit = new MenuManager("編集(&E)");
      menuManager.add(menuManager_edit);
      menuManager_edit.add(action_edit_delete);
      menuManager_edit.add(action_edit_rename);
    }
    {
      MenuManager menuManager_view = new MenuManager("表示(&V)");
      menuManager.add(menuManager_view);
      menuManager_view.add(action_view_fullscreen);
      menuManager_view.add(action_view_showImageInfo);
      menuManager_view.add(action_view_refresh);
      menuManager_view.add(new Separator());
      menuManager_view.add(action_view_prevDir);
      menuManager_view.add(action_view_nextDir);
    }
    {
      MenuManager menuManager_image = new MenuManager("画像(&I)");
      menuManager.add(menuManager_image);
      menuManager_image.add(action_image_rotate);
      menuManager_image.add(action_image_rotateReverse);
      menuManager_image.add(new Separator());
      menuManager_image.add(action_image_toggle2PageMode);
      menuManager_image.add(action_image_toggle2PageRL);
    }
    {
      MenuManager menuManager_zoom = new MenuManager("ズーム(&Z)");
      menuManager.add(menuManager_zoom);
      menuManager_zoom.add(action_zoom_in);
      menuManager_zoom.add(action_zoom_out);
      menuManager_zoom.add(new Separator());
      menuManager_zoom.add(action_zoom_originalSize);
      menuManager_zoom.add(action_zoom_fitToWindow);
    }
    {
      MenuManager menuManager_tab = new MenuManager("タブ(&T)");
      menuManager.add(menuManager_tab);
      menuManager_tab.add(action_tab_prevTab);
      menuManager_tab.add(action_tab_nextTab);
    }
    return menuManager;
  }

  /**
   * Create the toolbar manager.
   * @return the toolbar manager
   */
  @Override
  protected ToolBarManager createToolBarManager(int style) {
    ToolBarManager toolBarManager = new ToolBarManager(style);
    toolBarManager.add(action_screen_close);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_edit_copy);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_edit_delete);
    toolBarManager.add(action_edit_rename);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_view_fullscreen);
    toolBarManager.add(action_view_refresh);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_view_prevDir);
    toolBarManager.add(action_view_nextDir);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_image_rotate);
    toolBarManager.add(action_image_rotateReverse);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_zoom_in);
    toolBarManager.add(action_zoom_out);
    toolBarManager.add(new Separator());
    toolBarManager.add(action_zoom_fitToWindow);
    toolBarManager.add(action_zoom_originalSize);
    return toolBarManager;
  }

  /**
   * Create the status line manager.
   * @return the status line manager
   */
  @Override
  protected StatusLineManager createStatusLineManager() {
    StatusLineManager statusLineManager = new StatusLineManager();
    statusFileItem = new StatusLineContributionItem("FILE_STATUS");
    statusLineManager.add(statusFileItem);
    return statusLineManager;
  }

  /**
   * Configure the shell.
   * @param newShell
   */
  @Override
  protected void configureShell(Shell newShell) {
    super.configureShell(newShell);
    newShell.setText(Config.APP_NAME);
    newShell.addShellListener(new ShellAdapter() {
      @Override
      public void shellActivated(ShellEvent e) {
        initializeApp();
      }
    });

    // 画面の大きさを設定
    newShell.setBounds(WindowSelector.instance().getMainWindowBounds());
    newShell.setMaximized(WindowSelector.instance().isMainWindowMaximized());
  }

  /**
   * Return the initial size of the window.
   */
  @Override
  protected Point getInitialSize() {
    return new Point(450, 300);
  }
}
