package cvj2.app.gui.window.view;

import java.util.Date;

import org.eclipse.swt.custom.CTabItem;

import cvj2.app.Config;
import cvj2.app.command.Command;
import cvj2.app.command.CommandHandler;
import cvj2.app.gui.widgets.ListPanel;
import cvj2.app.gui.widgets.ViewPanel;
import cvj2.app.gui.widgets.ViewPanelState;
import cvj2.app.gui.window.WindowController;
import cvj2.app.gui.window.WindowSelector;
import cvj2.app.gui.window.WindowSelector.Type;
import cvj2.app.gui.window.main.MainWindow;
import cvj2.app.gui.window.main.MainWindowFileOperation;
import cvj2.gui.swt.ui.input.InputCombination;
import cvj2.util.Log;

/**
 * ViewWindow, ViewWindowFullscreen で発生したコマンドを処理するクラス。<br>
 * 
 * ViewWindow と ViewPanel の間で処理のやりとりを行う
 */
public class ViewWindowCommandHandler extends CommandHandler {
  ViewWindow wnd;
  ViewWindowFullscreen wndFull;
  WindowController wndController;
  
  /**
   * コンストラクタ
   */
  public ViewWindowCommandHandler(ViewWindow wnd, MainWindow mainWnd) {
    this.wnd = wnd;
    this.wndController = new WindowController(mainWnd, wnd);
    this.wndController.initViewPanelStateFileInfo();
  }

  /**
   * コンストラクタ
   */
  public ViewWindowCommandHandler(ViewWindowFullscreen wnd, MainWindow mainWnd) {
    this.wndFull = wnd;
    this.wndController = new WindowController(mainWnd, wnd);
    this.wndController.initViewPanelStateFileInfo();
  }

  /**
   * ViewPanel の入力ハンドラー
   */
  public void handle(InputCombination input) {
    ViewPanel vp = getCurrentViewPanel();
    if (vp != null) vp.setFocus();
    
    Command cmd = Config.get().getCmdMapView().findCommand(input);
    if (cmd == null) return;
    execCmd(cmd);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_CLOSE() {
    Log.start();
    WindowSelector.instance().select(Type.MAIN);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_DELETE() {
    MainWindowFileOperation.delete(wndController.getCurrentListPanel());
    updateViewPanelFile();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_EDIT_RENAME() {
    MainWindowFileOperation.rename(wndController.getCurrentListPanel());
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_REFRESH() {
    wndController.refresh();
    updateViewPanelFile();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_PREV_DIR() {
    wndController.moveSiblingDir(-1);
    updateViewPanelFile();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_NEXT_DIR() {
    wndController.moveSiblingDir(1);
    updateViewPanelFile();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_PARENT_DIR() {
    wndController.moveToParent();
    WindowSelector.instance().select(Type.MAIN);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_TAB_PREV() {
    wndController.moveTabSelect(-1);
    updateViewPanelFile();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_TAB_NEXT() {
    wndController.moveTabSelect(1);
    updateViewPanelFile();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_SWITCH_FULLSCREEN() {
    if (wnd != null) {
      Config.get().viewWindowFullscreen = true;
      WindowSelector.instance().select(Type.VIEW_FULL);
    }
    else {
      Config.get().viewWindowFullscreen = false;
      WindowSelector.instance().select(Type.VIEW);
    }
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_SHOW_IMAGE_INFO() {
    Config conf = Config.get();
    conf.viewWindowShowImageInfo = !conf.viewWindowShowImageInfo;
    updateViewPanelFile();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ROTATE() {
    ViewPanelState state = getCurrentViewPanelState();
    state.rotate.rotate(1);
    setCurrentViewPanelState(state);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ROTATE_REVERSE() {
    ViewPanelState state = getCurrentViewPanelState();
    state.rotate.rotate(-1);
    setCurrentViewPanelState(state);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_TOGGLE_2PAGE_MODE() {
    ViewPanelState state = getCurrentViewPanelState();
    state.page2 = !state.page2;
    setCurrentViewPanelState(state);
    updateViewPanelFile();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_TOGGLE_2PAGE_RL() {
    ViewPanelState state = getCurrentViewPanelState();
    state.page2Reverse = !state.page2Reverse;
    setCurrentViewPanelState(state);
    updateViewPanelFile();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_IN() {
    getCurrentViewPanel().setZoom(1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_OUT() {
    getCurrentViewPanel().setZoom(-1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_ORIGINAL_SIZE() {
    ViewPanelState state = getCurrentViewPanelState();
    state.zoom.reset();
    state.originalSize = true;
    state.location.reset(); 
    setCurrentViewPanelState(state);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_FIT_TO_WINDOW() {
    ViewPanelState state = getCurrentViewPanelState();
    state.zoom.reset();
    state.originalSize = false;
    state.location.reset(); 
    setCurrentViewPanelState(state);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_FIT_TO_WINDOW_OR_RETURN_MAIN() {
    ViewPanelState state = getCurrentViewPanelState();

    if (state.zoom.getLevel() == 0 && state.originalSize == false) {
      // 既に画面にフィットしている状態なら、メイン画面に戻る
      WindowSelector.instance().select(Type.MAIN);
      return;
    }

    handle_VIEW_ZOOM_FIT_TO_WINDOW();
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_MOVE_UP() {
    getCurrentViewPanel().moveImageLocation(0, 1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_MOVE_DOWN() {
    getCurrentViewPanel().moveImageLocation(0, -1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_MOVE_RIGHT() {
    getCurrentViewPanel().moveImageLocation(-1, 0);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_MOVE_LEFT() {
    getCurrentViewPanel().moveImageLocation(1, 0);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_ZOOM_MOVE_CENTER() {
    ViewPanelState state = getCurrentViewPanelState();
    state.location.reset(); 
    setCurrentViewPanelState(state);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_IMAGE_PREV_COLUMN() {
    moveCursor(-1, 0, 0, 0);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_IMAGE_NEXT_COLUMN() {
    moveCursor(1, 0, 0, 0);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_IMAGE_PREV_ROW() {
    moveCursor(0, -1, 0, 0);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_IMAGE_NEXT_ROW() {
    moveCursor(0, 1, 0, 0);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_IMAGE_PREV_PAGE() {
    moveCursor(0, 0, -1, 0);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_IMAGE_NEXT_PAGE() {
    moveCursor(0, 0, 1, 0);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_IMAGE_TOP() {
    moveCursor(0, 0, 0, -1);
  }

  /**
   * ※) このメソッドは、CommandHandler からリフレクションで呼ばれる
   */
  public void handle_VIEW_IMAGE_TAIL() {
    moveCursor(0, 0, 0, 1);
  }
  
  /**
   * タブの初期化
   * ※) このメソッドは、ViewWindow の初期化時にのみ呼ばれる
   */
  public void createInitialTab() {
    wndController.initializeViewWindowTab(this);

    CTabItem item = wnd.getTabFolder().getSelection();
    if (item == null) return;
    item.getControl().setFocus();
  }
  
  private long lastMoveCursor = 0;
  /*
   * カーソル移動
   */
  private void moveCursor(int moveCol, int moveRow, int movePage, int headTailDirection) {
    Config conf = Config.get();
    
    ViewPanelState state = getCurrentViewPanelState();
    int x = state.page2 ? 2 : 1;

    // あまり速く移動しすぎないようにする
    long now = new Date().getTime();
    if (moveCol != 0 && now - lastMoveCursor < conf.getViewWindowMoveCursorColumnDelay() * x) return;
    if (moveRow != 0 && now - lastMoveCursor < conf.getViewWindowMoveCursorRowDelay() * x) return;
    lastMoveCursor = now;
    
    wndController.moveCursor(moveCol * x, moveRow * x, movePage * x, headTailDirection);
    updateViewPanelFile();
  }

  /**
   * 表示画像更新
   * ※) ViewWindow と ViewWindowFullscreen の初期表示からも呼ばれるので、public にする。
   */
  public void updateViewPanelFile() {
    ListPanel lp = wndController.getCurrentListPanel();
    assert lp != null;

    getCurrentViewPanel().updateListPanelSelectionImage();
  }

  /**
   * 選択中のタブの ListPanel を取得
   */
  private ViewPanel getCurrentViewPanel() {
    if (wnd != null) {
      CTabItem item = wnd.getTabFolder().getSelection();
      if (item == null) return null;
      return (ViewPanel) item.getControl();
    }
    else {
      return wndFull.getViewPanel();
    }
  }
  
  /*
   * 現在選択されているタブの画像表示状態を取得する
   */
  private ViewPanelState getCurrentViewPanelState() {
    return getCurrentViewPanel().getCurrentTabState();
  }
  
  /*
   * 現在選択されているタブの画像表示状態を設定する
   */
  private void setCurrentViewPanelState(ViewPanelState state) {
    getCurrentViewPanel().setCurrentTabState(state);
  }
}
