package cvj2.app.gui.window.view;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CTabFolder;
import org.eclipse.swt.events.ShellAdapter;
import org.eclipse.swt.events.ShellEvent;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import cvj2.app.Main;
import cvj2.app.gui.data.FileListSortElement;
import cvj2.app.gui.widgets.ViewPanel;
import cvj2.app.gui.window.CommonApplicationWindow;
import cvj2.app.gui.window.WindowController;
import cvj2.app.gui.window.main.MainWindow;
import cvj2.util.Log;

/**
 * 一枚画像表示用の画面 (フルスクリーン)
 */
public class ViewWindowFullscreen extends Shell implements CommonApplicationWindow {
  private ViewPanel viewPanel;
  private ViewWindowCommandHandler cmd;
  private boolean closed = false;
  
  /**
   * コンストラクタ
   */
  public ViewWindowFullscreen(MainWindow parent) {
    super(Display.getCurrent(), SWT.NO_TRIM);
    addShellListener(new ShellAdapter() {
      @Override
      public void shellClosed(ShellEvent e) {
        Log.start();
        if (isViewWindowFullscreenClosed()) return;
        closed = true;
        cmd.handle_VIEW_CLOSE();
      }
    });
    setLayout(new FillLayout(SWT.HORIZONTAL));
    cmd = new ViewWindowCommandHandler(this, parent);
    viewPanel = new ViewPanel(this, cmd, new WindowController(parent, this));
  }
  
  /**
   * 画面を表示する
   */
  public void showViewWindowFullscreen() {
    Display display = Display.getCurrent();
    setBounds(display.getBounds());
    open();
    layout();
    getShell().setImage(Main.getAppIcon());
    cmd.updateViewPanelFile();
  }

  /**
   * オーバーライド
   * @see CommonApplicationWindow
   */
  @Override
  public Shell getShell() {
    return this;
  }

  /**
   * close() の代わりに使用する。<br>
   * ※) Shell は閉じる前の通知がないため、一度閉じられたことを憶えておくフラグが必要になる<br>
   * ※) ALT+F4 で閉じられたときは、ShellListener#shellClosed() がいきなり呼ばれるので、
   *     2回以上 close() を呼ばないようにするためにフラグをチェックする。
   */
  public void closeViewWindowFullscreen() {
    if (closed) return;
    closed = true;
    super.close();
  }
  
  /**
   * 既に閉じられているかどうか
   */
  public boolean isViewWindowFullscreenClosed() {
    return closed;
  }

  /**
   * ステータスバーのメッセージ
   */
  @Override
  public void setStatus(String message, boolean isTemporary) {
    // ViewPanel に表示する
    viewPanel.setStatusMessage(isTemporary ? message : "");
  }

  /**
   * ファイルステータスバーのメッセージ
   */
  @Override
  public void setStatusFileMessage(String s) {
    /* なにもしない */
  }

  /**
   * メニュー状態を更新する
   */
  @Override
  public void updateMenuState(boolean thumbnailMode, FileListSortElement sort) {
    /* なにもしない */
  }

  /**
   * タブ取得
   */
  @Override
  public CTabFolder getTabFolder() {
    /* 呼ばれないはず */
    throw new UnsupportedOperationException();
  }
  
  @Override
  protected void checkSubclass() {
    // Disable the check that prevents subclassing of SWT components
  }

  /**
   * ViewPanel の取得
   */
  public ViewPanel getViewPanel() {
    return viewPanel;
  }
}
