package cvj2.file.fs;

import java.text.Collator;
import java.util.Comparator;

import cvj2.util.StringComparator;

/**
 * ファイルの比較クラス。<br>
 * (同じディレクトリにあるファイルの名前のみの比較。フルパスには対応していない)
 */
public class FileInfoComparator implements Comparator<FileInfo> {
  static public final StringComparator customCmp = new StringComparator();
  static private final Collator collator = Collator.getInstance();

  private Type type;

  /** ソート種別 */
  public enum Type {
    /** Unicodeの文字コード順 */
    UNICODE("Unicodeの文字コード順"),
    /** Javaの自然言語比較順 (UNICODEの4倍遅い) */
    JAVA("Javaの自然言語比較順"),
    /** カスタム比較 (UNICODEの2倍遅い) */
    CUSTOM("カスタム比較");
    
    private final String name;
    private Type(String name) {
      this.name = name;
    }
    @Override public String toString() {
      return name;
    }
  }
  
  /**
   * コンストラクタ
   */
  public FileInfoComparator(Type type) {
    this.type = type;
  }
  
  /**
   * 比較
   */
  @Override
  public int compare(FileInfo f1, FileInfo f2) {
    switch (type) {
    case JAVA: return collator.compare(f1.getName(), f2.getName());
    case UNICODE: return f1.getName().compareTo(f2.getName());
    case CUSTOM: return customCmp.compare(f1.getName(), f2.getName());
    default: throw new RuntimeException("??? " + type);
    }
  }
}
