package cvj2.file.fs;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.apache.commons.io.FilenameUtils;

import cvj2.img.ImageUtil;

/**
 * 実ファイルシステムに対するFileInfo
 */
public class FileInfoFs extends FileInfo {
  /**
   * コンストラクタ
   */
  FileInfoFs(File f) {
    super(f);
  }

  @Override
  public FileInfo getParent() {
    if (file.getParentFile() == null) return FileInfo.ROOT;
    // Windows で、"\\hostnane" が normalize() で null になるので、その対策
    if (FilenameUtils.normalize(file.getParentFile().getPath()) == null) return FileInfo.ROOT;
    // ROOTでなければ、必ず FileInfoFs
    return new FileInfoFs(file.getParentFile());
  }

  @Override
  public FileInfoList getList() {
    // isDirectory() が true でも、このメソッドが呼ばれたときにどうなっているかわからないので、チェックはしない。
    // パーミッションの無いディレクトリや、ディレクトリでない場合は null になるので変換する
    File[] list = file.listFiles();
    if (list == null) list = new File[0];
    return new FileInfoFsList(getFile(), list);
  }

  @Override
  protected void initAttribute() {
    if (isAttributeInitialized) return;
    
    exists = file.exists();
    isDirectory = file.isDirectory();
    length = isDirectory ? 0 : file.length();
    lastModified = file.lastModified();

    isAttributeInitialized = true;
  }

  @Override
  public FileInfo getArchiveRoot() {
    throw new RuntimeException(this.toString());
  }

  @Override
  public InputStream openStream() throws IOException {
    synchronized (FILE_IO_SYNC_OBJ) {
      return new BufferedInputStream(new FileInputStream(file));
    }
  }

  @Override
  public boolean isImageFile() {
    return ImageUtil.isImgFileName(getName());
  }
}
