package cvj2.file.fs;

import java.io.File;
import java.util.Arrays;

import cvj2.util.CacheMap;

/**
 * FileInfoFs のリスト
 */
public class FileInfoFsList extends FileInfoList {
  // ファイル数が多くなるとソートする時間がかかるのでキャッシュする
  static private class CacheElm {
    public CacheElm(File[] list, FileInfo[] sortedList) {
      this.list = list;
      this.sortedList = sortedList;
    }
    File[] list;
    FileInfo[] sortedList;
  }
  static private CacheMap<File, CacheElm> sortedCache = new CacheMap<File, FileInfoFsList.CacheElm>(16);
  
  /**
   * コンストラクタ
   */
  FileInfoFsList(File parent, File[] list) {
    super(create(parent, list));
  }
  
  private static FileInfo[] create(File parent, File[] list) {
    FileInfo[] fileList = new FileInfo[list.length];

    // キャッシュにあるかどうか調べる
    CacheElm cache = sortedCache.get(parent);
    if (cache != null && Arrays.equals(list, cache.list)) {
      for (int i = 0; i < list.length; i++) {
        fileList[i] = FileInfoFactory.createChild(cache.sortedList[i].getFile());
      }
      return fileList;
    }
    
    // キャッシュになかったので作成してソートする
    for (int i = 0; i < list.length; i++) {
      fileList[i] = FileInfoFactory.createChild(list[i]);
    }
    Arrays.sort(fileList);
    sortedCache.put(parent, new CacheElm(list, fileList));
    
    return fileList;
  }
}
