package cvj2.file.fs;

import java.util.AbstractList;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.regex.Pattern;

/**
 * FileInfo のリスト
 */
public abstract class FileInfoList extends AbstractList<FileInfo> {
  public static final FileInfoList NULL_LIST = new NullList();

  private static Pattern hideNameRegExps = null;
  private static String lastHideNameRegExp = new String("");

  public static String hideNameRegExp = null;
  public static boolean hideFiles = false;

  private final FileInfo[] noFilterList;
  private final FileInfo[] list;
  private final int hideCount;

  /**
   * コンストラクタ
   */
  protected FileInfoList(FileInfo[] list) {
    assert list != null;
    this.noFilterList = list;
    this.list = filter(list);
    this.hideCount = list.length - this.list.length;
    
    int newLimit = Math.max(FileInfoComparator.customCmp.getCacheLimit(), (int)(list.length * 1.5));
    FileInfoComparator.customCmp.setCacheLimit(newLimit);
  }
  
  /*
   * 非表示フィルタリング
   */
  private FileInfo[] filter(FileInfo[] list) {
    if (!hideFiles) return list;
    
    if (lastHideNameRegExp != hideNameRegExp) {
      lastHideNameRegExp = hideNameRegExp;
      hideNameRegExps = Pattern.compile(hideNameRegExp);
    }
    
    ArrayList<FileInfo> showList = new ArrayList<FileInfo>();
    for (FileInfo f : list) {
      boolean match = hideNameRegExps.matcher(f.getName()).matches();
      if (!match) showList.add(f);
    }
    return showList.toArray(new FileInfo[0]);
  }
  
  /**
   * 非表示を含めた全てを取得
   */
  public FileInfo[] getNoFilterList() {
    return noFilterList;
  }
  
  /**
   * 非表示になった数を取得
   */
  public int getHideCount() {
    return hideCount;
  }
  
  /**
   * リストサイズ
   */
  @Override
  public int size() {
    return list.length;
  }
  
  /**
   * 要素取得
   */
  @Override
  public FileInfo get(int i) {
    return list[i];
  }
  
  /**
   * 探す
   * @see java.util.Arrays#binarySearch(Object[], Object)
   */
  public int find(FileInfo f) {
    return Arrays.binarySearch(list, f);
  }
  
  // -----------------------------------------------------------------------------------------------------------
  /**
   * 空リストクラス
   */
  private static class NullList extends FileInfoList {
    private NullList() {
      super(new FileInfo[0]);
    }
  }
}
