package cvj2.file.fs;

import java.io.File;

/**
 * Zipファイルのルート。
 * 普通のディレクトリで、名前が "*.zip" の場合もこのオブジェクトが作成される。
 * 
 * Windows のネットワーク環境で、ディレクトリかファイルかを判別するのに時間がかかるので、
 * 名前が一致したら、どちらであろうともこのオブジェクトが生成されれる。
 * 
 * このクラスのインスタンスは、{@link FileInfoFactory#create(File)} などで簡易的に作成され、
 * リスト取得時に、{@link FileInfoZipManager#getFileInfoZipRoot(File)} で別のインスタンスが改めて作成される。
 */
public class FileInfoZipRoot extends FileInfoFs {
  // 子リスト
  FileInfoZipList zipList;
  
  /**
   * コンストラクタ
   */
  FileInfoZipRoot(File f) {
    super(f);
  }

  @Override
  public boolean isDirectory() {
    return true;
  }

  @Override
  public boolean isArchiveRoot() {
    // 普通のディレクトリの場合
    if (super.isDirectory()) {
      return false;
    }
    return true;
  }
  
  @Override
  public FileInfoList getList() {
    // 普通のディレクトリだった場合
    if (!isArchiveRoot()) {
      assert zipList == null;
      return super.getList();
    }
    
    if (zipList != null) return zipList;
    FileInfoZipRoot other = FileInfoZipManager.getFileInfoZipRoot(file);
    if (other != null) zipList = other.zipList;
    if (zipList == null) zipList = new FileInfoZipList(new FileInfoZip[0]);
    return zipList;
  }
}
