package cvj2.file.util;

import java.io.File;

/**
 * ファイル関連のユーティリティー
 */
public class FileUtil {
  private FileUtil() {}
  
  /**
   * 名前と拡張子に分ける
   * @return [名前, .拡張子]
   *    例) "/a/b/c.txt" => [ "c", ".txt" ]
   *        "/a/b/.xxx"  => [ ".xxx", "" ]
   *        "/a/b/aaa."  => [ "aaa." "" ]
   *    拡張子の長さが 0 でないときは、1文字目は必ず "." になる
   */
  static public String[] splitBasenameExt(File f) {
    return splitBasenameExt(f.getName());
  }
  /**
   * 名前と拡張子に分ける
   */
  static public String[] splitBasenameExt(String name) {
    int i = name.lastIndexOf('.');
    if (i < 0 || i == 0 || i == name.length() - 1) return new String[] { name, "" };
    return new String[] { name.substring(0, i), name.substring(i, name.length()) };
  }
  
  /**
   * 名前と拡張子を結合する
   * @param splitBasenameExt splitBasenameExt() で分割された文字列
   */
  static public String joinBasenameExt(String[] splitBasenameExt) {
    assert splitBasenameExt.length == 2 && splitBasenameExt[0] != null && splitBasenameExt[1] != null;
    return splitBasenameExt[0] + splitBasenameExt[1];
  }
  
  /**
   * ファイルシステム中に存在する一番近い親を取得する
   * @return 存在しない時は、null。引数のファイルかディレクトリが存在すれば、そのまま返す。
   */
  static public File getExisitParentInFs(File f) {
    while (f != null && !f.exists()) {
      f = f.getParentFile();
    }
    return f;
  }
  
  /**
   * 世代が離れた親子関係があるかどうかを調べる
   */
  static public boolean isAncestor(File parent, File child) {
    while (!parent.equals(child)) {
      child = child.getParentFile();
      if (child == null) return false;
    }
    return true;
  }
}
