package cvj2.file.util;

import java.io.Closeable;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.zip.ZipEntry;

import org.apache.commons.compress.archivers.zip.ZipArchiveEntry;

/**
 * java.util.zip.ZipFile のラッパークラス。<br>
 * 日本語のファイル名にも対応する
 */
abstract public class ZipFile implements Closeable {
  public static ZipFile newInstance(File file, String encoding) throws IOException {
    if (encoding.equals("UTF-8") || encoding.equals("UTF8")) {
      return new ZipFile_java(file);
    }
    else {
      return new ZipFile_apache(file, encoding);    
    }
  }
  public abstract ZipEntry getEntry(String name);
  public abstract Enumeration<? extends ZipEntry> entries();
  public abstract InputStream getInputStream(ZipEntry entry) throws IOException;
  @Override
  public abstract void close() throws IOException;
}

/**
 * ZipFile の java.util.zip による実装
 */
class ZipFile_java extends ZipFile {
  java.util.zip.ZipFile org;
  
  ZipFile_java(File file) throws IOException {
    org = new java.util.zip.ZipFile(file);
  }
  
  @Override
  public ZipEntry getEntry(String name) {
    return org.getEntry(name);
  }

  @Override
  public Enumeration<? extends ZipEntry> entries() {
    return org.entries();
  }

  @Override
  public InputStream getInputStream(ZipEntry entry) throws IOException {
    return org.getInputStream(entry);
  }

  @Override
  public void close() throws IOException {
    org.close();
  }
}

/**
 * ZipFile の java.util.zip による実装
 */
class ZipFile_apache extends ZipFile {
  org.apache.commons.compress.archivers.zip.ZipFile org;
  
  ZipFile_apache(File file, String encoding) throws IOException {
    org = new org.apache.commons.compress.archivers.zip.ZipFile(file, encoding);
  }
  
  @Override
  public ZipEntry getEntry(String name) {
    return org.getEntry(name);
  }

  @Override
  public Enumeration<ZipArchiveEntry> entries() {
    return org.getEntries();
  }

  @Override
  public InputStream getInputStream(ZipEntry entry) throws IOException {
    return org.getInputStream((ZipArchiveEntry) entry);
  }

  @Override
  public void close() throws IOException {
    org.close();
  }
}
