package cvj2.gui.swt.dnd;

import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.FileTransfer;
import org.eclipse.swt.dnd.HTMLTransfer;
import org.eclipse.swt.dnd.ImageTransfer;
import org.eclipse.swt.dnd.RTFTransfer;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.dnd.URLTransfer;
import org.eclipse.ui.part.EditorInputTransfer;
import org.eclipse.ui.part.PluginTransfer;

/**
 * Drag and Drop に関するユーティリティ
 */
public class DndUtil {
  static private Transfer[] allTransfers = {
    EditorInputTransfer.getInstance(),
    FileTransfer.getInstance(),
    HTMLTransfer.getInstance(),
    ImageTransfer.getInstance(),
    LocalSelectionTransfer.getTransfer(),
    PluginTransfer.getInstance(),
    RTFTransfer.getInstance(),
    TextTransfer.getInstance(),
    URLTransfer.getInstance(),
  };

  private DndUtil() {}
  
  /**
   * デバッグ用に使用できる Transfer を取得
   */
  public static Transfer[] getAvailableTranser() {
    return allTransfers;
  }
  
  /**
   * TransderData の配列を文字列にする (デバッグ用)
   */
  public static String toString(TransferData[] transferDatas) {
    StringBuilder sb = new StringBuilder();
    sb.append("{");
    for (TransferData d : transferDatas) {
      sb.append(toString(d));
      sb.append(", ");
    }
    sb.append("}");
    return sb.toString();
  }
  
  /**
   * TransderData の配列を文字列にする (デバッグ用)
   */
  public static String toString(TransferData transferData) {
    for (Transfer t : allTransfers) {
      if (!t.isSupportedType(transferData)) continue;
      String s = t.getClass().getSimpleName();
      int index = s.lastIndexOf("Transfer");
      if (index < 0) return s;
      return s.substring(0, index);
    }
    return "???";
  }
  
  /**
   * DragSource, DropTarget の style を文字列にする (デバッグ用)
   * @param style DND.DROP_COPY, DND.DROP_MOVE, DND.DROP_LINK の or 
   */
  public static String getDndStyleString(int style) {
    if (style == 0) return "NONE";
    StringBuilder sb = new StringBuilder();
    if ((style & DND.DROP_COPY) != 0) sb.append("COPY|");
    if ((style & DND.DROP_MOVE) != 0) sb.append("MOVE|");
    if ((style & DND.DROP_LINK) != 0) sb.append("LINK|");
    assert sb.length() > 0;
    return sb.substring(0, sb.length() - 1);
  }
}
