package cvj2.gui.swt.dnd;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.IOException;

import org.eclipse.swt.dnd.ByteArrayTransfer;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.TransferData;

import cvj2.util.Log;

/**
 * 移動とコピーの区別のある FileTransfer
 */
public class MyFileTransfer extends ByteArrayTransfer {
  private static final String MYTYPENAME = "cbj2_my_file_transfer";
  private static final int MYTYPEID = registerType(MYTYPENAME);
  private static MyFileTransfer _instance = new MyFileTransfer();

  public static MyFileTransfer getInstance() {
    return _instance;
  }

  public void javaToNative(Object object, TransferData transferData) {
    if (!checkMyType(object) || !isSupportedType(transferData)) {
      DND.error(DND.ERROR_INVALID_DATA);
    }
    
    MyFileTransferData myData = (MyFileTransferData) object;
    try {
      ByteArrayOutputStream byteStream = new ByteArrayOutputStream();
      DataOutputStream out = new DataOutputStream(byteStream);
      out.writeInt(myData.isCopy ? 1 : 0);
      for (String s : myData.files) {
        byte[] buff = s.getBytes();
        out.writeInt(buff.length);
        out.write(buff);
      }
      byte[] buffer = byteStream.toByteArray();
      out.close();
      super.javaToNative(buffer, transferData);
    }
    catch (IOException e) {
      /* とくになにもしない SWTのソースでもなにもしていない */
      Log.e(e);
    }
  }
  
  public Object nativeToJava(TransferData transferData) {
    if (!isSupportedType(transferData)) return null;

    byte[] buffer = (byte[]) super.nativeToJava(transferData);
    if (buffer == null) return null;

    MyFileTransferData myData = new MyFileTransferData();
    try {
      ByteArrayInputStream byteStream = new ByteArrayInputStream(buffer);
      DataInputStream in = new DataInputStream(byteStream);
      myData.isCopy = in.readInt() != 0;
      while (in.available() > 4) {
        int size = in.readInt();
        if (size > 1024 * 64) throw new IOException("too long filename: " + size);
        byte[] name = new byte[size];
        int readBytes = in.read(name);
        if (readBytes != size) throw new IOException("size = " + size + ", read len = " + readBytes);
        myData.files.add(new String(name));
      }
      in.close();
    }
    catch (IOException e) {
      Log.e(e);
      return null;
    }
    return myData;
  }
  
  private boolean checkMyType(Object object) {
    if (object == null) return false;
    if (object.getClass() != MyFileTransferData.class) return false;
    MyFileTransferData data = (MyFileTransferData) object;
    if (data.files == null || data.files.size() == 0) return false;
    for (String s : data.files) {
      if (s == null) return false;
    }
    return true;
  }

  @Override
  protected String[] getTypeNames() {
    return new String[] { MYTYPENAME };
  }

  @Override
  protected int[] getTypeIds() {
    return new int[] { MYTYPEID };
  }
}
