package cvj2.gui.swt.ui.input;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.action.LegacyActionTools;
import org.eclipse.swt.SWT;

import cvj2.util.Log;

/**
 * キーコードや、マスク値を文字列にするユーティリティークラス
 */
public class KeyCodeUtil {
  static private Map<Integer, String> keyCodeMap;
  
  // 初期化
  static {
    keyCodeMap = new HashMap<Integer, String>();
    // char の範囲だけと特殊キーの場合
    init(SWT.DEL, "Delete");
    init(SWT.ESC, "Esc");
    init(SWT.TAB, "Tab");
    init(SWT.BS, "Backspace");
    init('\r', "Enter");
    init(' ', "Space");

    init(SWT.SCROLL_LOCK, "Scroll Lock");
    init(SWT.PAUSE, "Pause");
    init(SWT.BREAK, "Break");
    init(SWT.PRINT_SCREEN, "Print Screen");

    init(SWT.CONTROL, "Control");
    init(SWT.SHIFT, "Shift");
    init(SWT.ALT, "Alt");
    init(SWT.COMMAND, "Command");
    
    init(SWT.CAPS_LOCK, "Caps Lock");
    init(SWT.NUM_LOCK, "Num Lock");

    init(SWT.KEYPAD_0, "Keypad 0");
    init(SWT.KEYPAD_1, "Keypad 1");
    init(SWT.KEYPAD_2, "Keypad 2");
    init(SWT.KEYPAD_3, "Keypad 3");
    init(SWT.KEYPAD_4, "Keypad 4");
    init(SWT.KEYPAD_5, "Keypad 5");
    init(SWT.KEYPAD_6, "Keypad 6");
    init(SWT.KEYPAD_7, "Keypad 7");
    init(SWT.KEYPAD_8, "Keypad 8");
    init(SWT.KEYPAD_9, "Keypad 9");
    init(SWT.KEYPAD_CR, "Keypad Enter");
    init(SWT.KEYPAD_ADD, "Keypad +");
    init(SWT.KEYPAD_SUBTRACT, "Keypad -");
    init(SWT.KEYPAD_MULTIPLY, "Keypad *");
    init(SWT.KEYPAD_DIVIDE, "Keypad /");
    init(SWT.KEYPAD_DECIMAL, "Keypad .");
    init(SWT.KEYPAD_EQUAL, "Keypad =");
    
    init(SWT.ARROW_UP, "Arrow Up");
    init(SWT.ARROW_DOWN, "Arrow Down");
    init(SWT.ARROW_LEFT, "Arrow Left");
    init(SWT.ARROW_RIGHT, "Arrow Right");
    
    init(SWT.PAGE_UP, "Page Up");
    init(SWT.PAGE_DOWN, "Page Down");
    init(SWT.HOME, "Home");
    init(SWT.END, "End");
    init(SWT.INSERT, "Insert");
    
    init(SWT.F1, "F1");
    init(SWT.F2, "F2");
    init(SWT.F3, "F3");
    init(SWT.F4, "F4");
    init(SWT.F5, "F5");
    init(SWT.F6, "F6");
    init(SWT.F7, "F7");
    init(SWT.F8, "F8");
    init(SWT.F9, "F9");
    init(SWT.F10, "F10");
    init(SWT.F11, "F11");
    init(SWT.F12, "F12");
    init(SWT.F13, "F13");
    init(SWT.F14, "F14");
    init(SWT.F15, "F15");
    init(SWT.F16, "F16");
    init(SWT.F17, "F17");
    init(SWT.F18, "F18");
    init(SWT.F19, "F19");
    init(SWT.F20, "F20");
    
    init(SWT.HELP, "Help");
  }
  static private void init(int c, String s) { keyCodeMap.put(c, s); }

  private KeyCodeUtil() {}
  
  /**
   * キー入力を文字列に変換する
   * @return "CTRL+F1"、"CTRL+SHIFT+End" など
   */
  static public String getKeyCodeAndStateMaskString(int keyCodeAndStateMask) {
    int keyCode = keyCodeAndStateMask & ~SWT.MODIFIER_MASK;
    int stateMask = keyCodeAndStateMask & SWT.MODIFIER_MASK;
 
    String s = getStateMaskString(stateMask);
    if (keyCode == 0) {
      assert s.length() > 0;
      return s;
    }
    if (s.length() != 0) {
      return s + "+" + getKeyCodeString(keyCode);
    }
    return getKeyCodeString(keyCode);
  }

  /**
   * 表示できない文字をエスケープする
   */
  static public String escapeChar(char c) {
    if (Character.isISOControl(c)) {
      switch (c) {
      case '\n': return "\\n";
      case '\r': return "\\r";
      case '\t': return "\\t";
      default: return String.format("\\u%02x", (int)c);
      }
    }
    return "" + c;
  }
  
  /**
   * キーコードを文字列に変換する
   * @return "a", "Enter", "Tab" など
   */
  static public String getKeyCodeString(int keyCode) {
    String s = keyCodeMap.get(keyCode);
    if (s != null) return s;

    // char の範囲なら、文字をそのまま返す
    if ((keyCode & SWT.KEYCODE_BIT) == 0) {
      if (keyCode > Character.MAX_VALUE) Log.wf("invalid keycode_bit? 0x%08x", keyCode);
      return escapeChar((char)keyCode); 
    }
    
    Log.wf("unknown keycode? 0x%08x", keyCode);
    s = LegacyActionTools.findKeyString(keyCode);
    if (s == null || s.charAt(0) == '\0') return "";
    return s;
  }

  /**
   * キーボードの ctrl, alt, shift の状態を文字列に変換する
   * @return "CTRL", "CTRL+SHIFT", "CTRL+SHIFT+ALT" など
   */
  static public String getStateMaskString(int stateMask) {
    if (stateMask == 0) return "";
    
    // MASK 以外のビットがあるかどうか調べる
    if ((stateMask & SWT.MODIFIER_MASK) == 0 && (stateMask & SWT.BUTTON_MASK) == 0) {
      assert false;
      return String.format("0x%08x", stateMask);
    }
    
    StringBuilder sb = new StringBuilder();
    if ((stateMask & SWT.CONTROL) != 0) sb.append("CTRL+");
    if ((stateMask & SWT.SHIFT) != 0) sb.append("SHIFT+");
    if ((stateMask & SWT.ALT) != 0) sb.append("ALT+");
    if ((stateMask & SWT.COMMAND) != 0) sb.append("COMMAND+");
    
    if ((stateMask & SWT.BUTTON1) != 0) sb.append("BUTTON1+");
    if ((stateMask & SWT.BUTTON2) != 0) sb.append("BUTTON2+");
    if ((stateMask & SWT.BUTTON3) != 0) sb.append("BUTTON3+");
    if ((stateMask & SWT.BUTTON4) != 0) sb.append("BUTTON4+");
    if ((stateMask & SWT.BUTTON5) != 0) sb.append("BUTTON5+");
    
    assert sb.length() > 0;
    sb.deleteCharAt(sb.length() - 1);
    return sb.toString();
  }
}
