package cvj2.gui.swt.util;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.widgets.Display;

/**
 * Fontの管理。<br>
 * このクラスを使用したら、アプリケーション終了前に dispose() を呼ぶこと
 */
public class FontResourceManager {
  static private FontResourceManager instance = new FontResourceManager();
  private Map<String, Font> fontMap = new HashMap<String, Font>();

  private FontResourceManager() { }
  
  /**
   * インスタンス取得
   */
  static public FontResourceManager instance() {
    return instance;
  }
  
  /**
   * FontData を文字列に
   */
  public static String[] getFontDataStrings(FontData[] fontDatas) {
    String[] fontDataStrings = new String[fontDatas.length];
    for (int i = 0; i < fontDatas.length; i++) {
      fontDataStrings[i] = fontDatas[i].toString();
    }
    return fontDataStrings;
  }
  
  /**
   * 文字列をFontDataに
   */
  public static FontData[] getFontDatas(String[] fontDataStrings) {
    FontData[] fds = new FontData[fontDataStrings.length];
    for (int i = 0; i < fds.length; i++) {
      fds[i] = new FontData(fontDataStrings[i]);
    }
    return fds;
  }
  
  /**
   * 文字列から Fontへ
   */
  public Font getFont(String[] fontDataStrings) {
    // 既に生成済みかどうか調べる
    String key = Arrays.deepToString(fontDataStrings);
    Font font = fontMap.get(key);
    if (font != null) return font;
    
    // フォントを生成する
    FontData[] fds = new FontData[fontDataStrings.length];
    for (int i = 0; i < fds.length; i++) {
      fds[i] = new FontData(fontDataStrings[i]);
    }
    font = new Font(Display.getDefault(), fds);
    fontMap.put(key, font);
    
    return font;
  }
  
  /**
   * リソースを破棄する
   */
  public void dispose() {
    for (String key : fontMap.keySet()) {
      fontMap.get(key).dispose();
    }
    fontMap.clear();
  }
}
