package cvj2.gui.swt.util;

import org.eclipse.swt.graphics.FontMetrics;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

import cvj2.util.DelimString;
import cvj2.util.LateBindString;

/**
 * SWT関連のユーティリティー
 */
public class SWTUtil {
  private SWTUtil() {}
  
  /**
   * Rectangle をデバッグ文字列に
   */
  static public String toString(Rectangle r) {
    if (r == null) return "" + r;
    return "{" + r.x + "," + r.y + "," + r.width + "," + r.height + "}";
  }
  
  /**
   * Rectangle の clone()
   */
  static public Rectangle clone(Rectangle r) {
    return new Rectangle(r.x, r.y, r.width, r.height);
  }
  
  /**
   * Point の clone()
   */
  static public Point clone(Point p) {
    return new Point(p.x, p.y);
  }
  
  /**
   * FontMetrics のデバッグ文字列
   */
  static public String toString(FontMetrics fm) {
    return "w" + fm.getAverageCharWidth() + "|a" + fm.getAscent() + "|d" + fm.getDescent() + "|h" + fm.getHeight() + "|l" + fm.getLeading();
  }
  
  /**
   * SWTコントロールに表示する文字列のエスケープ
   */
  static public String escapeControlText(String s) {
    return s.replaceAll("[&]", "\\&\\&");
  }
  
  /**
   * ImageData のデバッグ文字列
   */
  static public LateBindString dump(final ImageData id) {
    return new LateBindString() {
      @Override
      public String create() {
        DelimString s = new DelimString(", ", "=");
        s.append("width", id.width);
        s.append("height", id.height);
        s.append("depth", id.depth);
        s.append("transparentPixel", id.transparentPixel);
        s.append("p.isDirect", id.palette.isDirect);
        return s.toString();
      }
    };
  }
}
