package cvj2.gui.swt.widgets;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;

import cvj2.util.Log;

/**
 * ダブルバッファリングを行うCanvas。<br>
 * Windowsの場合はこれをしないと画面がチラチラする
 */
public class DoubleBufferedCanvas extends Canvas {
  private List<PaintListener> listeners = new ArrayList<PaintListener>();
  private Image buff;
  private boolean noBackground;

  /**
   * コンストラクタ
   */
  public DoubleBufferedCanvas(Composite parent, int style) {
    super(parent, style | SWT.NO_BACKGROUND);
    super.addPaintListener(new LocalPaintListener());
    super.addControlListener(new LocalControlListener());
    noBackground = (style & SWT.NO_BACKGROUND) != 0; 
  }

  private class LocalPaintListener implements PaintListener {
    @Override
    public void paintControl(PaintEvent e) {
      GC orgGc = e.gc;
      GC gc = new GC(buff);
      e.gc = gc;
      gc.setClipping(orgGc.getClipping());
      Error error = null;
      RuntimeException runtimeEx = null;
      gc.setBackground(orgGc.getBackground());
      gc.setForeground(orgGc.getForeground());
      gc.setFont(orgGc.getFont());
      if (!noBackground) gc.fillRectangle(buff.getBounds());
      
      for (PaintListener listener : listeners) {
        try {
          listener.paintControl(e);
        }
        catch (Error ex) {
          Log.e(ex);
          error = ex;
        }
        catch (RuntimeException ex) {
          Log.e(ex);
          runtimeEx = ex;
        }
      }
      orgGc.drawImage(buff, 0, 0);
      e.gc = orgGc;
      gc.dispose();
      
      if (error != null) throw error;
      if (runtimeEx != null) throw runtimeEx;
    }
  }
  
  private class LocalControlListener extends ControlAdapter {
    @Override
    public void controlResized(ControlEvent e) {
      if (buff != null) {
        buff.dispose();
      }
      Point size = getSize();
      buff = new Image(Display.getCurrent(), Math.max(1, size.x), Math.max(1, size.y));
    }
  }
  
  @Override
  public void addPaintListener(PaintListener listener) {
    listeners.add(listener);
  }

  @Override
  public void removePaintListener(PaintListener listener) {
    listeners.remove(listener);    
  }
  
  @Override
  public void dispose() {
    super.dispose();
    if (buff != null && !buff.isDisposed()) buff.dispose();
  }
}
