package cvj2.gui.swt.widgets.gdidpanel;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.PaintEvent;
import org.eclipse.swt.events.PaintListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.wb.swt.SWTResourceManager;

import cvj2.gui.swt.util.GCX;
import cvj2.gui.swt.widgets.WidgetPaintUtil;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel.ColumnPainter;
import cvj2.gui.swt.widgets.gdidpanel.GridPanel.ColumnSelectionListener;
import cvj2.util.Log;

/**
 * 列ヘッダー (リスト表示の時のみ表示する)
 */
public class GridListHeader extends Canvas {
  private ColumnPainter columnData;
  private ColumnSelectionListener listener;
  private Point mouseDownPt = null;
  private Rectangle[] columnRects;
  
  /**
   * コンストラクタ
   */
  public GridListHeader(Composite parent, int style) {
    super(parent, style);

    addPaintListener(new PaintHandler());
    addMouseListener(new MouseHandler());
    addControlListener(new ControlHandler());
  }
  
  /**
   * データセット
   */
  public void setColumnData(ColumnPainter columnData) {
    this.columnData = columnData;
  }
  
  /**
   * 列選択リスナーの登録
   */
  public void setColumnSelectionListener(ColumnSelectionListener listener) {
    this.listener = listener;
  }

  @Override
  public Point computeSize(int wHint, int hHint, boolean changed) {
    if (columnData == null) return new Point(getClientArea().width, 0);
    return new Point(getClientArea().width, columnData.getColumnHeight(this));
  }

  /*
   * クリックされた列インデックスを取得
   */
  private int getClickedColumn(int x, int y) {
    for (int i = 0; i < columnRects.length; i++) {
      if (columnRects[i].contains(x, y)) return i;
    }
    return -1;
  }
  
  /*
   * 描画
   */
  private class PaintHandler implements PaintListener {
    @Override
    public void paintControl(PaintEvent e) {
      if (columnData == null) return;
      GCX gcx = new GCX(e.gc);
      int clickColumn = -1;
      if (mouseDownPt != null) clickColumn = getClickedColumn(mouseDownPt.x, mouseDownPt.y);
      
      for (int i = 0; i < columnRects.length; i++) {
        Rectangle rect = columnRects[i];
        Object data = columnData.getColumnData(i);
        if (columnData.paintControl(e.gc, rect, data, i, clickColumn == i)) continue;
        // デフォルト描画
        WidgetPaintUtil.drawButtonBg(gcx, rect.x, rect.y, rect.width, rect.height, clickColumn == i);
        gcx.setBackground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_BACKGROUND));
        gcx.setForeground(SWTResourceManager.getColor(SWT.COLOR_WIDGET_FOREGROUND));
        gcx.drawString("" + data, rect.x + 5, rect.y + 2);
        WidgetPaintUtil.drawButtonFg(gcx, rect.x, rect.y, rect.width, rect.height, clickColumn == i);
      }
    }
  }
  
  /*
   * マウスハンドラー
   */
  private class MouseHandler extends MouseAdapter {
    @Override
    public void mouseDown(MouseEvent e) {
      setCapture(true);
      mouseDownPt = new Point(e.x, e.y);
      redraw();
    }

    @Override
    public void mouseUp(MouseEvent e) {
      setCapture(false);
      if (mouseDownPt == null) return;
      int column0 = getClickedColumn(mouseDownPt.x, mouseDownPt.y);
      int column1 = getClickedColumn(e.x, e.y);
      if (column0 == column1 && listener != null) listener.columnSelected(column0);
      mouseDownPt = null;
      redraw();
    }
  }

  /*
   * サイズ変更
   */
  private class ControlHandler extends ControlAdapter {
    @Override
    public void controlResized(ControlEvent e) {
      if (columnData == null) return;
      Rectangle rect = getClientArea();
      int[] widths = columnData.getColumnWidths(rect.width);
      columnRects = new Rectangle[widths.length];
      Log.d(widths);
      int x = 0;
      for (int i = 0; i < widths.length; i++) {
        columnRects[i] = new Rectangle(rect.x + x, rect.y, widths[i], rect.height);
        x += widths[i];
      }
    }
  }
}
