package cvj2.img;

import java.awt.Dimension;

/**
 * ズーム状態
 */
public class ZoomLevel implements Cloneable {
  static public double zoomInRatio = 2.0;
  static public double zoomOutRatio = 2.0;
  private int level = 0;

  /**
   * ズームした画像サイズを取得する
   */
  public Dimension cnvSize(Dimension size) {
    double ratio = getRatio();
    return new Dimension((int)(size.width * ratio), (int)(size.height * ratio));
  }
  
  /**
   * ズームレベルをリセットする
   */
  public void reset() {
    level = 0;
  }
  
  /**
   * ズームレベルを取得する
   * @return 0: 原寸、負: 縮小、正: 拡大
   */
  public int getLevel() {
    return level;
  }
  
  /**
   * 拡大縮小率を取得する
   */
  public double getRatio() {
    double ratio = 1.0;
    if (level > 0) {
      // 拡大
      for (int i = 0; i < level; i++) {
        ratio *= zoomInRatio;
      }
    }
    else {
      // 縮小
      for (int i = 0; i > level; i--) {
        ratio /= zoomOutRatio;
      }
    }
    return ratio;
  }

  /**
   * 拡大
   */
  public void zoomIn() {
    level++;
  }
  
  /**
   * 縮小
   */
  public void zoomOut() {
    level--;
  }

  @Override
  public String toString() {
    return "ZoomLevel[" + level + "]";
  }

  @Override
  public ZoomLevel clone() {
    try {
      return (ZoomLevel) super.clone();
    }
    catch (CloneNotSupportedException e) {
      throw new RuntimeException(e);
    }
  }
}
