package cvj2.img.converter;

import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;

import cvj2.img.ImageRotate;

/**
 * 回転
 */
public class ImageConverterRotate implements ImageConverter {
  private ImageRotate rotate;
  
  public ImageConverterRotate(ImageRotate rotate) {
    this.rotate = rotate;
  }

  @Override
  public BufferedImage cnv(BufferedImage img) {
    if (rotate.getRotate() == 0) return img;

    // 移動 (180度)
    int tranX = 0;
    int tranY = 0;
    // 回転中心 (180度)
    int centerX = img.getWidth() / 2;
    int centerY = img.getHeight() / 2;

    if (rotate.getRotate() == 1) {
      // (90度)
      if (img.getWidth() > img.getHeight()) {
        // 横長
        tranX = -(img.getWidth() - img.getHeight());
        centerX = centerY = img.getHeight() / 2;
      }
      else {
        // 縦長
        centerX = centerY = img.getWidth() / 2;
      }
    }
    else if (rotate.getRotate() == 3) {
      // (270度)
      if (img.getWidth() > img.getHeight()) {
        // 横長
        centerX = centerY = img.getHeight() / 2;
      }
      else {
        // 縦長
        tranY = -(img.getHeight() - img.getWidth());
        centerX = centerY = img.getWidth() / 2;
      }
    }

    AffineTransform rotateAF = AffineTransform.getRotateInstance(-Math.toRadians(rotate.getRotate() * 90), centerX, centerY);
    AffineTransform at = rotateAF;
    if (tranX != 0 || tranY != 0) {
      at = AffineTransform.getTranslateInstance(tranX, tranY);
      at.preConcatenate(rotateAF);
    }
    AffineTransformOp aop = new AffineTransformOp(at, AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
    return aop.filter(img, null);
  }

}
