package cvj2.img.converter;

import java.awt.Dimension;
import java.awt.image.BufferedImage;

import cvj2.img.ImageUtil;

/**
 * サイズ変換の抽象クラス
 */
public abstract class ImageSizeConverter implements ImageConverter {
  private Dimension displaySize;
  private Double zoomRatio;
  
  public ImageSizeConverter(Dimension displaySize) {
    this.displaySize = displaySize;
  }

  public ImageSizeConverter(Double zoomRatio) {
    this.zoomRatio = zoomRatio;
  }

  public double getZoomRatio(BufferedImage img) {
    if (displaySize != null) {
      Dimension size = getResultSize(img);
      return (((double)size.width) / img.getWidth() + ((double)size.height) / img.getHeight()) / 2.0;
    }
    else {
      return zoomRatio;
    }
  }
  
  public Dimension getResultSize(BufferedImage img) {
    Dimension size;
    if (displaySize != null) {
      size = ImageUtil.calcImageFitSize(img.getWidth(), img.getHeight(), displaySize.width, displaySize.height);
    }
    else {
      size = new Dimension((int)(img.getWidth() * zoomRatio), (int)(img.getHeight() * zoomRatio));
    }
    if (size.width <= 0) size.width = 1;
    if (size.height <= 0) size.height = 1;
    return size;
  }
}
