package cvj2.img.loader;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.IOException;

import cvj2.file.fs.FileInfo;
import cvj2.img.ImageInfo;

/**
 * 2つのファイルをロードする
 */
public class ImageLoader2 {
  public static Color backgroundColor = null;
  public static int type;
  
  private ImageInfo[] imageInfos = { null, null };
  
  /**
   * 画像ロード
   * PDF ファイルを読む場合があるので、dpi か targetSize のどちらかを指定しなければならない
   * 
   * @param dpi null でないときは この解像度で画像を生成する
   * @param targetSize null でないときは このサイズで画像を生成する
   */
  public BufferedImage load(FileInfo file1, FileInfo file2, Integer dpi, Dimension targetSize) throws IOException {
    assert dpi != null || targetSize != null;
    assert file1 != null;
    
    ImageLoader loader = new ImageLoader();
    if (file2 == null) {
      BufferedImage img = loader.load(file1, dpi, targetSize);
      imageInfos[0] = new ImageInfo(img);
      return img;
    }
    
    BufferedImage img1 = loader.load(file1, dpi, targetSize);
    BufferedImage img2 = loader.load(file2, dpi, targetSize);
    int width = img1.getWidth() + img2.getWidth();
    int height = Math.max(img1.getHeight(), img2.getHeight());
    BufferedImage img = new BufferedImage(width, height, type);
    Graphics2D g2 = (Graphics2D) img.getGraphics();
    if (backgroundColor != null) {
      g2.setColor(backgroundColor);
      g2.fillRect(0, 0, width, height);
    }
    g2.drawImage(img2, null, 0, (height - img2.getHeight()) / 2);
    g2.drawImage(img1, null, img2.getWidth(), (height - img1.getHeight()) / 2);
    g2.dispose();
    imageInfos[0] = new ImageInfo(img1);
    imageInfos[1] = new ImageInfo(img2);
    return img;
  }
  
  /**
   * 読み込んだ個別の画像情報を取得
   */
  public ImageInfo[] getImageInfos() {
    return imageInfos;
  }
}
