package cvj2.img.loader;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collections;
import java.util.Set;
import java.util.TreeSet;

/**
 * 画像ロードプラグインの抽象クラス
 */
public abstract class ImageLoaderPlugin {
  private String name;
  private Set<String> extSet = new TreeSet<String>();

  /**
   * コンストラクタ
   */
  protected ImageLoaderPlugin(String name) {
    this.name = name;
  }

  /**
   * 名前
   */
  public String getName() {
    return name;
  }
  
  /**
   * サポートする拡張子を取得
   */
  public Set<String> getSupportExt() {
    return Collections.unmodifiableSet(extSet);
  }
  /**
   * 拡張子追加
   */
  protected void addExt(String ext) {
    assert !extSet.contains(ext);
    assert ext.toLowerCase().equals(ext);
    extSet.add(ext);
  }

  /**
   * ロードする
   * @return null は返らない
   */
  public abstract BufferedImage load(InputStream file) throws IOException;

  @Override
  public boolean equals(Object obj) {
    if (obj == null) return false;
    if (!(obj instanceof ImageLoaderPlugin)) return false;
    return getName().equals(((ImageLoaderPlugin)obj).getName());
  }
}
