package cvj2.img.loader;

import java.awt.Component;
import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.io.InputStream;

import cvj2.util.Util;

/**
 * java.awt.Toolkit.createImage() を使用した画像ロードプラグイン
 */
public class ImageLoaderPluginAWTToolkit extends ImageLoaderPlugin {
  public static int outputBufferedImageType = BufferedImage.TYPE_INT_ARGB;
  
  {
    addExt("jpg");
    addExt("jpeg");
    addExt("gif");
    addExt("png");
  }

  public ImageLoaderPluginAWTToolkit() {
    super("AWTToolkit");
  }
  
  @Override
  public BufferedImage load(InputStream in) throws IOException {
    Image i = Toolkit.getDefaultToolkit().createImage(Util.toByteArray(in));
    Component comp = new Container();
    MediaTracker mt = new MediaTracker(comp);
    mt.addImage(i, 0);
    try {
      mt.waitForAll();
    }
    catch (InterruptedException e) {
      throw new IOException(e);
    }
    int width =i.getWidth(comp);
    int height = i.getHeight(comp);
    if (width <= 0 || height <= 0) throw new IOException("width = " + width + ", height = " + height);
    BufferedImage bi = new BufferedImage(width, height, outputBufferedImageType);
    Graphics2D g2 = bi.createGraphics();
    g2.drawImage(i, 0, 0, comp);
    g2.dispose();
    return bi;
  }
}
