package cvj2.migemo;


/**
 * Migemo のキー入力イベントを処理する。<br>
 * (キーボードから途切れなく入力された文字を1つの文字列として認識する)
 */
public class MigemoKeyHandler {
  // キー入力のタイムアウト
  private static int inputTimeoutMilli = 1500;

  // 入力された文字列
  private StringBuilder inputStr = new StringBuilder();
  // 最終文字入力時刻 (この時刻からタイムアウト以内なら、一連の文字列として認識する)
  private long lastInput = 0;

  /**
   * コンストラクタ
   */
  public MigemoKeyHandler() { }

  /**
   * 入力文字の判定
   */
  public void handleKeyChar(char c, long when) {
    // タイムアウトしたら、新しい文字列になる
    if (lastInput + inputTimeoutMilli < when) {
      inputStr = new StringBuilder();
    }
    
    lastInput = when;
    inputStr.append(c);
  }
  
  /**
   * 一連の文字入力を完了する
   */
  public void cancelKeySequence() {
    lastInput = 0;
  }
  
  /**
   * 入力された文字列
   */
  public String getInputStr() {
    return inputStr.toString();
  }
  
  /**
   * タイムアウト設定
   */
  public static void setInputTimeoutMilli(int timeout) {
    inputTimeoutMilli = timeout;
  }
}
