package cvj2.migemo;

import java.io.File;
import java.io.IOException;
import java.lang.Character.UnicodeBlock;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import org.apache.commons.io.FileUtils;

import cvj2.util.CnvHira2Kata;
import cvj2.util.Log;

/**
 * Migemo 辞書を流用して、[漢字]->[カタカナ読み] の変換辞書を作成する
 */
public class MigemoReverseDict {
  private static MigemoReverseDict instance = new MigemoReverseDict();
  private HashMap<String, HashSet<String>> dict;
  
  private MigemoReverseDict() {}
  
  /**
   * インスタンス取得
   */
  public static MigemoReverseDict instance() {
    return instance;
  }
  
  /**
   * 漢字から読みを探す
   */
  public Set<String> find(String kanji) {
    assert dict != null;
    if (!dict.containsKey(kanji)) return null;
    return Collections.unmodifiableSet(dict.get(kanji));
  }

  /**
   * Migemo辞書ファイルを読み込む
   */
  public void initialize() throws IOException {
    Log.start();
    
    assert dict == null;
    dict = new HashMap<String, HashSet<String>>();
    load(new File(System.getProperty("user.dir"), "lib/migemo-dict"));
    load(new File(System.getProperty("user.dir"), "lib/migemo-dict-add"));
    add("～", "ー");
  }

  /*
   * 辞書ファイルを１つ読む
   */
  private void load(File file) throws IOException {
    Log.start(file);
    
    for (String line : FileUtils.readLines(file, "JISAutoDetect")) {
      line = line.trim();
      if (line.length() == 0) continue;
      if (line.startsWith(";")) continue;
      String[] spl = line.split("\t");
      if (spl.length < 2) continue;
      for (int i = 1; i < spl.length; i++) {
        add(spl[i], spl[0]);
      }
    }
  }

  /*
   * 単語登録
   */
  private void add(String kanji, String yomi) {
    yomi = CnvHira2Kata.cnv(yomi);
//    if (isAllKanji(kanji) && isAllKatakana(yomi)) {
    if (isAllKatakana(yomi)) {
      HashSet<String> list = dict.get(kanji);
      if (list == null) {
        list = new HashSet<String>();
        dict.put(kanji, list);
      }
      list.add(yomi);
//      Log.d("o " + kanji + "\t" + yomi);
    }
    else {
//      Log.d("x " + kanji + "\t" + yomi);
    }
  }
  
//  /*
//   * 文字列が全て漢字か
//   */
//  private boolean isAllKanji(String s) {
//    for (int i = 0; i < s.length(); i++) {
//      if (UnicodeBlock.of(s.charAt(i)) != UnicodeBlock.CJK_UNIFIED_IDEOGRAPHS) return false;
//    }
//    return true;
//  }
  
  /*
   * 文字列が全てカタカナか
   */
  private boolean isAllKatakana(String s) {
    for (int i = 0; i < s.length(); i++) {
      if (UnicodeBlock.of(s.charAt(i)) != UnicodeBlock.KATAKANA) return false;
    }
    return true;
  }
}
