package cvj2.migemo;

import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import cvj2.migemo.ListSearch.StringConverter;
import cvj2.util.Log;

/**
 * バックグラウンドでのSen解析
 * @author yos
 *
 */
public class SenBackgroundAnalyzer {
  private BlockingQueue<TaskElement> queue = new LinkedBlockingQueue<TaskElement>();
  private BgThread bgThread;
  private Sen sen = new Sen(false);
  
  /**
   * コンストラクタ
   */
  public SenBackgroundAnalyzer() {
    Log.ut_start();
    bgThread = new BgThread();
    bgThread.setDaemon(true);
    bgThread.setPriority(Thread.NORM_PRIORITY - 1);
    bgThread.setName("SenAnalyze");
    bgThread.start();
  };
  
  /**
   * 解析依頼
   */
  public <E> void analyze(List<E> list, ListSearch.StringConverter<E> cnv) {
    Log.ut_start("list.size = ", list.size());
    
    Sen.updateAnalyzeCacheSize(list.size());
    try {
      queue.put(new TaskElement(list, cnv));
    }
    catch (InterruptedException e) {
      // 起こらないはず
      Log.e(e);
      throw new RuntimeException(e);
    }
  }
  
  /**
   * 解析依頼のデータ
   */
  @SuppressWarnings("rawtypes")
  private class TaskElement {
    List list;
    ListSearch.StringConverter cnv;
    public TaskElement(List list, StringConverter cnv) {
      this.list = list;
      this.cnv = cnv;
    }
  }

  /**
   * 解析スレッド
   */
  private class BgThread extends Thread {
    @SuppressWarnings("unchecked")
    @Override
    public void run() {
      Log.ut_start();
      while (true) {
        try {
          TaskElement elm = queue.take();
          Log.ut("analyze start: ", elm.list.size());

          for (int i = 0; i < elm.list.size(); i++) {
            // リストの要素を解析してキャッシュする
            sen.analyze(elm.cnv.cnv(elm.list.get(i)));
            
            // 新しい仕事が来たら、中断する
            if (queue.size() > 0) break;
          }
          Log.ut("analyze end: cache.size = ", Sen.analyzeCache.getLimit());
        }
        catch (InterruptedException e) {
          // 起こらないはず
          throw new RuntimeException(e);
        }
      }
    }
  }
}
