package cvj2.util;

import java.util.HashMap;

import org.apache.commons.lang3.CharUtils;
import org.apache.commons.lang3.StringUtils;

/**
 * ローマ字カタカナ変換
 */
public class CnvRomaji2Kata {
  static boolean ignoreTrailingAlphabet = true;
  static boolean ignoreTrailingAlphabetRmN = true;
  
  static private int MAX_ROMA_LEN = 4;
  static private char[] aiueo = { 'a', 'i', 'u', 'e', 'o', };
  static private String[] tbl = {
    "",  "アイウエオ",
    "k", "カキクケコ",
    "s", "サシスセソ",
    "t", "タチツテト",
    "n", "ナニヌネノ",
    "h", "ハヒフヘホ",
    "m", "マミムメモ",
    "y", "ヤイユ\0ヨ",
    "r", "ラリルレロ",
    "w", "ワ\0ウ\0ヲ",
    "g", "ガギグゲゴ",
    "z", "ザジズゼゾ",
    "d", "ダヂヅデド",
    "b", "バビブベボ",
    "p", "パピプペポ",
    "x", "ァィゥェォ",
    "l", "ァィゥェォ",
    "ly","ャィュェョ",
    "xy","ャィュェョ",

    "ky", "キャキィキュキェキョ",
    "sy", "シャシィシュシェショ",
    "sh", "シャシ\0シュシェショ",
    "th", "テャティテュテェテョ",
    "ch", "チャチ\0チュチェチョ",
    "cy", "チャチィチュチェチョ",
    "ty", "チャチィチュチェチョ",
    "ny", "ニャニィニュニェニョ",
    "hy", "ヒャヒィヒュヒェヒョ",
    "fy", "フャフィフュフェフョ",
    "my", "ミャミィミュミェミョ",
    "ry", "リャリィリュリェリョ",

    "gy", "ギャギィギュギェギョ",
    "zy", "ジャジィジュジェジョ",
    "j",  "ジャジ\0ジュジェジョ",
    "jy", "ジャジィジュジェジョ",
    "dy", "ヂャヂィヂュヂェヂョ",
    "dh", "デャディデュデェデョ",
    "by", "ビャビィビュビェビョ",
    "py", "ピャピィピュピェピョ",

    "wh", "ウァウィ\0\0ウェウォ",
    "ts", "ツァツィツ\0ツェツォ",
    "f",  "ファフィフ\0フェフォ",
    "v",  "ヴァヴィヴ\0ヴェヴォ",

    "y",  "\0\0\0\0\0\0イェ\0\0",
    "w",  "\0\0ウィ\0\0ウェ\0\0",
    "lk", "ヵ\0\0\0\0\0ヶ\0\0\0",
    "xk", "ヵ\0\0\0\0\0ヶ\0\0\0",
    "kw", "クァ\0\0\0\0\0\0\0\0",
    "gw", "グァ\0\0\0\0\0\0\0\0",
    "dw", "\0\0\0\0ドゥ\0\0\0\0",
    "lt", "\0\0\0\0ッ\0\0\0\0\0",
    "lts","\0\0\0\0ッ\0\0\0\0\0",
    "lw", "ヮ\0\0\0\0\0\0\0\0\0",
    "tw", "\0\0\0\0トゥ\0\0\0\0",
    "wy", "\0\0ヰ\0\0\0ヱ\0\0\0",
    "xt", "\0\0\0\0ッ\0\0\0\0\0",
    "xts","\0\0\0\0ッ\0\0\0\0\0",
    "xw", "ヮ\0\0\0\0\0\0\0\0\0",
  };
  private static HashMap<String, String> cnvTbl = new HashMap<String, String>();
  
  /**
   * 初期化
   */
  static private void initialize() {
    if (cnvTbl.size() > 0) return;
    Log.ut_start();
    
    for (int i = 0; i < tbl.length; i += 2) {
      Log.ut(tbl[i], ", ", tbl[i + 1]);
      
      String[] kana = null;
      if (tbl[i + 1].length() == 5) {
        kana = spl5(tbl[i + 1]);
      }
      else if (tbl[i + 1].length() == 10) {
        kana = spl10(tbl[i + 1]);
      }
      else {
        throw new RuntimeException("tbl[" + (i + 1) + "].len=" + tbl[i + 1].length() + ": " + tbl[i + 1]);
      }
      addTbl(tbl[i], kana);
    }
    addTbl("nn", "ン");
  }
  static private String[] spl5(String s) {
    assert s.length() == 5;
    String[] ret = new String[5];
    for (int i = 0; i < ret.length; i++) {
      ret[i] = s.substring(i, i + 1);
      if (ret[i].charAt(0) == '\0') ret[i] = null;
    }
    return ret;
  }
  private static String[] spl10(String s) {
    assert s.length() == 10;
    String[] ret = new String[5];
    for (int i = 0; i < ret.length; i++) {
      ret[i] = s.substring(i* 2, i * 2 + 2);
      if (ret[i].charAt(1) == '\0') ret[i] = ret[i].substring(0, 1);
      if (ret[i].charAt(0) == '\0') ret[i] = null;
    }
    return ret;
  }
  private static void addTbl(String prefix, String[] spl) {
    for (int i = 0; i < aiueo.length; i++) {
      if (spl[i] == null) continue;
      String s = prefix + aiueo[i];
      addTbl(s, spl[i]);
    }
  }
  private static void addTbl(String roma, String kata) {
    assert !cnvTbl.containsKey(roma);
    Log.ut(roma, " => ", kata);
    cnvTbl.put(roma, kata);
  }

  /**
   * 末尾の変換不能アルファベットを削除するかどうか
   * "aax" =(変換)=> "アアx" =(削除)=> "アア"
   * 
   * @param rmN 末尾の単独の "N" も削除するか？
   *     例 (true) "hon" -> "ホ"
   *        (false)"hon" -> "ホン"
   *     ※) 削除しないと、"ホネ" をインクリメンタル検索する途中で、
   *         "hon"(ホン) が出現してしまい、検索結果がおかしくなる場合がある
   */
  static public void setIgnoreTrailingAlphabet(boolean ignoreTrailingAlphabet, boolean rmN) {
    CnvRomaji2Kata.ignoreTrailingAlphabet = ignoreTrailingAlphabet;
    CnvRomaji2Kata.ignoreTrailingAlphabetRmN = rmN;
  }
  
  /**
   * ローマ字 -> カタカナ 変換
   */
  static public String cnv(String roma) {
    initialize();
    roma = roma.toLowerCase();
    StringBuilder sb = new StringBuilder();
    int i = 0;
    while (i < roma.length()) {
      boolean found = false;
      // 変換表の中を探す
      for (int len = MAX_ROMA_LEN; len > 0; len--) {
        String s = StringUtils.substring(roma, i, i + len);
        String kana = cnvTbl.get(s);
        if (kana == null) continue;
        sb.append(kana);
        i += len;
        found = true;
        break;
      }
      if (found) continue;
      
      char c = roma.charAt(i);

      // "ー"
      if (c ==  '-') {
        sb.append('ー'); 
        i++;
        continue;
      }

      // "ン"
      if (c ==  'n') {
        if (ignoreTrailingAlphabetRmN) {
          // 末尾の "N" は無変換
          if (i == roma.length() - 1) {
            sb.append(c);
            i++;
            continue;
          }
          // 末尾の "NY" は無変換
          if (i == roma.length() - 2 && roma.charAt(i + 1) == 'y') {
            sb.append(c);
            i++;
            continue;
          }
        }
        sb.append('ン'); 
        i++;
        continue;
      }
      
      // "っっっ…"
      if (roma.length() > i + 1 && roma.charAt(i + 1) == c && CharUtils.isAsciiAlpha(c)) {
        sb.append('ッ');
        i++;
        continue;
      }
      
      // 無変換
      sb.append(c);
      i++;
    }

    // 末尾の未変換アルファベットを削除する
    while (ignoreTrailingAlphabet && sb.length() > 0) {
      if (!CharUtils.isAsciiAlpha(sb.charAt(sb.length() - 1))) break;
      sb.delete(sb.length() - 1, sb.length());
    }
    return sb.toString();
  }
}
