package cvj2.util;

import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * int範囲のIterator
 */
public class RangeIterator implements Iterable<Integer>, Iterator<Integer> {
  private int from;
  private int to;
  private int current;
  
  /**
   * コンストラクタ
   * @param from 開始インデックス
   * @param to 終了インデックス + 1
   */
  public RangeIterator(int from, int to) {
    this.from = from;
    this.to = to;
    this.current = from;
  }

  /**
   * サイズ取得
   */
  public int size() {
    return Math.max(0, to - from);
  }
  
  /**
   * 値が範囲内かどうか
   */
  public boolean contains(int i) {
    return from <= i && i < to;
  }
  
  public int getFrom() {
    return from;
  }
  
  public int getTo() {
    return to;
  }
  
  @Override
  public boolean hasNext() {
    return current < to;
  }

  @Override
  public Integer next() {
    if (!hasNext()) throw new NoSuchElementException();
    return current++;
  }

  @Override
  public void remove() {
    throw new UnsupportedOperationException();
  }

  @Override
  public Iterator<Integer> iterator() {
    return this;
  }
}
