package cvj2.util;

import java.beans.ExceptionListener;
import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.Iterator;

/**
 * ユーティリティークラス
 */
public class Util {
  /**
   * Iterator を Iterable に変換する
   */
  static public <T> Iterable<T> getIterable(final Iterator<T> iterator) {
    return new Iterable<T>() {
      @Override
      public Iterator<T> iterator() {
        return iterator;
      }
    };
  }
  
  /**
   * InputStreamをバイト配列に変換する
   */
  static public byte[] toByteArray(InputStream in) throws IOException {
    ByteArrayOutputStream out = new ByteArrayOutputStream();
    byte[] buff = new byte[1024 * 4];
    while (true) {
      int read = in.read(buff);
      if (read < 0) break;
      out.write(buff, 0, read);
    }
    return out.toByteArray();
  }

  /**
   * オブジェクトの deep clone
   */
  static public Object deepClone(Object o) {
    try {
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      ObjectOutputStream oos = new ObjectOutputStream(baos);
      oos.writeObject(o);
      ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
      ObjectInputStream ois = new ObjectInputStream(bais);
      return ois.readObject();
    }
    catch (IOException e) {
      throw new RuntimeException(e);
    }
    catch (ClassNotFoundException e) {
      throw new RuntimeException(e);
    }
  }

  /**
   * XMLからオブジェクトを読み込む
   */
  static public Object readXml(File f) throws IOException {
    XMLDecoder dec = null;
    try {
      dec = new XMLDecoder(new FileInputStream(f));
      dec.setExceptionListener(new ExceptionListener() {
        @Override
        public void exceptionThrown(Exception e) {
          throw new RuntimeException(e);
        }
      });
      return dec.readObject();
    }
    finally {
      if (dec != null) dec.close();
    }
  }
  
  /**
   * オブジェクトをXMLで出力する
   */
  static public void writeXml(File f, Object obj) throws IOException {
    XMLEncoder enc = null;
    try {
      enc = new XMLEncoder(new FileOutputStream(f));
      enc.setExceptionListener(new ExceptionListener() {
        @Override
        public void exceptionThrown(Exception e) {
          throw new RuntimeException(e);
        }
      });
      enc.writeObject(obj);
    }
    finally {
      if (enc != null) enc.close();
    }
  }
  
  /**
   * String の使用メモリを推定する
   */
  static public long getObjectBytes(String s) {
    // (value), (strPtr, offset, count, hash)
    return (s.length() * 2 + 4) + (4 * 4);
  }
  
  /**
   * String の Collection の使用メモリを推定する
   */
  static public long getObjectBytesStringCollection(Collection<String> c) {
    long ret = c.size() * 4 + 4;
    for (String s : c) {
      ret += Util.getObjectBytes(s);
    }
    return ret;
  }

  /**
   * Collection の使用メモリを推定する
   */
  static public <E extends SizeCountable> long getObjectBytes(Collection<E> c) {
    long ret = c.size() * 4 + 4;
    for (SizeCountable s : c) {
      ret += s.getObjectBytes();
    }
    return ret;
  }
}
